@@ -1,8 +1,10 @@
-# Take multiple screenshots
-# 
-# Usage:
-#  multi_screenshot <num> [<base>]
-#  
+set_help_text multi_screenshot \
+{Take multiple screenshots
+
+Usage:
+ multi_screenshot <num> [<base>]
+}
+
 proc multi_screenshot {num {base ""} } {
 	multi_screenshot_helper 1 $num $base
 	return ""
@@ -10,7 +12,7 @@ proc multi_screenshot {num {base ""} } {
 
 proc multi_screenshot_helper {acc max {base ""} } {
 	if {$acc <= $max} {
-		if {$base == ""} { screenshot } { screenshot ${base}${acc}.png }
+		if {$base == ""} { screenshot } { screenshot -prefix ${base} }
 		after frame "multi_screenshot_helper [expr $acc + 1] $max $base"
 	}
 }
