@@ -1,9 +1,12 @@
 #
-# print memory nicely formatted
-#
-# Usage:
-#    showmem <address> [<linecount>]
+# show_mem
 #
+set_help_text showmem \
+{Print memory content nicely formatted
+
+Usage:
+   showmem <address> [<linecount>]
+}
 
 proc showmem_line { address } {
 	set data ""
