@@ -1,4 +1,4 @@
-// $Id: Autofire.cc,v 1.11 2005/03/11 16:22:56 m9710797 Exp $
+// $Id: Autofire.cc,v 1.16 2006/01/24 18:03:35 m9710797 Exp $
 
 #include "Autofire.hh"
 #include "IntegerSetting.hh"
@@ -7,11 +7,13 @@ using std::string;
 
 namespace openmsx {
 
-Autofire::Autofire(unsigned newMinInts, unsigned newMaxInts, const string& name)
+Autofire::Autofire(CommandController& commandController,
+                   unsigned newMinInts, unsigned newMaxInts, const string& name)
 	: min_ints(newMinInts)
 	, max_ints(newMaxInts)
-	, speedSetting(new IntegerSetting(name,
+	, speedSetting(new IntegerSetting(commandController, name,
 	               "controls the speed of this autofire circuit", 0, 0, 100))
+	, clock(EmuTime::zero)
 {
 	if (min_ints < 1) {
 		min_ints = 1;
@@ -21,12 +23,12 @@ Autofire::Autofire(unsigned newMinInts, unsigned newMaxInts, const string& name)
 	}
 	setClock();
 
-	speedSetting->addListener(this);
+	speedSetting->attach(*this);
 }
 
 Autofire::~Autofire()
 {
-	speedSetting->removeListener(this);
+	speedSetting->detach(*this);
 }
 
 void Autofire::setClock()
@@ -36,10 +38,10 @@ void Autofire::setClock()
 	    (2 * 50 * 60) / (max_ints - (speed * (max_ints - min_ints)) / 100));
 }
 
-void Autofire::update(const Setting* setting)
+void Autofire::update(const Setting& setting)
 {
-	if (setting); // avoid warning
-	assert(setting == speedSetting.get());
+	(void)setting;
+	assert(&setting == speedSetting.get());
 	setClock();
 }
 
