@@ -1,13 +1,7 @@
-// $Id: CommandLineParser.cc,v 1.132 2005/03/12 20:47:10 m9710797 Exp $
+// $Id: CommandLineParser.cc,v 1.158 2006/05/31 19:24:12 m9710797 Exp $
 
-#include <cassert>
-#include <string>
-#include <sstream>
-#include <iostream>
-#include <cstdio>
-#include "ReadDir.hh"
 #include "CommandLineParser.hh"
-#include "HardwareConfig.hh"
+#include "CommandController.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
@@ -18,10 +12,16 @@
 #include "CliExtension.hh"
 #include "CassettePlayer.hh"
 #include "DiskImageCLI.hh"
+#include "HDImageCLI.hh"
+#include "CDImageCLI.hh"
 #include "ConfigException.hh"
 #include "FileException.hh"
-#include "SettingsManager.hh"
 #include "EnumSetting.hh"
+#include "HostCPU.hh"
+#include "Reactor.hh"
+#include <cassert>
+#include <iostream>
+#include <cstdio>
 
 using std::cout;
 using std::endl;
@@ -53,49 +53,50 @@ string CLIOption::peekArgument(const list<string>& cmdLine) const
 
 // class CommandLineParser
 
-CommandLineParser& CommandLineParser::instance()
-{
-	static CommandLineParser oneInstance;
-	return oneInstance;
-}
-
-CommandLineParser::CommandLineParser()
+CommandLineParser::CommandLineParser(Reactor& reactor_)
 	: parseStatus(UNPARSED)
-	, sound(true)
-	, hardwareConfig(HardwareConfig::instance())
-	, settingsConfig(SettingsConfig::instance())
-	, output(CliComm::instance())
-	, settingsManager(SettingsManager::instance()) // lifetime management
+	, reactor(reactor_)
+	, settingsConfig(reactor.getCommandController().getSettingsConfig())
+	, output(reactor.getCliComm())
 	, helpOption(*this)
 	, versionOption(*this)
 	, controlOption(*this)
 	, machineOption(*this)
 	, settingOption(*this)
-	, noSoundOption(*this)
+	, testConfigOption(*this)
 	, msxRomCLI(new MSXRomCLI(*this))
 	, cliExtension(new CliExtension(*this))
 	, cassettePlayerCLI(new MSXCassettePlayerCLI(*this))
 	, diskImageCLI(new DiskImageCLI(*this))
+	, hdImageCLI(new HDImageCLI(*this))
+	, cdImageCLI(new CDImageCLI(*this))
 {
 	haveConfig = false;
 	haveSettings = false;
-	issuedHelp = false;
 
-	registerOption("-machine",  &machineOption, 3);
-	registerOption("-setting",  &settingOption, 2);
-	registerOption("-nosound",  &noSoundOption, 1, 1);
-	registerOption("-h",        &helpOption, 1, 1);
-	registerOption("--help",    &helpOption, 1, 1);
-	registerOption("-v",        &versionOption, 1, 1);
-	registerOption("--version", &versionOption, 1, 1);
-	registerOption("-control",  &controlOption, 1, 1);
+	registerOption("-machine",    &machineOption, 4);
+	registerOption("-setting",    &settingOption, 2);
+	registerOption("-h",          &helpOption, 1, 1);
+	registerOption("--help",      &helpOption, 1, 1);
+	registerOption("-v",          &versionOption, 1, 1);
+	registerOption("--version",   &versionOption, 1, 1);
+	registerOption("-control",    &controlOption, 1, 1);
+	registerOption("-script",     &scriptOption, 1, 1);
+	#ifdef ASM_X86
+	registerOption("-nommx",      &noMMXOption, 1, 1);
+	registerOption("-nommxext",   &noMMXEXTOption, 1, 1);
+	#endif
+	registerOption("-testconfig", &testConfigOption, 1, 1);
+	
+	registerFileTypes();
 }
 
 CommandLineParser::~CommandLineParser()
 {
 }
 
-void CommandLineParser::registerOption(const string &str, CLIOption* cliOption, byte prio, byte length)
+void CommandLineParser::registerOption(
+	const string& str, CLIOption* cliOption, byte prio, byte length)
 {
 	OptionData temp;
 	temp.option = cliOption;
@@ -104,13 +105,13 @@ void CommandLineParser::registerOption(const string &str, CLIOption* cliOption,
 	optionMap[str] = temp;
 }
 
-void CommandLineParser::registerFileClass(const string &str,
-		CLIFileType* cliFileType)
+void CommandLineParser::registerFileClass(
+	const string& str, CLIFileType* cliFileType)
 {
 	fileClassMap[str] = cliFileType;
 }
 
-void CommandLineParser::postRegisterFileTypes()
+void CommandLineParser::registerFileTypes()
 {
 	const XMLElement* config = settingsConfig.findChild("FileTypes");
 	if (config) {
@@ -142,13 +143,19 @@ void CommandLineParser::postRegisterFileTypes()
 	}
 }
 
-bool CommandLineParser::parseOption(const string& arg, list<string>& cmdLine, byte priority)
+bool CommandLineParser::parseOption(
+	const string& arg, list<string>& cmdLine, byte priority)
 {
 	map<string, OptionData>::const_iterator it1 = optionMap.find(arg);
 	if (it1 != optionMap.end()) {
 		// parse option
 		if (it1->second.prio <= priority) {
-			return it1->second.option->parseOption(arg, cmdLine);
+			try {
+				return it1->second.option->parseOption(
+					arg, cmdLine);
+			} catch (MSXException& e) {
+				throw FatalError(e.getMessage());
+			}
 		}
 	}
 	return false; // unknown
@@ -158,7 +165,8 @@ bool CommandLineParser::parseFileName(const string& arg, list<string>& cmdLine)
 {
 	string originalName(arg);
 	try {
-		File file(arg);
+		UserFileContext context(getReactor().getCommandController());
+		File file(context.resolve(arg));
 		originalName = file.getOriginalName();
 	} catch (FileException& e) {
 		// ignore
@@ -166,19 +174,16 @@ bool CommandLineParser::parseFileName(const string& arg, list<string>& cmdLine)
 	string::size_type begin = originalName.find_last_of('.');
 	if (begin != string::npos) {
 		// there is an extension
-		// TODO <filename>,<mappertype> is deprecated
-		string::size_type end = originalName.find_last_of(',');
-		string extension;
-		if ((end == string::npos) || (end <= begin)) {
-			extension = originalName.substr(begin + 1);
-		} else {
-			extension = originalName.substr(begin + 1, end - begin - 1);
-		}
+		string extension = originalName.substr(begin + 1);
 		FileTypeMap::const_iterator it2 = fileTypeMap.find(extension);
 		if (it2 != fileTypeMap.end()) {
-			// parse filetype
-			it2->second->parseFileType(arg, cmdLine);
-			return true; // file processed
+			try {
+				// parse filetype
+				it2->second->parseFileType(arg, cmdLine);
+				return true; // file processed
+			} catch (MSXException& e) {
+				throw FatalError(e.getMessage());
+			}
 		}
 	}
 	return false; // unknown
@@ -195,9 +200,9 @@ void CommandLineParser::parse(int argc, char** argv)
 		cmdLine.push_back(FileOperations::getConventionalPath(argv[i]));
 	}
 
-	for (int priority = 1; priority <= 7; priority++) {
+	for (int priority = 1; (priority <= 8) && (parseStatus != EXIT); ++priority) {
 		switch (priority) {
-		case 4:
+		case 3:
 			if (!haveSettings) {
 				// Load default settings file in case the user didn't specify
 				// one.
@@ -213,14 +218,12 @@ void CommandLineParser::parse(int argc, char** argv)
 				// Consider an attempt to load the settings good enough.
 				haveSettings = true;
 			}
-			postRegisterFileTypes();
 			break;
 		case 5: {
-			createMachineSetting();
-
-			if (!issuedHelp && !haveConfig) {
+			if (!haveConfig) {
 				// load default config file in case the user didn't specify one
-				const string& machine = machineSetting->getValueString();
+				const string& machine =
+					reactor.getMachineSetting().getValueString();
 				output.printInfo("Using default machine: " + machine);
 				loadMachine(machine);
 				haveConfig = true;
@@ -235,7 +238,8 @@ void CommandLineParser::parse(int argc, char** argv)
 				// first try options
 				if (!parseOption(arg, cmdLine, priority)) {
 					// next try the registered filetypes (xml)
-					if (!parseFileName(arg, cmdLine)) {
+					if ((priority < 7) ||
+					    !parseFileName(arg, cmdLine)) {
 						// no option or known file
 						backupCmdLine.push_back(arg);
 						map<string, OptionData>::const_iterator it1 =
@@ -256,11 +260,18 @@ void CommandLineParser::parse(int argc, char** argv)
 			break;
 		}
 	}
-	if (!cmdLine.empty()) {
+	if (!cmdLine.empty() && (parseStatus != EXIT)) {
 		throw FatalError(
 			"Error parsing command line: " + cmdLine.front() + "\n" +
 			"Use \"openmsx -h\" to see a list of available options" );
 	}
+
+	hiddenStartup = (parseStatus == CONTROL || parseStatus == TEST );
+}
+
+bool CommandLineParser::isHiddenStartup() const
+{
+	return hiddenStartup;
 }
 
 CommandLineParser::ParseStatus CommandLineParser::getParseStatus() const
@@ -269,87 +280,44 @@ CommandLineParser::ParseStatus CommandLineParser::getParseStatus() const
 	return parseStatus;
 }
 
-bool CommandLineParser::wantSound() const
+const CommandLineParser::Scripts& CommandLineParser::getStartupScripts() const
 {
-	assert(parseStatus != UNPARSED);
-	return sound;
+	return scriptOption.getScripts();
+}
+
+Reactor& CommandLineParser::getReactor() const
+{
+	return reactor;
+}
+
+MSXMotherBoard* CommandLineParser::getMotherBoard() const
+{
+	return getReactor().getMotherBoard();
 }
 
 void CommandLineParser::loadMachine(const string& machine)
 {
 	try {
-		hardwareConfig.loadHardware(hardwareConfig, "machines", machine);
-	} catch (FileException& e) {
-		throw FatalError("Machine \"" + machine + "\" not found (" +
-		                 e.getMessage() + ").");
-	} catch (ConfigException& e) {
-		throw FatalError("Error in \"" + machine + "\" machine (" +
-		                 e.getMessage() + ").");
+		getReactor().createMotherBoard(machine);
+	} catch (MSXException& e) {
+		throw FatalError(e.getMessage());
 	}
 }
 
-static int select(const string& basepath, const struct dirent* d)
-{
-	// entry must be a directory and must contain the file "hardwareconfig.xml"
-	string name = basepath + '/' + d->d_name;
-	return FileOperations::isDirectory(name) &&
-	       FileOperations::isRegularFile(name + "/hardwareconfig.xml");
-}
-
-static void searchMachines(const string& basepath, EnumSetting<int>::Map& machines)
-{
-	static int unique = 1;
-	ReadDir dir(basepath);
-	while (dirent* d = dir.getEntry()) {
-		if (select(basepath, d)) {
-			machines[d->d_name] = unique++; // dummy value
-		}
-	}
-}
-
-void CommandLineParser::createMachineSetting()
-{
-	EnumSetting<int>::Map machines;
-
-	SystemFileContext context;
-	const vector<string>& paths = context.getPaths();
-	for (vector<string>::const_iterator it = paths.begin();
-	     it != paths.end(); ++it) {
-		searchMachines(*it + "machines", machines);
-	}
-
-	machines["C-BIOS_MSX2+"] = 0; // default machine
-
-	machineSetting.reset(new EnumSetting<int>("machine",
-		"default machine (takes effect next time openMSX is started)",
-		0, machines));
-}
-
 // Control option
 
-CommandLineParser::ControlOption::ControlOption(CommandLineParser& parent_)
-	: parent(parent_)
-{
-}
-
-CommandLineParser::ControlOption::~ControlOption()
+CommandLineParser::ControlOption::ControlOption(CommandLineParser& parser_)
+	: parser(parser_)
 {
 }
 
 bool CommandLineParser::ControlOption::parseOption(const string& option,
 		list<string>& cmdLine)
 {
-	string arg(getArgument(option, cmdLine));
-	string::size_type colon = arg.find(':');
-	string type_name;
-	string arguments;
-	if (colon == string::npos) {
-		type_name = arg;
-		//arguments = "";
-	} else {
-		type_name = arg.substr(0, colon);
-		arguments = arg.substr(colon + 1);
-	}
+	string type_name, arguments;
+	StringOp::splitOnFirst(getArgument(option, cmdLine),
+	                       ":", type_name, arguments);
+
 	map<string, CommandLineParser::ControlType> controlTypeMap;
 	controlTypeMap["stdio"] = IO_STD;
 #ifdef _WIN32
@@ -360,8 +328,8 @@ bool CommandLineParser::ControlOption::parseOption(const string& option,
 	}
 	CommandLineParser::ControlType type = controlTypeMap[type_name];
 
-	CliComm::instance().startInput(type, arguments);
-	parent.parseStatus = CONTROL;
+	parser.getReactor().getCliComm().startInput(type, arguments);
+	parser.parseStatus = CONTROL;
 	return true;
 }
 
@@ -372,6 +340,28 @@ const string& CommandLineParser::ControlOption::optionHelp() const
 }
 
 
+// Script option
+
+const CommandLineParser::Scripts&
+CommandLineParser::ScriptOption::getScripts() const
+{
+	return scripts;
+}
+
+bool CommandLineParser::ScriptOption::parseOption(const string& option,
+		list<string>& cmdLine)
+{
+	scripts.push_back(getArgument(option, cmdLine));
+	return true;
+}
+
+const string& CommandLineParser::ScriptOption::optionHelp() const
+{
+	static const string text("Run extra startup script");
+	return text;
+}
+
+
 // Help option
 
 static string formatSet(const set<string>& inputSet, string::size_type columns)
@@ -437,20 +427,15 @@ static void printItemMap(const map<string, set<string> >& itemMap)
 	}
 }
 
-CommandLineParser::HelpOption::HelpOption(CommandLineParser& parent_)
-	: parent(parent_)
-{
-}
-
-CommandLineParser::HelpOption::~HelpOption()
+CommandLineParser::HelpOption::HelpOption(CommandLineParser& parser_)
+	: parser(parser_)
 {
 }
 
 bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
-	parent.issuedHelp = true;
-	if (!parent.haveSettings) {
+	if (!parser.haveSettings) {
 		return false; // not parsed yet, load settings first
 	}
 	cout << Version::FULL_VERSION << endl;
@@ -462,8 +447,8 @@ bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
 	cout << "  this is the list of supported options:" << endl;
 
 	map<string, set<string> > optionMap;
-	for (map<string, OptionData>::const_iterator it = parent.optionMap.begin();
-	     it != parent.optionMap.end(); ++it) {
+	for (map<string, OptionData>::const_iterator it = parser.optionMap.begin();
+	     it != parser.optionMap.end(); ++it) {
 		optionMap[it->second.option->optionHelp()].insert(it->first);
 	}
 	printItemMap(optionMap);
@@ -472,13 +457,13 @@ bool CommandLineParser::HelpOption::parseOption(const string& /*option*/,
 	cout << "  this is the list of supported file types:" << endl;
 
 	map<string, set<string> > extMap;
-	for (FileTypeMap::const_iterator it = parent.fileTypeMap.begin();
-	     it != parent.fileTypeMap.end(); ++it) {
+	for (FileTypeMap::const_iterator it = parser.fileTypeMap.begin();
+	     it != parser.fileTypeMap.end(); ++it) {
 		extMap[it->second->fileTypeHelp()].insert(it->first);
 	}
 	printItemMap(extMap);
 
-	parent.parseStatus = EXIT;
+	parser.parseStatus = EXIT;
 	return true;
 }
 
@@ -491,21 +476,16 @@ const string& CommandLineParser::HelpOption::optionHelp() const
 
 // class VersionOption
 
-CommandLineParser::VersionOption::VersionOption(CommandLineParser& parent_)
-	: parent(parent_)
-{
-}
-
-CommandLineParser::VersionOption::~VersionOption()
+CommandLineParser::VersionOption::VersionOption(CommandLineParser& parser_)
+	: parser(parser_)
 {
 }
 
 bool CommandLineParser::VersionOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
-	parent.issuedHelp = true;
 	cout << Version::FULL_VERSION << endl;
-	parent.parseStatus = EXIT;
+	parser.parseStatus = EXIT;
 	return true;
 }
 
@@ -518,28 +498,21 @@ const string& CommandLineParser::VersionOption::optionHelp() const
 
 // Machine option
 
-CommandLineParser::MachineOption::MachineOption(CommandLineParser& parent_)
-	: parent(parent_)
+CommandLineParser::MachineOption::MachineOption(CommandLineParser& parser_)
+	: parser(parser_)
 {
 }
 
-CommandLineParser::MachineOption::~MachineOption()
+bool CommandLineParser::MachineOption::parseOption(const string& option,
+		list<string>& cmdLine)
 {
-}
-
-bool CommandLineParser::MachineOption::parseOption(const string &option,
-		list<string> &cmdLine)
-{
-	if (parent.haveConfig) {
+	if (parser.haveConfig) {
 		throw FatalError("Only one machine option allowed");
 	}
 	string machine(getArgument(option, cmdLine));
-	if (parent.issuedHelp) {
-		return true;
-	}
-	parent.output.printInfo("Using specified machine: " + machine);
-	parent.loadMachine(machine);
-	parent.haveConfig = true;
+	parser.output.printInfo("Using specified machine: " + machine);
+	parser.loadMachine(machine);
+	parser.haveConfig = true;
 	return true;
 }
 const string& CommandLineParser::MachineOption::optionHelp() const
@@ -551,26 +524,24 @@ const string& CommandLineParser::MachineOption::optionHelp() const
 
 // Setting Option
 
-CommandLineParser::SettingOption::SettingOption(CommandLineParser& parent_)
-	: parent(parent_)
-{
-}
-
-CommandLineParser::SettingOption::~SettingOption()
+CommandLineParser::SettingOption::SettingOption(CommandLineParser& parser_)
+	: parser(parser_)
 {
 }
 
 bool CommandLineParser::SettingOption::parseOption(const string &option,
 		list<string> &cmdLine)
 {
-	if (parent.haveSettings) {
+	if (parser.haveSettings) {
 		throw FatalError("Only one setting option allowed");
 	}
 	try {
-		UserFileContext context("", true); // skip user directories
-		parent.settingsConfig.loadSetting(
+		UserFileContext context(
+		        parser.getReactor().getCommandController(),
+		        "", true); // skip user directories
+		parser.settingsConfig.loadSetting(
 			context, getArgument(option, cmdLine));
-		parent.haveSettings = true;
+		parser.haveSettings = true;
 	} catch (FileException& e) {
 		throw FatalError(e.getMessage());
 	} catch (ConfigException& e) {
@@ -578,6 +549,7 @@ bool CommandLineParser::SettingOption::parseOption(const string &option,
 	}
 	return true;
 }
+
 const string& CommandLineParser::SettingOption::optionHelp() const
 {
 	static const string text("Load an alternative settings file");
@@ -585,27 +557,59 @@ const string& CommandLineParser::SettingOption::optionHelp() const
 }
 
 
-// NoSoundOption
+// class NoMMXOption
 
-CommandLineParser::NoSoundOption::NoSoundOption(CommandLineParser& parent_)
-	: parent(parent_)
-{
-}
-
-CommandLineParser::NoSoundOption::~NoSoundOption()
-{
-}
-
-bool CommandLineParser::NoSoundOption::parseOption(const string& /*option*/,
+bool CommandLineParser::NoMMXOption::parseOption(const string& /*option*/,
 		list<string>& /*cmdLine*/)
 {
-	parent.sound = false;
+	cout << "Disabling MMX " << endl;
+	HostCPU::getInstance().forceDisableMMX();
 	return true;
 }
 
-const string& CommandLineParser::NoSoundOption::optionHelp() const
+const string& CommandLineParser::NoMMXOption::optionHelp() const
 {
-	static const string text("Disable sound output");
+	static const string text(
+		"Disables usage of MMX, including extensions (for debugging)");
+	return text;
+}
+
+
+// class NoMMXEXTOption
+
+bool CommandLineParser::NoMMXEXTOption::parseOption(const string& /*option*/,
+		list<string>& /*cmdLine*/)
+{
+	cout << "Disabling MMX extensions" << endl;
+	HostCPU::getInstance().forceDisableMMXEXT();
+	return true;
+}
+
+const string& CommandLineParser::NoMMXEXTOption::optionHelp() const
+{
+	static const string text(
+		"Disables usage of MMX extensions (for debugging)");
+	return text;
+}
+
+
+// TestConfig option
+
+CommandLineParser::TestConfigOption::TestConfigOption(CommandLineParser& parser_)
+	: parser(parser_)
+{
+}
+
+bool CommandLineParser::TestConfigOption::parseOption(const string& /*option*/,
+		list<string>& /*cmdLine*/)
+{
+	parser.parseStatus = TEST;
+	return true;
+}
+
+const string& CommandLineParser::TestConfigOption::optionHelp() const
+{
+	static const string text("Test if the specified config works and exit");
 	return text;
 }
 
