@@ -1,4 +1,4 @@
-// $Id: Connector.cc,v 1.12 2005/01/13 18:25:36 m9710797 Exp $
+// $Id: Connector.cc,v 1.13 2005/11/11 20:53:05 m9710797 Exp $
 
 #include "Connector.hh"
 #include "Pluggable.hh"
@@ -20,10 +20,10 @@ const std::string& Connector::getName() const
 	return name;
 }
 
-void Connector::plug(Pluggable* device, const EmuTime& time)
+void Connector::plug(Pluggable& device, const EmuTime& time)
 {
-	device->plug(this, time);
-	plugged = device; // not executed if plug fails
+	device.plug(*this, time);
+	plugged = &device; // not executed if plug fails
 }
 
 void Connector::unplug(const EmuTime& time)
