@@ -1,4 +1,4 @@
-// $Id: DebugDevice.cc,v 1.22 2005/05/11 18:54:17 m9710797 Exp $
+// $Id: DebugDevice.cc,v 1.28 2006/01/23 19:35:25 m9710797 Exp $
 
 #include <iostream>
 #include <iomanip>
@@ -7,17 +7,20 @@
 #include "FileOperations.hh"
 #include "FilenameSetting.hh"
 #include "XMLElement.hh"
+#include "MSXMotherBoard.hh"
 
 using std::string;
 
 namespace openmsx {
 
-DebugDevice::DebugDevice(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
+DebugDevice::DebugDevice(MSXMotherBoard& motherBoard, const XMLElement& config,
+                         const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
 {
 	mode = OFF;
 	string outputFile = config.getChildData("filename", "stdout");
-	fileNameSetting.reset(new FilenameSetting("debugoutput",
+	fileNameSetting.reset(new FilenameSetting(
+		motherBoard.getCommandController(), "debugoutput",
 		"name of the file the debugdevice outputs to", outputFile));
 	fileNameString = fileNameSetting->getValueString();
 	openOutput(fileNameString);
@@ -27,21 +30,21 @@ DebugDevice::~DebugDevice()
 {
 }
 
-void DebugDevice::writeIO(byte port, byte value, const EmuTime& time)
+void DebugDevice::writeIO(word port, byte value, const EmuTime& time)
 {
 	string currentName = fileNameSetting->getValueString();
 	if (currentName != fileNameString) {
 		fileNameString = currentName;
 		openOutput(fileNameString);
 	}
-	
+
 	switch (port & 0x01) {
 		case 0:
 			switch ((value & 0x30) >> 4) {
-				case 0: 
+				case 0:
 					mode = OFF;
 					break;
-				case 1: 
+				case 1:
 					mode = SINGLEBYTE;
 					modeParameter = value & 0x0F;
 					break;
@@ -52,7 +55,7 @@ void DebugDevice::writeIO(byte port, byte value, const EmuTime& time)
 				case 3:
 					break;
 			}
-			if (!(value & 0x40)){ 
+			if (!(value & 0x40)){
 				(*outputstrm) << std::endl;
 			}
 			break;
@@ -92,8 +95,8 @@ void DebugDevice::outputSingleByte(byte value, const EmuTime& time)
 		}
 		(*outputstrm) << "' ";
 	}
-	Clock<3579545> zero;
-	(*outputstrm) << "emutime: " << zero.getTicksTill(time); 
+	Clock<3579545> zero(EmuTime::zero);
+	(*outputstrm) << "emutime: " << std::dec << zero.getTicksTill(time);
 	if ((modeParameter & 0x08) && ((value < ' ') || (value == 127))) {
 		displayByte(value, ASC); // do special effects
 	}
@@ -104,7 +107,7 @@ void DebugDevice::outputMultiByte(byte value)
 {
 	DisplayType dispType;
 	switch (modeParameter) {
-		case 0: 
+		case 0:
 			dispType = HEX;
 			break;
 		case 1:
@@ -117,7 +120,7 @@ void DebugDevice::outputMultiByte(byte value)
 		default:
 			dispType = ASC;
 			break;
-	}	
+	}
 	displayByte(value, dispType);
 }
 
@@ -158,7 +161,7 @@ void DebugDevice::openOutput(const string& name)
 	debugOut.close();
 	if (name == "stdout") {
 		outputstrm = &std::cout;
-	} else if (name == "stderr") { 
+	} else if (name == "stderr") {
 		outputstrm = &std::cerr;
 	} else {
 		string realName = FileOperations::expandTilde(name);
