@@ -1,17 +1,21 @@
-// $Id: FirmwareSwitch.cc,v 1.3 2005/01/13 18:25:36 m9710797 Exp $
+// $Id: FirmwareSwitch.cc,v 1.5 2005/11/12 21:53:20 m9710797 Exp $
 
 #include "FirmwareSwitch.hh"
 #include "BooleanSetting.hh"
 
 namespace openmsx {
 
-FirmwareSwitch::FirmwareSwitch()
-	: setting(new BooleanSetting("firmwareswitch",
+FirmwareSwitch::FirmwareSwitch(CommandController& commandController)
+	: setting(new BooleanSetting(commandController, "firmwareswitch",
 	          "This setting controls the firmware switch",
 	          false))
 {
 }
 
+FirmwareSwitch::~FirmwareSwitch()
+{
+}
+
 bool FirmwareSwitch::getStatus() const
 {
 	return setting->getValue();
