@@ -1,21 +1,16 @@
-// $Id: HostCPU.cc,v 1.6 2004/12/15 09:09:42 m9710797 Exp $
+// $Id: HostCPU.cc,v 1.9 2006/03/26 17:05:00 mthuurne Exp $
 
 #include "HostCPU.hh"
 #include "openmsx.hh"
-#include <cassert>
-#include <cstdio>
-#include <iostream>
-#include <iomanip>
 
 namespace openmsx {
 
 HostCPU::HostCPU()
 {
-	PRT_DEBUG("probing host CPU...");
-
 	mmxFlag = false;
 	mmxExtFlag = false;
-	#ifdef ASM_X86 
+	#ifdef ASM_X86
+		// Note: On Mac OS X, EBX is in use by the OS, so we have to restore it.
 		// Is CPUID instruction supported?
 		unsigned hasCPUID;
 		asm (
@@ -23,7 +18,7 @@ HostCPU::HostCPU()
 			"pushfl;"
 			"popl	%%eax;"
 			// Save current value.
-			"movl	%%eax,%%ebx;"
+			"movl	%%eax,%%ecx;"
 			// Toggle bit 21.
 			"xorl	$0x200000, %%eax;"
 			// Load EAX into EFLAGS.
@@ -33,31 +28,34 @@ HostCPU::HostCPU()
 			"pushfl;"
 			"popl	%%eax;"
 			// Did bit 21 change?
-			"xor	%%ebx, %%eax;"
+			"xor	%%ecx, %%eax;"
 			"andl	$0x200000, %%eax;"
 			: "=a" (hasCPUID) // 0
 			: // no input
-			: "ebx"
+			: "ecx"
 			);
 		if (hasCPUID) {
 			// Which CPUID calls are supported?
 			unsigned highest;
 			asm (
+				"pushl	%%ebx;"
 				"cpuid;"
+				"popl	%%ebx;"
 				: "=a" (highest) // 0
 				: "0" (0) // 1: function
-				: "ebx", "ecx", "edx"
+				: "ecx", "edx"
 				);
 			if (highest >= 1) {
 				// Get features flags.
 				unsigned features;
 				asm (
+					"pushl	%%ebx;"
 					"cpuid;"
+					"popl	%%ebx;"
 					: "=d" (features) // 0
 					: "a" (1) // 1: function
-					: "ebx", "ecx"
+					: "ecx"
 					);
-				PRT_DEBUG("CPU flags: " << std::hex << std::setw(8) << std::setfill('0') << std::uppercase << features);
 				mmxFlag = features & 0x800000;
 				bool sseFlag = features & 0x2000000;
 				mmxExtFlag = mmxFlag && sseFlag;
@@ -70,10 +68,3 @@ HostCPU::HostCPU()
 }
 
 } // namespace openmsx
-
-/*
-int main(char** argv, int argc) {
-	openmsx::HostCPU::getInstance();
-	return 0;
-}
-*/
