@@ -1,4 +1,4 @@
-// $Id: IPSPatch.cc,v 1.4 2005/04/23 18:58:35 m9710797 Exp $
+// $Id: IPSPatch.cc,v 1.8 2006/01/25 19:26:31 m9710797 Exp $
 
 #include "IPSPatch.hh"
 #include "File.hh"
@@ -10,7 +10,7 @@ using std::vector;
 
 namespace openmsx {
 
-unsigned IPSPatch::getStop(const PatchMap::const_iterator& it)
+unsigned IPSPatch::getStop(const IPSPatch::PatchMap::const_iterator& it)
 {
 	return it->first + it->second.size();
 }
@@ -24,7 +24,7 @@ IPSPatch::IPSPatch(const std::string& filename,
 	byte buf[5];
 	ipsFile.read(buf, 5);
 	if (memcmp(buf, "PATCH", 5) != 0) {
-		throw FatalError("Invalid IPS file: " + filename);
+		throw MSXException("Invalid IPS file: " + filename);
 	}
 	ipsFile.read(buf, 3);
 	while (memcmp(buf, "EOF", 3) != 0) {
@@ -68,7 +68,7 @@ IPSPatch::IPSPatch(const std::string& filename,
 			// add new region
 			patchMap[offset] = v;
 		}
-		
+
 		ipsFile.read(buf, 3);
 	}
 	if (patchMap.empty()) {
@@ -82,7 +82,7 @@ IPSPatch::IPSPatch(const std::string& filename,
 void IPSPatch::copyBlock(unsigned src, byte* dst, unsigned num) const
 {
 	parent->copyBlock(src, dst, num);
-	
+
 	PatchMap::const_iterator b = patchMap.lower_bound(src);
 	if (b != patchMap.begin()) --b;
 	PatchMap::const_iterator e = patchMap.upper_bound(src + num - 1);
