@@ -1,26 +1,23 @@
-// $Id: MSXE6Timer.cc,v 1.28 2004/10/19 19:47:43 m9710797 Exp $
+// $Id: MSXE6Timer.cc,v 1.32 2006/01/23 19:35:25 m9710797 Exp $
 
-#include <cassert>
 #include "MSXE6Timer.hh"
 
 namespace openmsx {
 
-MSXE6Timer::MSXE6Timer(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
+MSXE6Timer::MSXE6Timer(MSXMotherBoard& motherBoard, const XMLElement& config,
+                       const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, reference(time)
 {
 	reset(time);
 }
 
-MSXE6Timer::~MSXE6Timer()
-{
-}
-
 void MSXE6Timer::reset(const EmuTime& time)
 {
 	reference.advance(time);
 }
 
-void MSXE6Timer::writeIO(byte /*port*/, byte /*value*/, const EmuTime& time)
+void MSXE6Timer::writeIO(word /*port*/, byte /*value*/, const EmuTime& time)
 {
 	/*
 	The Clock class rounds down time to its clock resolution.
@@ -40,23 +37,15 @@ void MSXE6Timer::writeIO(byte /*port*/, byte /*value*/, const EmuTime& time)
 	reference.advance(time);
 }
 
-byte MSXE6Timer::readIO(byte port, const EmuTime& time)
+byte MSXE6Timer::readIO(word port, const EmuTime& time)
 {
 	return peekIO(port, time);
 }
 
-byte MSXE6Timer::peekIO(byte port, const EmuTime& time) const
+byte MSXE6Timer::peekIO(word port, const EmuTime& time) const
 {
 	int counter = reference.getTicksTill(time);
-	switch (port & 0x01) {
-	case 0:
-		return counter & 0xFF;
-	case 1:
-		return (counter >> 8) & 0xFF;
-	default: // unreachable, avoid warning
-		assert(false);
-		return 0;
-	}
+	return (port & 1) ? ((counter >> 8) & 0xFF) : (counter & 0xFF);
 }
 
 } // namespace openmsx
