@@ -1,17 +1,19 @@
-// $Id: MSXKanji.cc,v 1.45 2004/10/19 19:47:43 m9710797 Exp $
+// $Id: MSXKanji.cc,v 1.49 2006/01/25 19:26:31 m9710797 Exp $
 
-#include <cassert>
 #include "MSXKanji.hh"
+#include "Rom.hh"
 #include "MSXException.hh"
 
 namespace openmsx {
 
-MSXKanji::MSXKanji(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time), rom(getName(), "Kanji ROM", config)
+MSXKanji::MSXKanji(MSXMotherBoard& motherBoard, const XMLElement& config,
+                   const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, rom(new Rom(motherBoard, getName(), "Kanji ROM", config))
 {
-	int size = rom.getSize();
+	int size = rom->getSize();
 	if ((size != 0x20000) && (size != 0x40000)) {
-		throw FatalError("MSXKanji: wrong kanji rom");
+		throw MSXException("MSXKanji: wrong kanji rom");
 	}
 
 	reset(time);
@@ -27,7 +29,7 @@ void MSXKanji::reset(const EmuTime& /*time*/)
 	adr2 = 0x20000;	// TODO check this
 }
 
-void MSXKanji::writeIO(byte port, byte value, const EmuTime& /*time*/)
+void MSXKanji::writeIO(word port, byte value, const EmuTime& /*time*/)
 {
 	//PRT_DEBUG("MSXKanji: write " << (int)port << " " << (int)value);
 	switch (port & 0x03) {
@@ -43,12 +45,10 @@ void MSXKanji::writeIO(byte port, byte value, const EmuTime& /*time*/)
 	case 3:
 		adr2 = (adr2 & 0x207e0) | ((value & 0x3f) << 11);
 		break;
-	default:
-		assert(false);
 	}
 }
 
-byte MSXKanji::readIO(byte port, const EmuTime& time)
+byte MSXKanji::readIO(word port, const EmuTime& time)
 {
 	byte result = peekIO(port, time);
 	switch (port & 0x03) {
@@ -63,16 +63,16 @@ byte MSXKanji::readIO(byte port, const EmuTime& time)
 	return result;
 }
 
-byte MSXKanji::peekIO(byte port, const EmuTime& /*time*/) const
+byte MSXKanji::peekIO(word port, const EmuTime& /*time*/) const
 {
 	byte result;
 	switch (port & 0x03) {
 	case 1:
-		result = rom[adr1];
+		result = (*rom)[adr1];
 		break;
 	case 3:
-		if (rom.getSize() == 0x40000) { // temp workaround
-			result = rom[adr2];
+		if (rom->getSize() == 0x40000) { // temp workaround
+			result = (*rom)[adr2];
 		} else {
 			result = 0xFF;
 		}
