@@ -1,31 +1,35 @@
-// $Id: MSXPPI.cc,v 1.73 2005/03/01 17:16:39 m9710797 Exp $
+// $Id: MSXPPI.cc,v 1.82 2006/05/01 13:21:48 m9710797 Exp $
 
-#include <cassert>
 #include "MSXPPI.hh"
 #include "Keyboard.hh"
 #include "LedEvent.hh"
 #include "EventDistributor.hh"
 #include "MSXCPUInterface.hh"
+#include "MSXMotherBoard.hh"
 #include "KeyClick.hh"
 #include "CassettePort.hh"
 #include "XMLElement.hh"
 #include "RenShaTurbo.hh"
+#include <cassert>
 
 namespace openmsx {
 
 // MSXDevice
 
-MSXPPI::MSXPPI(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, cassettePort(CassettePortFactory::instance())
-	, cpuInterface(MSXCPUInterface::instance())
-	, renshaTurbo(RenShaTurbo::instance())
+MSXPPI::MSXPPI(MSXMotherBoard& motherBoard, const XMLElement& config,
+               const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, cassettePort(motherBoard.getCassettePort())
+	, renshaTurbo(motherBoard.getRenShaTurbo())
 	, prevBits(15)
 {
 	bool keyGhosting = deviceConfig.getChildDataAsBool("key_ghosting", true);
-	keyboard.reset(new Keyboard(keyGhosting));
+	keyboard.reset(new Keyboard(motherBoard.getScheduler(),
+	                            motherBoard.getCommandController(),
+	                            motherBoard.getUserInputEventDistributor(),
+	                            keyGhosting));
 	i8255.reset(new I8255(*this, time));
-	click.reset(new KeyClick(config, time));
+	click.reset(new KeyClick(motherBoard.getMixer(), config, time));
 
 	reset(time);
 }
@@ -42,11 +46,11 @@ void MSXPPI::reset(const EmuTime& time)
 
 void MSXPPI::powerDown(const EmuTime& /*time*/)
 {
-	EventDistributor::instance().distributeEvent(
+	getMotherBoard().getEventDistributor().distributeEvent(
 		new LedEvent(LedEvent::CAPS, false));
 }
 
-byte MSXPPI::readIO(byte port, const EmuTime& time)
+byte MSXPPI::readIO(word port, const EmuTime& time)
 {
 	switch (port & 0x03) {
 	case 0:
@@ -63,7 +67,7 @@ byte MSXPPI::readIO(byte port, const EmuTime& time)
 	}
 }
 
-byte MSXPPI::peekIO(byte port, const EmuTime& time) const
+byte MSXPPI::peekIO(word port, const EmuTime& time) const
 {
 	switch (port & 0x03) {
 	case 0:
@@ -80,7 +84,7 @@ byte MSXPPI::peekIO(byte port, const EmuTime& time) const
 	}
 }
 
-void MSXPPI::writeIO(byte port, byte value, const EmuTime& time)
+void MSXPPI::writeIO(word port, byte value, const EmuTime& time)
 {
 	switch (port & 0x03) {
 	case 0:
@@ -111,11 +115,15 @@ byte MSXPPI::peekA(const EmuTime& /*time*/) const
 {
 	// port A is normally an output on MSX, reading from an output port
 	// is handled internally in the 8255
-	return 255;	//TODO check this
+	// TODO check this on a real MSX
+	// TODO returning 0 fixes the 'get_selected_slot' script right after
+	//      reset (when PPI directions are not yet set). For now this
+	//      solution is good enough.
+	return 0;
 }
 void MSXPPI::writeA(byte value, const EmuTime& /*time*/)
 {
-	cpuInterface.setPrimarySlots(value);
+	getMotherBoard().getCPUInterface().setPrimarySlots(value);
 }
 
 byte MSXPPI::readB(const EmuTime& time)
@@ -141,7 +149,7 @@ nibble MSXPPI::readC1(const EmuTime& time)
 }
 nibble MSXPPI::peekC1(const EmuTime& /*time*/) const
 {
-	return 15;	// TODO check this
+	return 15; // TODO check this
 }
 nibble MSXPPI::readC0(const EmuTime& time)
 {
@@ -149,7 +157,7 @@ nibble MSXPPI::readC0(const EmuTime& time)
 }
 nibble MSXPPI::peekC0(const EmuTime& /*time*/) const
 {
-	return 15;	// TODO check this
+	return 15; // TODO check this
 }
 void MSXPPI::writeC1(nibble value, const EmuTime& time)
 {
@@ -160,7 +168,7 @@ void MSXPPI::writeC1(nibble value, const EmuTime& time)
 		cassettePort.cassetteOut(value & 2, time);
 	}
 	if ((prevBits ^ value) & 4) {
-		EventDistributor::instance().distributeEvent(
+		getMotherBoard().getEventDistributor().distributeEvent(
 			new LedEvent(LedEvent::CAPS, !(value & 4)));
 	}
 	if ((prevBits ^ value) & 8) {
