@@ -1,4 +1,4 @@
-// $Id: MSXS1985.cc,v 1.14 2005/01/17 19:29:31 m9710797 Exp $
+// $Id: MSXS1985.cc,v 1.17 2005/12/03 18:58:10 arnoldmnl Exp $
 
 #include "MSXS1985.hh"
 #include "Ram.hh"
@@ -7,10 +7,11 @@ namespace openmsx {
 
 const byte ID = 0xFE;
 
-MSXS1985::MSXS1985(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, MSXSwitchedDevice(ID)
-	, ram(new Ram(getName() + " RAM", "S1985 RAM", 0x10))
+MSXS1985::MSXS1985(MSXMotherBoard& motherBoard, const XMLElement& config,
+                   const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, MSXSwitchedDevice(motherBoard, ID)
+	, ram(new Ram(motherBoard, getName() + " RAM", "S1985 RAM", 0x10))
 {
 	// TODO load ram
 }
@@ -24,7 +25,7 @@ void MSXS1985::reset(const EmuTime& /*time*/)
 {
 }
 
-byte MSXS1985::readIO(byte port, const EmuTime& time)
+byte MSXS1985::readIO(word port, const EmuTime& time)
 {
 	byte result = peekIO(port, time);
 	switch (port & 0x0F) {
@@ -36,7 +37,7 @@ byte MSXS1985::readIO(byte port, const EmuTime& time)
 	return result;
 }
 
-byte MSXS1985::peekIO(byte port, const EmuTime& /*time*/) const
+byte MSXS1985::peekIO(word port, const EmuTime& /*time*/) const
 {
 	byte result;
 	switch (port & 0x0F) {
@@ -55,7 +56,7 @@ byte MSXS1985::peekIO(byte port, const EmuTime& /*time*/) const
 	return result;
 }
 
-void MSXS1985::writeIO(byte port, byte value, const EmuTime& /*time*/)
+void MSXS1985::writeIO(word port, byte value, const EmuTime& /*time*/)
 {
 	//PRT_DEBUG("S1985: write " << (int) port << " " << (int)value);
 	switch (port & 0x0F) {
