@@ -1,8 +1,8 @@
-// $Id: Pluggable.cc,v 1.8 2004/12/15 09:09:42 m9710797 Exp $
+// $Id: Pluggable.cc,v 1.9 2005/11/11 20:53:05 m9710797 Exp $
 
-#include <cassert>
 #include "Pluggable.hh"
 #include "Connector.hh"
+#include <cassert>
 
 using std::string;
 
@@ -23,16 +23,16 @@ const string& Pluggable::getName() const
 	return name;
 }
 
-void Pluggable::plug(Connector* newConnector, const EmuTime& time)
+void Pluggable::plug(Connector& newConnector, const EmuTime& time)
 {
-	assert(getClass() == newConnector->getClass());
+	assert(getClass() == newConnector.getClass());
 
 	if (connector) {
 		throw PlugException(getName() + " already plugged in " +
 		                    connector->getName() + ".");
 	}
 	plugHelper(newConnector, time);
-	connector = newConnector;
+	connector = &newConnector;
 }
 
 void Pluggable::unplug(const EmuTime& time)
