@@ -1,11 +1,10 @@
-// $Id: RP5C01.cc,v 1.26 2005/01/17 19:29:31 m9710797 Exp $
+// $Id: RP5C01.cc,v 1.32 2006/06/22 18:05:46 m9710797 Exp $
 
 #include "RP5C01.hh"
-#include "SettingsConfig.hh"
 #include "EnumSetting.hh"
+#include "SRAM.hh"
 #include <cassert>
 #include <ctime>
-#include <string>
 
 namespace openmsx {
 
@@ -25,8 +24,8 @@ const nibble MODE_TIMERENABLE = 0x8;
 
 const nibble TEST_SECONDS = 0x1;
 const nibble TEST_MINUTES = 0x2;
-const nibble TEST_HOURS   = 0x4;
-const nibble TEST_DAYS    = 0x8;
+const nibble TEST_DAYS    = 0x4;
+const nibble TEST_YEARS   = 0x8;
 
 const nibble RESET_ALARM    = 0x1;
 const nibble RESET_FRACTION = 0x2;
@@ -41,15 +40,16 @@ static const nibble mask[4][13] = {
 };
 
 
-RP5C01::RP5C01(byte* data, const EmuTime& time)
-	: reg(data), reference(time)
+RP5C01::RP5C01(CommandController& commandController, SRAM& regs_,
+               const EmuTime& time)
+	: regs(regs_), reference(time)
 {
 	EnumSetting<RTCMode>::Map modeMap;
 	modeMap["EmuTime"] = EMUTIME;
 	modeMap["RealTime"] = REALTIME;
-	modeSetting.reset(new EnumSetting<RTCMode>(
+	modeSetting.reset(new EnumSetting<RTCMode>(commandController,
 		"rtcmode", "Real Time Clock mode", EMUTIME, modeMap));
-	
+
 	initializeTime();
 	reset(time);
 }
@@ -80,7 +80,7 @@ nibble RP5C01::readPort(nibble port, const EmuTime& time)
 			int block = modeReg & MODE_BLOKSELECT;
 			if (block == TIME_BLOCK)
 				updateTimeRegs(time);
-			nibble tmp = reg[block * 13 + port];
+			nibble tmp = regs[block * 13 + port];
 			return tmp & mask[block][port];
 	}
 }
@@ -108,7 +108,7 @@ void RP5C01::writePort(nibble port, nibble value, const EmuTime& time)
 			int block = modeReg & MODE_BLOKSELECT;
 			if (block == TIME_BLOCK)
 				updateTimeRegs(time);
-			reg[block * 13 + port] = value & mask[block][port];
+			regs.write(block * 13 + port, value & mask[block][port]);
 			if (block == TIME_BLOCK)
 				regs2Time();
 	}
@@ -132,86 +132,96 @@ void RP5C01::initializeTime()
 
 void RP5C01::regs2Time()
 {
-	seconds  = reg[TIME_BLOCK * 13 + 0] + 10 * reg[TIME_BLOCK * 13 + 1];
-	minutes  = reg[TIME_BLOCK * 13 + 2] + 10 * reg[TIME_BLOCK * 13 + 3];
-	hours    = reg[TIME_BLOCK * 13 + 4] + 10 * reg[TIME_BLOCK * 13 + 5];
-	dayWeek  = reg[TIME_BLOCK * 13 + 6];
-	days     = reg[TIME_BLOCK * 13 + 7] + 10 * reg[TIME_BLOCK * 13 + 8] - 1;
-	months   = reg[TIME_BLOCK * 13 + 9] + 10 * reg[TIME_BLOCK * 13 +10] - 1;
-	years    = reg[TIME_BLOCK * 13 +11] + 10 * reg[TIME_BLOCK * 13 +12];
-	leapYear = reg[ALARM_BLOCK * 13 +11];
+	seconds  = regs[TIME_BLOCK * 13 + 0] + 10 * regs[TIME_BLOCK * 13 + 1];
+	minutes  = regs[TIME_BLOCK * 13 + 2] + 10 * regs[TIME_BLOCK * 13 + 3];
+	hours    = regs[TIME_BLOCK * 13 + 4] + 10 * regs[TIME_BLOCK * 13 + 5];
+	dayWeek  = regs[TIME_BLOCK * 13 + 6];
+	days     = regs[TIME_BLOCK * 13 + 7] + 10 * regs[TIME_BLOCK * 13 + 8] - 1;
+	months   = regs[TIME_BLOCK * 13 + 9] + 10 * regs[TIME_BLOCK * 13 +10] - 1;
+	years    = regs[TIME_BLOCK * 13 +11] + 10 * regs[TIME_BLOCK * 13 +12];
+	leapYear = regs[ALARM_BLOCK * 13 +11];
 
-	if (!reg[ALARM_BLOCK * 13 + 10]) {
+	if (!regs[ALARM_BLOCK * 13 + 10]) {
 		// 12 hours mode
 		if (hours >= 20) hours = (hours - 20) + 12;
 	}
 }
-	
+
 void RP5C01::time2Regs()
 {
 	int hours_ = hours;
-	if (!reg[ALARM_BLOCK * 13 + 10]) {
+	if (!regs[ALARM_BLOCK * 13 + 10]) {
 		// 12 hours mode
 		if (hours >= 12) hours_ = (hours - 12) + 20;
 	}
-	
-	reg[TIME_BLOCK  * 13 +  0] =  seconds   % 10;
-	reg[TIME_BLOCK  * 13 +  1] =  seconds   / 10;
-	reg[TIME_BLOCK  * 13 +  2] =  minutes   % 10;
-	reg[TIME_BLOCK  * 13 +  3] =  minutes   / 10;
-	reg[TIME_BLOCK  * 13 +  4] =  hours_    % 10;
-	reg[TIME_BLOCK  * 13 +  5] =  hours_    / 10;
-	reg[TIME_BLOCK  * 13 +  6] =  dayWeek;
-	reg[TIME_BLOCK  * 13 +  7] = (days+1)   % 10;	// 0-30 -> 1-31
-	reg[TIME_BLOCK  * 13 +  8] = (days+1)   / 10;	// 0-11 -> 1-12
-	reg[TIME_BLOCK  * 13 +  9] = (months+1) % 10;
-	reg[TIME_BLOCK  * 13 + 10] = (months+1) / 10;
-	reg[TIME_BLOCK  * 13 + 11] =  years     % 10;
-	reg[TIME_BLOCK  * 13 + 12] =  years     / 10;
-	reg[ALARM_BLOCK * 13 + 11] =  leapYear;
+
+	regs.write(TIME_BLOCK  * 13 +  0,  seconds   % 10);
+	regs.write(TIME_BLOCK  * 13 +  1,  seconds   / 10);
+	regs.write(TIME_BLOCK  * 13 +  2,  minutes   % 10);
+	regs.write(TIME_BLOCK  * 13 +  3,  minutes   / 10);
+	regs.write(TIME_BLOCK  * 13 +  4,  hours_    % 10);
+	regs.write(TIME_BLOCK  * 13 +  5,  hours_    / 10);
+	regs.write(TIME_BLOCK  * 13 +  6,  dayWeek);
+	regs.write(TIME_BLOCK  * 13 +  7, (days+1)   % 10);	// 0-30 -> 1-31
+	regs.write(TIME_BLOCK  * 13 +  8, (days+1)   / 10);	// 0-11 -> 1-12
+	regs.write(TIME_BLOCK  * 13 +  9, (months+1) % 10);
+	regs.write(TIME_BLOCK  * 13 + 10, (months+1) / 10);
+	regs.write(TIME_BLOCK  * 13 + 11,  years     % 10);
+	regs.write(TIME_BLOCK  * 13 + 12,  years     / 10);
+	regs.write(ALARM_BLOCK * 13 + 11,  leapYear);
 }
 
-static int daysInMonth(int month, int leapYear) 
+static int daysInMonth(int month, int leapYear)
 {
 	const int daysInMonths[12] = {
 		31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
 	};
-	
+
 	month %= 12;
-	return ((month == 1) && (leapYear == 0)) ? 29
-	                                         : daysInMonths[month];
+	return ((month == 1) && (leapYear == 0)) ? 29 : daysInMonths[month];
 }
 
 void RP5C01::updateTimeRegs(const EmuTime& time)
 {
 	if (modeSetting->getValue() == EMUTIME) {
 		// sync with EmuTime, perfect emulation
-		uint64 elapsed = (modeReg & MODE_TIMERENABLE)
-			? reference.getTicksTill(time)
-			: 0;
+		uint64 elapsed = reference.getTicksTill(time);
 		reference.advance(time);
 
 		// in test mode increase sec/min/.. at a rate of 16384Hz
-		uint64 testSeconds = (testReg & TEST_SECONDS) ? elapsed : 0;
-		uint64 testMinutes = (testReg & TEST_MINUTES) ? elapsed : 0;
-		uint64 testHours   = (testReg & TEST_HOURS  ) ? elapsed : 0;
-		uint64 testDays    = (testReg & TEST_DAYS   ) ? elapsed : 0;
-		
-		fraction += elapsed;
-		seconds  += fraction/FREQ  + testSeconds; fraction %= FREQ;
-		minutes  += seconds / 60   + testMinutes; seconds  %= 60;
-		hours    += minutes / 60   + testHours;   minutes  %= 60;
-		int carryDays = hours / 24 + testDays; 
-		days     += carryDays;      hours   %= 24;
-		dayWeek = (dayWeek + carryDays) % 7;
+		fraction += (modeReg & MODE_TIMERENABLE) ? elapsed : 0;
+		int carrySeconds = (testReg & TEST_SECONDS)
+		                 ? elapsed : fraction / FREQ;
+		seconds  += carrySeconds;
+		int carryMinutes = (testReg & TEST_MINUTES)
+		                 ? elapsed : seconds / 60;
+		minutes  += carryMinutes;
+		hours    += minutes / 60;
+		int carryDays = (testReg & TEST_DAYS)
+		              ? elapsed : hours / 24;
+		days     += carryDays;
+		dayWeek  += carryDays;
 		while (days >= daysInMonth(months, leapYear)) {
+			// TODO not correct because leapYear is not updated
+			//      is only triggered when we update several months
+			//      at a time (but might happen in TEST_DAY mode)
 			days -= daysInMonth(months, leapYear);
 			months++;
 		}
-		int carryYears = months / 12;
-		years = (years + carryYears) % 100; months %= 12;
-		leapYear = (leapYear + carryYears) % 4;
+		int carryYears = (testReg & TEST_YEARS)
+		               ? elapsed : months / 12;
+		years    += carryYears;
+		leapYear += carryYears;
 		
+		fraction %= FREQ;
+		seconds  %= 60;
+		minutes  %= 60;
+		hours    %= 24;
+		dayWeek  %= 7;
+		months   %= 12;
+		years    %= 100;
+		leapYear %= 4;
+
 		time2Regs();
 	} else {
 		// sync with host clock
@@ -223,7 +233,7 @@ void RP5C01::updateTimeRegs(const EmuTime& time)
 void RP5C01::resetAlarm()
 {
 	for (int i = 2; i <= 8; i++) {
-		reg[ALARM_BLOCK * 13 + i] = 0;
+		regs.write(ALARM_BLOCK * 13 + i, 0);
 	}
 }
 
