@@ -1,24 +1,20 @@
-// $Id: RenShaTurbo.cc,v 1.11 2004/07/11 18:51:34 m9710797 Exp $
+// $Id: RenShaTurbo.cc,v 1.15 2006/01/18 22:29:44 m9710797 Exp $
 
 #include "RenShaTurbo.hh"
-#include "HardwareConfig.hh"
+#include "XMLElement.hh"
 #include "Autofire.hh"
 
 namespace openmsx {
 
-RenShaTurbo& RenShaTurbo::instance()
+RenShaTurbo::RenShaTurbo(CommandController& commandController,
+                         const XMLElement& machineConfig)
 {
-	static RenShaTurbo oneInstance;
-	return oneInstance;
-}
-
-RenShaTurbo::RenShaTurbo()
-{
-	const XMLElement* config = HardwareConfig::instance().findChild("RenShaTurbo");
+	const XMLElement* config = machineConfig.findChild("RenShaTurbo");
 	if (config) {
 		int min_ints = config->getChildDataAsInt("min_ints", 47);
 		int max_ints = config->getChildDataAsInt("max_ints", 221);
-		autofire.reset(new Autofire(min_ints, max_ints, "renshaturbo"));
+		autofire.reset(new Autofire(commandController, min_ints,
+		                            max_ints, "renshaturbo"));
 	}
 }
 
