@@ -1,69 +1,61 @@
-// $Id: Scheduler.cc,v 1.145 2005/05/14 12:49:39 m9710797 Exp $
+// $Id: Scheduler.cc,v 1.155 2006/05/25 12:40:08 m9710797 Exp $
 
 #include "Scheduler.hh"
 #include "Schedulable.hh"
-#include "InputEventGenerator.hh"
-#include "Interpreter.hh"
+#include "Thread.hh"
 #include <cassert>
 #include <algorithm>
 
-using std::swap;
-using std::make_heap;
-using std::push_heap;
-using std::pop_heap;
-
-
 namespace openmsx {
 
-const EmuTime Scheduler::ASAP;
-
 Scheduler::Scheduler()
-	: sem(1)
+	: scheduleTime(EmuTime::zero)
 {
 }
 
 Scheduler::~Scheduler()
 {
-}
-
-Scheduler& Scheduler::instance()
-{
-	static Scheduler oneInstance;
-	return oneInstance;
+	assert(syncPoints.empty());
 }
 
 void Scheduler::setSyncPoint(const EmuTime& time, Schedulable& device, int userData)
 {
 	//PRT_DEBUG("Sched: registering " << device->schedName() <<
 	//          " " << userData << " for emulation at " << time);
-
-	ScopedLock lock(sem);
-	assert(time == ASAP || time >= scheduleTime);
+	assert(Thread::isMainThread());
+	assert(time >= scheduleTime);
 
 	// Push sync point into queue.
-	syncPoints.push_back(SynchronizationPoint(time, &device, userData));
-	push_heap(syncPoints.begin(), syncPoints.end());
+	SyncPoints::iterator it =
+		std::upper_bound(syncPoints.begin(), syncPoints.end(), time,
+		            LessSyncPoint());
+	syncPoints.insert(it, SynchronizationPoint(time, &device, userData));
 }
 
 void Scheduler::removeSyncPoint(Schedulable& device, int userData)
 {
-	ScopedLock lock(sem);
+	assert(Thread::isMainThread());
 	for (SyncPoints::iterator it = syncPoints.begin();
 	     it != syncPoints.end(); ++it) {
-		SynchronizationPoint& sp = *it;
-		if ((sp.getDevice() == &device) &&
-		    (sp.getUserData() == userData)) {
-			swap(sp, syncPoints.back());
-			syncPoints.pop_back();
-			make_heap(syncPoints.begin(), syncPoints.end());
-			return;
+		if (((*it).getDevice() == &device) &&
+		    ((*it).getUserData() == userData)) {
+			syncPoints.erase(it);
+			break;
 		}
 	}
 }
 
+void Scheduler::removeSyncPoints(Schedulable& device)
+{
+	assert(Thread::isMainThread());
+	syncPoints.erase(remove_if(syncPoints.begin(), syncPoints.end(),
+	                           FindSchedulable(device)),
+	                 syncPoints.end());
+}
+
 bool Scheduler::pendingSyncPoint(Schedulable& device, int userData)
 {
-	ScopedLock lock(sem);
+	assert(Thread::isMainThread());
 	for (SyncPoints::iterator it = syncPoints.begin();
 	     it != syncPoints.end(); ++it) {
 		if ((it->getDevice() == &device) &&
@@ -76,34 +68,27 @@ bool Scheduler::pendingSyncPoint(Schedulable& device, int userData)
 
 const EmuTime& Scheduler::getCurrentTime() const
 {
+	assert(Thread::isMainThread());
 	return scheduleTime;
 }
 
 void Scheduler::scheduleHelper(const EmuTime& limit)
 {
-	InputEventGenerator::instance().poll();
-	Interpreter::instance().poll();
-
 	while (true) {
 		// Get next sync point.
-		sem.down();
 		const SynchronizationPoint sp =
 			  syncPoints.empty()
 			? SynchronizationPoint(EmuTime::infinity, NULL, 0)
 			: syncPoints.front();
-		const EmuTime& sp_time = sp.getTime();
-		if (sp_time > limit) {
-			sem.up();
+		const EmuTime& time = sp.getTime();
+		if (time > limit) {
 			break;
 		}
 
-		EmuTime time((sp_time == ASAP) ? scheduleTime : sp_time);
 		assert(scheduleTime <= time);
 		scheduleTime = time;
 
-		pop_heap(syncPoints.begin(), syncPoints.end());
-		syncPoints.pop_back();
-		sem.up();
+		syncPoints.erase(syncPoints.begin());
 
 		Schedulable* device = sp.getDevice();
 		assert(device);
@@ -112,7 +97,6 @@ void Scheduler::scheduleHelper(const EmuTime& limit)
 		//		<< " " << userData << " till " << time);
 		device->executeUntil(time, userData);
 	}
-	scheduleTime = limit;
 }
 
 } // namespace openmsx
