@@ -1,15 +1,9 @@
-// $Id: DummyCassetteDevice.cc,v 1.12 2004/12/15 09:09:43 m9710797 Exp $
+// $Id: DummyCassetteDevice.cc,v 1.14 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "DummyCassetteDevice.hh"
 
-using std::string;
-
 namespace openmsx {
 
-DummyCassetteDevice::DummyCassetteDevice()
-{
-}
-
 void DummyCassetteDevice::setMotor(bool /*status*/, const EmuTime& /*time*/)
 {
 	// do nothing
@@ -20,22 +14,18 @@ short DummyCassetteDevice::readSample(const EmuTime& /*time*/)
 	return 32767;	// TODO check value
 }
 
-void DummyCassetteDevice::writeWave(short* /*buf*/, int /*length*/)
+void DummyCassetteDevice::setSignal(bool /*output*/, const EmuTime& /*time*/)
 {
 	// do nothing
 }
-int DummyCassetteDevice::getWriteSampleRate()
-{
-	return 0;	// 0 means not interested
-}
 
-const string& DummyCassetteDevice::getDescription() const
+const std::string& DummyCassetteDevice::getDescription() const
 {
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyCassetteDevice::plugHelper(Connector* /*connector*/,
+void DummyCassetteDevice::plugHelper(Connector& /*connector*/,
                                      const EmuTime& /*time*/)
 {
 }
