@@ -1,19 +1,26 @@
-// $Id: CommandController.cc,v 1.41 2005/03/12 20:47:12 m9710797 Exp $
+// $Id: CommandController.cc,v 1.58 2006/07/17 18:58:14 m9710797 Exp $
 
-#include <cassert>
-#include <cstdlib>
 #include "CommandController.hh"
+#include "Command.hh"
+#include "InfoTopic.hh"
 #include "CommandConsole.hh"
 #include "FileOperations.hh"
 #include "FileContext.hh"
 #include "File.hh"
 #include "openmsx.hh"
 #include "CliComm.hh"
+#include "HotKey.hh"
 #include "Interpreter.hh"
 #include "InfoCommand.hh"
 #include "ReadDir.hh"
 #include "CommandException.hh"
-#include "FileException.hh"
+#include "SettingsConfig.hh"
+#include "GlobalSettings.hh"
+#include "RomInfoTopic.hh"
+#include "TclObject.hh"
+#include "Version.hh"
+#include <cassert>
+#include <cstdlib>
 
 using std::set;
 using std::string;
@@ -21,65 +28,142 @@ using std::vector;
 
 namespace openmsx {
 
-CommandController::CommandController()
-	: helpCmd(*this), cmdConsole(NULL)
-	, infoCommand(InfoCommand::instance())
-	, interpreter(Interpreter::instance())
+class HelpCmd : public Command
+{
+public:
+	HelpCmd(CommandController& parent);
+	virtual void execute(const vector<TclObject*>& tokens, TclObject& result);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+private:
+	CommandController& parent;
+};
+
+class TabCompletionCmd : public Command
+{
+public:
+	TabCompletionCmd(CommandController& parent);
+	virtual void execute(const vector<TclObject*>& tokens, TclObject& result);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+private:
+	CommandController& parent;
+};
+
+class VersionInfo : public InfoTopic
+{
+public:
+	VersionInfo(CommandController& commandController);
+	virtual void execute(const std::vector<TclObject*>& tokens,
+	                     TclObject& result) const;
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+};
+
+
+CommandController::CommandController(EventDistributor& eventDistributor_)
+	: cmdConsole(NULL)
+	, cliComm(NULL)
+	, connection(NULL)
+	, eventDistributor(eventDistributor_)
+	, infoCommand(new InfoCommand(*this))
+	, helpCmd(new HelpCmd(*this))
+	, tabCompletionCmd(new TabCompletionCmd(*this))
+	, versionInfo(new VersionInfo(*this))
+	, romInfoTopic(new RomInfoTopic(*this))
 {
-	registerCommand(&helpCmd, "help");
-	registerCommand(&infoCommand, "openmsx_info");
 }
 
 CommandController::~CommandController()
 {
-	unregisterCommand(&infoCommand, "openmsx_info");
-	unregisterCommand(&helpCmd, "help");
-	assert(commands.empty());
-	assert(commandCompleters.empty());
+	//assert(commands.empty());            // TODO
+	//assert(commandCompleters.empty());   // TODO
 	assert(!cmdConsole);
 }
 
-CommandController& CommandController::instance()
+void CommandController::setCliComm(CliComm* cliComm_)
 {
-	static CommandController oneInstance;
-	return oneInstance;
+	cliComm = cliComm_;
 }
 
+CliComm& CommandController::getCliComm()
+{
+	assert(cliComm);
+	return *cliComm;
+}
 
-void CommandController::registerCommand(Command* command,
+CliConnection* CommandController::getConnection() const
+{
+	return connection;
+}
+
+Interpreter& CommandController::getInterpreter()
+{
+	if (!interpreter.get()) {
+		interpreter.reset(new Interpreter(eventDistributor));
+	}
+	return *interpreter;
+}
+
+InfoCommand& CommandController::getInfoCommand()
+{
+	return *infoCommand;
+}
+
+HotKey& CommandController::getHotKey()
+{
+	if (!hotKey.get()) {
+		hotKey.reset(new HotKey(*this, eventDistributor));
+	}
+	return *hotKey;
+}
+
+SettingsConfig& CommandController::getSettingsConfig()
+{
+	if (!settingsConfig.get()) {
+		settingsConfig.reset(new SettingsConfig(*this, getHotKey()));
+	}
+	return *settingsConfig;
+}
+
+GlobalSettings& CommandController::getGlobalSettings()
+{
+	if (!globalSettings.get()) {
+		globalSettings.reset(new GlobalSettings(*this));
+	}
+	return *globalSettings;
+}
+
+void CommandController::registerCommand(Command& command,
                                         const string& str)
 {
 	assert(commands.find(str) == commands.end());
 
-	registerCompleter(command, str);
-	commands[str] = command;
-	interpreter.registerCommand(str, *command);
+	commands[str] = &command;
+	getInterpreter().registerCommand(str, command);
 }
 
-void CommandController::unregisterCommand(Command* command,
+void CommandController::unregisterCommand(Command& command,
                                           const string& str)
 {
 	assert(commands.find(str) != commands.end());
-	assert(commands.find(str)->second == command);
+	assert(commands.find(str)->second == &command);
 
-	interpreter.unregisterCommand(str, *command);
+	getInterpreter().unregisterCommand(str, command);
 	commands.erase(str);
-	unregisterCompleter(command, str);
 }
 
-void CommandController::registerCompleter(CommandCompleter* completer,
+void CommandController::registerCompleter(CommandCompleter& completer,
                                           const string& str)
 {
 	assert(commandCompleters.find(str) == commandCompleters.end());
-	commandCompleters[str] = completer;
+	commandCompleters[str] = &completer;
 }
 
-void CommandController::unregisterCompleter(CommandCompleter* completer,
+void CommandController::unregisterCompleter(CommandCompleter& completer,
                                             const string& str)
 {
-	if (completer); // avoid warning
+	(void)completer;
 	assert(commandCompleters.find(str) != commandCompleters.end());
-	assert(commandCompleters.find(str)->second == completer);
+	assert(commandCompleters.find(str)->second == &completer);
 	commandCompleters.erase(str);
 }
 
@@ -186,7 +270,7 @@ static string escapeChars(const string& str, const string& chars)
 			result += '\\';
 		}
 		result += chr;
-		
+
 	}
 	return result;
 }
@@ -224,26 +308,32 @@ string CommandController::join(const vector<string>& tokens, char delimiter)
 	return result;
 }
 
-bool CommandController::isComplete(const string& command) const
+bool CommandController::isComplete(const string& command)
 {
-	return interpreter.isComplete(command);
+	return getInterpreter().isComplete(command);
 }
 
-string CommandController::executeCommand(const string& cmd)
+string CommandController::executeCommand(
+	const string& cmd, CliConnection* connection_)
 {
-	return interpreter.execute(cmd);
+	struct Restore {
+		Restore(CliConnection*& connection) : c(connection) {}
+		~Restore() { c = NULL; }
+		CliConnection*& c;
+	} restore(connection);
+
+	assert(connection == NULL);
+	connection = connection_;
+	return getInterpreter().execute(cmd);
 }
 
-void CommandController::autoCommands()
+void CommandController::source(const std::string& script)
 {
 	try {
-		SystemFileContext context(true); // only in system dir
-		File file(context.resolve("init.tcl"));
-		interpreter.executeFile(file.getLocalName());
-	} catch (FileException& e) {
-		// no init.tcl
+		File file(script);
+		getInterpreter().executeFile(file.getLocalName());
 	} catch (CommandException& e) {
-		CliComm::instance().printWarning(
+		getCliComm().printWarning(
 			 "While executing init.tcl: " + e.getMessage());
 	}
 }
@@ -253,7 +343,7 @@ void CommandController::setCommandConsole(CommandConsole* console)
 	cmdConsole = console;
 }
 
-void CommandController::tabCompletion(string &command)
+void CommandController::tabCompletion(string& command)
 {
 	// split in sub commands
 	vector<string> subcmds;
@@ -261,14 +351,14 @@ void CommandController::tabCompletion(string &command)
 	if (subcmds.empty()) {
 		subcmds.push_back("");
 	}
-	
+
 	// split command string in tokens
 	vector<string> originalTokens;
 	split(subcmds.back(), originalTokens, ' ');
 	if (originalTokens.empty()) {
 		originalTokens.push_back("");
 	}
-	
+
 	// complete last token
 	vector<string> tokens;
 	removeEscaping(originalTokens, tokens, true);
@@ -276,7 +366,7 @@ void CommandController::tabCompletion(string &command)
 	tabCompletion(tokens);
 	unsigned newNum = tokens.size();
 	bool tokenFinished = oldNum != newNum;
-	
+
 	// replace last token
 	string& original = originalTokens.back();
 	string& completed = tokens[oldNum - 1];
@@ -289,13 +379,13 @@ void CommandController::tabCompletion(string &command)
 		assert(tokens.back().empty());
 		originalTokens.push_back("");
 	}
-	
+
 	// rebuild command string
 	subcmds.back() = join(originalTokens, ' ');
 	command = join(subcmds, ';');
 }
 
-void CommandController::tabCompletion(vector<string> &tokens)
+void CommandController::tabCompletion(vector<string>& tokens)
 {
 	if (tokens.empty()) {
 		// nothing typed yet
@@ -304,18 +394,35 @@ void CommandController::tabCompletion(vector<string> &tokens)
 	if (tokens.size() == 1) {
 		// build a list of all command strings
 		set<string> cmds;
-		interpreter.getCommandNames(cmds);
+		getInterpreter().getCommandNames(cmds);
 		completeString(tokens, cmds);
 	} else {
 		CompleterMap::const_iterator it = commandCompleters.find(tokens.front());
 		if (it != commandCompleters.end()) {
 			it->second->tabCompletion(tokens);
+		} else {
+			TclObject command(getInterpreter());
+			command.addListElement("__tabcompletion");
+			for (vector<string>::const_iterator it2 = tokens.begin();
+			     it2 != tokens.end(); ++it2) {
+				command.addListElement(*it2);
+			}
+			try {
+				string result = command.executeCommand();
+				vector<string> split;
+				getInterpreter().splitList(result, split);
+				set<string> completions(split.begin(), split.end());
+				completeString(tokens, completions);
+			} catch (CommandException& e) {
+				cliComm->printWarning(
+					"Error while executing tab-completion "
+					"proc: " + e.getMessage());
+			}
 		}
 	}
 }
 
-bool CommandController::equal(const string& s1, const string& s2,
-                              bool caseSensitive)
+static bool equal(const string& s1, const string& s2, bool caseSensitive)
 {
 	if (caseSensitive) {
 		return s1 == s2;
@@ -327,7 +434,6 @@ bool CommandController::equal(const string& s1, const string& s2,
 bool CommandController::completeString2(string &str, set<string>& st,
                                         bool caseSensitive)
 {
-	CommandConsole* cmdConsole = CommandController::instance().cmdConsole;
 	assert(cmdConsole);
 	set<string>::iterator it = st.begin();
 	while (it != st.end()) {
@@ -355,7 +461,7 @@ bool CommandController::completeString2(string &str, set<string>& st,
 			// match is as long as first word
 			goto out;	// TODO rewrite this
 		}
-		// expand with one char and check all strings 
+		// expand with one char and check all strings
 		string string2 = (*it).substr(0, str.size() + 1);
 		for (;  it != st.end(); it++) {
 			if (!equal(string2, (*it).substr(0, string2.size()),
@@ -388,15 +494,23 @@ void CommandController::completeString(vector<string>& tokens,
 
 void CommandController::completeFileName(vector<string>& tokens)
 {
-	UserFileContext context;
+	UserFileContext context(*this);
 	completeFileName(tokens, context);
 }
 
 void CommandController::completeFileName(vector<string>& tokens,
                                          const FileContext& context)
+{
+	set<string> empty;
+	completeFileName(tokens, context, empty);
+}
+
+void CommandController::completeFileName(vector<string>& tokens,
+                                         const FileContext& context,
+                                         const set<string>& extra)
 {
 	vector<string> paths(context.getPaths());
-	
+
 	string& filename = tokens[tokens.size() - 1];
 	filename = FileOperations::expandTilde(filename);
 	filename = FileOperations::expandCurrentDirFromDrive(filename);
@@ -405,12 +519,12 @@ void CommandController::completeFileName(vector<string>& tokens,
 		paths.push_back("");
 	}
 
-	set<string> filenames;
+	set<string> filenames(extra);
 	for (vector<string>::const_iterator it = paths.begin();
 	     it != paths.end();
 	     ++it) {
 		string dirname = *it + basename;
-		ReadDir dir(FileOperations::getNativePath(dirname).c_str());
+		ReadDir dir(FileOperations::getNativePath(dirname));
 		while (dirent* de = dir.getEntry()) {
 			string name = dirname + de->d_name;
 			if (FileOperations::exists(name)) {
@@ -432,44 +546,58 @@ void CommandController::completeFileName(vector<string>& tokens,
 
 // Help Command
 
-CommandController::HelpCmd::HelpCmd(CommandController& parent_)
-	: parent(parent_)
+HelpCmd::HelpCmd(CommandController& commandController)
+	: Command(commandController, "help")
+	, parent(commandController)
 {
 }
 
-string CommandController::HelpCmd::execute(const vector<string>& tokens)
+void HelpCmd::execute(const vector<TclObject*>& tokens, TclObject& result)
 {
-	string result;
 	switch (tokens.size()) {
-	case 1: 
-		result += "Use 'help [command]' to get help for a specific command\n";
-		result += "The following commands exist:\n";
-		for (CompleterMap::const_iterator it =
+	case 1: {
+		string text =
+			"Use 'help [command]' to get help for a specific command\n"
+			"The following commands exist:\n";
+		for (CommandController::CompleterMap::const_iterator it =
 		         parent.commandCompleters.begin();
 		     it != parent.commandCompleters.end(); ++it) {
-			result += it->first;
-			result += '\n';
+			text += it->first;
+			text += '\n';
 		}
+		result.setString(text);
 		break;
+	}
 	default: {
-		CompleterMap::const_iterator it =
-			parent.commandCompleters.find(tokens[1]);
-		if (it == parent.commandCompleters.end()) {
-			throw CommandException(tokens[1] + ": unknown command");
+		 CommandController::CompleterMap::const_iterator it =
+			parent.commandCompleters.find(tokens[1]->getString());
+		if (it != parent.commandCompleters.end()) {
+			vector<string> tokens2;
+			vector<TclObject*>::const_iterator it2 = tokens.begin();
+			for (++it2; it2 != tokens.end(); ++it2) {
+				tokens2.push_back((*it2)->getString());
+			}
+			result.setString(it->second->help(tokens2));
+		} else {
+			TclObject command(result.getInterpreter());
+			command.addListElement("__help");
+			vector<TclObject*>::const_iterator it2 = tokens.begin();
+			for (++it2; it2 != tokens.end(); ++it2) {
+				command.addListElement(**it2);
+			}
+			result.setString(command.executeCommand());
 		}
-		vector<string>::const_iterator remainder = tokens.begin();
-		vector<string> tokens2(++remainder, tokens.end());
-		result += it->second->help(tokens2);
 		break;
 	}
 	}
-	return result;
 }
-string CommandController::HelpCmd::help(const vector<string>& /*tokens*/) const
+
+string HelpCmd::help(const vector<string>& /*tokens*/) const
 {
 	return "prints help information for commands\n";
 }
-void CommandController::HelpCmd::tabCompletion(vector<string>& tokens) const
+
+void HelpCmd::tabCompletion(vector<string>& tokens) const
 {
 	string front = tokens.front();
 	tokens.erase(tokens.begin());
@@ -477,4 +605,55 @@ void CommandController::HelpCmd::tabCompletion(vector<string>& tokens) const
 	tokens.insert(tokens.begin(), front);
 }
 
+
+// TabCompletionCmd Command
+
+TabCompletionCmd::TabCompletionCmd(CommandController& commandController)
+	: Command(commandController, "tabcompletion")
+	, parent(commandController)
+{
+}
+
+void TabCompletionCmd::execute(const vector<TclObject*>& tokens, TclObject& result)
+{
+	switch (tokens.size()) {
+	case 2: {
+		// TODO this prints list of possible completions in the console
+		string command = tokens[1]->getString();
+		parent.tabCompletion(command);
+		result.setString(command);
+		break;
+	}
+	default:
+		throw SyntaxError();
+	}
+}
+
+string TabCompletionCmd::help(const vector<string>& /*tokens*/) const
+{
+	return "!!! This command will change in the future !!!\n"
+	       "Tries to completes the given argument as if it were typed in "
+	       "the console. This command is only useful to provide "
+	       "tabcompletion to external console interfaces.";
+}
+
+
+// Version info
+
+VersionInfo::VersionInfo(CommandController& commandController)
+	: InfoTopic(commandController, "version")
+{
+}
+
+void VersionInfo::execute(const vector<TclObject*>& /*tokens*/,
+                          TclObject& result) const
+{
+	result.setString(Version::FULL_VERSION);
+}
+
+string VersionInfo::help(const vector<string>& /*tokens*/) const
+{
+	return "Prints openMSX version.";
+}
+
 } // namespace openmsx
