@@ -1,4 +1,4 @@
-// $Id: CommandException.cc,v 1.2 2004/12/15 09:09:43 m9710797 Exp $
+// $Id: CommandException.cc,v 1.3 2005/11/10 18:50:23 m9710797 Exp $
 
 #include "CommandException.hh"
 
@@ -10,18 +10,10 @@ CommandException::CommandException(const std::string& desc)
 {
 }
 
-CommandException::~CommandException()
-{
-}
-
 SyntaxError::SyntaxError()
 	: CommandException("Syntax error")
 {
 }
 
-SyntaxError::~SyntaxError()
-{
-}
-
 } // namespace openmsx
 
