@@ -1,11 +1,10 @@
-// $Id: InfoCommand.cc,v 1.12 2005/03/11 16:22:57 m9710797 Exp $
+// $Id: InfoCommand.cc,v 1.18 2006/07/06 18:14:57 m9710797 Exp $
 
 #include "InfoCommand.hh"
-#include "CommandController.hh"
-#include "CommandArgument.hh"
+#include "InfoTopic.hh"
+#include "TclObject.hh"
 #include "CommandException.hh"
-#include "RomInfoTopic.hh"
-#include "Version.hh"
+#include <iostream>
 #include <cassert>
 
 using std::map;
@@ -15,45 +14,40 @@ using std::vector;
 
 namespace openmsx {
 
-InfoCommand::InfoCommand()
-	: romInfoTopic(new RomInfoTopic())
+InfoCommand::InfoCommand(CommandController& commandController)
+	: Command(commandController, "openmsx_info")
 {
-	registerTopic("version", &versionInfo);
-	registerTopic("romtype", romInfoTopic.get());
 }
 
 InfoCommand::~InfoCommand()
 {
-	unregisterTopic("romtype", romInfoTopic.get());
-	unregisterTopic("version", &versionInfo);
+	assert(infoTopics.empty());
 }
 
-InfoCommand& InfoCommand::instance()
+void InfoCommand::registerTopic(InfoTopic& topic, const string& name)
 {
-	static InfoCommand oneInstance;
-	return oneInstance;
+#ifndef NDEBUG
+	if (infoTopics.find(name) != infoTopics.end()) {
+		std::cerr << "INTERNAL ERROR: already have a info topic with "
+		             "name " << name << std::endl;
+		assert(false);
+	}
+#endif
+	infoTopics[name] = &topic;
 }
 
-void InfoCommand::registerTopic(const string& name,
-                                const InfoTopic* topic)
+void InfoCommand::unregisterTopic(InfoTopic& topic, const string& name)
 {
-	assert(infoTopics.find(name) == infoTopics.end());
-	infoTopics[name] = topic;
-}
-
-void InfoCommand::unregisterTopic(const string& name,
-                                  const InfoTopic* topic)
-{
-	if (topic); // avoid warning
+	(void)topic;
 	assert(infoTopics.find(name) != infoTopics.end());
-	assert(infoTopics[name] == topic);
+	assert(infoTopics[name] == &topic);
 	infoTopics.erase(name);
 }
 
 // Command
 
-void InfoCommand::execute(const vector<CommandArgument>& tokens,
-                          CommandArgument& result)
+void InfoCommand::execute(const vector<TclObject*>& tokens,
+                          TclObject& result)
 {
 	switch (tokens.size()) {
 	case 1:
@@ -66,7 +60,7 @@ void InfoCommand::execute(const vector<CommandArgument>& tokens,
 	default:
 		// show info about topic
 		assert(tokens.size() >= 2);
-		string topic = tokens[1].getString();
+		string topic = tokens[1]->getString();
 		map<string, const InfoTopic*>::const_iterator it =
 			infoTopics.find(topic);
 		if (it == infoTopics.end()) {
@@ -110,7 +104,7 @@ void InfoCommand::tabCompletion(vector<string>& tokens) const
 		       infoTopics.begin(); it != infoTopics.end(); ++it) {
 			topics.insert(it->first);
 		}
-		CommandController::completeString(tokens, topics);
+		completeString(tokens, topics);
 		break;
 	}
 	default:
@@ -125,17 +119,4 @@ void InfoCommand::tabCompletion(vector<string>& tokens) const
 	}
 }
 
-// Version info
-
-void InfoCommand::VersionInfo::execute(const vector<CommandArgument>& /*tokens*/,
-                                       CommandArgument& result) const
-{
-	result.setString(Version::FULL_VERSION);
-}
-
-string InfoCommand::VersionInfo::help(const vector<string>& /*tokens*/) const
-{
-	return "Prints openMSX version.";
-}
-
 } // namespace openmsx
