@@ -1,13 +1,14 @@
-// $Id: Interpreter.cc,v 1.13 2005/05/04 20:11:40 m9710797 Exp $
+// $Id: Interpreter.cc,v 1.30 2006/07/28 19:13:04 manuelbi Exp $
 
-#include "openmsx.hh"
+#include "Interpreter.hh"
+#include "EventDistributor.hh"
 #include "Command.hh"
-#include "CommandConsole.hh"
-#include "CommandArgument.hh"
+#include "TclObject.hh"
 #include "CommandException.hh"
 #include "Setting.hh"
-#include "Scheduler.hh"
-#include "Interpreter.hh"
+#include "InterpreterOutput.hh"
+#include "openmsx.hh"
+#include "FileOperations.hh"
 //#include <tk.h>
 
 using std::set;
@@ -34,21 +35,21 @@ static int dummyGetHandle(ClientData /*instanceData*/, int /*direction*/,
 	return TCL_ERROR;
 }
 Tcl_ChannelType Interpreter::channelType = {
-	"openMSX console",	// Type name
-	NULL,			// Always non-blocking
-	dummyClose,		// Close proc
-	dummyInput,		// Input proc
-	Interpreter::outputProc,// Output proc
-	NULL,			// Seek proc
-	NULL,			// Set option proc
-	NULL,			// Get option proc
-	dummyWatch,		// Watch for events on console
-	dummyGetHandle,		// Get a handle from the device
-	NULL,			// Tcl_DriverClose2Proc
-	NULL,			// Tcl_DriverBlockModeProc
-	NULL,			// Tcl_DriverFlushProc
-	NULL,			// Tcl_DriverHandlerProc
-	NULL,			// Tcl_DriverWideSeekProc
+	const_cast<char*>("openMSX console"),// Type name
+	NULL,			 // Always non-blocking
+	dummyClose,		 // Close proc
+	dummyInput,		 // Input proc
+	Interpreter::outputProc, // Output proc
+	NULL,			 // Seek proc
+	NULL,			 // Set option proc
+	NULL,			 // Get option proc
+	dummyWatch,		 // Watch for events on console
+	dummyGetHandle,		 // Get a handle from the device
+	NULL,			 // Tcl_DriverClose2Proc
+	NULL,			 // Tcl_DriverBlockModeProc
+	NULL,			 // Tcl_DriverFlushProc
+	NULL,			 // Tcl_DriverHandlerProc
+	NULL,			 // Tcl_DriverWideSeekProc
 };
 
 void Interpreter::init(const char* programName)
@@ -56,11 +57,12 @@ void Interpreter::init(const char* programName)
 	Tcl_FindExecutable(programName);
 }
 
-Interpreter::Interpreter()
+Interpreter::Interpreter(EventDistributor& eventDistributor_)
+	: eventDistributor(eventDistributor_)
 {
 	interp = Tcl_CreateInterp();
 	Tcl_Preserve(interp);
-	
+
 	// TODO need to investigate this: doesn't work on windows
 	/*
 	if (Tcl_Init(interp) != TCL_OK) {
@@ -75,35 +77,50 @@ Interpreter::Interpreter()
 	*/
 
 	Tcl_Channel channel = Tcl_CreateChannel(&channelType,
-		"openMSX console", NULL, TCL_WRITABLE);
+		"openMSX console", this, TCL_WRITABLE);
 	if (channel != NULL) {
 		Tcl_SetChannelOption(interp, channel, "-translation", "binary");
 		Tcl_SetChannelOption(interp, channel, "-buffering", "line");
 		Tcl_SetChannelOption(interp, channel, "-encoding", "binary");
 	}
 	Tcl_SetStdChannel(channel, TCL_STDOUT);
+
+	setVariable("env(OPENMSX_USER_DATA)", FileOperations::getUserDataDir());
+	setVariable("env(OPENMSX_SYSTEM_DATA)", FileOperations::getSystemDataDir());
+
+	eventDistributor.registerEventListener(OPENMSX_POLL_EVENT, *this);
+
 }
 
 Interpreter::~Interpreter()
 {
+	eventDistributor.unregisterEventListener(OPENMSX_POLL_EVENT, *this);
+
 	if (!Tcl_InterpDeleted(interp)) {
 		Tcl_DeleteInterp(interp);
 	}
 	Tcl_Release(interp);
+
+	Tcl_Finalize();
 }
 
-Interpreter& Interpreter::instance()
+void Interpreter::setOutput(InterpreterOutput* output_)
 {
-	static Interpreter oneInstance;
-	return oneInstance;
+	output = output_;
 }
 
-int Interpreter::outputProc(ClientData /*clientData*/, const char* buf,
+int Interpreter::outputProc(ClientData clientData, const char* buf,
                  int toWrite, int* /*errorCodePtr*/)
 {
-	string output(buf, toWrite);
-	if (!output.empty()) {
-		CommandConsole::instance().print(output);
+	try {
+		InterpreterOutput* output = ((Interpreter*)clientData)->output;
+
+		string text(buf, toWrite);
+		if (!text.empty() && output) {
+			output->output(text);
+		}
+	} catch (...) {
+		assert(false); // we cannot let exceptions pass through TCL
 	}
 	return toWrite;
 }
@@ -118,25 +135,38 @@ void Interpreter::registerCommand(const string& name, Command& command)
 
 void Interpreter::unregisterCommand(const string& name, Command& /*command*/)
 {
-	assert(commandTokenMap.find(name) != commandTokenMap.end());
-	Tcl_DeleteCommandFromToken(interp, commandTokenMap[name]);
+	CommandTokenMap::iterator it = commandTokenMap.find(name);
+	assert(it != commandTokenMap.end());
+	Tcl_DeleteCommandFromToken(interp, it->second);
+	commandTokenMap.erase(it);
 }
 
 int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
                            int objc, Tcl_Obj* const objv[])
 {
-	Command& command = *static_cast<Command*>(clientData);
-	vector<CommandArgument> tokens;
-	tokens.reserve(objc);
-	for (int i = 0; i < objc; ++i) {
-		tokens.push_back(CommandArgument(interp, objv[i]));
-	}
-	CommandResult result(interp);
 	try {
-		command.execute(tokens, result);
-		return TCL_OK;
-	} catch (MSXException& e) {
-		result.setString(e.getMessage());
+		Command& command = *static_cast<Command*>(clientData);
+		vector<TclObject*> tokens;
+		tokens.reserve(objc);
+		for (int i = 0; i < objc; ++i) {
+			tokens.push_back(new TclObject(interp, objv[i]));
+		}
+		int res = TCL_OK;
+		TclObject result(interp);
+		try {
+			command.execute(tokens, result);
+		} catch (MSXException& e) {
+			result.setString(e.getMessage());
+			res = TCL_ERROR;
+		}
+		Tcl_SetObjResult(interp, result.getTclObject());
+		for (vector<TclObject*>::const_iterator it = tokens.begin();
+		     it != tokens.end(); ++it) {
+			delete *it;
+		}
+		return res;
+	} catch (...) {
+		assert(false); // we cannot let exceptions pass through TCL
 		return TCL_ERROR;
 	}
 }
@@ -148,15 +178,13 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
 void Interpreter::getCommandNames(set<string>& result)
 {
 	string list = execute("info commands");
-	
+
 	int argc;
 	const char** argv;
 	if (Tcl_SplitList(interp, list.c_str(), &argc, &argv) != TCL_OK) {
 		return;
 	}
-	for (int i = 0; i < argc; ++i) {
-		result.insert(argv[i]);
-	}
+	result.insert(&argv[0], &argv[argc]);
 	Tcl_Free((char*)argv);
 }
 
@@ -228,33 +256,42 @@ void Interpreter::unregisterSetting(Setting& variable)
 char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
                            const char* part1, const char* /*part2*/, int flags)
 {
-	static string static_string;
-	
-	Setting* variable = static_cast<Setting*>(clientData);
-	
-	if (flags & TCL_TRACE_READS) {
-		Tcl_SetVar(interp, part1, variable->getValueString().c_str(), 0);
-	}
-	if (flags & TCL_TRACE_WRITES) {
-		try {
-			string newValue = Tcl_GetVar(interp, part1, 0);
-			variable->setValueString(newValue);
-			string newValue2 = variable->getValueString();
-			if (newValue != newValue2) {
-				Tcl_SetVar(interp, part1, newValue2.c_str(), 0);
-			}
-		} catch (MSXException& e) {
-			Tcl_SetVar(interp, part1, variable->getValueString().c_str(), 0);
-			static_string = e.getMessage();
-			return const_cast<char*>(static_string.c_str());
+	try {
+		static string static_string;
+
+		Setting* variable = static_cast<Setting*>(clientData);
+
+		if (flags & TCL_TRACE_READS) {
+			Tcl_SetVar(interp, part1,
+			           variable->getValueString().c_str(), 0);
 		}
-	}
-	if (flags & TCL_TRACE_UNSETS) {
-		variable->restoreDefault();
-		Tcl_SetVar(interp, part1, variable->getValueString().c_str(), 0);
-		Tcl_TraceVar(interp, part1,
-		             TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
-		             traceProc, static_cast<ClientData>(variable));
+		if (flags & TCL_TRACE_WRITES) {
+			try {
+				string newValue = Tcl_GetVar(interp, part1, 0);
+				variable->setValueString(newValue);
+				string newValue2 = variable->getValueString();
+				if (newValue != newValue2) {
+					Tcl_SetVar(interp, part1,
+					           newValue2.c_str(), 0);
+				}
+			} catch (MSXException& e) {
+				Tcl_SetVar(interp, part1,
+				           variable->getValueString().c_str(), 0);
+				static_string = e.getMessage();
+				return const_cast<char*>(static_string.c_str());
+			}
+		}
+		if (flags & TCL_TRACE_UNSETS) {
+			variable->restoreDefault();
+			Tcl_SetVar(interp, part1,
+			           variable->getValueString().c_str(), 0);
+			Tcl_TraceVar(interp, part1, TCL_TRACE_READS |
+			                TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
+			             traceProc,
+			             static_cast<ClientData>(variable));
+		}
+	} catch (...) {
+		assert(false); // we cannot let exceptions pass through TCL
 	}
 	return NULL;
 }
@@ -270,6 +307,12 @@ void Interpreter::splitList(const string& list, vector<string>& result)
 	Tcl_Free((char*)argv);
 }
 
+void Interpreter::signalEvent(const Event& event)
+{
+	(void)event;
+	assert(event.getType() == OPENMSX_POLL_EVENT);
+	poll();
+}
 
 void Interpreter::poll()
 {
