@@ -1,7 +1,6 @@
-// $Id: XMLElement.cc,v 1.4 2005/01/17 20:37:29 m9710797 Exp $
+// $Id: XMLElement.cc,v 1.11 2006/05/20 19:35:20 mthuurne Exp $
 
 #include "XMLElement.hh"
-#include "XMLElementListener.hh"
 #include "StringOp.hh"
 #include "FileContext.hh"
 #include "ConfigException.hh"
@@ -9,9 +8,7 @@
 #include <cassert>
 #include <algorithm>
 
-
 using std::auto_ptr;
-using std::remove;
 using std::string;
 
 namespace openmsx {
@@ -29,11 +26,7 @@ XMLElement::XMLElement(const XMLElement& element)
 
 XMLElement::~XMLElement()
 {
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		delete *it;
-	}
-	assert(listeners.empty());
+	removeAllChildren();
 }
 
 XMLElement* XMLElement::getParent()
@@ -53,11 +46,6 @@ void XMLElement::addChild(auto_ptr<XMLElement> child)
 	child->parent = this;
 	XMLElement* child2 = child.release();
 	children.push_back(child2);
-	
-	for (Listeners::const_iterator it = listeners.begin();
-	     it != listeners.end(); ++it) {
-		(*it)->childAdded(*this, *child2);
-	}
 }
 
 auto_ptr<XMLElement> XMLElement::removeChild(const XMLElement& child)
@@ -99,10 +87,6 @@ void XMLElement::setData(const string& data_)
 {
 	//assert(children.empty()); // no mixed-content elements
 	data = data_;
-	for (Listeners::const_iterator it = listeners.begin();
-	     it != listeners.end(); ++it) {
-		(*it)->updateData(*this);
-	}
 }
 
 void XMLElement::getChildren(const string& name, Children& result) const
@@ -131,6 +115,27 @@ const XMLElement* XMLElement::findChild(const string& name) const
 	return const_cast<XMLElement*>(this)->findChild(name);
 }
 
+XMLElement* XMLElement::findChildWithAttribute(const string& name,
+	const string& attName, const string& attValue)
+{
+	Children children;
+	getChildren(name, children);
+	for (Children::const_iterator it = children.begin();
+	     it != children.end(); ++it) {
+		if ((*it)->getAttribute(attName) == attValue) {
+			return *it;
+		}
+	}
+	return NULL;
+}
+
+const XMLElement* XMLElement::findChildWithAttribute(const string& name,
+	const string& attName, const string& attValue) const
+{
+	return const_cast<XMLElement*>(this)->findChildWithAttribute(
+		name, attName, attValue);
+}
+
 XMLElement& XMLElement::getChild(const string& name)
 {
 	XMLElement* elem = findChild(name);
@@ -160,17 +165,12 @@ XMLElement& XMLElement::getCreateChildWithAttribute(
 	const string& name, const string& attName,
 	const string& attValue, const string& defaultValue)
 {
-	Children children;
-	getChildren(name, children);
-	for (Children::iterator it = children.begin();
-	     it != children.end(); ++it) {
-		if ((*it)->getAttribute(attName) == attValue) {
-			return **it;
-		}
+	XMLElement* result = findChildWithAttribute(name, attName, attValue);
+	if (!result) {
+		result = new XMLElement(name, defaultValue);
+		result->addAttribute(attName, attValue);
+		addChild(auto_ptr<XMLElement>(result));
 	}
-	XMLElement* result = new XMLElement(name, defaultValue);
-	result->addAttribute(attName, attValue);
-	addChild(auto_ptr<XMLElement>(result));
 	return *result;
 }
 
@@ -199,6 +199,15 @@ int XMLElement::getChildDataAsInt(const string& name, int defaultValue) const
 	return child ? StringOp::stringToInt(child->getData()) : defaultValue;
 }
 
+void XMLElement::removeAllChildren()
+{
+	for (Children::const_iterator it = children.begin();
+	     it != children.end(); ++it) {
+		delete *it;
+	}
+	children.clear();
+}
+
 bool XMLElement::hasAttribute(const string& name) const
 {
 	return attributes.find(name) != attributes.end();
@@ -273,11 +282,8 @@ const XMLElement& XMLElement::operator=(const XMLElement& element)
 	name = element.name;
 	data = element.data;
 	attributes = element.attributes;
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		delete *it;
-	}
-	children.clear();
+
+	removeAllChildren();
 	for (Children::const_iterator it = element.children.begin();
 	     it != element.children.end(); ++it) {
 		addChild(auto_ptr<XMLElement>(new XMLElement(**it)));
@@ -334,11 +340,11 @@ void XMLElement::merge(const XMLElement& source)
 	for (Children::const_iterator it = sourceChildren.begin();
 	     it != sourceChildren.end(); ++it) {
 		const XMLElement& srcChild = **it;
-		Children::iterator it = std::find_if(srcChildrenCopy.begin(),
+		Children::iterator it2 = std::find_if(srcChildrenCopy.begin(),
 			srcChildrenCopy.end(), ShallowEqualTo(srcChild));
-		if (it != srcChildrenCopy.end()) {
-			(*it)->merge(srcChild);
-			srcChildrenCopy.erase(it); // don't merge to same child twice
+		if (it2 != srcChildrenCopy.end()) {
+			(*it2)->merge(srcChild);
+			srcChildrenCopy.erase(it2); // don't merge to same child twice
 		} else {
 			addChild(auto_ptr<XMLElement>(new XMLElement(srcChild)));
 		}
@@ -352,17 +358,6 @@ bool XMLElement::isShallowEqual(const XMLElement& other) const
 	       (getAttributes() == other.getAttributes());
 }
 
-void XMLElement::addListener(XMLElementListener& listener)
-{
-	listeners.push_back(&listener);
-}
-
-void XMLElement::removeListener(XMLElementListener& listener)
-{
-	assert(std::count(listeners.begin(), listeners.end(), &listener) == 1);
-	listeners.erase(std::find(listeners.begin(), listeners.end(), &listener));
-}
-
 string XMLElement::XMLEscape(const string& str)
 {
 	xmlChar* buffer = xmlEncodeEntitiesReentrant(NULL, (const xmlChar*)str.c_str());
