@@ -1,8 +1,9 @@
-// $Id: CommandConsole.cc,v 1.52 2005/05/24 20:39:33 m9710797 Exp $
+// $Id: CommandConsole.cc,v 1.58 2006/02/21 18:06:48 m9710797 Exp $
 
 #include "CommandConsole.hh"
 #include "CommandException.hh"
 #include "CommandController.hh"
+#include "Interpreter.hh"
 #include "Keys.hh"
 #include "FileContext.hh"
 #include "FileOperations.hh"
@@ -11,8 +12,12 @@
 #include "SettingsConfig.hh"
 #include "InputEvents.hh"
 #include "Display.hh"
+#include "EventDistributor.hh"
+#include "GlobalSettings.hh"
+#include "BooleanSetting.hh"
 #include <algorithm>
 #include <fstream>
+#include <cassert>
 
 using std::min;
 using std::max;
@@ -26,10 +31,13 @@ namespace openmsx {
 const char* const PROMPT1 = "> ";
 const char* const PROMPT2 = "| ";
 
-CommandConsole::CommandConsole()
-	: settingsConfig(SettingsConfig::instance())
-	, commandController(CommandController::instance())
-	, output(CliComm::instance())
+CommandConsole::CommandConsole(
+		CommandController& commandController_,
+		EventDistributor& eventDistributor_)
+	: commandController(commandController_)
+	, eventDistributor(eventDistributor_)
+	, consoleSetting(commandController.getGlobalSettings().getConsoleSetting())
+	, display(NULL)
 {
 	resetScrollBack();
 	prompt = PROMPT1;
@@ -37,7 +45,8 @@ CommandConsole::CommandConsole()
 	putPrompt();
 	maxHistory = 100;
 	removeDoubles = true;
-	if (const XMLElement* config = settingsConfig.findChild("Console")) {
+	if (const XMLElement* config = commandController.getSettingsConfig().
+	                                                 findChild("Console")) {
 		maxHistory = config->getChildDataAsInt(
 			"historysize", maxHistory);
 		removeDoubles = config->getChildDataAsBool(
@@ -45,24 +54,31 @@ CommandConsole::CommandConsole()
 	}
 	loadHistory();
 	commandController.setCommandConsole(this);
+
+	commandController.getInterpreter().setOutput(this);
+	eventDistributor.registerEventListener(OPENMSX_KEY_UP_EVENT,   *this);
+	eventDistributor.registerEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
 }
 
 CommandConsole::~CommandConsole()
 {
+	eventDistributor.unregisterEventListener(OPENMSX_KEY_UP_EVENT,   *this);
+	eventDistributor.unregisterEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
+	commandController.getInterpreter().setOutput(NULL);
 	commandController.setCommandConsole(NULL);
 	saveHistory();
+	assert(display == NULL);
 }
 
-CommandConsole& CommandConsole::instance()
+void CommandConsole::setDisplay(Display* display_)
 {
-	static CommandConsole oneInstance;
-	return oneInstance;
+	display = display_;
 }
 
 void CommandConsole::saveHistory()
 {
 	try {
-		UserFileContext context("console");
+		UserFileContext context(commandController, "console");
 		std::ofstream outputfile(FileOperations::expandTilde(
 		        context.resolveCreate("history.txt")).c_str());
 		if (!outputfile) {
@@ -74,14 +90,14 @@ void CommandConsole::saveHistory()
 			outputfile << it->substr(prompt.length()) << std::endl;
 		}
 	} catch (FileException& e) {
-		output.printWarning(e.getMessage());
+		commandController.getCliComm().printWarning(e.getMessage());
 	}
 }
 
 void CommandConsole::loadHistory()
 {
 	try {
-		UserFileContext context("console");
+		UserFileContext context(commandController, "console");
 		std::ifstream inputfile(FileOperations::expandTilde(
 		        context.resolveCreate("history.txt")).c_str());
 		if (!inputfile) {
@@ -125,16 +141,20 @@ string CommandConsole::getLine(unsigned line) const
 	return "";
 }
 
-bool CommandConsole::signalEvent(const Event& event)
+void CommandConsole::signalEvent(const Event& event)
 {
-	Display::instance().repaintDelayed(40000); // 25fps
-
-	if (event.getType() == OPENMSX_KEY_UP_EVENT) {
-		return false;	// don't pass event to MSX-Keyboard
-	}
-
 	assert(dynamic_cast<const KeyEvent*>(&event));
 	const KeyEvent& keyEvent = static_cast<const KeyEvent&>(event);
+	if ((event.getType() == OPENMSX_KEY_DOWN_EVENT) &&
+	    consoleSetting.getValue()) {
+		handleEvent(keyEvent);
+		assert(display);
+		display->repaintDelayed(40000); // 25fps
+	}
+}
+
+void CommandConsole::handleEvent(const KeyEvent& keyEvent)
+{
 	Keys::KeyCode keyCode = keyEvent.getKeyCode();
 	switch (keyCode) {
 		case (Keys::K_PAGEUP | Keys::KM_SHIFT):
@@ -197,7 +217,11 @@ bool CommandConsole::signalEvent(const Event& event)
 				normalKey(keyEvent.getUnicode());
 			}
 	}
-	return false;	// don't pass event to MSX-Keyboard
+}
+
+void CommandConsole::output(const std::string& text)
+{
+	print(text);
 }
 
 void CommandConsole::print(string text)
