@@ -1,22 +1,22 @@
-// $Id: GLConsole.cc,v 1.30 2005/05/04 18:40:42 m9710797 Exp $
+// $Id: GLConsole.cc,v 1.38 2006/02/27 21:49:16 m9710797 Exp $
 
-#include <cassert>
-#include "Display.hh"
 #include "GLConsole.hh"
 #include "GLFont.hh"
-#include "Console.hh"
 #include "GLImage.hh"
+#include "Console.hh"
+#include "Timer.hh"
+#include <cassert>
+#include <SDL.h>
 
 using std::string;
 
 namespace openmsx {
 
-GLConsole::GLConsole(Console& console_)
-	: OSDConsoleRenderer(console_)
+GLConsole::GLConsole(Reactor& reactor)
+	: OSDConsoleRenderer(reactor)
 	, backgroundTexture(0)
 {
 	initConsole();
-	Display::instance().addLayer(this);
 }
 
 GLConsole::~GLConsole()
@@ -56,19 +56,23 @@ void GLConsole::loadBackground(const string& filename)
 
 unsigned GLConsole::getScreenW() const
 {
-	return 640;
+	// TODO refactor
+	SDL_Surface* surface = SDL_GetVideoSurface();
+	return surface->w;
 }
 
 unsigned GLConsole::getScreenH() const
 {
-	return 480;
+	// TODO refactor
+	SDL_Surface* surface = SDL_GetVideoSurface();
+	return surface->h;
 }
 
 void GLConsole::paint()
 {
 	byte visibility = getVisibility();
 	if (!visibility) return;
-	
+
 	glPushAttrib(GL_ENABLE_BIT);
 	glDisable(GL_DEPTH_TEST);
 	glDisable(GL_CULL_FACE);
@@ -104,6 +108,7 @@ void GLConsole::paint()
 	glVertex2i(destW, 0);
 	glEnd();
 
+	Console& console = getConsole();
 	int screenlines = destH / font->getHeight();
 	for (int loop = 0; loop < screenlines; loop++) {
 		int num = loop + console.getScrollBack();
@@ -115,16 +120,17 @@ void GLConsole::paint()
 	}
 
 	// Check if the blink period is over
-	if (SDL_GetTicks() > lastBlinkTime) {
-		lastBlinkTime = SDL_GetTicks() + BLINK_RATE;
+	unsigned long long now = Timer::getTime();
+	if (lastBlinkTime < now) {
+		lastBlinkTime = now + BLINK_RATE;
 		blink = !blink;
 	}
-	
+
 	unsigned cursorX, cursorY;
 	console.getCursorPosition(cursorX, cursorY);
 	if ((cursorX != lastCursorX) || (cursorY != lastCursorY)) {
 		blink = true; // force cursor
-		lastBlinkTime=SDL_GetTicks() + BLINK_RATE; // maximum time
+		lastBlinkTime = now + BLINK_RATE; // maximum time
 		lastCursorX = cursorX;
 		lastCursorY = cursorY;
 	}
@@ -135,7 +141,6 @@ void GLConsole::paint()
 				CHAR_BORDER + cursorX * font->getWidth(),
 				destH - (font->getHeight() * (cursorY + 1)),
 				visibility);
-
 		}
 	}
 
