@@ -1,20 +1,22 @@
-// $Id: OSDConsoleRenderer.cc,v 1.66 2005/05/28 18:09:40 manuelbi Exp $
+// $Id: OSDConsoleRenderer.cc,v 1.78 2006/02/27 21:49:16 m9710797 Exp $
 
 #include "OSDConsoleRenderer.hh"
-#include "Console.hh"
+#include "CommandConsole.hh"
 #include "EnumSetting.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "GlobalSettings.hh"
 #include "Display.hh"
+#include "Event.hh"
+#include "InputEvents.hh"
 #include "EventDistributor.hh"
 #include "InputEventGenerator.hh"
 #include "Timer.hh"
 #include "DummyFont.hh"
 #include "FileContext.hh"
 #include "CliComm.hh"
+#include "Reactor.hh"
 #include <algorithm>
-#include <SDL.h>
 
 using std::string;
 
@@ -22,36 +24,36 @@ namespace openmsx {
 
 // class OSDConsoleRenderer
 
-OSDConsoleRenderer::OSDConsoleRenderer(Console& console_)
+OSDConsoleRenderer::OSDConsoleRenderer(Reactor& reactor_)
 	: Layer(COVER_NONE, Z_CONSOLE)
-	, console(console_)
-	, eventDistributor(EventDistributor::instance())
-	, inputEventGenerator(InputEventGenerator::instance())
-	, consoleSetting(GlobalSettings::instance().getConsoleSetting())
+	, reactor(reactor_)
+	, consoleSetting(reactor.getCommandController().getGlobalSettings().
+	                    getConsoleSetting())
 {
 	destX = destY = destW = destH = 0; // avoid UMR
 	font.reset(new DummyFont());
 	blink = false;
-	lastBlinkTime = 0;
+	lastBlinkTime = Timer::getTime();
 	lastCursorX = lastCursorY = 0;
 
 	active = false;
 	time = 0;
 	setCoverage(COVER_PARTIAL);
-	consoleSetting.addListener(this);
+	consoleSetting.attach(*this);
 	setActive(consoleSetting.getValue());
 }
 
 OSDConsoleRenderer::~OSDConsoleRenderer()
 {
-	consoleSetting.removeListener(this);
+	consoleSetting.detach(*this);
 	setActive(false);
 }
 
 void OSDConsoleRenderer::initConsole()
 {
+	CommandController& commandController = reactor.getCommandController();
 	// font
-	fontSetting.reset(new FilenameSetting(
+	fontSetting.reset(new FilenameSetting(commandController,
 		"consolefont", "console font file",
 		"skins/ConsoleFontRaveLShaded.png"));
 	try {
@@ -60,16 +62,17 @@ void OSDConsoleRenderer::initConsole()
 		// we really need a font
 		throw FatalError(e.getMessage());
 	}
-	
+
 	// rows / columns
 	int columns = (((getScreenW() - CHAR_BORDER) / font->getWidth()) * 30) / 32;
 	int rows = ((getScreenH() / font->getHeight()) * 6) / 15;
-	consoleColumnsSetting.reset(new IntegerSetting("consolecolumns",
-		"number of columns in the console", columns, 32, 999));
-	consoleRowsSetting.reset(new IntegerSetting("consolerows",
-		"number of rows in the console", rows, 1, 99));
+	consoleColumnsSetting.reset(new IntegerSetting(commandController,
+		"consolecolumns", "number of columns in the console", columns,
+		32, 999));
+	consoleRowsSetting.reset(new IntegerSetting(commandController,
+		"consolerows", "number of rows in the console", rows, 1, 99));
 	adjustColRow();
-	
+
 	// placement
 	typedef EnumSetting<Placement>::Map PlaceMap;
 	PlaceMap placeMap;
@@ -82,20 +85,20 @@ void OSDConsoleRenderer::initConsole()
 	placeMap["bottomleft"]  = CP_BOTTOMLEFT;
 	placeMap["bottom"]      = CP_BOTTOM;
 	placeMap["bottomright"] = CP_BOTTOMRIGHT;
-	consolePlacementSetting.reset(new EnumSetting<Placement>(
+	consolePlacementSetting.reset(new EnumSetting<Placement>(commandController,
 		"consoleplacement", "position of the console within the emulator",
 		CP_BOTTOM, placeMap));
-	
+
 	updateConsoleRect();
-	
+
 	// background
-	backgroundSetting.reset(new FilenameSetting(
+	backgroundSetting.reset(new FilenameSetting(commandController,
 		"consolebackground", "console background file",
 		"skins/ConsoleBackgroundGrey.png"));
 	try {
 		backgroundSetting->setChecker(this);
 	} catch (MSXException& e) {
-		CliComm::instance().printWarning(e.getMessage());
+		reactor.getCliComm().printWarning(e.getMessage());
 	}
 }
 
@@ -107,14 +110,14 @@ void OSDConsoleRenderer::adjustColRow()
 	unsigned consoleRows = std::min<unsigned>(
 		consoleRowsSetting->getValue(),
 		getScreenH() / font->getHeight());
-	console.setColumns(consoleColumns);
-	console.setRows(consoleRows);
+	getConsole().setColumns(consoleColumns);
+	getConsole().setRows(consoleRows);
 }
 
-void OSDConsoleRenderer::update(const Setting* setting)
+void OSDConsoleRenderer::update(const Setting& setting)
 {
-	if (setting); // avoid warning
-	assert(setting == &consoleSetting);
+	(void)setting;
+	assert(&setting == &consoleSetting);
 	setActive(consoleSetting.getValue());
 }
 
@@ -123,25 +126,17 @@ void OSDConsoleRenderer::setActive(bool active_)
 	if (active == active_) return;
 	active = active_;
 
-	Display::instance().repaintDelayed(40000); // 25 fps
-	
+	getDisplay().repaintDelayed(40000); // 25 fps
+
 	time = Timer::getTime();
 
-	inputEventGenerator.setKeyRepeat(active);
+	reactor.getInputEventGenerator().setKeyRepeat(active);
 	if (active) {
-		eventDistributor.registerEventListener(
-			OPENMSX_KEY_UP_EVENT,   console, EventDistributor::NATIVE);
-		eventDistributor.registerEventListener(
-			OPENMSX_KEY_DOWN_EVENT, console, EventDistributor::NATIVE);
-		eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_CONSOLE_ON_EVENT>());
+		reactor.getEventDistributor().distributeEvent(
+			new ConsoleEvent(OPENMSX_CONSOLE_ON_EVENT));
 	} else {
-		eventDistributor.unregisterEventListener(
-			OPENMSX_KEY_DOWN_EVENT, console, EventDistributor::NATIVE);
-		eventDistributor.unregisterEventListener(
-			OPENMSX_KEY_UP_EVENT,   console, EventDistributor::NATIVE);
-		eventDistributor.distributeEvent(
-			new SimpleEvent<OPENMSX_CONSOLE_OFF_EVENT>());
+		reactor.getEventDistributor().distributeEvent(
+			new ConsoleEvent(OPENMSX_CONSOLE_OFF_EVENT));
 	}
 }
 
@@ -156,14 +151,14 @@ byte OSDConsoleRenderer::getVisibility() const
 		if (dur > FADE_IN_DURATION) {
 			return 255;
 		} else {
-			Display::instance().repaintDelayed(40000); // 25 fps
+			getDisplay().repaintDelayed(40000); // 25 fps
 			return (dur * 255) / FADE_IN_DURATION;
 		}
 	} else {
 		if (dur > FADE_OUT_DURATION) {
 			return 0;
 		} else {
-			Display::instance().repaintDelayed(40000); // 25 fps
+			getDisplay().repaintDelayed(40000); // 25 fps
 			return 255 - ((dur * 255) / FADE_OUT_DURATION);
 		}
 	}
@@ -176,9 +171,9 @@ bool OSDConsoleRenderer::updateConsoleRect()
 	adjustColRow();
 
 	unsigned x, y, w, h;
-	h = font->getHeight() * console.getRows();
-	w = (font->getWidth() * console.getColumns()) + CHAR_BORDER;
-	
+	h = font->getHeight() * getConsole().getRows();
+	w = (font->getWidth() * getConsole().getColumns()) + CHAR_BORDER;
+
 	// TODO use setting listener in the future
 	switch (consolePlacementSetting->getValue()) {
 		case CP_TOPLEFT:
@@ -226,11 +221,8 @@ bool OSDConsoleRenderer::updateConsoleRect()
 void OSDConsoleRenderer::check(SettingImpl<FilenameSetting::Policy>& setting,
                                string& value)
 {
-	assert(dynamic_cast<FilenameSetting*>(&setting));
-	
-	string filename = value.empty() ? value :
-		static_cast<FilenameSetting&>(setting).
-			getFileContext().resolve(value);
+	SystemFileContext context;
+	string filename = value.empty() ? value : context.resolve(value);
 	if (&setting == backgroundSetting.get()) {
 		loadBackground(filename);
 	} else if (&setting == fontSetting.get()) {
@@ -240,4 +232,14 @@ void OSDConsoleRenderer::check(SettingImpl<FilenameSetting::Policy>& setting,
 	}
 }
 
+Display& OSDConsoleRenderer::getDisplay() const
+{
+	return reactor.getDisplay();
+}
+
+Console& OSDConsoleRenderer::getConsole() const
+{
+	return reactor.getCommandConsole();
+}
+
 } // namespace openmsx
