@@ -1,4 +1,4 @@
-// $Id: SDLConsole.cc,v 1.38 2005/05/04 18:40:42 m9710797 Exp $
+// $Id: SDLConsole.cc,v 1.46 2006/02/27 21:49:16 m9710797 Exp $
 
 /*  Written By: Garrett Banuk <mongoose@mongeese.org>
  *  Adapted to C++ and openMSX needs by David Heremans
@@ -6,31 +6,24 @@
  *  in any of your programs.
  */
 
-#include <cassert>
-#include "Display.hh"
 #include "SDLConsole.hh"
-#include "Console.hh"
-#include "SDLFont.hh"
-#include "File.hh"
 #include "SDLImage.hh"
+#include "SDLFont.hh"
+#include "Console.hh"
+#include "File.hh"
+#include "Timer.hh"
+#include <cassert>
+#include <SDL.h>
 
 using std::string;
 
 namespace openmsx {
 
-SDLConsole::SDLConsole(Console& console_, SDL_Surface* screen)
-	: OSDConsoleRenderer(console_)
+SDLConsole::SDLConsole(Reactor& reactor, SDL_Surface* screen)
+	: OSDConsoleRenderer(reactor)
 	, outputScreen(screen)
 {
-	blink = false;
-	lastBlinkTime = 0;
-
 	initConsole();
-	Display::instance().addLayer(this);
-}
-
-SDLConsole::~SDLConsole()
-{
 }
 
 void SDLConsole::updateConsoleRect()
@@ -48,7 +41,7 @@ void SDLConsole::paint()
 {
 	byte visibility = getVisibility();
 	if (!visibility) return;
-	
+
 	updateConsoleRect();
 
 	// draw the background image if there is one
@@ -65,6 +58,7 @@ void SDLConsole::paint()
 		backgroundImage->draw(destX, destY, visibility);
 	}
 
+	Console& console = getConsole();
 	int screenlines = destH / font->getHeight();
 	for (int loop = 0; loop < screenlines; ++loop) {
 		int num = loop + console.getScrollBack();
@@ -75,8 +69,9 @@ void SDLConsole::paint()
 	}
 
 	// Check if the blink period is over
-	if (SDL_GetTicks() > lastBlinkTime) {
-		lastBlinkTime = SDL_GetTicks() + BLINK_RATE;
+	unsigned long long now = Timer::getTime();
+	if (lastBlinkTime < now) {
+		lastBlinkTime = now + BLINK_RATE;
 		blink = !blink;
 	}
 
@@ -84,7 +79,7 @@ void SDLConsole::paint()
 	console.getCursorPosition(cursorX, cursorY);
 	if ((cursorX != lastCursorX) || (cursorY != lastCursorY)) {
 		blink = true; // force cursor
-		lastBlinkTime = SDL_GetTicks() + BLINK_RATE; // maximum time
+		lastBlinkTime = now + BLINK_RATE; // maximum time
 		lastCursorX = cursorX;
 		lastCursorY = cursorY;
 	}
@@ -111,8 +106,8 @@ void SDLConsole::loadBackground(const string& filename)
 		backgroundImage.reset();
 		return;
 	}
-	backgroundImage.reset(new SDLImage(outputScreen, filename,
-		destW, destH, CONSOLE_ALPHA));
+	backgroundImage.reset(
+		new SDLImage(outputScreen, filename, destW, destH));
 	backgroundName = filename;
 }
 
