@@ -1,76 +1,83 @@
-// $Id: MSXCPU.cc,v 1.61 2005/05/20 20:21:02 m9710797 Exp $
+// $Id: MSXCPU.cc,v 1.78 2006/01/24 18:03:35 m9710797 Exp $
 
 #include "MSXCPU.hh"
-#include "CPU.hh"
-#include "InfoCommand.hh"
+#include "MSXMotherBoard.hh"
 #include "Debugger.hh"
+#include "SimpleDebuggable.hh"
 #include "BooleanSetting.hh"
 #include "CPUCore.hh"
 #include "Z80.hh"
 #include "R800.hh"
-#include <sstream>
+#include "BreakPoint.hh"
+#include "InfoTopic.hh"
 #include <cassert>
 
-using std::ostringstream;
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-MSXCPU::MSXCPU()
-	: traceSetting(new BooleanSetting("cputrace", "CPU tracing on/off", false))
-	, z80 (new CPUCore<Z80TYPE> ("z80",  *traceSetting, EmuTime::zero))
-	, r800(new CPUCore<R800TYPE>("r800", *traceSetting, EmuTime::zero))
-	, timeInfo(*this)
-	, infoCmd(InfoCommand::instance())
-	, debugger(Debugger::instance())
+class TimeInfoTopic : public InfoTopic
+{
+public:
+	TimeInfoTopic(CommandController& commandController,
+	              MSXCPU& msxcpu);
+	virtual void execute(const vector<TclObject*>& tokens,
+	                     TclObject& result) const;
+	virtual std::string help (const vector<string>& tokens) const;
+private:
+	MSXCPU& msxcpu;
+};
+
+class MSXCPUDebuggable : public SimpleDebuggable
+{
+public:
+	MSXCPUDebuggable(MSXMotherBoard& motherboard, MSXCPU& cpu);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value);
+private:
+	MSXCPU& cpu;
+};
+
+
+MSXCPU::MSXCPU(MSXMotherBoard& motherboard_)
+	: motherboard(motherboard_)
+	, traceSetting(new BooleanSetting(motherboard.getCommandController(),
+	                              "cputrace", "CPU tracing on/off", false))
+	, z80 (new CPUCore<Z80TYPE> (motherboard, "z80",  *traceSetting,
+	                             EmuTime::zero))
+	, r800(new CPUCore<R800TYPE>(motherboard, "r800", *traceSetting,
+	                             EmuTime::zero))
+	, reference(EmuTime::zero)
+	, timeInfo(new TimeInfoTopic(motherboard.getCommandController(), *this))
+	, debuggable(new MSXCPUDebuggable(motherboard_, *this))
 {
 	activeCPU = z80.get();	// setActiveCPU(CPU_Z80);
-	reset(EmuTime::zero);
 
-	infoCmd.registerTopic("time", &timeInfo);
-	debugger.setCPU(this);
-	debugger.registerDebuggable("CPU regs", *this);
-
-	traceSetting->addListener(this);
+	motherboard.getDebugger().setCPU(this);
+	traceSetting->attach(*this);
 }
 
 MSXCPU::~MSXCPU()
 {
-	traceSetting->removeListener(this);
-
-	debugger.unregisterDebuggable("CPU regs", *this);
-	debugger.setCPU(0);
-	infoCmd.unregisterTopic("time", &timeInfo);
-}
-
-MSXCPU& MSXCPU::instance()
-{
-	static MSXCPU oneInstance;
-	return oneInstance;
-}
-
-void MSXCPU::setMotherboard(MSXMotherBoard* motherboard)
-{
-	z80 ->setMotherboard(motherboard);
-	r800->setMotherboard(motherboard);
+	traceSetting->detach(*this);
+	motherboard.getDebugger().setCPU(0);
 }
 
 void MSXCPU::setInterface(MSXCPUInterface* interface)
 {
-	z80 ->setInterface(interface); 
-	r800->setInterface(interface); 
+	z80 ->setInterface(interface);
+	r800->setInterface(interface);
 }
 
-void MSXCPU::reset(const EmuTime& time)
+void MSXCPU::doReset(const EmuTime& time)
 {
-	z80 ->reset(time);
-	r800->reset(time);
+	z80 ->doReset(time);
+	r800->doReset(time);
 
 	reference = time;
 }
 
-
 void MSXCPU::setActiveCPU(CPUType cpu)
 {
 	CPU* newCPU;
@@ -88,7 +95,7 @@ void MSXCPU::setActiveCPU(CPUType cpu)
 			newCPU = NULL;	// prevent warning
 	}
 	if (newCPU != activeCPU) {
-		newCPU->advance(activeCPU->getCurrentTime());
+		newCPU->warp(activeCPU->getCurrentTime());
 		newCPU->invalidateMemCache(0x0000, 0x10000);
 		exitCPULoop();
 		activeCPU = newCPU;
@@ -164,14 +171,86 @@ void MSXCPU::wait(const EmuTime& time)
 	activeCPU->wait(time);
 }
 
-void MSXCPU::update(const Setting* setting)
+void MSXCPU::update(const Setting& setting)
 {
-	if (setting); // avoid warning
-	assert(setting == traceSetting.get());
+	(void)setting;
+	assert(&setting == traceSetting.get());
 	exitCPULoop();
 }
 
-// DebugInterface
+// Command
+
+void MSXCPU::doStep()
+{
+	activeCPU->doStep();
+}
+
+void MSXCPU::doContinue()
+{
+	activeCPU->doContinue();
+}
+
+void MSXCPU::doBreak()
+{
+	activeCPU->doBreak();
+}
+
+bool MSXCPU::isBreaked()
+{
+	return activeCPU->isBreaked();
+}
+
+void MSXCPU::disasmCommand(const vector<TclObject*>& tokens,
+                           TclObject& result) const
+{
+	activeCPU->disasmCommand(tokens, result);
+}
+
+void MSXCPU::insertBreakPoint(std::auto_ptr<BreakPoint> bp)
+{
+	activeCPU->insertBreakPoint(bp);
+}
+
+void MSXCPU::removeBreakPoint(const BreakPoint& bp)
+{
+	activeCPU->removeBreakPoint(bp);
+}
+
+const CPU::BreakPoints& MSXCPU::getBreakPoints() const
+{
+	return activeCPU->getBreakPoints();
+}
+
+
+void MSXCPU::setPaused(bool paused)
+{
+	activeCPU->setPaused(paused);
+}
+
+
+// class TimeInfoTopic
+
+TimeInfoTopic::TimeInfoTopic(CommandController& commandController,
+                             MSXCPU& msxcpu_)
+	: InfoTopic(commandController, "time")
+	, msxcpu(msxcpu_)
+{
+}
+
+void TimeInfoTopic::execute(const vector<TclObject*>& /*tokens*/,
+                            TclObject& result) const
+{
+	EmuDuration dur = msxcpu.getCurrentTime() - msxcpu.reference;
+	result.setDouble(dur.toDouble());
+}
+
+string TimeInfoTopic::help(const vector<string>& /*tokens*/) const
+{
+	return "Prints the time in seconds that the MSX is powered on\n";
+}
+
+
+// class MSXCPUDebuggable
 
 //  0 ->  A      1 ->  F      2 -> B       3 -> C
 //  4 ->  D      5 ->  E      6 -> H       7 -> L
@@ -181,24 +260,19 @@ void MSXCPU::update(const Setting* setting)
 // 20 -> PCH    21 -> PCL    22 -> SPH    23 -> SPL
 // 24 ->  I     25 ->  R     26 -> IM     27 -> IFF1/2
 
-unsigned MSXCPU::getSize() const
+MSXCPUDebuggable::MSXCPUDebuggable(MSXMotherBoard& motherboard, MSXCPU& cpu_)
+	: SimpleDebuggable(motherboard, "CPU regs",
+	                   "Registers of the active CPU (Z80 or R800)", 28)
+	, cpu(cpu_)
 {
-	return 28; // number of 8 bits registers (16 bits = 2 registers)
 }
 
-const string& MSXCPU::getDescription() const
+byte MSXCPUDebuggable::read(unsigned address)
 {
-	static const string desc = 
-		"Registers of the active CPU (Z80 or R800)";
-	return desc;
-}
-
-byte MSXCPU::read(unsigned address)
-{
-	const CPU::CPURegs& regs = activeCPU->getRegisters(); 
+	const CPU::CPURegs& regs = cpu.activeCPU->getRegisters();
 	const word* registers[] = {
-		&regs.AF,  &regs.BC,  &regs.DE,  &regs.HL, 
-		&regs.AF2, &regs.BC2, &regs.DE2, &regs.HL2, 
+		&regs.AF,  &regs.BC,  &regs.DE,  &regs.HL,
+		&regs.AF2, &regs.BC2, &regs.DE2, &regs.HL2,
 		&regs.IX,  &regs.IY,  &regs.PC,  &regs.SP
 	};
 
@@ -221,12 +295,12 @@ byte MSXCPU::read(unsigned address)
 	}
 }
 
-void MSXCPU::write(unsigned address, byte value)
+void MSXCPUDebuggable::write(unsigned address, byte value)
 {
-	CPU::CPURegs& regs = activeCPU->getRegisters(); 
+	CPU::CPURegs& regs = cpu.activeCPU->getRegisters();
 	word* registers[] = {
-		&regs.AF,  &regs.BC,  &regs.DE,  &regs.HL, 
-		&regs.AF2, &regs.BC2, &regs.DE2, &regs.HL2, 
+		&regs.AF,  &regs.BC,  &regs.DE,  &regs.HL,
+		&regs.AF2, &regs.BC2, &regs.DE2, &regs.HL2,
 		&regs.IX,  &regs.IY,  &regs.PC,  &regs.SP
 	};
 
@@ -258,74 +332,4 @@ void MSXCPU::write(unsigned address, byte value)
 	}
 }
 
-
-// Command
-
-string MSXCPU::doStep()
-{
-	activeCPU->doStep();
-	return "";
-}
-
-string MSXCPU::doContinue()
-{
-	activeCPU->doContinue();
-	return "";
-}
-
-string MSXCPU::doBreak()
-{
-	activeCPU->doBreak();
-	return "";
-}
-
-void MSXCPU::disasmCommand(const vector<CommandArgument>& tokens,
-                           CommandArgument& result) const
-{
-	activeCPU->disasmCommand(tokens, result);
-}
-
-string MSXCPU::setBreakPoint(word addr)
-{
-	activeCPU->insertBreakPoint(addr);
-	return "";
-}
-
-string MSXCPU::removeBreakPoint(word addr)
-{
-	activeCPU->removeBreakPoint(addr);
-	return "";
-}
-
-string MSXCPU::listBreakPoints() const
-{
-	const CPU::BreakPoints& breakPoints = activeCPU->getBreakPoints();
-	ostringstream os;
-	for (CPU::BreakPoints::const_iterator it = breakPoints.begin();
-	     it != breakPoints.end(); ++it) {
-		os << std::hex << *it << '\n';
-	}
-	return os.str();
-}
-
-
-// class TimeInfoTopic
-
-MSXCPU::TimeInfoTopic::TimeInfoTopic(MSXCPU& parent_)
-	: parent(parent_)
-{
-}
-
-void MSXCPU::TimeInfoTopic::execute(const vector<CommandArgument>& /*tokens*/,
-                                    CommandArgument& result) const
-{
-	EmuDuration dur = parent.getCurrentTime() - parent.reference;
-	result.setDouble(dur.toDouble());
-}
-
-string MSXCPU::TimeInfoTopic::help(const vector<string>& /*tokens*/) const
-{
-	return "Prints the time in seconds that the MSX is powered on\n";
-}
-
 } // namespace openmsx
