@@ -1,4 +1,4 @@
-// $Id: AfterCommand.cc,v 1.20 2005/05/24 20:39:34 m9710797 Exp $
+// $Id: AfterCommand.cc,v 1.30 2006/02/21 18:06:48 m9710797 Exp $
 
 #include "AfterCommand.hh"
 #include "CommandController.hh"
@@ -16,58 +16,69 @@ using std::set;
 
 namespace openmsx {
 
-AfterCommand::AfterCmdMap AfterCommand::afterCmds;
-
-AfterCommand::AfterCommand()
+AfterCommand::AfterCommand(Scheduler& scheduler_,
+                           EventDistributor& eventDistributor_,
+                           CommandController& commandController_)
+	: SimpleCommand(commandController_, "after")
+	, scheduler(scheduler_)
+	, eventDistributor(eventDistributor_)
+	, commandController(commandController_)
 {
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_KEY_UP_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_KEY_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_MOTION_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_BUTTON_UP_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_AXIS_MOTION_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_BUTTON_UP_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_FINISH_FRAME_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_BREAK_EVENT, *this, EventDistributor::NATIVE);
-
-	CommandController::instance().registerCommand(this, "after");
+	// TODO DETACHED <-> EMU types should be cleaned up
+	//      (moved to event iso listener?)
+	eventDistributor.registerEventListener(
+		OPENMSX_KEY_UP_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_KEY_DOWN_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_MOUSE_MOTION_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_MOUSE_BUTTON_UP_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_JOY_AXIS_MOTION_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_JOY_BUTTON_UP_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_FINISH_FRAME_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_BREAK_EVENT, *this);
 }
 
 AfterCommand::~AfterCommand()
 {
-	CommandController::instance().unregisterCommand(this, "after");
-	
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_BREAK_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_FINISH_FRAME_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_BUTTON_UP_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_AXIS_MOTION_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_BUTTON_UP_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_MOTION_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_KEY_DOWN_EVENT, *this, EventDistributor::NATIVE);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_KEY_UP_EVENT, *this, EventDistributor::NATIVE);
+	while (!afterCmds.empty()) {
+		delete afterCmds.begin()->second; // removes itself from map
+	}
+
+	eventDistributor.unregisterEventListener(
+		OPENMSX_BREAK_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_FINISH_FRAME_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_JOY_BUTTON_UP_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_JOY_AXIS_MOTION_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_MOUSE_BUTTON_UP_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_MOUSE_MOTION_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_KEY_DOWN_EVENT, *this);
+	eventDistributor.unregisterEventListener(
+		OPENMSX_KEY_UP_EVENT, *this);
+}
+
+CommandController& AfterCommand::getCommandController() const
+{
+	return commandController;
 }
 
 string AfterCommand::execute(const vector<string>& tokens)
@@ -109,7 +120,7 @@ string AfterCommand::afterTime(const vector<string>& tokens)
 		throw SyntaxError();
 	}
 	double time = getTime(tokens[2]);
-	AfterTimeCmd* cmd = new AfterTimeCmd(tokens[3], time);
+	AfterTimeCmd* cmd = new AfterTimeCmd(scheduler, *this, tokens[3], time);
 	return cmd->getId();
 }
 
@@ -119,7 +130,7 @@ string AfterCommand::afterIdle(const vector<string>& tokens)
 		throw SyntaxError();
 	}
 	double time = getTime(tokens[2]);
-	AfterIdleCmd* cmd = new AfterIdleCmd(tokens[3], time);
+	AfterIdleCmd* cmd = new AfterIdleCmd(scheduler, *this, tokens[3], time);
 	return cmd->getId();
 }
 
@@ -129,7 +140,7 @@ string AfterCommand::afterEvent(const vector<string>& tokens)
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	AfterEventCmd<T>* cmd = new AfterEventCmd<T>(tokens[2]);
+	AfterEventCmd<T>* cmd = new AfterEventCmd<T>(*this, tokens[2]);
 	return cmd->getId();
 }
 
@@ -186,13 +197,13 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 		cmds.insert("break");
 		cmds.insert("info");
 		cmds.insert("cancel");
-		CommandController::completeString(tokens, cmds);
+		completeString(tokens, cmds);
 	}
 	// TODO : make more complete
 }
 
 
-bool AfterCommand::signalEvent(const Event& event)
+void AfterCommand::signalEvent(const Event& event)
 {
 	if (event.getType() == OPENMSX_FINISH_FRAME_EVENT) {
 		executeEvents<OPENMSX_FINISH_FRAME_EVENT>();
@@ -206,7 +217,6 @@ bool AfterCommand::signalEvent(const Event& event)
 			}
 		}
 	}
-	return true;
 }
 
 template<EventType T> void AfterCommand::executeEvents()
@@ -229,19 +239,20 @@ template<EventType T> void AfterCommand::executeEvents()
 
 unsigned AfterCommand::AfterCmd::lastAfterId = 0;
 
-AfterCommand::AfterCmd::AfterCmd(const string& command_)
-	: command(command_)
+AfterCommand::AfterCmd::AfterCmd(AfterCommand& afterCommand_,
+                                 const string& command_)
+	: afterCommand(afterCommand_), command(command_)
 {
 	ostringstream str;
 	str << "after#" << ++lastAfterId;
 	id = str.str();
-	
-	afterCmds[id] = this;
+
+	afterCommand.afterCmds[id] = this;
 }
 
 AfterCommand::AfterCmd::~AfterCmd()
 {
-	afterCmds.erase(id);
+	afterCommand.afterCmds.erase(id);
 }
 
 const string& AfterCommand::AfterCmd::getCommand() const
@@ -257,9 +268,9 @@ const string& AfterCommand::AfterCmd::getId() const
 void AfterCommand::AfterCmd::execute()
 {
 	try {
-		CommandController::instance().executeCommand(command);
+		afterCommand.getCommandController().executeCommand(command);
 	} catch (CommandException& e) {
-		CliComm::instance().printWarning(
+		afterCommand.getCommandController().getCliComm().printWarning(
 			"Error executig delayed command: " + e.getMessage());
 	}
 	delete this;
@@ -268,17 +279,18 @@ void AfterCommand::AfterCmd::execute()
 
 // class  AfterTimedCmd
 
-AfterCommand::AfterTimedCmd::AfterTimedCmd(const string& command, double time_)
-	: AfterCmd(command), time(time_)
+AfterCommand::AfterTimedCmd::AfterTimedCmd(
+		Scheduler& scheduler,
+		AfterCommand& afterCommand,
+		const string& command, double time_)
+	: AfterCmd(afterCommand, command)
+	, Schedulable(scheduler)
+	, time(time_)
+
 {
 	reschedule();
 }
 
-AfterCommand::AfterTimedCmd::~AfterTimedCmd()
-{
-	Scheduler::instance().removeSyncPoint(*this);
-}
-
 double AfterCommand::AfterTimedCmd::getTime() const
 {
 	return time;
@@ -286,9 +298,9 @@ double AfterCommand::AfterTimedCmd::getTime() const
 
 void AfterCommand::AfterTimedCmd::reschedule()
 {
-	Scheduler::instance().removeSyncPoint(*this);
-	EmuTime t = Scheduler::instance().getCurrentTime() + EmuDuration(time);
-	Scheduler::instance().setSyncPoint(t, *this);
+	removeSyncPoint();
+	EmuTime t = getScheduler().getCurrentTime() + EmuDuration(time);
+	setSyncPoint(t);
 }
 
 void AfterCommand::AfterTimedCmd::executeUntil(const EmuTime& /*time*/,
@@ -306,8 +318,11 @@ const string& AfterCommand::AfterTimedCmd::schedName() const
 
 // class AfterTimeCmd
 
-AfterCommand::AfterTimeCmd::AfterTimeCmd(const string& command, double time)
-	: AfterTimedCmd(command, time)
+AfterCommand::AfterTimeCmd::AfterTimeCmd(
+		Scheduler& scheduler,
+		AfterCommand& afterCommand,
+		const string& command, double time)
+	: AfterTimedCmd(scheduler, afterCommand, command, time)
 {
 }
 
@@ -320,8 +335,11 @@ const string& AfterCommand::AfterTimeCmd::getType() const
 
 // class AfterIdleCmd
 
-AfterCommand::AfterIdleCmd::AfterIdleCmd(const string& command, double time)
-	: AfterTimedCmd(command, time)
+AfterCommand::AfterIdleCmd::AfterIdleCmd(
+		Scheduler& scheduler,
+		AfterCommand& afterCommand,
+		const string& command, double time)
+	: AfterTimedCmd(scheduler, afterCommand, command, time)
 {
 }
 
@@ -335,8 +353,9 @@ const string& AfterCommand::AfterIdleCmd::getType() const
 // class AfterEventCmd
 
 template<EventType T>
-AfterCommand::AfterEventCmd<T>::AfterEventCmd(const string& command)
-	: AfterCmd(command)
+AfterCommand::AfterEventCmd<T>::AfterEventCmd(
+		AfterCommand& afterCommand, const string& command)
+	: AfterCmd(afterCommand, command)
 {
 }
 
