@@ -1,47 +1,53 @@
-// $Id: InputEventGenerator.cc,v 1.13 2005/05/24 20:39:35 m9710797 Exp $
+// $Id: InputEventGenerator.cc,v 1.26 2006/06/18 12:28:25 mthuurne Exp $
 
-#include <cassert>
-#include "openmsx.hh"
+#include "Command.hh"
 #include "InputEventGenerator.hh"
 #include "EventDistributor.hh"
-#include "CommandController.hh"
 #include "InputEvents.hh"
 #include "BooleanSetting.hh"
+#include "openmsx.hh"
+#include <cassert>
 
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-InputEventGenerator::InputEventGenerator()
-	: grabInput(new BooleanSetting("grabinput",
+class EscapeGrabCmd : public SimpleCommand
+{
+public:
+	EscapeGrabCmd(CommandController& commandController,
+		      InputEventGenerator& inputEventGenerator);
+	virtual std::string execute(const std::vector<std::string>& tokens);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+private:
+	InputEventGenerator& inputEventGenerator;
+};
+
+
+InputEventGenerator::InputEventGenerator(CommandController& commandController,
+                                         EventDistributor& eventDistributor_)
+	: grabInput(new BooleanSetting(commandController, "grabinput",
 		"This setting controls if openmsx takes over mouse and keyboard input",
-		false))
-	, escapeGrabCmd(*this)
+		false, Setting::DONT_SAVE))
+	, escapeGrabState(ESCAPE_GRAB_WAIT_CMD)
+	, escapeGrabCmd(new EscapeGrabCmd(commandController, *this))
 	, keyRepeat(false)
-	, distributor(EventDistributor::instance())
+	, eventDistributor(eventDistributor_)
 {
 	setGrabInput(grabInput->getValue());
-	grabInput->addListener(this);
-	distributor.registerEventListener(OPENMSX_FOCUS_EVENT, *this,
-	                                  EventDistributor::NATIVE);
-	CommandController::instance().registerCommand(&escapeGrabCmd, "escape_grab");
+	grabInput->attach(*this);
+	eventDistributor.registerEventListener(OPENMSX_FOCUS_EVENT, *this);
+	eventDistributor.registerEventListener(OPENMSX_POLL_EVENT,  *this);
 
 	reinit();
 }
 
 InputEventGenerator::~InputEventGenerator()
 {
-	CommandController::instance().unregisterCommand(&escapeGrabCmd, "escape_grab");
-	distributor.unregisterEventListener(OPENMSX_FOCUS_EVENT, *this,
-	                                  EventDistributor::NATIVE);
-	grabInput->removeListener(this);
-}
-
-InputEventGenerator& InputEventGenerator::instance()
-{
-	static InputEventGenerator oneInstance;
-	return oneInstance;
+	eventDistributor.unregisterEventListener(OPENMSX_POLL_EVENT,  *this);
+	eventDistributor.unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
+	grabInput->detach(*this);
 }
 
 void InputEventGenerator::reinit()
@@ -119,75 +125,77 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 {
 	switch (evt.type) {
 	case SDL_KEYUP: {
-		Event* event = new KeyUpEvent(Keys::getCode(evt.key.keysym.sym,
-		                                            evt.key.keysym.mod,
-		                                            true),
-		                              evt.key.keysym.unicode);
-		distributor.distributeEvent(event);
+		Event* event = new KeyUpEvent(
+		        Keys::getCode(evt.key.keysym.sym,
+		                      evt.key.keysym.mod,
+		                      true),
+		        evt.key.keysym.unicode);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 	case SDL_KEYDOWN: {
-		Event* event = new KeyDownEvent(Keys::getCode(evt.key.keysym.sym,
-		                                              evt.key.keysym.mod,
-		                                              false),
-		                                evt.key.keysym.unicode);
-		distributor.distributeEvent(event);
+		Event* event = new KeyDownEvent(
+		        Keys::getCode(evt.key.keysym.sym,
+		                      evt.key.keysym.mod,
+		                      false),
+		        evt.key.keysym.unicode);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 
 	case SDL_MOUSEBUTTONUP: {
 		Event* event = new MouseButtonUpEvent(
 		                        convertMouseButton(evt.button.button));
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 	case SDL_MOUSEBUTTONDOWN: {
 		Event* event = new MouseButtonDownEvent(
 		                        convertMouseButton(evt.button.button));
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 	case SDL_MOUSEMOTION: {
 		Event* event = new MouseMotionEvent(evt.motion.xrel,
 		                                    evt.motion.yrel);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 
 	case SDL_JOYBUTTONUP: {
 		Event* event = new JoystickButtonUpEvent(evt.jbutton.which,
 		                                         evt.jbutton.button);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 	case SDL_JOYBUTTONDOWN: {
 		Event* event = new JoystickButtonDownEvent(evt.jbutton.which,
 		                                           evt.jbutton.button);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 	case SDL_JOYAXISMOTION: {
 		Event* event = new JoystickAxisMotionEvent(evt.jaxis.which,
 		                              convertJoyAxis(evt.jaxis.axis),
 		                              evt.jaxis.value);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
-	      
+
 	case SDL_ACTIVEEVENT: {
 		Event* event = new FocusEvent(evt.active.gain);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
 
 	case SDL_VIDEORESIZE: {
 		Event* event = new ResizeEvent(evt.resize.w, evt.resize.h);
-		distributor.distributeEvent(event);
+		eventDistributor.distributeEvent(event);
 		break;
 	}
-	
+
 	case SDL_QUIT:
-		distributor.distributeEvent(new QuitEvent());
+		eventDistributor.distributeEvent(new QuitEvent());
 		break;
 
 	default:
@@ -196,38 +204,39 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 }
 
 
-void InputEventGenerator::update(const Setting* setting)
+void InputEventGenerator::update(const Setting& setting)
 {
-	if (setting); // avoid warning
-	assert(setting == grabInput.get());
+	(void)setting;
+	assert(&setting == grabInput.get());
 	escapeGrabState = ESCAPE_GRAB_WAIT_CMD;
 	setGrabInput(grabInput->getValue());
 }
 
-bool InputEventGenerator::signalEvent(const Event& event)
+void InputEventGenerator::signalEvent(const Event& event)
 {
-	assert(event.getType() == OPENMSX_FOCUS_EVENT);
-
-	const FocusEvent& focusEvent = static_cast<const FocusEvent&>(event);
-	switch (escapeGrabState) {
-		case ESCAPE_GRAB_WAIT_CMD:
-			// nothing
-			break;
-		case ESCAPE_GRAB_WAIT_LOST:
-			if (focusEvent.getGain() == false) {
-				escapeGrabState = ESCAPE_GRAB_WAIT_GAIN;
-			}
-			break;
-		case ESCAPE_GRAB_WAIT_GAIN:
-			if (focusEvent.getGain() == true) {
-				escapeGrabState = ESCAPE_GRAB_WAIT_CMD;
-			}
-			setGrabInput(true);
-			break;
-		default:
-			assert(false);
+	if (event.getType() == OPENMSX_FOCUS_EVENT) {
+		const FocusEvent& focusEvent = static_cast<const FocusEvent&>(event);
+		switch (escapeGrabState) {
+			case ESCAPE_GRAB_WAIT_CMD:
+				// nothing
+				break;
+			case ESCAPE_GRAB_WAIT_LOST:
+				if (focusEvent.getGain() == false) {
+					escapeGrabState = ESCAPE_GRAB_WAIT_GAIN;
+				}
+				break;
+			case ESCAPE_GRAB_WAIT_GAIN:
+				if (focusEvent.getGain() == true) {
+					escapeGrabState = ESCAPE_GRAB_WAIT_CMD;
+				}
+				setGrabInput(true);
+				break;
+			default:
+				assert(false);
+		}
+	} else if (event.getType() == OPENMSX_POLL_EVENT) {
+		poll();
 	}
-	return true;
 }
 
 void InputEventGenerator::setGrabInput(bool grab)
@@ -235,22 +244,26 @@ void InputEventGenerator::setGrabInput(bool grab)
 	SDL_WM_GrabInput(grab ? SDL_GRAB_ON : SDL_GRAB_OFF);
 }
 
+
 // class EscapeGrabCmd
-InputEventGenerator::EscapeGrabCmd::EscapeGrabCmd(InputEventGenerator& parent_)
-	: parent(parent_)
+EscapeGrabCmd::EscapeGrabCmd(CommandController& commandController,
+                             InputEventGenerator& inputEventGenerator_)
+	: SimpleCommand(commandController, "escape_grab")
+	, inputEventGenerator(inputEventGenerator_)
 {
 }
 
-string InputEventGenerator::EscapeGrabCmd::execute(const vector<string>& /*tokens*/)
+string EscapeGrabCmd::execute(const vector<string>& /*tokens*/)
 {
-	if (parent.grabInput->getValue()) {
-		parent.escapeGrabState = ESCAPE_GRAB_WAIT_LOST;
-		parent.setGrabInput(false);
+	if (inputEventGenerator.grabInput->getValue()) {
+		inputEventGenerator.escapeGrabState =
+			InputEventGenerator::ESCAPE_GRAB_WAIT_LOST;
+		inputEventGenerator.setGrabInput(false);
 	}
 	return "";
 }
 
-string InputEventGenerator::EscapeGrabCmd::help(const vector<string>& /*tokens*/) const
+string EscapeGrabCmd::help(const vector<string>& /*tokens*/) const
 {
 	return "Temporarily release input grab.";
 }
