@@ -1,4 +1,4 @@
-// $Id: DSKDiskImage.cc,v 1.12 2005/04/05 17:41:20 m9710797 Exp $
+// $Id: DSKDiskImage.cc,v 1.14 2006/02/06 21:05:40 m9710797 Exp $
 
 #include "DSKDiskImage.hh"
 #include "File.hh"
@@ -9,7 +9,8 @@ using std::string;
 namespace openmsx {
 
 DSKDiskImage::DSKDiskImage(const string& fileName)
-	: file(new File(fileName))
+	: SectorBasedDisk(fileName)
+	, file(new File(fileName, File::PRE_CACHE))
 {
 	nbSectors = file->getSize() / SECTOR_SIZE;
 }
