@@ -1,11 +1,11 @@
-// $Id: Disk.cc,v 1.9 2005/02/28 18:57:35 m9710797 Exp $
+// $Id: Disk.cc,v 1.12 2006/02/06 21:05:40 m9710797 Exp $
 
 #include "Disk.hh"
 
 namespace openmsx {
 
-Disk::Disk()
-	: nbSides(0)
+Disk::Disk(const std::string& name_)
+	: nbSides(0), name(name_)
 {
 }
 
@@ -13,6 +13,11 @@ Disk::~Disk()
 {
 }
 
+const std::string& Disk::getName() const
+{
+	return name;
+}
+
 void Disk::getTrackHeader(byte /*track*/, byte /*side*/, byte* /*buf*/)
 {
 	PRT_DEBUG("Disk::getTrackHeader [unimplemented]");
@@ -65,7 +70,7 @@ int Disk::physToLog(byte track, byte side, byte sector)
 	return result;
 }
 
-void Disk::logToPhys(int log, byte &track, byte &side, byte &sector)
+void Disk::logToPhys(int log, byte& track, byte& side, byte& sector)
 {
 	if (!nbSides) {
 		// try to guess values from boot sector
@@ -81,20 +86,20 @@ void Disk::detectGeometryFallback() // if all else fails, use statistics
 	// TODO maybe also check for 8*80*512 for 8 sectors per track
 	sectorsPerTrack = 9; // most of the time (sorry 5.25" disk users...)
 	// 360k disks are likely to be single sided:
-	nbSides = (getImageSize() == 720*512) ? 1 : 2;
+	nbSides = (getImageSize() == (720 * 512)) ? 1 : 2;
 }
 
 void Disk::detectGeometry()
 {
 	// From the MSX Red Book (p265):
-	// 
+	//
 	//  How to determine media types
 	//
 	//  a) Read the boot sector (track 0, sector 1) of the target drive
-	//  
+	//
 	//  b) Check if the first byte is either 0E9H or 0EBH (the JMP
 	//     instruction on 8086)
-	//     
+	//
 	//  c) If step b) fails, the disk is a version prior to MS-DOS 2.0;
 	//     therefore, use the first byte of the FAT passed from the caller
 	//     and make sure it is between 0F8h and 0FFh.
@@ -110,7 +115,7 @@ void Disk::detectGeometry()
 	//     +18 +19  Sectors per track
 	//     +1A +1B  Number of heads
 	//     ...
-	
+
 	//
 	//     Media Descriptor  0F8H  0F9H  0FAh  0FBH  0FCH  0FDH  0FEH  0FFH
 	//       byte (FATID)
@@ -118,7 +123,7 @@ void Disk::detectGeometry()
 	//     No. of sides         1     2     1     2     1     2     1     2
 	//     Tracks/side         80    80    80    80    40    40    40    40
 	//     ...
-	
+
 	try {
 		byte buf[512];
 		read(0, 1, 0, 512, buf);
