@@ -1,11 +1,13 @@
-// $Id: DiskDrive.cc,v 1.77 2005/05/25 17:17:38 m9710797 Exp $
+// $Id: DiskDrive.cc,v 1.87 2006/02/21 18:06:49 m9710797 Exp $
 
-#include "DiskDrive.hh"
 #include "Disk.hh"
+#include "DiskDrive.hh"
 #include "DiskChanger.hh"
 #include "EventDistributor.hh"
 #include "LedEvent.hh"
-#include "FileManipulator.hh"
+#include "CommandController.hh"
+#include "GlobalSettings.hh"
+#include "ThrottleManager.hh"
 #include <bitset>
 
 using std::string;
@@ -21,14 +23,6 @@ DiskDrive::~DiskDrive()
 
 /// class DummyDrive ///
 
-DummyDrive::DummyDrive()
-{
-}
-
-DummyDrive::~DummyDrive()
-{
-}
-
 bool DummyDrive::ready()
 {
 	return false;
@@ -134,6 +128,11 @@ bool DummyDrive::diskChanged()
 	return false;
 }
 
+bool DummyDrive::peekDiskChanged() const
+{
+	return false;
+}
+
 bool DummyDrive::dummyDrive()
 {
 	return true;
@@ -144,20 +143,32 @@ bool DummyDrive::dummyDrive()
 
 static std::bitset<RealDrive::MAX_DRIVES> drivesInUse;
 
-RealDrive::RealDrive(const EmuTime& time)
-	: headPos(0), motorStatus(false), motorTimer(time)
+RealDrive::RealDrive(CommandController& commandController,
+                     EventDistributor& eventDistributor_,
+                     Scheduler& scheduler,
+                     FileManipulator& fileManipulator, const EmuTime& time)
+	: Schedulable(scheduler)
+	, headPos(0), motorStatus(false), motorTimer(time)
 	, headLoadStatus(false), headLoadTimer(time)
+	, eventDistributor(eventDistributor_)
+	, throttleManager(commandController.getGlobalSettings().getThrottleManager())
+	, isLoading(false)
+	, timeOut(false)
 {
 	int i = 0;
 	while (drivesInUse[i]) {
 		if (++i == MAX_DRIVES) {
-			throw FatalError("Too many disk drives.");
+			throw MSXException("Too many disk drives.");
 		}
 	}
 	drivesInUse[i] = true;
 	string driveName = string("disk") + static_cast<char>('a' + i);
 
-	changer.reset(new DiskChanger(driveName, FileManipulator::instance()));
+	if (commandController.hasCommand(driveName)) {
+		throw MSXException("Duplicated drive name: " + driveName);
+	}
+	changer.reset(new DiskChanger(driveName, commandController,
+		                      fileManipulator));
 }
 
 RealDrive::~RealDrive()
@@ -178,7 +189,7 @@ bool RealDrive::writeProtected()
 	return motorStatus && changer->getDisk().writeProtected();
 }
 
-void RealDrive::step(bool direction, const EmuTime& /*time*/)
+void RealDrive::step(bool direction, const EmuTime& time)
 {
 	if (direction) {
 		// step in
@@ -192,10 +203,12 @@ void RealDrive::step(bool direction, const EmuTime& /*time*/)
 		}
 	}
 	PRT_DEBUG("DiskDrive track " << headPos);
+	resetTimeOut(time);
 }
 
-bool RealDrive::track00(const EmuTime& /*time*/)
+bool RealDrive::track00(const EmuTime& time)
 {
+	resetTimeOut(time);
 	// track00 bit is always 0 when motor is off
 	// verified on NMS8280
 	return motorStatus && (headPos == 0);
@@ -209,9 +222,11 @@ void RealDrive::setMotor(bool status, const EmuTime& time)
 		/* The following is a hack to emulate the drive LED behaviour.
 		 * This is in real life dependent on the FDC and should be
 		 * investigated in detail to implement it properly... TODO */
-		EventDistributor::instance().distributeEvent(
+		eventDistributor.distributeEvent(
 			new LedEvent(LedEvent::FDD, motorStatus));
+		updateLoadingState();
 	}
+
 }
 
 bool RealDrive::indexPulse(const EmuTime& time)
@@ -250,7 +265,7 @@ EmuTime RealDrive::getTimeTillSector(byte sector, const EmuTime& time)
 	//std::cout << "DEBUG a1: " << angle
 	//          << " a2: " << sectorAngle
 	//          << " delta: " << delta << std::endl;
-	
+
 	EmuDuration dur = Clock<TICKS_PER_ROTATION * ROTATIONS_PER_SECOND>::
 	                      duration(delta);
 	return time + dur;
@@ -278,7 +293,7 @@ void RealDrive::setHeadLoaded(bool status, const EmuTime& time)
 
 bool RealDrive::headLoaded(const EmuTime& time)
 {
-	return headLoadStatus && 
+	return headLoadStatus &&
 	       (headLoadTimer.getTicksTill(time) > 10);
 }
 
@@ -287,20 +302,57 @@ bool RealDrive::diskChanged()
 	return changer->diskChanged();
 }
 
+bool RealDrive::peekDiskChanged() const
+{
+	return changer->peekDiskChanged();
+}
+
 bool RealDrive::dummyDrive()
 {
 	return false;
 }
 
+void RealDrive::executeUntil(const EmuTime& /*time*/, int /*userData*/)
+{
+	timeOut = true;
+	updateLoadingState();
+}
+
+void RealDrive::updateLoadingState()
+{
+	bool newState = motorStatus && (!timeOut);
+        if (isLoading != newState) {
+                isLoading = newState;
+                throttleManager.indicateLoadingState(isLoading);
+        }
+}
+
+
+const std::string& RealDrive::schedName() const
+{
+        static const string schedName = "RealDrive";
+        return schedName;
+}
+
+void RealDrive::resetTimeOut(const EmuTime& time)
+{
+	timeOut = false;
+	updateLoadingState();
+	removeSyncPoint();
+	Clock<1000> now(time);
+	setSyncPoint(now + 1000);
+}
 
 /// class SingleSidedDrive ///
 
-SingleSidedDrive::SingleSidedDrive(const EmuTime& time)
-	: RealDrive(time)
-{
-}
-
-SingleSidedDrive::~SingleSidedDrive()
+SingleSidedDrive::SingleSidedDrive(
+		CommandController& commandController,
+		EventDistributor& eventDistributor,
+		Scheduler& scheduler,
+		FileManipulator& fileManipulator,
+		const EmuTime& time)
+	: RealDrive(commandController, eventDistributor, scheduler,
+	            fileManipulator, time)
 {
 }
 
@@ -359,16 +411,18 @@ void SingleSidedDrive::writeTrackData(byte data)
 
 /// class DoubleSidedDrive ///
 
-DoubleSidedDrive::DoubleSidedDrive(const EmuTime& time)
-	: RealDrive(time)
+DoubleSidedDrive::DoubleSidedDrive(
+		CommandController& commandController,
+		EventDistributor& eventDistributor,
+		Scheduler& scheduler,
+		FileManipulator& fileManipulator,
+		const EmuTime& time)
+	: RealDrive(commandController, eventDistributor, scheduler,
+	            fileManipulator, time)
 {
 	side = 0;
 }
 
-DoubleSidedDrive::~DoubleSidedDrive()
-{
-}
-
 bool DoubleSidedDrive::doubleSided()
 {
 	return changer->getDisk().doubleSided();
