@@ -1,4 +1,4 @@
-// $Id: EmptyDiskPatch.cc,v 1.3 2005/03/11 16:22:59 m9710797 Exp $
+// $Id: EmptyDiskPatch.cc,v 1.4 2006/01/24 18:03:35 m9710797 Exp $
 
 #include "EmptyDiskPatch.hh"
 #include "SectorBasedDisk.hh"
@@ -13,7 +13,7 @@ EmptyDiskPatch::EmptyDiskPatch(SectorBasedDisk& disk_)
 
 void EmptyDiskPatch::copyBlock(unsigned src, byte* dst, unsigned num) const
 {
-	if (num); // avoid warning
+	(void)num;
 	assert(num == SectorBasedDisk::SECTOR_SIZE);
 	disk.readLogicalSector(src / SectorBasedDisk::SECTOR_SIZE, dst);
 }
