@@ -1,4 +1,4 @@
-// $Id: SectorBasedDisk.cc,v 1.10 2005/04/11 11:15:08 m9710797 Exp $
+// $Id: SectorBasedDisk.cc,v 1.13 2006/02/06 21:05:40 m9710797 Exp $
 
 #include "SectorBasedDisk.hh"
 #include "EmptyDiskPatch.hh"
@@ -7,8 +7,9 @@
 
 namespace openmsx {
 
-SectorBasedDisk::SectorBasedDisk()
-	: patch(new EmptyDiskPatch(*this))
+SectorBasedDisk::SectorBasedDisk(const std::string& name)
+	: Disk(name)
+	, patch(new EmptyDiskPatch(*this))
 {
 }
 
@@ -19,7 +20,7 @@ SectorBasedDisk::~SectorBasedDisk()
 void SectorBasedDisk::read(byte track, byte sector, byte side,
                            unsigned size, byte* buf)
 {
-	if (size); // avoid warning
+	(void)size;
 	assert(size == SECTOR_SIZE);
 	unsigned logicalSector = physToLog(track, side, sector);
 	if (logicalSector >= nbSectors) {
@@ -32,10 +33,10 @@ void SectorBasedDisk::read(byte track, byte sector, byte side,
 	}
 }
 
-void SectorBasedDisk::write(byte track, byte sector, byte side, 
+void SectorBasedDisk::write(byte track, byte sector, byte side,
                             unsigned size, const byte* buf)
 {
-	if (size); // avoid warning
+	(void)size;
 	assert(size == SECTOR_SIZE);
 	if (writeProtected()) {
 		throw WriteProtectedException("");
@@ -61,7 +62,7 @@ void SectorBasedDisk::initWriteTrack(byte track, byte side)
 	if (writeProtected()) {
 		throw WriteProtectedException("");
 	}
-	
+
 	writeTrackBufCur = 0;
 	writeTrack_track = track;
 	writeTrack_side = side;
@@ -74,13 +75,13 @@ void SectorBasedDisk::writeTrackData(byte data)
 	if (writeProtected()) {
 		throw WriteProtectedException("");
 	}
-	
+
 	// if it is a 0xF7 ("two CRC characters") then the previous 512
 	// bytes could be actual sectordata bytes
 	if (data == 0xF7) {
 		if (writeTrack_CRCcount & 1) {
 			// first CRC is sector header CRC, second CRC is actual
-			// sector data CRC so write them 
+			// sector data CRC so write them
 			byte tempWriteBuf[512];
 			for (int i = 0; i < 512; i++) {
 				tempWriteBuf[i] =
@@ -90,7 +91,7 @@ void SectorBasedDisk::writeTrackData(byte data)
 			      writeTrack_side, 512, tempWriteBuf);
 			writeTrack_sector++; // update sector counter
 		}
-		writeTrack_CRCcount++; 
+		writeTrack_CRCcount++;
 	} else {
 		writeTrackBuf[writeTrackBufCur++] = data;
 		writeTrackBufCur &= 511;
@@ -114,7 +115,7 @@ void SectorBasedDisk::initReadTrack(byte track, byte side)
 	// '00' x 12, 'A1' x 3, 'FE' x 1,
 	// C,H,R,N,CRC(2bytes), '4E' x 22, '00' x 12,
 	// 'A1' x 4,'FB'('F8') x 1, data(512 bytes),CRC(2bytes),'4E'(gap3)
-	
+
 	readTrackDataCount = 0;
 	byte* tmppoint = readTrackDataBuf;
 	// Fill track header
@@ -177,20 +178,20 @@ unsigned SectorBasedDisk::getNbSectors() const
 void SectorBasedDisk::detectGeometry()
 {
 	// the following are just heuristics...
-	
+
 	if (nbSectors == 1440) {
 		// explicitly check for 720kb filesize
-		 
+
 		// "trojka.dsk" is 720kb, but has bootsector and FAT media ID
 		// for a single sided disk. From an emulator point of view it
 		// must be accessed as a double sided disk.
-		 
+
 		// "SDSNAT2.DSK" has invalid media ID in both FAT and
 		// bootsector, other data in the bootsector is invalid as well.
 		// Altough the first byte of the bootsector is 0xE9 to indicate
 		// valid bootsector data. The only way to detect the format is
 		// to look at the diskimage filesize.
-		
+
 		sectorsPerTrack = 9;
 		nbSides = 2;
 
