@@ -1,4 +1,4 @@
-// $Id: TC8566AF.cc,v 1.18 2005/02/28 18:57:35 m9710797 Exp $
+// $Id: TC8566AF.cc,v 1.25 2006/06/25 08:03:42 m9710797 Exp $
 
 /*
  * Based on code from NLMSX written by Frits Hilderink
@@ -35,6 +35,7 @@ const byte CMD_SENSE_DEVICE_STATUS    = 15;
 
 
 TC8566AF::TC8566AF(DiskDrive* drv[4], const EmuTime& time)
+	: delayTime(EmuTime::zero)
 {
 	drive[0] = drv[0];
 	drive[1] = drv[1];
@@ -43,10 +44,6 @@ TC8566AF::TC8566AF(DiskDrive* drv[4], const EmuTime& time)
 	reset(time);
 }
 
-TC8566AF::~TC8566AF()
-{
-}
-
 void TC8566AF::reset(const EmuTime& time)
 {
 	// Control register 0
@@ -106,7 +103,7 @@ void TC8566AF::reset(const EmuTime& time)
 	PCN = 0;		// Present Cylinder Number
 }
 
-byte TC8566AF::makeST0()
+byte TC8566AF::makeST0() const
 {
 	return  (ST0_IC << 6) |		// bit 7,6	Interrupt Code
 	        (ST0_SE << 5) |		// bit 5	Seek End
@@ -116,7 +113,7 @@ byte TC8566AF::makeST0()
 	        (ST0_DS << 0);		// bit 1,0	Drive Select
 }
 
-byte TC8566AF::makeST1()
+byte TC8566AF::makeST1() const
 {
 	return  (ST1_EN << 7) |		// bit 7	End of Cylinder
 	        (ST1_DE << 5) |		// bit 5	data Error
@@ -126,7 +123,7 @@ byte TC8566AF::makeST1()
 	        (ST1_MA << 0);		// bit 0	Missing Address Mark
 }
 
-byte TC8566AF::makeST2()
+byte TC8566AF::makeST2() const
 {
 	return  (ST2_CM << 6) |		// bit 6	Control Mark
 	        (ST2_DD << 5) |		// bit 5	data Error in data Field
@@ -137,7 +134,7 @@ byte TC8566AF::makeST2()
 	        (ST2_MD << 0);		// bit 0	Missing Address Mark in data Field
 }
 
-byte TC8566AF::makeST3()
+byte TC8566AF::makeST3() const
 {
 	return  (ST3_FLT << 7) |	// bit 7	Fault
 	        (ST3_WP  << 6) |	// bit 6	Write Protect
@@ -150,40 +147,69 @@ byte TC8566AF::makeST3()
 
 byte TC8566AF::readReg(int reg, const EmuTime& time)
 {
-	byte result = 0xFF;
+	byte result;
 	switch (reg) {
-	case 4: // Main Status Register
+	case 4: { // Main Status Register
 		if (delayTime.before(time)) {
 			RequestForMaster = 1;
 		}
+		bool dma = NonDMAMode && (Phase == PHASE_DATATRANSFER);
 		result = (RequestForMaster << 7) |
 		         (dataInputOutput  << 6) |
-		         (NonDMAMode       << 5) |
+		         (dma              << 5) |
 		         (FDCBusy          << 4) |
 		         (FDD3Busy         << 3) |
 		         (FDD2Busy         << 2) |
 		         (FDD1Busy         << 1) |
 		         (FDD0Busy         << 0);
 		break;
-
+	}
 	case 5: // data port
 		switch (Phase) {
 		case PHASE_DATATRANSFER:
-			result = readDataTransferPhase(time);
+			result = readDataTransferPhase();
 			RequestForMaster = 0;
-			delayTime.advance(time);
+			delayTime.reset(time);
 			delayTime += 15;
 			break;
 		case PHASE_RESULT:
-			result = readDataResultPhase(time);
+			result = readDataResultPhase();
 			break;
+		default:
+			result = 0;
 		}
 		break;
+
+	default:
+		result = 0xFF;
+	}
+	return result;
+}
+
+byte TC8566AF::peekReg(int reg, const EmuTime& time)
+{
+	byte result;
+	switch (reg) {
+	case 5: // data port
+		switch (Phase) {
+		case PHASE_DATATRANSFER:
+			result = peekDataTransferPhase();
+			break;
+		case PHASE_RESULT:
+			result = peekDataResultPhase();
+			break;
+		default:
+			result = readReg(reg, time);
+		}
+		break;
+	default:
+		result = readReg(reg, time);
+		break;
 	}
 	return result;
 }
 
-byte TC8566AF::readDataTransferPhase(const EmuTime& /*time*/)
+byte TC8566AF::readDataTransferPhase()
 {
 	byte result = 0xFF;
 	switch (Command) {
@@ -201,14 +227,27 @@ byte TC8566AF::readDataTransferPhase(const EmuTime& /*time*/)
 	return result;
 }
 
-byte TC8566AF::readDataResultPhase(const EmuTime& /*time*/)
+byte TC8566AF::peekDataTransferPhase() const
+{
+	byte result = 0xFF;
+	switch (Command) {
+	case CMD_READ_DATA:
+		if (SectorByteCount > 0) {
+			result = Sector[SectorPtr];
+		}
+		break;
+	}
+	return result;
+}
+
+byte TC8566AF::readDataResultPhase()
 {
 	byte result = 0xFF;
 	switch (Command) {
 	case CMD_READ_DATA:
 	case CMD_WRITE_DATA:
 	case CMD_FORMAT:
-		switch	(PhaseStep++) {
+		switch (PhaseStep++) {
 		case 0:
 			result = makeST0();
 			break;
@@ -267,6 +306,60 @@ byte TC8566AF::readDataResultPhase(const EmuTime& /*time*/)
 	return result;
 }
 
+byte TC8566AF::peekDataResultPhase() const
+{
+	byte result = 0xFF;
+	switch (Command) {
+	case CMD_READ_DATA:
+	case CMD_WRITE_DATA:
+	case CMD_FORMAT:
+		switch (PhaseStep) {
+		case 0:
+			result = makeST0();
+			break;
+		case 1:
+			result = makeST1();
+			break;
+		case 2:
+			result = makeST2();
+			break;
+		case 3:
+			result = StartCylinder;
+			break;
+		case 4:
+			result = StartHead;
+			break;
+		case 5:
+			result = StartRecord;
+			break;
+		case 6:
+			result = StartN;
+			break;
+		}
+		break;
+
+	case CMD_SENSE_INTERRUPT_STATUS:
+		switch (PhaseStep) {
+		case 0:
+			result = makeST0();
+			break;
+		case 1:
+			result = PCN;
+			break;
+		}
+		break;
+
+	case CMD_SENSE_DEVICE_STATUS:
+		switch	(PhaseStep) {
+		case 0:
+			result = makeST3();
+			break;
+		}
+		break;
+	}
+	return result;
+}
+
 
 void TC8566AF::writeReg(int reg, byte data, const EmuTime& time)
 {
@@ -298,7 +391,7 @@ void TC8566AF::writeReg(int reg, byte data, const EmuTime& time)
 		case PHASE_DATATRANSFER:
 			writeDataTransferPhase(data, time);
 			RequestForMaster = 0;
-			delayTime.advance(time);
+			delayTime.reset(time);
 			delayTime += 15;
 			break;
 
@@ -339,6 +432,7 @@ void TC8566AF::writeDataIdlePhase(byte data, const EmuTime& /*time*/)
 		case CMD_FORMAT:
 			ST0_IC = 0;
 			ST1_ND = 0;
+			ST1_NW = 0;
 			MT  = (data & 0x80) != 0;
 			MFM = (data & 0x40) != 0;
 			SK  = (data & 0x20) != 0;
@@ -570,10 +664,11 @@ void TC8566AF::writeDataTransferPhase(byte data, const EmuTime& /*time*/)
 				PhaseStep = 0;
 				dataInputOutput = 1;
 			} catch (MSXException& e) {
+				ST1_NW = 1;
 			}
 		}
 		break;
-	
+
 	case CMD_FORMAT:
 		switch(PhaseStep & 3) {
 		case 0:
@@ -589,6 +684,7 @@ void TC8566AF::writeDataTransferPhase(byte data, const EmuTime& /*time*/)
 				    StartRecord, Sector, dummy,
 				    dummy, dummy, dummy2);
 			} catch (MSXException& e) {
+				ST1_NW = 1;
 			}
 			break;
 		case 2:
@@ -610,4 +706,9 @@ bool TC8566AF::diskChanged(int driveno)
 	return drive[driveno]->diskChanged();
 }
 
+bool TC8566AF::peekDiskChanged(int driveno) const
+{
+	return drive[driveno]->peekDiskChanged();
+}
+
 } // namespace openmsx
