@@ -1,4 +1,4 @@
-// $Id: TurboRFDC.cc,v 1.23 2005/01/17 19:29:56 m9710797 Exp $
+// $Id: TurboRFDC.cc,v 1.26 2005/09/12 17:09:23 m9710797 Exp $
 
 /*
  * Based on code from NLMSX written by Frits Hilderink
@@ -8,12 +8,14 @@
 #include "TC8566AF.hh"
 #include "Rom.hh"
 #include "MSXCPU.hh"
+#include "MSXMotherBoard.hh"
 #include "CPU.hh"
 
 namespace openmsx {
 
-TurboRFDC::TurboRFDC(const XMLElement& config, const EmuTime& time)
-	: MSXFDC(config, time)
+TurboRFDC::TurboRFDC(MSXMotherBoard& motherBoard, const XMLElement& config,
+                     const EmuTime& time)
+	: MSXFDC(motherBoard, config, time)
 	, controller(new TC8566AF(reinterpret_cast<DiskDrive**>(drives), time))
 {
 	blockMask = (rom->getSize() / 0x4000) - 1;
@@ -27,6 +29,7 @@ TurboRFDC::~TurboRFDC()
 void TurboRFDC::reset(const EmuTime& time)
 {
 	memory = &(*rom)[0];
+	getMotherBoard().getCPU().invalidateMemCache(0x4000, 0x4000);
 	controller->reset(time);
 }
 
@@ -36,11 +39,6 @@ byte TurboRFDC::readMem(word address, const EmuTime& time)
 	if (0x3FF0 <= (address & 0x3FFF)) {
 		switch (address & 0x3FFF) {
 		case 0x3FF1:
-			// bit 0	FD2HD1	High Density detect drive 1
-			// bit 1	FD2HD2	High Density detect drive 2
-			// bit 4	FDCHG1	Disk Change detect on drive 1
-			// bit 5	FDCHG2	Disk Change detect on drive 2
-			// active low
 			result = 0x33;
 			if (controller->diskChanged(0)) result &= ~0x10;
 			if (controller->diskChanged(1)) result &= ~0x20;
@@ -53,6 +51,39 @@ byte TurboRFDC::readMem(word address, const EmuTime& time)
 		case 0x3FFB:
 			result = controller->readReg(5, time);
 			break;
+		default:
+			result = peekMem(address, time);
+			break;
+		}
+	} else {
+		result = peekMem(address, time);
+	}
+	return result;
+}
+
+byte TurboRFDC::peekMem(word address, const EmuTime& time) const
+{
+	byte result;
+	if (0x3FF0 <= (address & 0x3FFF)) {
+		switch (address & 0x3FFF) {
+		case 0x3FF1:
+			// bit 0	FD2HD1	High Density detect drive 1
+			// bit 1	FD2HD2	High Density detect drive 2
+			// bit 4	FDCHG1	Disk Change detect on drive 1
+			// bit 5	FDCHG2	Disk Change detect on drive 2
+			// active low
+			result = 0x33;
+			if (controller->peekDiskChanged(0)) result &= ~0x10;
+			if (controller->peekDiskChanged(1)) result &= ~0x20;
+			break;
+		case 0x3FF4:
+		case 0x3FFA:
+			result = controller->peekReg(4, time);
+			break;
+		case 0x3FF5:
+		case 0x3FFB:
+			result = controller->peekReg(5, time);
+			break;
 		default:
 			result = 0xFF;
 			break;
@@ -77,11 +108,11 @@ const byte* TurboRFDC::getReadCacheLine(word start) const
 	}
 }
 
-void TurboRFDC::writeMem(word address, byte value, const EmuTime& time)  
+void TurboRFDC::writeMem(word address, byte value, const EmuTime& time)
 {
 	//PRT_DEBUG("TurboRFDC: write 0x" << hex << (int)address << " 0x" << (int)value << dec);
 	if ((address == 0x6000) || (address == 0x7FF0) || (address == 0x7FFE)) {
-		MSXCPU::instance().invalidateMemCache(0x4000, 0x4000);
+		getMotherBoard().getCPU().invalidateMemCache(0x4000, 0x4000);
 		memory = &(*rom)[0x4000 * (value & blockMask)];
 		return;
 	} else {
