@@ -1,12 +1,9 @@
-// $Id: WD2793.cc,v 1.43 2005/05/14 12:49:39 m9710797 Exp $
+// $Id: WD2793.cc,v 1.49 2006/01/23 19:35:25 m9710797 Exp $
 
 #include "WD2793.hh"
 #include "DiskDrive.hh"
-#include "Scheduler.hh"
 #include "MSXException.hh"
 
-using std::string;
-
 namespace openmsx {
 
 // Status register
@@ -38,32 +35,30 @@ static const int IMM_IRQ    = 0x08;
 // Sync point types
 enum SyncPointType { SCHED_FSM, SCHED_IDX_IRQ };
 
-WD2793::WD2793(DiskDrive& drive_, const EmuTime& time)
-	: drive(drive_)
+WD2793::WD2793(Scheduler& scheduler, DiskDrive& drive_, const EmuTime& time)
+	: Schedulable(scheduler)
+	, drive(drive_)
+	, commandStart(time)
+	, DRQTimer(time)
 {
 	reset(time);
 }
 
-WD2793::~WD2793()
-{
-}
-
 void WD2793::reset(const EmuTime& time)
 {
-	//PRT_DEBUG("WD2793::reset()");
-	Scheduler::instance().removeSyncPoint(*this, SCHED_FSM);
-	Scheduler::instance().removeSyncPoint(*this, SCHED_IDX_IRQ);
+	removeSyncPoint(SCHED_FSM);
+	removeSyncPoint(SCHED_IDX_IRQ);
 	fsmState = FSM_NONE;
 
 	statusReg = 0;
 	trackReg = 0;
 	dataReg = 0;
 	directionIn = true;
-	
+
 	setDRQ(false, time);
 	resetIRQ();
 	immediateIRQ = false;
-	
+
 	needInitWriteTrack = false;
 	formatting = false;
 	transferring = false;
@@ -107,12 +102,22 @@ bool WD2793::getDTRQ(const EmuTime& time)
 	return DRQ;
 }
 
+bool WD2793::peekDTRQ(const EmuTime& /*time*/)
+{
+	return DRQ; // TODO can be improved
+}
+
 bool WD2793::getIRQ(const EmuTime& /*time*/)
 {
 	//PRT_DEBUG("WD2793::getIRQ() " << INTRQ);
 	return INTRQ | immediateIRQ;
 }
 
+bool WD2793::peekIRQ(const EmuTime& time)
+{
+	return getIRQ(time);
+}
+
 void WD2793::setIRQ()
 {
 	INTRQ = true;
@@ -132,7 +137,7 @@ void WD2793::setDRQ(bool drq, const EmuTime& time)
 void WD2793::setCommandReg(byte value, const EmuTime& time)
 {
 	//PRT_DEBUG("WD2793::setCommandReg() 0x" << std::hex << (int)value);
-	Scheduler::instance().removeSyncPoint(*this, SCHED_FSM);
+	removeSyncPoint(SCHED_FSM);
 
 	commandReg = value;
 	resetIRQ();
@@ -155,13 +160,13 @@ void WD2793::setCommandReg(byte value, const EmuTime& time)
 		case 0xB0: // write sector (multi)
 			startType2Cmd(time);
 			break;
-			
+
 		case 0xC0: // Read Address
 		case 0xE0: // read track
 		case 0xF0: // write track
 			startType3Cmd(time);
 			break;
-		
+
 		case 0xD0: // Force interrupt
 			startType4Cmd(time);
 			break;
@@ -171,7 +176,7 @@ void WD2793::setCommandReg(byte value, const EmuTime& time)
 byte WD2793::getStatusReg(const EmuTime& time)
 {
 	if (((commandReg & 0x80) == 0) || ((commandReg & 0xF0) == 0xD0)) {
-		// Type I or type IV command 
+		// Type I or type IV command
 		statusReg &= ~(INDEX | TRACK00 | HEAD_LOADED | WRITE_PROTECTED);
 		if (drive.indexPulse(time)) {
 			statusReg |=  INDEX;
@@ -205,6 +210,11 @@ byte WD2793::getStatusReg(const EmuTime& time)
 	return statusReg;
 }
 
+byte WD2793::peekStatusReg(const EmuTime& time)
+{
+	return getStatusReg(time);
+}
+
 void WD2793::setTrackReg(byte value, const EmuTime& /*time*/)
 {
 	//PRT_DEBUG("WD2793::setTrackReg() 0x" << std::hex << (int)value);
@@ -216,6 +226,11 @@ byte WD2793::getTrackReg(const EmuTime& /*time*/)
 	return trackReg;
 }
 
+byte WD2793::peekTrackReg (const EmuTime& time)
+{
+	return getTrackReg(time);
+}
+
 void WD2793::setSectorReg(byte value, const EmuTime& /*time*/)
 {
 	//PRT_DEBUG("WD2793::setSectorReg() 0x" << std::hex << (int)value);
@@ -227,6 +242,11 @@ byte WD2793::getSectorReg(const EmuTime& /*time*/)
 	return sectorReg;
 }
 
+byte WD2793::peekSectorReg(const EmuTime& time)
+{
+	return getSectorReg(time);
+}
+
 void WD2793::setDataReg(byte value, const EmuTime& time)
 {
 	//PRT_DEBUG("WD2793::setDataReg() 0x" << std::hex << (int)value);
@@ -288,7 +308,7 @@ void WD2793::setDataReg(byte value, const EmuTime& time)
 			drive.initWriteTrack();
 		}
 		setDRQ(false, time);
-		
+
 		//indexmark related timing
 		int pulses = drive.indexPulseCount(commandStart, time);
 		switch (pulses) {
@@ -350,7 +370,7 @@ void WD2793::setDataReg(byte value, const EmuTime& time)
 		   if (indexmark) {
 		   statusReg &= ~0x03;	// reset status on Busy and DRQ
 		   setIRQ();
-		   DRQ = false; 
+		   DRQ = false;
 		   }
 		 */
 	}
@@ -380,6 +400,16 @@ byte WD2793::getDataReg(const EmuTime& time)
 	return dataReg;
 }
 
+byte WD2793::peekDataReg(const EmuTime& /*time*/)
+{
+	if (((commandReg & 0xE0) == 0x80) && (statusReg & BUSY)) {
+		// READ SECTOR
+		return dataBuffer[dataCurrent];
+	} else {
+		return dataReg;
+	}
+}
+
 void WD2793::tryToReadSector()
 {
 	try {
@@ -408,9 +438,9 @@ void WD2793::tryToReadSector()
 
 void WD2793::schedule(FSMState state, const EmuTime& time)
 {
-	assert(!Scheduler::instance().pendingSyncPoint(*this, SCHED_FSM));
+	assert(!pendingSyncPoint(SCHED_FSM));
 	fsmState = state;
-	Scheduler::instance().setSyncPoint(time, *this, SCHED_FSM);
+	setSyncPoint(time, SCHED_FSM);
 }
 
 void WD2793::executeUntil(const EmuTime& time, int userData)
@@ -450,14 +480,14 @@ void WD2793::executeUntil(const EmuTime& time, int userData)
 			break;
 		case FSM_TYPE3_WAIT_LOAD:
 			if (((commandReg & 0xC0) == 0xC0) &&
-			    ((commandReg & 0xF0) != 0xD0)) { 
+			    ((commandReg & 0xF0) != 0xD0)) {
 				// Type III command
 				type3WaitLoad(time);
 			}
 			break;
 		case FSM_TYPE3_LOADED:
 			if (((commandReg & 0xC0) == 0xC0) &&
-			    ((commandReg & 0xF0) != 0xD0)) { 
+			    ((commandReg & 0xF0) != 0xD0)) {
 				// Type III command
 				type3Loaded(time);
 			}
@@ -467,9 +497,9 @@ void WD2793::executeUntil(const EmuTime& time, int userData)
 	}
 }
 
-const string& WD2793::schedName() const
+const std::string& WD2793::schedName() const
 {
-	static const string name("WD2793");
+	static const std::string name("WD2793");
 	return name;
 }
 
@@ -487,22 +517,22 @@ void WD2793::startType1Cmd(const EmuTime& time)
 			dataReg  = 0x00;
 			seek(time);
 			break;
-			
+
 		case 0x10: // seek
 			seek(time);
 			break;
-			
+
 		case 0x20: // step
 		case 0x30: // step (Update trackRegister)
 			step(time);
 			break;
-			
+
 		case 0x40: // step-in
 		case 0x50: // step-in (Update trackRegister)
 			directionIn = true;
 			step(time);
 			break;
-			
+
 		case 0x60: // step-out
 		case 0x70: // step-out (Update trackRegister)
 			directionIn = false;
@@ -714,7 +744,7 @@ void WD2793::startType4Cmd(const EmuTime& time)
 {
 	// Force interrupt
 	PRT_DEBUG("WD2793 command: Force interrupt");
-	
+
 	byte flags = commandReg & 0x0F;
 	if (flags & (N2R_IRQ | R2N_IRQ)) {
 		// all flags not yet supported
@@ -725,15 +755,14 @@ void WD2793::startType4Cmd(const EmuTime& time)
 		immediateIRQ = false;
 	}
 	if ((flags & IDX_IRQ) && drive.ready()) {
-		Scheduler::instance().setSyncPoint(
-			drive.getTimeTillIndexPulse(time), *this, SCHED_IDX_IRQ);
+		setSyncPoint(drive.getTimeTillIndexPulse(time), SCHED_IDX_IRQ);
 	} else {
-		Scheduler::instance().removeSyncPoint(*this, SCHED_IDX_IRQ);
+		removeSyncPoint(SCHED_IDX_IRQ);
 	}
 	if (flags & IMM_IRQ) {
 		immediateIRQ = true;
 	}
-	
+
 	setDRQ(false, time);
 	statusReg &= ~BUSY;	// reset status on Busy
 }
