@@ -1,4 +1,4 @@
-// $Id: File.cc,v 1.27 2005/01/11 20:52:45 m9710797 Exp $
+// $Id: File.cc,v 1.29 2006/01/25 19:26:32 m9710797 Exp $
 
 #include "File.hh"
 #include "FileBase.hh"
@@ -28,7 +28,7 @@ File::File(const string& url, OpenMode mode)
 	if (protocol == "file") {
 		file.reset(new LocalFile(name, mode));
 	} else {
-		throw FatalError("Unsupported protocol: " + protocol);
+		throw MSXException("Unsupported protocol: " + protocol);
 	}
 
 	if (((pos = name.rfind(".gz")) != string::npos) &&
@@ -93,6 +93,11 @@ void File::truncate(unsigned size)
 	return file->truncate(size);
 }
 
+void File::flush()
+{
+	file->flush();
+}
+
 const string File::getURL() const
 {
 	return file->getURL();
