@@ -1,4 +1,4 @@
-// $Id: FileBase.cc,v 1.10 2005/01/11 20:52:45 m9710797 Exp $
+// $Id: FileBase.cc,v 1.12 2006/04/25 21:03:23 m9710797 Exp $
 
 #include <cassert>
 #include <algorithm>
@@ -41,21 +41,24 @@ void FileBase::munmap()
 	}
 }
 
-void FileBase::truncate(unsigned size)
+void FileBase::truncate(unsigned newSize)
 {
-	int grow = size - getSize();
-	if (grow < 0) {
+	unsigned oldSize = getSize();
+	if (newSize < oldSize) {
 		PRT_DEBUG("Default truncate() can't shrink file!");
 		return;
 	}
-	const int BUF_SIZE = 4096;
+	unsigned grow = newSize - oldSize;
+	seek(oldSize);
+
+	const unsigned BUF_SIZE = 4096;
 	byte buf[BUF_SIZE];
 	memset(buf, 0, BUF_SIZE);
 	while (grow > 0) {
 		write(buf, std::min(BUF_SIZE, grow));
 		grow -= BUF_SIZE;
 	}
-} 
+}
 
 const string FileBase::getOriginalName()
 {
