@@ -1,9 +1,10 @@
-// $Id: FileContext.cc,v 1.48 2005/03/12 20:47:14 m9710797 Exp $
+// $Id: FileContext.cc,v 1.52 2005/09/22 11:27:28 m9710797 Exp $
 
 #include "FileContext.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
 #include "GlobalSettings.hh"
+#include "CommandController.hh"
 #include "Interpreter.hh"
 #include "CliComm.hh"
 #include "CommandException.hh"
@@ -57,7 +58,7 @@ string FileContext::resolve(const vector<string>& pathList,
 		// protocol specified or absolute path, don't resolve
 		return filepath;
 	}
-	
+
 	for (vector<string>::const_iterator it = pathList.begin();
 	     it != pathList.end();
 	     ++it) {
@@ -124,7 +125,7 @@ SystemFileContext::SystemFileContext(bool preferSystemDir)
 {
 	string userDir   = FileOperations::getUserDataDir()   + '/';
 	string systemDir = FileOperations::getSystemDataDir() + '/';
-	
+
 	if (preferSystemDir) {
 		paths.push_back(systemDir);
 		paths.push_back(userDir);
@@ -180,15 +181,16 @@ SettingFileContext::SettingFileContext(const SettingFileContext& rhs)
 
 // class UserFileContext
 
-UserFileContext::UserFileContext(const string& savePath, bool skipUserDirs)
+UserFileContext::UserFileContext(CommandController& commandController,
+                                 const string& savePath, bool skipUserDirs)
 {
-	paths.push_back("./");
+	paths.push_back("");
 	if (!skipUserDirs) {
 		try {
 			vector<string> dirs;
-			const string& list = GlobalSettings::instance().
+			const string& list = commandController.getGlobalSettings().
 				getUserDirSetting().getValue();
-			Interpreter::instance().splitList(list, dirs);
+			commandController.getInterpreter().splitList(list, dirs);
 			for (vector<string>::const_iterator it = dirs.begin();
 			     it != dirs.end(); ++it) {
 				string path = *it;
@@ -202,11 +204,11 @@ UserFileContext::UserFileContext(const string& savePath, bool skipUserDirs)
 				paths.push_back(path);
 			}
 		} catch (CommandException& e) {
-			CliComm::instance().printWarning(
+			commandController.getCliComm().printWarning(
 				"user directories: " + e.getMessage());
 		}
 	}
-	
+
 	if (!savePath.empty()) {
 		savePaths.push_back(FileOperations::getUserOpenMSXDir() +
 		                    "/persistent/" + savePath + '/');
