@@ -1,4 +1,4 @@
-// $Id: FileOperations.cc,v 1.39 2005/04/21 17:32:28 mthuurne Exp $
+// $Id: FileOperations.cc,v 1.49 2006/02/02 01:00:35 mthuurne Exp $
 
 #ifdef	_WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -13,113 +13,39 @@
 #include <algorithm>
 #define	MAXPATHLEN	MAX_PATH
 #define	mode_t	unsigned short int
+#else
+#include <sys/types.h>
+#include <pwd.h>
 #endif
 
 #ifdef __APPLE__
 #include <Carbon/Carbon.h>
 #endif
 
-#include <cerrno>
-#include <cstdlib>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
+#include "ReadDir.hh"
 #include "build-info.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
 #include "openmsx.hh"
 #include "CliComm.hh"
+#include <cerrno>
+#include <cstdlib>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <dirent.h>
 
 using std::string;
 
-#ifdef _WIN32
-
-static	struct maint_setenv_table {
-	char *env;
-	struct maint_setenv_table *next;
-} *maint_setenv_table_p = NULL, *maint_setenv_table_c = NULL;
-
-/*
- * A clean up for A tiny setenv() for win32.
- */
-static void cleanSetEnv(void)
-{
-	struct maint_setenv_table* p = maint_setenv_table_p;
-	while (p) {
-		if (p->env) {
-			unsigned len = strlen(p->env);
-			for (unsigned i = 0; i < len; ++i) {
-				if (p->env[i] == '=') {
-					char buf[i + 3];
-					strncpy(buf, p->env, i+1);
-					buf[i + 2] = '\0';
-					// when no string after '=' it works as
-					// unset. So buf will be ok even if it
-					// is auto variable...
-					_putenv(buf);
-					break;
-				}
-			}
-			free(p->env);
-		}
-		struct maint_setenv_table* q = p;
-		p = p->next;
-		free(q);
-	}
-}
-
-/*
- * A tiny setenv() for win32.
- */
-static int setenv(const char *name, const char *value, int overwrite)
-{
-	if (!overwrite && getenv(name)) {
-		return	0;
-	} else {
-		struct maint_setenv_table *p;
-		char *buf;
-		p=NULL; buf=NULL;
-		if ((p=(struct maint_setenv_table*)malloc(
-		                     sizeof(struct maint_setenv_table)))
-		    && (buf=(char*)malloc(strlen(name)+strlen(value)+2))) {
-			strcpy(buf, name);
-			strcat(buf, "=");
-			strcat(buf, value);
-			if (!_putenv(buf)) {
-				p->next = NULL;
-				p->env  = buf;
-				if (maint_setenv_table_p) {
-					maint_setenv_table_c->next = p;
-					maint_setenv_table_c = p;
-				} else {
-					maint_setenv_table_p = p;
-					maint_setenv_table_c = p;
-					atexit(cleanSetEnv);
-				}
-				return	0;
-			} else {
-				return	-1;
-			}
-		} else {
-			if (p) {
-				free(p);
-			}
-			if (buf) {
-				free(buf);
-			}
-			return	-1;
-		}
-	}
-}
-
-#endif // _WIN32
-
-
 namespace openmsx {
 
+namespace FileOperations {
+
 #ifdef __APPLE__
 
 std::string findShareDir() {
+	// Find bundle location:
+	// for an app folder, this is the outer directory,
+	// for an unbundled executable, it is the executable itself.
 	ProcessSerialNumber psn;
 	if (GetCurrentProcess(&psn) != noErr) {
 		throw FatalError("Failed to get process serial number");
@@ -128,40 +54,43 @@ std::string findShareDir() {
 	if (GetProcessBundleLocation(&psn, &location) != noErr) {
 		throw FatalError("Failed to get process bundle location");
 	}
+	// Get info about the location.
+	FSCatalogInfo catalogInfo;
+	FSRef parentRef;
+	if (FSGetCatalogInfo(
+		&location, kFSCatInfoVolume | kFSCatInfoNodeFlags,
+		&catalogInfo, NULL, NULL, &parentRef
+		) != noErr) {
+		throw FatalError("Failed to get info about bundle path");
+	}
+	// Get reference to root directory of the volume we are searching.
+	// We will need this later to know when to give up.
 	FSRef root;
-	if (FSPathMakeRef(reinterpret_cast<const UInt8*>("/"), &root, NULL) != noErr) {
+	if (FSGetVolumeInfo(
+		catalogInfo.volume, 0, NULL, kFSVolInfoNone, NULL, NULL, &root
+		) != noErr) {
 		throw FatalError("Failed to get reference to root directory");
 	}
+	// Make sure we are looking at a directory.
+	if (~catalogInfo.nodeFlags & kFSNodeIsDirectoryMask) {
+		// Location is not a directory, so it is the path to the executable.
+		location = parentRef;
+	}
 	while (true) {
-		OSErr err;
-		// Are we in the root yet?
-		err = FSCompareFSRefs(&location, &root);
-		if (err == noErr) {
-			throw FatalError("Could not find \"share\" directory anywhere");
-		}
-		// Go up one level.
-		// Initial location is the executable file itself, so the first level up
-		// will take us to the directory containing the executable.
-		if (FSGetCatalogInfo(
-			&location, kFSCatInfoNone, NULL, NULL, NULL, &location
-			) != noErr
-		) {
-			throw FatalError("Failed to get parent directory");
-		}
-
+		// Iterate through the files in the directory.
 		FSIterator iterator;
 		if (FSOpenIterator(&location, kFSIterateFlat, &iterator) != noErr) {
 			throw FatalError("Failed to open iterator");
 		}
 		bool filesLeft = true; // iterator has files left for next call
 		while (filesLeft) {
-			// Get several files at a time, to reduce the number of system calls.
+			// Get info about several files at a time.
 			const int MAX_SCANNED_FILES = 100;
 			ItemCount actualObjects;
 			FSRef refs[MAX_SCANNED_FILES];
 			FSCatalogInfo catalogInfos[MAX_SCANNED_FILES];
 			HFSUniStr255 names[MAX_SCANNED_FILES];
-			err = FSGetCatalogInfoBulk(
+			OSErr err = FSGetCatalogInfoBulk(
 				iterator,
 				MAX_SCANNED_FILES,
 				&actualObjects,
@@ -175,10 +104,10 @@ std::string findShareDir() {
 			if (err == errFSNoMoreItems) {
 				filesLeft = false;
 			} else if (err != noErr) {
-				throw FatalError("Catalog search failed");
+				throw FatalError("Catalog get failed");
 			}
 			for (ItemCount i = 0; i < actualObjects; i++) {
-				// We're only interested in directories.
+				// We're only interested in subdirectories.
 				if (catalogInfos[i].nodeFlags & kFSNodeIsDirectoryMask) {
 					// Convert the name to a CFString.
 					CFStringRef name = CFStringCreateWithCharactersNoCopy(
@@ -193,11 +122,14 @@ std::string findShareDir() {
 					CFRelease(name);
 					if (cmp == kCFCompareEqualTo) {
 						// Clean up.
-						err = FSCloseIterator(iterator);
-						assert(err == noErr);
+						if (FSCloseIterator(iterator) != noErr) {
+							assert(false);
+						}
 						// Get full path of directory.
 						UInt8 path[256];
-						if (FSRefMakePath(&refs[i], path, sizeof(path)) != noErr) {
+						if (FSRefMakePath(
+							&refs[i], path, sizeof(path)) != noErr
+							) {
 							throw FatalError("Path too long");
 						}
 						return std::string(reinterpret_cast<char*>(path));
@@ -205,52 +137,67 @@ std::string findShareDir() {
 				}
 			}
 		}
-		err = FSCloseIterator(iterator);
-		assert(err == noErr);
+		if (FSCloseIterator(iterator) != noErr) {
+			assert(false);
+		}
+		// Are we in the root yet?
+		if (FSCompareFSRefs(&location, &root) == noErr) {
+			throw FatalError("Could not find \"share\" directory anywhere");
+		}
+		// Go up one level.
+		if (FSGetCatalogInfo(
+			&location, kFSCatInfoNone, NULL, NULL, NULL, &parentRef
+			) != noErr
+		) {
+			throw FatalError("Failed to get parent directory");
+		}
+		location = parentRef;
 	}
 }
 
 #endif // __APPLE__
 
-/* A wrapper for mkdir().  On some systems, mkdir() does not take permision in
- * arguments. For such systems, in this function, adjust arguments.
- */
-static int doMkdir(const char* name, mode_t mode)
+string expandTilde(const string& path)
+{
+	if (path.empty() || path[0] != '~') {
+		return path;
+	}
+	string::size_type pos = path.find_first_of('/');
+	string user = ((path.size() == 1) || (pos == 1)) ? "" :
+		path.substr(1, (pos == string::npos) ? pos : pos - 1);
+	string result = getUserHomeDir(user);
+	if (result.empty()) {
+		// failed to find homedir, return the path unchanged
+		return path;
+	}
+	if (pos == string::npos) {
+		return result;
+	}
+	if (result[result.size() - 1] != '/') {
+		result += '/';
+	}
+	result += path.substr(pos + 1);
+	return result;
+}
+
+void mkdir(const string& path, mode_t mode)
 {
 #if	defined(__MINGW32__) || defined(_MSC_VER)
-	if ((name[0]=='/' || name[0]=='\\') && name[1]=='\0' ||
-	    (name[1]==':' && name[3]=='\0' && (name[2]=='/' || name[2]=='\\' || name[2]=='\0')
-	    && ((name[0]>='A' && name[0]<='Z')||(name[0]>='a' && name[0]<='z')))) {
-		// *(_errno()) = EEXIST;
-		// return -1;
-		return 0;
-	} else {
-		return mkdir(name);
+	if ((path == "/") ||
+	    StringOp::endsWith(path, ":") ||
+	    StringOp::endsWith(path, ":/")) {
+		return;
 	}
+	int result = ::mkdir(getNativePath(path).c_str());
 #else
-	return mkdir(name, mode);
+	int result = ::mkdir(getNativePath(path).c_str(), mode);
 #endif
-}
-
-
-string FileOperations::expandTilde(const string& path)
-{
-	if ((path.size() <= 1) || (path[0] != '~')) {
-		return path;
-	}
-	if (path[1] == '/') {
-		// current user
-		return getUserHomeDir() + path.substr(1);
-	} else {
-		// other user
-		CliComm::instance().printWarning(
-			"~<user>/ not yet implemented");
-		return path;
+	if (result && (errno != EEXIST)) {
+		throw FileException("Error creating dir " + path);
 	}
 }
 
-
-void FileOperations::mkdirp(const string& path_)
+void mkdirp(const string& path_)
 {
 	if (path_.empty()) {
 		return;
@@ -260,10 +207,7 @@ void FileOperations::mkdirp(const string& path_)
 	string::size_type pos = 0;
 	do {
 		pos = path.find_first_of('/', pos + 1);
-		if (doMkdir(getNativePath(path).substr(0, pos).c_str(), 0755) &&
-		    (errno != EEXIST)) {
-			throw FileException("Error creating dir " + path);
-		}
+		mkdir(path.substr(0, pos), 0755);
 	} while (pos != string::npos);
 
 	struct stat st;
@@ -272,7 +216,7 @@ void FileOperations::mkdirp(const string& path_)
 	}
 }
 
-string FileOperations::getFilename(const string& path)
+string getFilename(const string& path)
 {
 	string::size_type pos = path.rfind('/');
 	if (pos == string::npos) {
@@ -282,7 +226,7 @@ string FileOperations::getFilename(const string& path)
 	}
 }
 
-string FileOperations::getBaseName(const string& path)
+string getBaseName(const string& path)
 {
 	string::size_type pos = path.rfind('/');
 	if (pos == string::npos) {
@@ -292,7 +236,7 @@ string FileOperations::getBaseName(const string& path)
 	}
 }
 
-string FileOperations::getNativePath(const string &path)
+string getNativePath(const string &path)
 {
 #if	defined(_WIN32)
 	string result(path);
@@ -303,7 +247,7 @@ string FileOperations::getNativePath(const string &path)
 #endif
 }
 
-string FileOperations::getConventionalPath(const string &path)
+string getConventionalPath(const string &path)
 {
 #if	defined(_WIN32)
 	string result(path);
@@ -314,7 +258,7 @@ string FileOperations::getConventionalPath(const string &path)
 #endif
 }
 
-bool FileOperations::isAbsolutePath(const string& path)
+bool isAbsolutePath(const string& path)
 {
 #if	defined(_WIN32)
 	if ((path.size() >= 3) && (path[1] == ':') && (path[2] == '/')) {
@@ -327,11 +271,12 @@ bool FileOperations::isAbsolutePath(const string& path)
 	return !path.empty() && (path[0] == '/');
 }
 
-const string& FileOperations::getUserHomeDir()
+string getUserHomeDir(const string& username)
 {
+#if	defined(_WIN32)
+	if (username.empty()); // ignore parameter, avoid warning
 	static string userDir;
 	if (userDir.empty()) {
-#if	defined(_WIN32)
 		HMODULE sh32dll = LoadLibraryA("SHELL32.DLL");
 		if (sh32dll) {
 			FARPROC funcp = GetProcAddress(sh32dll, "SHGetSpecialFolderPathA");
@@ -355,14 +300,27 @@ const string& FileOperations::getUserHomeDir()
 				userDir.erase(2,1);  // remove the trailing slash because other functions will add it, X:// will be seen as protocol
 			}
 		}
-#else
-		userDir = getenv("HOME");
-#endif
 	}
 	return userDir;
+#else
+	const char* dir = NULL;
+	struct passwd* pw = NULL;
+	if (username.empty()) {
+		dir = getenv("HOME");
+		if (!dir) {
+			pw = getpwuid(getuid());
+		}
+	} else {
+		pw = getpwnam(username.c_str());
+	}
+	if (pw) {
+		dir = pw->pw_dir;
+	}
+	return dir ? dir : "";
+#endif
 }
 
-const string& FileOperations::getUserOpenMSXDir()
+const string& getUserOpenMSXDir()
 {
 #if defined(_WIN32)
 	static const string OPENMSX_DIR = expandTilde("~/openMSX");
@@ -372,20 +330,14 @@ const string& FileOperations::getUserOpenMSXDir()
 	return OPENMSX_DIR;
 }
 
-string FileOperations::getUserDataDir()
+string getUserDataDir()
 {
 	const char* const NAME = "OPENMSX_USER_DATA";
 	char* value = getenv(NAME);
-	if (value) {
-		return value;
-	}
-
-	string newValue = getUserOpenMSXDir() + "/share";
-	setenv(NAME, newValue.c_str(), 0);
-	return newValue;
+	return value ? value : getUserOpenMSXDir() + "/share";
 }
 
-string FileOperations::getSystemDataDir()
+string getSystemDataDir()
 {
 	const char* const NAME = "OPENMSX_SYSTEM_DATA";
 	char* value = getenv(NAME);
@@ -411,11 +363,10 @@ string FileOperations::getSystemDataDir()
 	// defined in build-info.hh (default /opt/openMSX/share)
 	newValue = DATADIR;
 #endif
-	setenv(NAME, newValue.c_str(), 0);
 	return newValue;
 }
 
-string FileOperations::expandCurrentDirFromDrive (const string& path)
+string expandCurrentDirFromDrive(const string& path)
 {
 	string result = path;
 #ifdef _WIN32
@@ -441,24 +392,90 @@ string FileOperations::expandCurrentDirFromDrive (const string& path)
 	return result;
 }
 
-bool FileOperations::isRegularFile(const string& filename)
+bool isRegularFile(const string& filename)
 {
 	struct stat st;
 	int ret = stat(filename.c_str(), &st);
 	return (ret == 0) && S_ISREG(st.st_mode);
 }
 
-bool FileOperations::isDirectory(const string& directory)
+bool isDirectory(const string& directory)
 {
 	struct stat st;
 	int ret = stat(directory.c_str(), &st);
 	return (ret == 0) && S_ISDIR(st.st_mode);
 }
 
-bool FileOperations::exists(const string& filename)
+bool exists(const string& filename)
 {
 	struct stat st;
 	return stat(filename.c_str(), &st) == 0;
 }
 
+static int getNextNum(dirent* d, const string& prefix, const string& extension,
+                      const unsigned int nofdigits)
+{
+	const unsigned int extensionlen = extension.length();
+	const unsigned int prefixlen = prefix.length();
+	string name(d->d_name);
+
+	if ((name.length() != (prefixlen + nofdigits + extensionlen)) ||
+	    (name.substr(0, prefixlen) != prefix) ||
+	    (name.substr(prefixlen + nofdigits, extensionlen) != extension)) {
+		return 0;
+	}
+	string num(name.substr(prefixlen, nofdigits));
+	char* endpos;
+	unsigned long n = strtoul(num.c_str(), &endpos, 10);
+	if (*endpos != '\0') {
+		return 0;
+	}
+	return n;
+}
+
+string getNextNumberedFileName(
+	const string& directory, const string& prefix, const string& extension)
+{
+	const unsigned int nofdigits = 4;
+
+	int max_num = 0;
+
+	string dirName = getUserOpenMSXDir() + "/" + directory;
+	try {
+		mkdirp(dirName);
+	} catch (FileException& e) {
+		// ignore
+	}
+
+	ReadDir dir(dirName.c_str());
+	while (dirent* d = dir.getEntry()) {
+		max_num = std::max(max_num, getNextNum(d, prefix, extension, nofdigits));
+	}
+
+	std::ostringstream os;
+	os << dirName << "/" << prefix;
+	os.width(nofdigits);
+	os.fill('0');
+	os << (max_num + 1) << extension;
+	return os.str();
+}
+
+string getTempDir()
+{
+	const char* result = NULL;
+	if (!result) result = getenv("TMPDIR");
+	if (!result) result = getenv("TMP");
+	if (!result) result = getenv("TEMP");
+	if (!result) {
+#ifdef _WIN32
+		result = "C:/WINDOWS/TEMP";
+#else
+		result = "/tmp";
+#endif
+	}
+	return result;
+}
+
+} // namespace FileOperations
+
 } // namespace openmsx
