@@ -1,4 +1,4 @@
-// $Id: LocalFile.cc,v 1.38 2004/12/15 09:09:47 m9710797 Exp $
+// $Id: LocalFile.cc,v 1.41 2006/01/11 18:57:34 m9710797 Exp $
 
 #include "probed_defs.hh"
 #include <unistd.h>
@@ -10,18 +10,19 @@
 #include "LocalFile.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
+#include "PreCacheFile.hh"
 
 using std::string;
 
 namespace openmsx {
 
-LocalFile::LocalFile(const string& filename_, OpenMode mode)
+LocalFile::LocalFile(const string& filename_, File::OpenMode mode)
 	: filename(FileOperations::expandTilde(filename_)),
 	  readOnly(false)
 {
 	PRT_DEBUG("LocalFile: " << filename);
 
-	if (mode == SAVE_PERSISTENT) {
+	if (mode == File::SAVE_PERSISTENT) {
 		string::size_type pos = filename.find_last_of('/');
 		if (pos != string::npos) {
 			FileOperations::mkdirp(filename.substr(0, pos));
@@ -29,10 +30,10 @@ LocalFile::LocalFile(const string& filename_, OpenMode mode)
 	}
 
 	const string name = FileOperations::getNativePath(filename);
-	if ((mode == SAVE_PERSISTENT) || (mode == TRUNCATE)) {
+	if ((mode == File::SAVE_PERSISTENT) || (mode == File::TRUNCATE)) {
 		// open file read/write truncated
 		file = fopen(name.c_str(), "wb+");
-	} else if (mode == CREATE) {
+	} else if (mode == File::CREATE) {
 		// open file read/write
 		file = fopen(name.c_str(), "rb+");
 		if (!file) {
@@ -51,6 +52,10 @@ LocalFile::LocalFile(const string& filename_, OpenMode mode)
 	if (!file) {
 		throw FileException("Error opening file " + filename);
 	}
+
+	if (mode == File::PRE_CACHE) {
+		cache.reset(new PreCacheFile(name));
+	}
 }
 
 LocalFile::~LocalFile()
@@ -138,6 +143,11 @@ void LocalFile::truncate(unsigned size)
 }
 #endif
 
+void LocalFile::flush()
+{
+	fflush(file);
+}
+
 const string LocalFile::getURL() const
 {
 	static const string prefix("file://");
