@@ -1,14 +1,12 @@
-// $Id: ReadDir.cc,v 1.2 2004/12/15 09:09:47 m9710797 Exp $
+// $Id: ReadDir.cc,v 1.4 2006/05/28 19:19:29 m9710797 Exp $
 
 #include "ReadDir.hh"
 
-using std::string;
-
 namespace openmsx {
 
-ReadDir::ReadDir(const string& directory)
+ReadDir::ReadDir(const std::string& directory)
 {
-	dir = opendir(directory.c_str());
+	dir = opendir(directory.empty() ? "." : directory.c_str());
 }
 
 ReadDir::~ReadDir()
