@@ -1,81 +1,41 @@
-// $Id: SunriseIDE.cc,v 1.34 2005/05/07 16:47:38 m9710797 Exp $
+// $Id: SunriseIDE.cc,v 1.42 2006/05/27 16:33:41 m9710797 Exp $
 
 #include "SunriseIDE.hh"
 #include "DummyIDEDevice.hh"
-#include "MSXCPU.hh"
 #include "IDEDeviceFactory.hh"
+#include "MSXMotherBoard.hh"
+#include "MSXCPU.hh"
 #include "Rom.hh"
-#include "FileManipulator.hh"
-#include "IDEHD.hh"
 #include "XMLElement.hh"
-#include "MSXException.hh"
-#include <bitset>
-
-using std::string;
 
 namespace openmsx {
 
-static const int MAX_INTERFACES = 26 / 2;
-static std::bitset<MAX_INTERFACES> interfaceInUse;
-
-SunriseIDE::SunriseIDE(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, rom(new Rom(getName() + " ROM", "rom", config))
+static IDEDevice* create(const XMLElement* elem, MSXMotherBoard& motherBoard,
+                         const EmuTime& time)
 {
-	int i = 0;
-	for ( ; i < MAX_INTERFACES; ++i) {
-		if (!interfaceInUse[i]) {
-			interfaceNum = i;
-			interfaceInUse[i] = true;
-			break;
-		}
-	}
-	if (i == MAX_INTERFACES) {
-		throw FatalError("Too many IDE interfaces.");
-	}
-	
-	const XMLElement* masterElem = config.findChild("master");
-	const XMLElement* slaveElem  = config.findChild("slave");
-	device[0].reset(masterElem 
-	          ? IDEDeviceFactory::create(*masterElem, time)
-	          : new DummyIDEDevice());
-	device[1].reset(slaveElem
-	          ? IDEDeviceFactory::create(*slaveElem, time)
-	          : new DummyIDEDevice());
+	return elem
+	     ? IDEDeviceFactory::create(motherBoard, *elem, time)
+	     : new DummyIDEDevice();
+}
 
-	registerDrive(0);
-	registerDrive(1);
+SunriseIDE::SunriseIDE(MSXMotherBoard& motherBoard, const XMLElement& config,
+                       const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, rom(new Rom(motherBoard, getName() + " ROM", "rom", config))
+{
+	device[0].reset(create(config.findChild("master"), motherBoard, time));
+	device[1].reset(create(config.findChild("slave" ), motherBoard, time));
 
 	// make valgrind happy
 	internalBank = 0;
 	ideRegsEnabled = false;
-	
+
 	writeControl(0xFF);
 	reset(time);
 }
 
 SunriseIDE::~SunriseIDE()
 {
-	unregisterDrive(1);
-	unregisterDrive(0);
-
-	interfaceInUse[interfaceNum] = false;
-}
-
-void SunriseIDE::registerDrive(int n)
-{
-	if (DiskContainer* cont = dynamic_cast<DiskContainer*>(device[n].get())) {
-		string name = string("hd") + (char)('a' + 2 * interfaceNum + n);
-		FileManipulator::instance().registerDrive(*cont, name);
-	}
-}
-
-void SunriseIDE::unregisterDrive(int n)
-{
-	if (DiskContainer* cont = dynamic_cast<DiskContainer*>(device[n].get())) {
-		string name = string("hd") + (char)('a' + 2 * interfaceNum + n);
-		FileManipulator::instance().unregisterDrive(*cont, name);
-	}
 }
 
 void SunriseIDE::reset(const EmuTime& time)
@@ -158,12 +118,13 @@ static byte reverse(byte a)
 
 void SunriseIDE::writeControl(byte value)
 {
+	MSXCPU& cpu = getMotherBoard().getCPU();
 	if (ideRegsEnabled != (value & 1)) {
 		ideRegsEnabled = value & 1;
-		MSXCPU::instance().invalidateMemCache(0x3C00, 0x0300);
-		MSXCPU::instance().invalidateMemCache(0x7C00, 0x0300);
-		MSXCPU::instance().invalidateMemCache(0xBC00, 0x0300);
-		MSXCPU::instance().invalidateMemCache(0xFC00, 0x0300);
+		cpu.invalidateMemCache(0x3C00, 0x0300);
+		cpu.invalidateMemCache(0x7C00, 0x0300);
+		cpu.invalidateMemCache(0xBC00, 0x0300);
+		cpu.invalidateMemCache(0xFC00, 0x0300);
 	}
 
 	byte bank = reverse(value & 0xF8);
@@ -172,7 +133,7 @@ void SunriseIDE::writeControl(byte value)
 	}
 	if (internalBank != &(*rom)[0x4000 * bank]) {
 		internalBank = &(*rom)[0x4000 * bank];
-		MSXCPU::instance().invalidateMemCache(0x4000, 0x4000);
+		cpu.invalidateMemCache(0x4000, 0x4000);
 	}
 }
 
@@ -207,7 +168,7 @@ byte SunriseIDE::readReg(nibble reg, const EmuTime& time)
 			// read status
 			result = 0xFF;	// BUSY
 		} else {
-			// all others 
+			// all others
 			result = 0x7F;	// don't care
 		}
 	} else {
