@@ -1,20 +1,9 @@
-// $Id: DummyJoystick.cc,v 1.7 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: DummyJoystick.cc,v 1.9 2005/11/11 22:00:02 m9710797 Exp $
 
 #include "DummyJoystick.hh"
 
-using std::string;
-
 namespace openmsx {
 
-DummyJoystick::DummyJoystick()
-{
-}
-
-DummyJoystick::~DummyJoystick()
-{
-}
-
-
 byte DummyJoystick::read(const EmuTime& /*time*/)
 {
 	return 0x3F;
@@ -25,13 +14,13 @@ void DummyJoystick::write(byte /*value*/, const EmuTime& /*time*/)
 	// do nothing
 }
 
-const string& DummyJoystick::getDescription() const
+const std::string& DummyJoystick::getDescription() const
 {
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyJoystick::plugHelper(Connector* /*connector*/,
+void DummyJoystick::plugHelper(Connector& /*connector*/,
                                const EmuTime& /*time*/)
 {
 }
