@@ -1,31 +1,32 @@
-// $Id: Joystick.cc,v 1.21 2005/05/24 20:39:35 m9710797 Exp $
+// $Id: Joystick.cc,v 1.27 2006/02/21 18:06:49 m9710797 Exp $
 
-#include <cassert>
-#include <SDL.h>	// TODO move this
 #include "Joystick.hh"
 #include "PluggingController.hh"
-#include "EventDistributor.hh"
+#include "UserInputEventDistributor.hh"
 #include "InputEvents.hh"
+#include <cassert>
 
 using std::string;
 
 namespace openmsx {
 
-void Joystick::registerAll(PluggingController *controller)
+void Joystick::registerAll(UserInputEventDistributor& eventDistributor,
+                           PluggingController& controller)
 {
 	if (!SDL_WasInit(SDL_INIT_JOYSTICK)) {
 		SDL_InitSubSystem(SDL_INIT_JOYSTICK);
-		SDL_JoystickEventState(SDL_ENABLE);	// joysticks generate events
+		SDL_JoystickEventState(SDL_ENABLE); // joysticks generate events
 	}
 
 	unsigned numJoysticks = SDL_NumJoysticks();
 	for (unsigned i = 0; i < numJoysticks; i++) {
-		controller->registerPluggable(new Joystick(i));
+		controller.registerPluggable(new Joystick(eventDistributor, i));
 	}
 }
 
-Joystick::Joystick(unsigned joyNum_)
-	: joyNum(joyNum_)
+Joystick::Joystick(UserInputEventDistributor& eventDistributor_, unsigned joyNum_)
+	: eventDistributor(eventDistributor_)
+	, joyNum(joyNum_)
 {
 	PRT_DEBUG("Creating a Joystick object for joystick " << joyNum);
 	assert(joyNum < (unsigned)SDL_NumJoysticks());
@@ -48,7 +49,7 @@ const string& Joystick::getDescription() const
 	return desc;
 }
 
-void Joystick::plugHelper(Connector* /*connector*/, const EmuTime& /*time*/)
+void Joystick::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
 {
 	PRT_DEBUG("Opening joystick " << SDL_JoystickName(joyNum));
 	joystick = SDL_JoystickOpen(joyNum);
@@ -56,12 +57,7 @@ void Joystick::plugHelper(Connector* /*connector*/, const EmuTime& /*time*/)
 		throw PlugException("Failed to open joystick device");
 	}
 
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_AXIS_MOTION_EVENT, *this);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_JOY_BUTTON_UP_EVENT,   *this);
+	eventDistributor.registerEventListener(*this);
 
 	status = JOY_UP | JOY_DOWN | JOY_LEFT | JOY_RIGHT |
 	         JOY_BUTTONA | JOY_BUTTONB;
@@ -69,13 +65,7 @@ void Joystick::plugHelper(Connector* /*connector*/, const EmuTime& /*time*/)
 
 void Joystick::unplugHelper(const EmuTime& /*time*/)
 {
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_AXIS_MOTION_EVENT, *this);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_BUTTON_DOWN_EVENT, *this);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_JOY_BUTTON_UP_EVENT,   *this);
-
+	eventDistributor.unregisterEventListener(*this);
 	SDL_JoystickClose(joystick);
 }
 
@@ -92,12 +82,17 @@ void Joystick::write(byte /*value*/, const EmuTime& /*time*/)
 }
 
 //EventListener
-bool Joystick::signalEvent(const Event& event)
+void Joystick::signalEvent(const Event& event)
 {
-	assert(dynamic_cast<const JoystickEvent*>(&event));
-	const JoystickEvent& joyEvent = static_cast<const JoystickEvent&>(event);
-	if (joyEvent.getJoystick() != joyNum) {
-		return true;
+	const JoystickEvent* joyEvent = dynamic_cast<const JoystickEvent*>(&event);
+	if (!joyEvent) {
+		return;
+	}
+
+	// TODO: It would be more efficient to make a dispatcher instead of
+	//       sending the event to all joysticks.
+	if (joyEvent->getJoystick() != joyNum) {
+		return;
 	}
 
 	switch (event.getType()) {
@@ -141,7 +136,7 @@ bool Joystick::signalEvent(const Event& event)
 		assert(dynamic_cast<const JoystickButtonEvent*>(&event));
 		const JoystickButtonEvent& buttonEvent =
 			static_cast<const JoystickButtonEvent&>(event);
-		switch (buttonEvent.getButton()) {
+		switch (buttonEvent.getButton() % 2) {
 		case 0:
 			status &= ~JOY_BUTTONA;
 			break;
@@ -158,7 +153,7 @@ bool Joystick::signalEvent(const Event& event)
 		assert(dynamic_cast<const JoystickButtonEvent*>(&event));
 		const JoystickButtonEvent& buttonEvent =
 			static_cast<const JoystickButtonEvent&>(event);
-		switch (buttonEvent.getButton()) {
+		switch (buttonEvent.getButton() % 2) {
 		case 0:
 			status |= JOY_BUTTONA;
 			break;
@@ -174,7 +169,6 @@ bool Joystick::signalEvent(const Event& event)
 	default:
 		assert(false);
 	}
-	return true;
 }
 
 } // namespace openmsx
