@@ -1,4 +1,4 @@
-// $Id: JoystickPort.cc,v 1.6 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: JoystickPort.cc,v 1.9 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "JoystickPort.hh"
 #include "DummyJoystick.hh"
@@ -8,16 +8,18 @@ using std::string;
 
 namespace openmsx {
 
-JoystickPort::JoystickPort(const string &name)
+JoystickPort::JoystickPort(PluggingController& pluggingController_,
+                           const string& name)
 	: Connector(name, std::auto_ptr<Pluggable>(new DummyJoystick()))
+	, pluggingController(pluggingController_)
+	, lastValue(255) // != 0
 {
-	lastValue = 255;	// != 0
-	PluggingController::instance().registerConnector(this);
+	pluggingController.registerConnector(*this);
 }
 
 JoystickPort::~JoystickPort()
 {
-	PluggingController::instance().unregisterConnector(this);
+	pluggingController.unregisterConnector(*this);
 }
 
 const string& JoystickPort::getDescription() const
@@ -37,18 +39,18 @@ JoystickDevice& JoystickPort::getPlugged() const
 	return static_cast<JoystickDevice&>(*plugged);
 }
 
-void JoystickPort::plug(Pluggable *device, const EmuTime &time)
+void JoystickPort::plug(Pluggable& device, const EmuTime& time)
 {
 	Connector::plug(device, time);
 	getPlugged().write(lastValue, time);
 }
 
-byte JoystickPort::read(const EmuTime &time)
+byte JoystickPort::read(const EmuTime& time)
 {
 	return getPlugged().read(time);
 }
 
-void JoystickPort::write(byte value, const EmuTime &time)
+void JoystickPort::write(byte value, const EmuTime& time)
 {
 	if (lastValue != value) {
 		lastValue = value;
