@@ -1,4 +1,4 @@
-// $Id: Keyboard.cc,v 1.38 2005/05/24 20:39:46 m9710797 Exp $
+// $Id: Keyboard.cc,v 1.49 2006/02/21 18:06:49 m9710797 Exp $
 
 #include <cstdio>
 #include <cstdlib>
@@ -6,14 +6,14 @@
 #include <cassert>
 #include "Clock.hh"
 #include "Keyboard.hh"
-#include "EventDistributor.hh"
+#include "UserInputEventDistributor.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "CommandController.hh"
+#include "Command.hh"
 #include "CommandException.hh"
-#include "Keys.hh"
 #include "InputEvents.hh"
 #include "Scheduler.hh"
 #include "Unicode.hh"
@@ -23,6 +23,54 @@ using std::vector;
 
 namespace openmsx {
 
+class KeyMatrixUpCmd : public SimpleCommand
+{
+public:
+	KeyMatrixUpCmd(CommandController& commandController,
+		       Keyboard& keyboard);
+	virtual std::string execute(const std::vector<std::string>& tokens);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+private:
+	Keyboard& keyboard;
+};
+
+class KeyMatrixDownCmd : public SimpleCommand
+{
+public:
+	KeyMatrixDownCmd(CommandController& commandController,
+			 Keyboard& keyboard);
+	virtual std::string execute(const std::vector<std::string>& tokens);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+private:
+	Keyboard& keyboard;
+};
+
+class KeyInserter : public SimpleCommand, private Schedulable
+{
+public:
+	KeyInserter(Scheduler& scheduler,
+		    CommandController& commandController,
+		    Keyboard& keyboard);
+	virtual ~KeyInserter();
+
+private:
+	void type(const std::string& str);
+	void reschedule(const EmuTime& time);
+
+	// Command
+	virtual std::string execute(const std::vector<std::string>& tokens);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+
+	// Schedulable
+	virtual void executeUntil(const EmuTime& time, int userData);
+	virtual const std::string& schedName() const;
+
+	Keyboard& keyboard;
+	char last;
+	std::string text;
+};
+
+
 void Keyboard::parseKeymapfile(const byte* buf, unsigned size)
 {
 	unsigned i = 0;
@@ -70,14 +118,18 @@ void Keyboard::loadKeymapfile(const string& filename)
 		File file(filename);
 		byte* buf = file.mmap();
 		parseKeymapfile(buf, file.getSize());
-		file.munmap();
 	} catch (FileException &e) {
 		throw FatalError("Couldn't load keymap file: " + filename);
 	}
 }
 
-Keyboard::Keyboard(bool keyG)
-	: keyMatrixUpCmd(*this), keyMatrixDownCmd(*this), keyTypeCmd(*this)
+Keyboard::Keyboard(Scheduler& scheduler, CommandController& commandController,
+                   UserInputEventDistributor& eventDistributor_, bool keyG)
+	: Schedulable(scheduler)
+	, keyMatrixUpCmd  (new KeyMatrixUpCmd  (commandController, *this))
+	, keyMatrixDownCmd(new KeyMatrixDownCmd(commandController, *this))
+	, keyTypeCmd(new KeyInserter(scheduler, commandController, *this))
+	, eventDistributor(eventDistributor_)
 {
 	keyGhosting = keyG;
 	keysChanged = false;
@@ -85,38 +137,24 @@ Keyboard::Keyboard(bool keyG)
 	memset(cmdKeyMatrix,  255, sizeof(cmdKeyMatrix));
 	memset(userKeyMatrix, 255, sizeof(userKeyMatrix));
 
-	const XMLElement* config = SettingsConfig::instance().findChild("KeyMap");
+	const XMLElement* config = commandController.getSettingsConfig().
+	                                                 findChild("KeyMap");
 	if (config) {
 		string filename = config->getData();
 		loadKeymapfile(config->getFileContext().resolve(filename));
 	}
 
-	EventDistributor & distributor(EventDistributor::instance());
-	distributor.registerEventListener(OPENMSX_KEY_DOWN_EVENT,   *this);
-	distributor.registerEventListener(OPENMSX_KEY_UP_EVENT,     *this);
-	distributor.registerEventListener(OPENMSX_CONSOLE_ON_EVENT, *this);
+	eventDistributor.registerEventListener(*this);
 	// We do not listen for CONSOLE_OFF_EVENTS because rescanning the
 	// keyboard can have unwanted side effects
 
 	// We may do something with focus-change events later, but SDL does
 	// not send them
-
-	CommandController::instance().registerCommand(&keyMatrixUpCmd,   "keymatrixup");
-	CommandController::instance().registerCommand(&keyMatrixDownCmd, "keymatrixdown");
-	CommandController::instance().registerCommand(&keyTypeCmd,       "type");
 }
 
 Keyboard::~Keyboard()
 {
-
-	CommandController::instance().unregisterCommand(&keyTypeCmd,       "type");
-	CommandController::instance().unregisterCommand(&keyMatrixDownCmd, "keymatrixdown");
-	CommandController::instance().unregisterCommand(&keyMatrixUpCmd,   "keymatrixup");
-
-	EventDistributor & distributor(EventDistributor::instance());
-	distributor.unregisterEventListener(OPENMSX_KEY_UP_EVENT,   *this);
-	distributor.unregisterEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
-	distributor.unregisterEventListener(OPENMSX_CONSOLE_ON_EVENT, *this);
+	eventDistributor.unregisterEventListener(*this);
 }
 
 
@@ -134,46 +172,56 @@ const byte* Keyboard::getKeys()
 	return keyMatrix;
 }
 
-void Keyboard::allUp()
+void Keyboard::signalEvent(const Event& event)
 {
-	for (unsigned int i=0 ; i<NR_KEYROWS ; ++i) 
-		userKeyMatrix[i]=0xFF;
-}
-
-bool Keyboard::signalEvent(const Event& event)
-{
-	switch (event.getType()) {
-	case OPENMSX_CONSOLE_ON_EVENT: 
-		allUp();
-		break;
-	case OPENMSX_CONSOLE_OFF_EVENT: 
-		// We do not rescan the keyboard since this may lead to an
-		// unwanted pressing of <return> in MSX after typing "set console
-		// off" in the console.
-		break;
-	default: // must be keyEvent 
+	EventType type = event.getType();
+	if ((type == OPENMSX_KEY_DOWN_EVENT) ||
+	    (type == OPENMSX_KEY_UP_EVENT)) {
+		// Ignore possible console on/off events:
+		// we do not rescan the keyboard since this may lead to
+		// an unwanted pressing of <return> in MSX after typing
+		// "set console off" in the console.
 		assert(dynamic_cast<const KeyEvent*>(&event));
-		Keys::KeyCode key = (Keys::KeyCode)((int)((KeyEvent&)event).getKeyCode() &
-	                                    (int)Keys::K_MASK);
-		if (key < MAX_KEYSYM) {
-			switch (event.getType()) {
-			case OPENMSX_KEY_DOWN_EVENT: {
-				// Key pressed: reset bit in keyMatrix
-				userKeyMatrix[keyTab[key][0]] &= ~keyTab[key][1];
-				break;
-			}
-			case OPENMSX_KEY_UP_EVENT: {
-				// Key released: set bit in keyMatrix
-				userKeyMatrix[keyTab[key][0]] |= keyTab[key][1];
-				break;
-			}
-			default:
-				assert(false);
-			}
+		Keys::KeyCode key = (Keys::KeyCode)
+			((int)((KeyEvent&)event).getKeyCode() &
+			 (int)Keys::K_MASK);
+		if (key != Keys::K_CAPSLOCK) {
+			processKey(type == OPENMSX_KEY_DOWN_EVENT, key);
+		} else {
+			// This is a workaround for a SDL 'feature' (it's
+			// actually a documented bug in SDL). In a future
+			// refactoring this workaround should be moved to a
+			// higher level.
+			processKey(true, Keys::K_CAPSLOCK);
+			Clock<1000> now(getScheduler().getCurrentTime());
+			setSyncPoint(now + 100);
 		}
 	}
-	keysChanged = true;	// do ghosting at next getKeys()
-	return true;
+}
+
+void Keyboard::executeUntil(const EmuTime& /*time*/, int /*userData*/)
+{
+	processKey(false, Keys::K_CAPSLOCK);
+}
+
+const std::string& Keyboard::schedName() const
+{
+	static const string schedName = "Keyboard";
+	return schedName;
+}
+
+void Keyboard::processKey(bool down, int key)
+{
+	if (key < MAX_KEYSYM) {
+		if (down) {
+			// Key pressed: reset bit in keyMatrix
+			userKeyMatrix[keyTab[key][0]] &= ~keyTab[key][1];
+		} else {
+			// Key released: set bit in keyMatrix
+			userKeyMatrix[keyTab[key][0]] |= keyTab[key][1];
+		}
+		keysChanged = true; // do ghosting at next getKeys()
+	}
 }
 
 
@@ -257,7 +305,7 @@ bool Keyboard::commonKeys(char asciiCode1, char asciiCode2)
 	byte mask11 = asciiTab[asciiCode1 & 0xFF][1] & 0xFF;
 	byte mask20 = asciiTab[asciiCode2 & 0xFF][0] & 0xFF;
 	byte mask21 = asciiTab[asciiCode2 & 0xFF][1] & 0xFF;
-	
+
 	// ignore modifier keys (shift, ctrl, graph, code)
 	if (row10 == 6) mask10 &= ~0x17;
 	if (row11 == 6) mask11 &= ~0x17;
@@ -272,19 +320,21 @@ bool Keyboard::commonKeys(char asciiCode1, char asciiCode2)
 
 // class KeyMatrixUpCmd
 
-Keyboard::KeyMatrixUpCmd::KeyMatrixUpCmd(Keyboard& parent_)
-	: parent(parent_)
+KeyMatrixUpCmd::KeyMatrixUpCmd(CommandController& commandController,
+                                         Keyboard& keyboard_)
+	: SimpleCommand(commandController, "keymatrixup")
+	, keyboard(keyboard_)
 {
 }
 
-string Keyboard::KeyMatrixUpCmd::execute(const vector<string>& tokens)
+string KeyMatrixUpCmd::execute(const vector<string>& tokens)
 {
-	return parent.processCmd(tokens, true);
+	return keyboard.processCmd(tokens, true);
 }
 
-string Keyboard::KeyMatrixUpCmd::help(const vector<string>& /*tokens*/) const
+string KeyMatrixUpCmd::help(const vector<string>& /*tokens*/) const
 {
-	static const string helpText = 
+	static const string helpText =
 		"keymatrixup <row> <bitmask>  release a key in the keyboardmatrix\n";
 	return helpText;
 }
@@ -292,19 +342,21 @@ string Keyboard::KeyMatrixUpCmd::help(const vector<string>& /*tokens*/) const
 
 // class KeyMatrixDownCmd
 
-Keyboard::KeyMatrixDownCmd::KeyMatrixDownCmd(Keyboard& parent_)
-	: parent(parent_)
+KeyMatrixDownCmd::KeyMatrixDownCmd(CommandController& commandController,
+                                             Keyboard& keyboard_)
+	: SimpleCommand(commandController, "keymatrixdown")
+	, keyboard(keyboard_)
 {
 }
 
-string Keyboard::KeyMatrixDownCmd::execute(const vector<string>& tokens)
+string KeyMatrixDownCmd::execute(const vector<string>& tokens)
 {
-	return parent.processCmd(tokens, false);
+	return keyboard.processCmd(tokens, false);
 }
 
-string Keyboard::KeyMatrixDownCmd::help(const vector<string>& /*tokens*/) const
+string KeyMatrixDownCmd::help(const vector<string>& /*tokens*/) const
 {
-	static const string helpText= 
+	static const string helpText=
 		"keymatrixdown <row> <bitmask>  press a key in the keyboardmatrix\n";
 	return helpText;
 }
@@ -312,16 +364,19 @@ string Keyboard::KeyMatrixDownCmd::help(const vector<string>& /*tokens*/) const
 
 // class KeyInserter
 
-Keyboard::KeyInserter::KeyInserter(Keyboard& parent_)
-	: parent(parent_), last(-1)
+KeyInserter::KeyInserter(Scheduler& scheduler,
+		CommandController& commandController, Keyboard& keyboard_)
+	: SimpleCommand(commandController, "type")
+	, Schedulable(scheduler)
+	, keyboard(keyboard_), last(-1)
 {
 }
 
-Keyboard::KeyInserter::~KeyInserter()
+KeyInserter::~KeyInserter()
 {
 }
 
-string Keyboard::KeyInserter::execute(const vector<string>& tokens)
+string KeyInserter::execute(const vector<string>& tokens)
 {
 	if (tokens.size() != 2) {
 		throw SyntaxError();
@@ -330,48 +385,47 @@ string Keyboard::KeyInserter::execute(const vector<string>& tokens)
 	return "";
 }
 
-string Keyboard::KeyInserter::help(const vector<string>& /*tokens*/) const
+string KeyInserter::help(const vector<string>& /*tokens*/) const
 {
 	static const string helpText = "Type a string in the emulated MSX.";
 	return helpText;
 }
 
-void Keyboard::KeyInserter::type(const string& str)
+void KeyInserter::type(const string& str)
 {
 	if (str.empty()) {
 		return;
 	}
-	Scheduler& scheduler = Scheduler::instance();
 	if (text.empty()) {
-		reschedule(scheduler.getCurrentTime());
+		reschedule(getScheduler().getCurrentTime());
 	}
 	text += Unicode::utf8ToAscii(str);
 }
 
-void Keyboard::KeyInserter::executeUntil(const EmuTime& time, int /*userData*/)
+void KeyInserter::executeUntil(const EmuTime& time, int /*userData*/)
 {
-	parent.pressAscii(last, false);
+	keyboard.pressAscii(last, false);
 	if (text.empty()) {
 		return;
 	}
 	char current = text[0];
-	if (parent.commonKeys(last, current)) {
+	if (keyboard.commonKeys(last, current)) {
 		last = -1;
 	} else {
-		parent.pressAscii(current, true);
+		keyboard.pressAscii(current, true);
 		last = current;
 		text = text.substr(1);
 	}
 	reschedule(time);
 }
 
-void Keyboard::KeyInserter::reschedule(const EmuTime& time)
+void KeyInserter::reschedule(const EmuTime& time)
 {
 	Clock<15> nextTime(time);
-	Scheduler::instance().setSyncPoint(nextTime + 1, *this);
+	setSyncPoint(nextTime + 1);
 }
 
-const string& Keyboard::KeyInserter::schedName() const
+const string& KeyInserter::schedName() const
 {
 	static const string schedName = "KeyInserter";
 	return schedName;
@@ -382,7 +436,7 @@ const string& Keyboard::KeyInserter::schedName() const
 
 // Key-Matrix table
 //
-// row/bit  7     6     5     4     3     2     1     0   
+// row/bit  7     6     5     4     3     2     1     0
 //       +-----+-----+-----+-----+-----+-----+-----+-----+
 //   0   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
 //       +-----+-----+-----+-----+-----+-----+-----+-----+
@@ -475,7 +529,7 @@ byte Keyboard::keyTab[MAX_KEYSYM][2] = {
   {0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00},{0,0x00}
 };
 
-// keypresses needed to type ascii characters 
+// keypresses needed to type ascii characters
 // for each character there are shorts S[0], S[2]
 // S[i] >> 8   adresses a row in the keyboard matrix
 // S[i] & 0xFF gives the keys that need to be pressed in that row
@@ -503,7 +557,7 @@ short Keyboard::asciiTab[256][2] =
 	{0x740},        {0x602, 0x540}, {0x602, 0x580}, {0x704},        // SEL, ^Y, EOF, ESC,
 	{0x880},        {0x810},        {0x820},        {0x840},        // RGT, LFT, UP, DWN,
 	// 0x20
-	{0x801},        {0x601, 0x002}, {0x601, 0x201}, {0x601, 0x008}, // SPC, !, ", #, 
+	{0x801},        {0x601, 0x002}, {0x601, 0x201}, {0x601, 0x008}, // SPC, !, ", #,
 	{0x601, 0x010}, {0x601, 0x020}, {0x601, 0x080}, {0x201},        // $, %, &, ',
 	{0x601, 0x102}, {0x601, 0x001}, {0x601, 0x101}, {0x601, 0x108}, // (, ), *, +,
 	{0x204},        {0x104},        {0x208},        {0x210},        // ,, -, ., /,
