@@ -1,7 +1,7 @@
-// $Id: Mouse.cc,v 1.24 2005/05/24 20:39:46 m9710797 Exp $
+// $Id: Mouse.cc,v 1.30 2006/02/21 18:06:49 m9710797 Exp $
 
 #include "Mouse.hh"
-#include "EventDistributor.hh"
+#include "UserInputEventDistributor.hh"
 #include "InputEvents.hh"
 
 using std::string;
@@ -17,29 +17,20 @@ const int FAZE_YLOW  = 3;
 const int STROBE = 0x04;
 
 
-Mouse::Mouse()
+Mouse::Mouse(UserInputEventDistributor& eventDistributor_)
+	: eventDistributor(eventDistributor_)
+	, lastTime(EmuTime::zero)
 {
 	status = JOY_BUTTONA | JOY_BUTTONB;
 	faze = FAZE_YLOW;
 	xrel = yrel = curxrel = curyrel = 0;
 	mouseMode = true;
-
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_MOTION_EVENT,      *this);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_MOUSE_BUTTON_UP_EVENT,   *this);
+	eventDistributor.registerEventListener(*this);
 }
 
 Mouse::~Mouse()
 {
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_MOTION_EVENT,      *this);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_BUTTON_DOWN_EVENT, *this);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_MOUSE_BUTTON_UP_EVENT,   *this);
+	eventDistributor.unregisterEventListener(*this);
 }
 
 
@@ -56,7 +47,7 @@ const string& Mouse::getDescription() const
 	return desc;
 }
 
-void Mouse::plugHelper(Connector* /*connector*/, const EmuTime& time)
+void Mouse::plugHelper(Connector& /*connector*/, const EmuTime& time)
 {
 	if (status & JOY_BUTTONA) {
 		// not pressed, mouse mode
@@ -108,7 +99,7 @@ void Mouse::emulateJoystick()
 	if ((absx < TRESHOLD) && (absy < TRESHOLD)) {
 		return;
 	}
-	
+
 	// tan(pi/8) ~= 5/12
 	if (deltax > 0) {
 		if (deltay > 0) {
@@ -150,7 +141,7 @@ void Mouse::write(byte value, const EmuTime& time)
 	if (mouseMode) {
 		// TODO figure out the timeout mechanism
 		//      does it exist at all?
-		
+
 		const int TIMEOUT = 1000;	// TODO find a good value
 		int delta = lastTime.getTicksTill(time);
 		lastTime.advance(time);
@@ -183,7 +174,7 @@ void Mouse::write(byte value, const EmuTime& time)
 
 
 //EventListener
-bool Mouse::signalEvent(const Event& event)
+void Mouse::signalEvent(const Event& event)
 {
 	switch (event.getType()) {
 	case OPENMSX_MOUSE_MOTION_EVENT: {
@@ -233,9 +224,9 @@ bool Mouse::signalEvent(const Event& event)
 		break;
 	}
 	default:
-		assert(false);
+		// ignore
+		break;
 	}
-	return true;
 }
 
 } // namespace openmsx
