@@ -1,9 +1,7 @@
-// $Id: SETetrisDongle.cc,v 1.5 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: SETetrisDongle.cc,v 1.6 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "SETetrisDongle.hh"
 
-using std::string;
-
 namespace openmsx {
 
 SETetrisDongle::SETetrisDongle()
@@ -12,24 +10,21 @@ SETetrisDongle::SETetrisDongle()
 	         JOY_BUTTONA | JOY_BUTTONB;
 }
 
-SETetrisDongle::~SETetrisDongle()
-{
-}
-
 //Pluggable
-const string& SETetrisDongle::getName() const
+const std::string& SETetrisDongle::getName() const
 {
-	static const string NAME = "tetris2-protection";
+	static const std::string NAME = "tetris2-protection";
 	return NAME;
 }
 
-const string& SETetrisDongle::getDescription() const
+const std::string& SETetrisDongle::getDescription() const
 {
-	static const string DESC = "SETetrisDongledongle";
+	static const std::string DESC = "SETetrisDongledongle";
 	return DESC;
 }
 
-void SETetrisDongle::plugHelper(Connector* /*connector*/, const EmuTime& /*time*/)
+void SETetrisDongle::plugHelper(
+	Connector& /*connector*/, const EmuTime& /*time*/)
 {
 }
 
