@@ -1,25 +1,26 @@
-// $Id: main.cc,v 1.69 2005/05/03 21:08:44 m9710797 Exp $
+// $Id: main.cc,v 1.98 2006/07/17 18:58:14 m9710797 Exp $
 
 /*
  *  openmsx - the MSX emulator that aims for perfection
  *
  */
 
+#include "Reactor.hh"
+#include "CommandController.hh"
+#include "CommandLineParser.hh"
+#include "SettingsConfig.hh"
+#include "CliServer.hh"
+#include "Interpreter.hh"
+#include "Display.hh"
+#include "RenderSettings.hh"
+#include "EnumSetting.hh"
+#include "MSXException.hh"
+#include "Thread.hh"
 #include <memory>
 #include <iostream>
 #include <exception>
+#include <cstdlib>
 #include <SDL.h>
-#include "MSXMotherBoard.hh"
-#include "CommandLineParser.hh"
-#include "CartridgeSlotManager.hh"
-#include "CliComm.hh"
-#include "HotKey.hh"
-#include "CliServer.hh"
-#include "AfterCommand.hh"
-#include "Interpreter.hh"
-#include "Display.hh"
-#include "RendererFactory.hh"
-#include "MSXException.hh"
 
 using std::auto_ptr;
 using std::cerr;
@@ -30,11 +31,11 @@ namespace openmsx {
 
 static void initializeSDL()
 {
-	int flags = SDL_INIT_VIDEO | SDL_INIT_TIMER;
+	int flags = SDL_INIT_TIMER;
 #ifndef NDEBUG
 	flags |= SDL_INIT_NOPARACHUTE;
 #endif
-	if (SDL_Init(flags) < 0) { 
+	if (SDL_Init(flags) < 0) {
 		throw FatalError(string("Couldn't init SDL: ") + SDL_GetError());
 	}
 }
@@ -47,22 +48,29 @@ static void unexpectedExceptionHandler()
 static int main(int argc, char **argv)
 {
 	std::set_unexpected(unexpectedExceptionHandler);
-	
+
 	int err = 0;
 	try {
-		Interpreter::instance().init(argv[0]);
-		initializeSDL();
-		CommandLineParser& parser = CommandLineParser::instance();
+		Thread::setMainThread();
+		Reactor reactor;
+		reactor.getCommandController().getInterpreter().init(argv[0]);
+
+		CommandLineParser parser(reactor);
 		parser.parse(argc, argv);
 		CommandLineParser::ParseStatus parseStatus = parser.getParseStatus();
+
 		if (parseStatus != CommandLineParser::EXIT) {
-			CartridgeSlotManager::instance().readConfig();
-			HotKey hotkey;
-			AfterCommand afterCommand;
-			RendererFactory::createVideoSystem();
-			MSXMotherBoard motherboard;
-			// CliServer cliServer; // disabled for security reasons
-			motherboard.run(parseStatus == CommandLineParser::RUN);
+			initializeSDL();
+			reactor.getIconStatus();
+			if (!parser.isHiddenStartup()) {
+				reactor.getDisplay().getRenderSettings().
+					getRenderer().restoreDefault();
+			}
+			if (parseStatus != CommandLineParser::TEST) {
+				CliServer cliServer(reactor.getCommandController(),
+				                    reactor.getEventDistributor());
+				reactor.run(parser);
+			}
 		}
 	} catch (FatalError& e) {
 		cerr << "Fatal error: " << e.getMessage() << endl;
@@ -78,7 +86,6 @@ static int main(int argc, char **argv)
 		err = 1;
 	}
 	// Clean up.
-	Display::instance().resetVideoSystem();
 	if (SDL_WasInit(SDL_INIT_EVERYTHING)) {
 		SDL_Quit();
 	}
@@ -90,5 +97,5 @@ static int main(int argc, char **argv)
 // Enter the openMSX namespace.
 int main(int argc, char **argv)
 {
-	return openmsx::main(argc, argv);
+	exit(openmsx::main(argc, argv)); // need exit() iso return on win32/SDL
 }
