@@ -1,23 +1,25 @@
-// $Id: PanasonicMemory.cc,v 1.9 2005/05/14 14:01:28 m9710797 Exp $
+// $Id: PanasonicMemory.cc,v 1.14 2006/03/09 22:03:57 m9710797 Exp $
 
 #include "PanasonicMemory.hh"
+#include "MSXMotherBoard.hh"
 #include "MSXCPU.hh"
 #include "Ram.hh"
 #include "Rom.hh"
-#include "HardwareConfig.hh"
+#include "MachineConfig.hh"
 #include "MSXException.hh"
 
 namespace openmsx {
 
-static XMLElement& getRomConfig()
+static const XMLElement& getRomConfig(MSXMotherBoard& motherBoard)
 {
-	return HardwareConfig::instance().getChild("PanasonicRom");
+	return motherBoard.getMachineConfig().getConfig().getChild("PanasonicRom");
 }
 
-PanasonicMemory::PanasonicMemory()
-	: rom(new Rom("PanasonicRom", "Turbo-R main ROM", getRomConfig()))
+PanasonicMemory::PanasonicMemory(MSXMotherBoard& motherBoard)
+	: rom(new Rom(motherBoard, "PanasonicRom", "Turbo-R main ROM",
+	              getRomConfig(motherBoard)))
 	, ram(NULL), dram(false)
-	, msxcpu(MSXCPU::instance())
+	, msxcpu(motherBoard.getCPU())
 {
 }
 
@@ -25,12 +27,6 @@ PanasonicMemory::~PanasonicMemory()
 {
 }
 
-PanasonicMemory& PanasonicMemory::instance()
-{
-	static PanasonicMemory oneInstance;
-	return oneInstance;
-}
-
 void PanasonicMemory::registerRam(Ram& ram_)
 {
 	ram = &ram_[0];
@@ -59,18 +55,18 @@ const byte* PanasonicMemory::getRomBlock(unsigned block)
 const byte* PanasonicMemory::getRomRange(unsigned first, unsigned last)
 {
 	if (last < first) {
-		throw FatalError("Error in config file: firstblock must "
-		                 "be smaller than lastblock");
+		throw MSXException("Error in config file: firstblock must "
+		                   "be smaller than lastblock");
 	}
 	unsigned start =  first     * 0x2000;
 	if (start >= rom->getSize()) {
-		throw FatalError("Error in config file: firstblock lies "
-		                 "outside of rom image.");
+		throw MSXException("Error in config file: firstblock lies "
+		                   "outside of rom image.");
 	}
 	unsigned stop  = (last + 1) * 0x2000;
 	if (stop > rom->getSize()) {
-		throw FatalError("Error in config file: lastblock lies "
-		                 "outside of rom image.");
+		throw MSXException("Error in config file: lastblock lies "
+		                   "outside of rom image.");
 	}
 	return &(*rom)[start];
 }
@@ -93,4 +89,9 @@ void PanasonicMemory::setDRAM(bool dram_)
 	}
 }
 
+bool PanasonicMemory::isWritable(unsigned address) const
+{
+	return !dram || (address < (ramSize - 0x10000));
+}
+
 } // namespace openmsx
