@@ -1,21 +1,23 @@
-// $Id: Rom.cc,v 1.45 2005/05/14 14:01:29 m9710797 Exp $
+// $Id: Rom.cc,v 1.56 2006/07/12 17:32:21 m9710797 Exp $
 
 #include <string>
 #include <sstream>
 #include "XMLElement.hh"
 #include "Rom.hh"
 #include "RomInfo.hh"
+#include "RomDatabase.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "PanasonicMemory.hh"
+#include "MSXMotherBoard.hh"
 #include "StringOp.hh"
 #include "Debugger.hh"
+#include "Debuggable.hh"
 #include "sha1.hh"
 #include "CliComm.hh"
 #include "FilePool.hh"
 #include "ConfigException.hh"
-#include "HardwareConfig.hh"
 #include "EmptyPatch.hh"
 #include "IPSPatch.hh"
 
@@ -24,30 +26,46 @@ using std::auto_ptr;
 
 namespace openmsx {
 
-Rom::Rom(const string& name_, const string& description_,
-         const XMLElement& config)
-	: name(name_), description(description_)
+class RomDebuggable : public Debuggable
 {
-	init(config.getChild("rom"));
+public:
+	RomDebuggable(Debugger& debugger, Rom& rom);
+	~RomDebuggable();
+	virtual unsigned getSize() const;
+	virtual const std::string& getDescription() const;
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value);
+private:
+	Debugger& debugger;
+	Rom& rom;
+};
+
+
+Rom::Rom(MSXMotherBoard& motherBoard_, const string& name_,
+         const string& description_, const XMLElement& config)
+	: motherBoard(motherBoard_), name(name_), description(description_)
+{
+	init(motherBoard.getCliComm(), config.getChild("rom"));
 }
 
-Rom::Rom(const string& name_, const string& description_,
-         const XMLElement& config, const string& id)
-	: name(name_), description(description_)
+Rom::Rom(MSXMotherBoard& motherBoard_, const string& name_,
+         const string& description_, const XMLElement& config,
+         const string& id)
+	: motherBoard(motherBoard_), name(name_), description(description_)
 {
 	XMLElement::Children romConfigs;
 	config.getChildren("rom", romConfigs);
 	for (XMLElement::Children::const_iterator it = romConfigs.begin();
 	     it != romConfigs.end(); ++it) {
 		if ((*it)->getId() == id) {
-			init(**it);
+			init(motherBoard.getCliComm(), **it);
 			return;
 		}
 	}
 	throw ConfigException("ROM tag \"" + id + "\" missing.");
 }
 
-void Rom::init(const XMLElement& config)
+void Rom::init(CliComm& cliComm, const XMLElement& config)
 {
 	extendedRom = NULL;
 	XMLElement::Children sums;
@@ -62,7 +80,7 @@ void Rom::init(const XMLElement& config)
 		for (XMLElement::Children::const_iterator it = sums.begin();
 		     it != sums.end(); ++it) {
 			const string& sha1 = (*it)->getData();
-			string sha1File = FilePool::instance().getFile(sha1);
+			string sha1File = motherBoard.getFilePool().getFile(sha1);
 			if (!sha1File.empty()) {
 				filename = sha1File;
 				sha1sum = sha1;
@@ -70,8 +88,8 @@ void Rom::init(const XMLElement& config)
 			}
 		}
 		if (filename.empty()) {
-			throw FatalError("Couldn't find ROM file for \"" +
-			                 config.getId() + "\".");
+			throw MSXException("Couldn't find ROM file for \"" +
+			                   config.getId() + "\".");
 		}
 		read(config, filename);
 
@@ -80,7 +98,7 @@ void Rom::init(const XMLElement& config)
 		int first = config.getChildDataAsInt("firstblock");
 		int last  = config.getChildDataAsInt("lastblock");
 		size = (last - first + 1) * 0x2000;
-		rom = PanasonicMemory::instance().getRomRange(first, last);
+		rom = motherBoard.getPanasonicMemory().getRomRange(first, last);
 		assert(rom);
 
 	} else {
@@ -114,16 +132,24 @@ void Rom::init(const XMLElement& config)
 			}
 		}
 	}
-	info = RomInfo::fetchRomInfo(*this);
+	info = RomDatabase::instance().fetchRomInfo(cliComm, *this);
 
 	// TODO fix this, this is a hack that depends heavily on MSXRomCLI.cc
-	if (!info->getTitle().empty() &&
-	    ((name.size() >= 6) && (name.substr(0, 6) == "MSXRom"))) {
-		name = HardwareConfig::instance().makeUnique(info->getTitle());
+	if (!info->getTitle().empty() && StringOp::startsWith(name, "MSXRom")) {
+		name = info->getTitle();
 	}
 
 	if (size) {
-		Debugger::instance().registerDebuggable(name, *this);
+		Debugger& debugger = motherBoard.getDebugger();
+		if (debugger.findDebuggable(name)) {
+			unsigned n = 0;
+			string tmp;
+			do {
+				tmp = name + " (" + StringOp::toString(++n) + ")";
+			} while (debugger.findDebuggable(tmp));
+			name = tmp;
+		}
+		romDebuggable.reset(new RomDebuggable(debugger, *this));
 	}
 }
 
@@ -135,9 +161,9 @@ void Rom::read(const XMLElement& config, const string& filename)
 	try {
 		file.reset(new File(config.getFileContext().resolve(filename)));
 	} catch (FileException& e) {
-		throw FatalError("Error reading ROM: " + filename);
+		throw MSXException("Error reading ROM: " + filename);
 	}
-	
+
 	// get filesize
 	int fileSize;
 	string fileSizeStr = config.getChildData("filesize", "auto");
@@ -146,29 +172,28 @@ void Rom::read(const XMLElement& config, const string& filename)
 	} else {
 		fileSize = StringOp::stringToInt(fileSizeStr);
 	}
-	
+
 	// get offset
 	int offset = config.getChildDataAsInt("skip_headerbytes", 0);
 	if (fileSize <= offset) {
-		throw FatalError("Offset greater than filesize");
+		throw MSXException("Offset greater than filesize");
 	}
 	size = fileSize - offset;
-	
+
 	// read file
 	byte* tmp = 0;	// avoid warning
 	try {
 		tmp = file->mmap() + offset;
 		rom = tmp;
 	} catch (FileException &e) {
-		throw FatalError("Error reading ROM image: " + filename);
+		throw MSXException("Error reading ROM image: " + filename);
 	}
 
 	// verify SHA1
 	if (!checkSHA1(config)) {
-		CliComm::instance().printWarning(
+		motherBoard.getCliComm().printWarning(
 			"SHA1 sum for '" + config.getId() +
-			"' does not match with sum of '" + filename +
-			"'.");
+			"' does not match with sum of '" + filename + "'.");
 	}
 }
 
@@ -191,35 +216,12 @@ bool Rom::checkSHA1(const XMLElement& config)
 
 Rom::~Rom()
 {
-	if (size) {
-		Debugger::instance().unregisterDebuggable(name, *this);
-	}
-	if (file.get()) {
-		file->munmap();
-	}
 	delete[] extendedRom;
 }
 
-
-unsigned Rom::getSize() const
+const RomInfo& Rom::getInfo() const
 {
-	return size;
-}
-
-const string& Rom::getDescription() const
-{
-	return description;
-}
-
-byte Rom::read(unsigned address)
-{
-	assert(address < size);
-	return rom[address];
-}
-
-void Rom::write(unsigned /*address*/, byte /*value*/)
-{
-	// ignore
+	return *info;
 }
 
 const string& Rom::getName() const
@@ -227,6 +229,11 @@ const string& Rom::getName() const
 	return name;
 }
 
+const string& Rom::getDescription() const
+{
+	return description;
+}
+
 const string& Rom::getSHA1Sum() const
 {
 	if (sha1sum.empty()) {
@@ -237,4 +244,37 @@ const string& Rom::getSHA1Sum() const
 	return sha1sum;
 }
 
+
+RomDebuggable::RomDebuggable(Debugger& debugger_, Rom& rom_)
+	: debugger(debugger_), rom(rom_)
+{
+	debugger.registerDebuggable(rom.getName(), *this);
+}
+
+RomDebuggable::~RomDebuggable()
+{
+	debugger.unregisterDebuggable(rom.getName(), *this);
+}
+
+unsigned RomDebuggable::getSize() const
+{
+	return rom.getSize();
+}
+
+const string& RomDebuggable::getDescription() const
+{
+	return rom.getDescription();
+}
+
+byte RomDebuggable::read(unsigned address)
+{
+	assert(address < getSize());
+	return rom[address];
+}
+
+void RomDebuggable::write(unsigned /*address*/, byte /*value*/)
+{
+	// ignore
+}
+
 } // namespace openmsx
