@@ -1,4 +1,4 @@
-// $Id: RomAscii16kB.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomAscii16kB.cc,v 1.12 2005/11/12 21:53:20 m9710797 Exp $
 
 // ASCII 16kB cartridges
 //
@@ -15,17 +15,14 @@
 
 namespace openmsx {
 
-RomAscii16kB::RomAscii16kB(const XMLElement& config, const EmuTime& time,
-                           std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomAscii16kB::RomAscii16kB(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomAscii16kB::~RomAscii16kB()
-{
-}
-
 void RomAscii16kB::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
