@@ -1,12 +1,12 @@
-// $Id: RomAscii8kB.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomAscii8kB.cc,v 1.13 2005/11/12 21:53:20 m9710797 Exp $
 
 // ASCII 8kB cartridges
-// 
+//
 // this type is used in many japanese-only cartridges.
-// example of cartridges: Valis(Fantasm Soldier), Dragon Slayer, Outrun, 
+// example of cartridges: Valis(Fantasm Soldier), Dragon Slayer, Outrun,
 //                        Ashguine 2, ...
 // The address to change banks:
-//  bank 1: 0x6000 - 0x67ff (0x6000 used) 
+//  bank 1: 0x6000 - 0x67ff (0x6000 used)
 //  bank 2: 0x6800 - 0x6fff (0x6800 used)
 //  bank 3: 0x7000 - 0x77ff (0x7000 used)
 //  bank 4: 0x7800 - 0x7fff (0x7800 used)
@@ -16,17 +16,13 @@
 
 namespace openmsx {
 
-RomAscii8kB::RomAscii8kB(const XMLElement& config, const EmuTime& time,
-                         std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomAscii8kB::RomAscii8kB(MSXMotherBoard& motherBoard, const XMLElement& config,
+                         const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomAscii8kB::~RomAscii8kB()
-{
-}
-
 void RomAscii8kB::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
