@@ -1,4 +1,4 @@
-// $Id: RomCrossBlaim.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomCrossBlaim.cc,v 1.12 2005/11/12 21:53:20 m9710797 Exp $
 
 #include "RomCrossBlaim.hh"
 #include "Rom.hh"
@@ -6,17 +6,14 @@
 
 namespace openmsx {
 
-RomCrossBlaim::RomCrossBlaim(const XMLElement& config, const EmuTime& time,
-                             std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomCrossBlaim::RomCrossBlaim(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomCrossBlaim::~RomCrossBlaim()
-{
-}
-
 void RomCrossBlaim::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
