@@ -1,4 +1,4 @@
-// $Id: RomFSA1FM.cc,v 1.17 2005/01/17 19:30:10 m9710797 Exp $
+// $Id: RomFSA1FM.cc,v 1.25 2006/05/26 10:45:54 m9710797 Exp $
 
 // FS-A1FM mappers
 //   based on info from:
@@ -39,43 +39,49 @@
 #include "CPU.hh"
 #include "Rom.hh"
 #include "SRAM.hh"
+#include "MSXMotherBoard.hh"
 #include "FirmwareSwitch.hh"
 #include "XMLElement.hh"
-#include "BooleanSetting.hh"
 
 namespace openmsx {
 
-// common sram //
+// 8kb shared sram //
 
-FSA1FMRam::FSA1FMRam(const XMLElement& config)
-	: sram(new SRAM(config.getId() + " SRAM", 0x2000, config))
+static SRAM* sram = 0;
+static int sramRefCounter = 0;
+
+static void allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
 {
+	if (sramRefCounter == 0) {
+		sram = new SRAM(motherBoard, config.getId() + " SRAM", 0x2000,
+		                config);
+	}
+	++sramRefCounter;
 }
 
-FSA1FMRam::~FSA1FMRam()
+static void releaseSRAM()
 {
-}
-
-byte* FSA1FMRam::getSRAM(const XMLElement& config)
-{
-	static FSA1FMRam oneInstance(config);
-	return &(*oneInstance.sram)[0];
+	-- sramRefCounter;
+	if (sramRefCounter == 0) {
+		delete sram;
+	}
 }
 
 
 // Mapper for slot 3-1 //
 
-RomFSA1FM1::RomFSA1FM1(const XMLElement& config, const EmuTime& time,
-                       std::auto_ptr<Rom> rom)
-	: MSXRom(config, time, rom)
-	, firmwareSwitch(new FirmwareSwitch())
+RomFSA1FM1::RomFSA1FM1(MSXMotherBoard& motherBoard, const XMLElement& config,
+                       const EmuTime& time, std::auto_ptr<Rom> rom)
+	: MSXRom(motherBoard, config, time, rom)
+	, firmwareSwitch(new FirmwareSwitch(motherBoard.getCommandController()))
 {
-	sram = FSA1FMRam::getSRAM(config);
+	allocSRAM(motherBoard, config);
 	reset(time);
 }
 
 RomFSA1FM1::~RomFSA1FM1()
 {
+	releaseSRAM();
 }
 
 void RomFSA1FM1::reset(const EmuTime& /*time*/)
@@ -83,16 +89,16 @@ void RomFSA1FM1::reset(const EmuTime& /*time*/)
 	// initial rom bank is undefined
 }
 
-byte RomFSA1FM1::readMem(word address, const EmuTime& /*time*/)
+byte RomFSA1FM1::peekMem(word address, const EmuTime& /*time*/) const
 {
 	if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return (*rom)[(0x2000 * (sram[0x1FC4] & 0x0F)) + 
+		return (*rom)[(0x2000 * ((*sram)[0x1FC4] & 0x0F)) +
 		              (address & 0x1FFF)];
 	} else if ((0x7FC0 <= address) && (address < 0x7FD0)) {
 		switch (address & 0x0F) {
 		case 4:
-			return sram[address & 0x1FFF];
+			return (*sram)[address & 0x1FFF];
 		case 6:
 			return firmwareSwitch->getStatus() ? 0xFB : 0xFF;
 		default:
@@ -101,12 +107,17 @@ byte RomFSA1FM1::readMem(word address, const EmuTime& /*time*/)
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
 		// read sram
 		// TODO are there multiple sram blocks?
-		return sram[address & 0x1FFF];
+		return (*sram)[address & 0x1FFF];
 	} else {
 		return 0xFF;
 	}
 }
 
+byte RomFSA1FM1::readMem(word address, const EmuTime& time)
+{
+	return peekMem(address, time);
+}
+
 const byte* RomFSA1FM1::getReadCacheLine(word address) const
 {
 	if (address == (0x7FC0 & CPU::CACHE_LINE_HIGH)) {
@@ -114,11 +125,11 @@ const byte* RomFSA1FM1::getReadCacheLine(word address) const
 		return NULL;
 	} else if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return &(*rom)[(0x2000 * (sram[0x1FC4] & 0x0F)) +
+		return &(*rom)[(0x2000 * ((*sram)[0x1FC4] & 0x0F)) +
 		               (address & 0x1FFF)];
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
 		// read sram
-		return &sram[address & 0x1FFF];
+		return &(*sram)[address & 0x1FFF];
 	} else {
 		return unmappedRead;
 	}
@@ -127,14 +138,14 @@ const byte* RomFSA1FM1::getReadCacheLine(word address) const
 void RomFSA1FM1::writeMem(word address, byte value, const EmuTime& /*time*/)
 {
 	// TODO 0x7FC0 - 0x7FCF is modem IO area
-	
+
 	//PRT_DEBUG("FSA1FM1: write "<<hex<<(int)address<<" "<<(int)value<<dec);
 	if ((0x6000 <= address) && (address < 0x8000)) {
 		if (address == 0x7FC4) {
 			// switch rom bank
-			cpu->invalidateMemCache(0x4000, 0x2000);
+			cpu.invalidateMemCache(0x4000, 0x2000);
 		}
-		sram[address & 0x1FFF] = value;
+		sram->write(address & 0x1FFF, value);
 	}
 }
 
@@ -144,7 +155,8 @@ byte* RomFSA1FM1::getWriteCacheLine(word address) const
 		// dont't cache IO area
 		return NULL;
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
-		return &sram[address & 0x1FFF];
+		// don't cache SRAM writes
+		return NULL;
 	} else {
 		return unmappedWrite;
 	}
@@ -153,16 +165,17 @@ byte* RomFSA1FM1::getWriteCacheLine(word address) const
 
 // Mapper for slot 3-3 //
 
-RomFSA1FM2::RomFSA1FM2(const XMLElement& config, const EmuTime& time,
-                       std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomFSA1FM2::RomFSA1FM2(MSXMotherBoard& motherBoard, const XMLElement& config,
+                       const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
-	sram = FSA1FMRam::getSRAM(config);
+	allocSRAM(motherBoard, config);
 	reset(time);
 }
 
 RomFSA1FM2::~RomFSA1FM2()
 {
+	releaseSRAM();
 }
 
 void RomFSA1FM2::reset(const EmuTime& /*time*/)
@@ -175,7 +188,7 @@ void RomFSA1FM2::reset(const EmuTime& /*time*/)
 	changeBank(7, 0);
 }
 
-byte RomFSA1FM2::readMem(word address, const EmuTime& time)
+byte RomFSA1FM2::peekMem(word address, const EmuTime& time) const
 {
 	byte result;
 	if (0xC000 <= address) {
@@ -184,16 +197,21 @@ byte RomFSA1FM2::readMem(word address, const EmuTime& time)
 		// read mapper state
 		result = bankSelect[address & 7];
 	} else if (isRam[address >> 13]) {
-		result = sram[address & 0x1FFF];
+		result = (*sram)[address & 0x1FFF];
 	} else if (isEmpty[address >> 13]) {
 		result = 0xFF;
 	} else {
-		result = Rom8kBBlocks::readMem(address, time);
+		result = Rom8kBBlocks::peekMem(address, time);
 	}
 	//PRT_DEBUG("FSA1FM2 read "<<hex<<(int)address<<" "<<(int)result<<dec);
 	return result;
 }
 
+byte RomFSA1FM2::readMem(word address, const EmuTime& time)
+{
+	return peekMem(address, time);
+}
+
 const byte* RomFSA1FM2::getReadCacheLine(word address) const
 {
 	if (0xC000 <= address) {
@@ -201,7 +219,7 @@ const byte* RomFSA1FM2::getReadCacheLine(word address) const
 	} else if ((0x7FF0 & CPU::CACHE_LINE_HIGH) == address) {
 		return NULL;
 	} else if (isRam[address >> 13]) {
-		return &sram[address & 0x1FFF];
+		return &(*sram)[address & 0x1FFF];
 	} else if (isEmpty[address >> 13]) {
 		return unmappedRead;
 	} else {
@@ -246,7 +264,7 @@ void RomFSA1FM2::writeMem(word address, byte value,
 		// write control byte
 		control = value;
 	} else if (isRam[address >> 13]) {
-		sram[address & 0x1FFF] = value;
+		sram->write(address & 0x1FFF, value);
 	}
 }
 
@@ -255,7 +273,7 @@ byte* RomFSA1FM2::getWriteCacheLine(word address) const
 	if ((0x6000 <= address) && (address < 0x8000)) {
 		return NULL;
 	} else if (isRam[address >> 13]) {
-		return &sram[address & 0x1FFF];
+		return NULL;
 	} else {
 		return unmappedWrite;
 	}
@@ -272,7 +290,7 @@ void RomFSA1FM2::changeBank(byte region, byte bank)
 			isRam[region]   = false;
 			isEmpty[region] = true;
 		}
-		cpu->invalidateMemCache(0x2000 * region, 0x2000);
+		cpu.invalidateMemCache(0x2000 * region, 0x2000);
 	} else {
 		isRam[region]   = false;
 		isEmpty[region] = false;
