@@ -1,21 +1,18 @@
-// $Id: RomGeneric16kB.cc,v 1.11 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomGeneric16kB.cc,v 1.13 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RomGeneric16kB.hh"
 #include "Rom.hh"
 
 namespace openmsx {
 
-RomGeneric16kB::RomGeneric16kB(const XMLElement& config, const EmuTime& time,
-                               std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomGeneric16kB::RomGeneric16kB(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomGeneric16kB::~RomGeneric16kB()
-{
-}
-
 void RomGeneric16kB::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
