@@ -1,21 +1,18 @@
-// $Id: RomGeneric8kB.cc,v 1.11 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomGeneric8kB.cc,v 1.13 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RomGeneric8kB.hh"
 #include "Rom.hh"
 
 namespace openmsx {
 
-RomGeneric8kB::RomGeneric8kB(const XMLElement& config, const EmuTime& time,
-                             std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomGeneric8kB::RomGeneric8kB(
+	MSXMotherBoard& motherBoard, const XMLElement& config, const EmuTime& time,
+	std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomGeneric8kB::~RomGeneric8kB()
-{
-}
-
 void RomGeneric8kB::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
