@@ -1,4 +1,4 @@
-// $Id: RomHalnote.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomHalnote.cc,v 1.12 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RomHalnote.hh"
 #include "CPU.hh"
@@ -6,17 +6,13 @@
 
 namespace openmsx {
 
-RomHalnote::RomHalnote(const XMLElement& config, const EmuTime& time,
-                       std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomHalnote::RomHalnote(MSXMotherBoard& motherBoard, const XMLElement& config,
+                       const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomHalnote::~RomHalnote()
-{
-}
-
 void RomHalnote::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
