@@ -1,4 +1,4 @@
-// $Id: RomHarryFox.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomHarryFox.cc,v 1.12 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RomHarryFox.hh"
 #include "CPU.hh"
@@ -6,17 +6,13 @@
 
 namespace openmsx {
 
-RomHarryFox::RomHarryFox(const XMLElement& config, const EmuTime& time,
-                         std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomHarryFox::RomHarryFox(MSXMotherBoard& motherBoard, const XMLElement& config,
+                         const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomHarryFox::~RomHarryFox()
-{
-}
-
 void RomHarryFox::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
