@@ -1,4 +1,4 @@
-// $Id: RomHolyQuran.cc,v 1.8 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomHolyQuran.cc,v 1.10 2005/11/12 21:53:21 m9710797 Exp $
 
 // Holy Qu'ran  cartridge
 //  It is like an ASCII 8KB, but using the 5000h, 5400h, 5800h and 5C00h
@@ -10,17 +10,14 @@
 
 namespace openmsx {
 
-RomHolyQuran::RomHolyQuran(const XMLElement& config, const EmuTime& time,
-                           std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomHolyQuran::RomHolyQuran(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomHolyQuran::~RomHolyQuran()
-{
-}
-
 void RomHolyQuran::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
