@@ -1,7 +1,7 @@
-// $Id: RomKorean126in1.cc,v 1.8 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomKorean126in1.cc,v 1.11 2005/11/12 21:53:21 m9710797 Exp $
 
 // Korean 126-in-1 cartridge
-// 
+//
 // Information obtained by studying MESS sources:
 //    0x4001 : 0x4000-0x7FFF
 //    0x4002 : 0x8000-0xBFFF
@@ -13,17 +13,14 @@
 
 namespace openmsx {
 
-RomKorean126in1::RomKorean126in1(const XMLElement& config, const EmuTime& time,
-                                 std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomKorean126in1::RomKorean126in1(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomKorean126in1::~RomKorean126in1()
-{
-}
-
 void RomKorean126in1::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
