@@ -1,4 +1,4 @@
-// $Id: RomKorean80in1.cc,v 1.8 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomKorean80in1.cc,v 1.10 2005/11/12 21:53:21 m9710797 Exp $
 
 // Korean 80-in-1 cartridge
 //
@@ -14,17 +14,14 @@
 
 namespace openmsx {
 
-RomKorean80in1::RomKorean80in1(const XMLElement& config, const EmuTime& time,
-                               std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(config, time, rom)
+RomKorean80in1::RomKorean80in1(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomKorean80in1::~RomKorean80in1()
-{
-}
-
 void RomKorean80in1::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
