@@ -1,4 +1,4 @@
-// $Id: RomNational.cc,v 1.13 2005/01/13 18:25:41 m9710797 Exp $
+// $Id: RomNational.cc,v 1.17 2006/05/26 10:45:54 m9710797 Exp $
 
 #include "RomNational.hh"
 #include "CPU.hh"
@@ -7,10 +7,10 @@
 
 namespace openmsx {
 
-RomNational::RomNational(const XMLElement& config, const EmuTime& time,
-                         std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
-	, sram(new SRAM(getName() + " SRAM", 0x1000, config))
+RomNational::RomNational(MSXMotherBoard& motherBoard, const XMLElement& config,
+                         const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
+	, sram(new SRAM(motherBoard, getName() + " SRAM", 0x1000, config))
 {
 	reset(time);
 }
@@ -29,7 +29,7 @@ void RomNational::reset(const EmuTime& /*time*/)
 	sramAddr = 0;	// TODO check this
 }
 
-byte RomNational::readMem(word address, const EmuTime& time)
+byte RomNational::peekMem(word address, const EmuTime& time) const
 {
 	if ((control & 0x04) && ((address & 0x7FF9) == 0x7FF0)) {
 		// TODO check mirrored
@@ -39,9 +39,18 @@ byte RomNational::readMem(word address, const EmuTime& time)
 	}
 	if ((control & 0x02) && ((address & 0x3FFF) == 0x3FFD)) {
 		// SRAM read
-		return (*sram)[sramAddr++ & 0x0FFF];
+		return (*sram)[sramAddr & 0x0FFF];
 	}
-	return Rom16kBBlocks::readMem(address, time);
+	return Rom16kBBlocks::peekMem(address, time);
+}
+
+byte RomNational::readMem(word address, const EmuTime& time)
+{
+	byte result = peekMem(address, time);
+	if ((control & 0x02) && ((address & 0x3FFF) == 0x3FFD)) {
+		++sramAddr;
+	}
+	return result;
 }
 
 const byte* RomNational::getReadCacheLine(word address) const
@@ -83,7 +92,7 @@ void RomNational::writeMem(word address, byte value, const EmuTime& /*time*/)
 			// SRAM address bits 7-0
 			sramAddr = (sramAddr & 0xFFFF00) | value;
 		} else if (address == 0x3FFD) {
-			(*sram)[sramAddr++ & 0x0FFF] = value;
+			sram->write((sramAddr++ & 0x0FFF), value);
 		}
 	}
 }
