@@ -1,13 +1,13 @@
-// $Id: RomPageNN.cc,v 1.11 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomPageNN.cc,v 1.13 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RomPageNN.hh"
 #include "Rom.hh"
 
 namespace openmsx {
 
-RomPageNN::RomPageNN(const XMLElement& config, const EmuTime& time,
-                     std::auto_ptr<Rom> rom, byte pages)
-	: Rom8kBBlocks(config, time, rom)
+RomPageNN::RomPageNN(MSXMotherBoard& motherBoard, const XMLElement& config,
+                     const EmuTime& time, std::auto_ptr<Rom> rom, byte pages)
+	: Rom8kBBlocks(motherBoard, config, time, rom)
 {
 	int bank = 0;
 	for (int page = 0; page < 4; page++) {
@@ -21,9 +21,5 @@ RomPageNN::RomPageNN(const XMLElement& config, const EmuTime& time,
 	}
 }
 
-RomPageNN::~RomPageNN()
-{
-}
-
 } // namespace openmsx
 
