@@ -1,7 +1,8 @@
-// $Id: RomPanasonic.cc,v 1.21 2005/01/15 12:55:45 m9710797 Exp $
+// $Id: RomPanasonic.cc,v 1.26 2006/05/26 10:45:54 m9710797 Exp $
 
 #include "RomPanasonic.hh"
 #include "PanasonicMemory.hh"
+#include "MSXMotherBoard.hh"
 #include "XMLElement.hh"
 #include "SRAM.hh"
 #include "CPU.hh"
@@ -13,13 +14,15 @@ const int SRAM_BASE = 0x80;
 const int RAM_BASE  = 0x180;
 
 
-RomPanasonic::RomPanasonic(const XMLElement& config, const EmuTime& time,
-                           std::auto_ptr<Rom> rom_)
-	: Rom8kBBlocks(config, time, rom_)
+RomPanasonic::RomPanasonic(
+	MSXMotherBoard& motherBoard, const XMLElement& config,
+	const EmuTime& time, std::auto_ptr<Rom> rom_)
+	: Rom8kBBlocks(motherBoard, config, time, rom_)
 {
 	int sramSize = config.getChildDataAsInt("sramsize", 0);
 	if (sramSize) {
-		sram.reset(new SRAM(getName() + " SRAM", sramSize * 1024, config));
+		sram.reset(new SRAM(motherBoard, getName() + " SRAM",
+		                    sramSize * 1024, config));
 	}
 
 	if (config.getChildDataAsBool("sram-mirrored", false)) {
@@ -27,7 +30,7 @@ RomPanasonic::RomPanasonic(const XMLElement& config, const EmuTime& time,
 	} else {
 		maxSRAMBank = SRAM_BASE + (sramSize / 8);
 	}
-	
+
 	reset(time);
 }
 
@@ -44,7 +47,7 @@ void RomPanasonic::reset(const EmuTime& /*time*/)
 	}
 }
 
-byte RomPanasonic::readMem(word address, const EmuTime& time)
+byte RomPanasonic::peekMem(word address, const EmuTime& time) const
 {
 	byte result;
 	if ((control & 0x04) && (0x7FF0 <= address) && (address < 0x7FF8)) {
@@ -63,16 +66,20 @@ byte RomPanasonic::readMem(word address, const EmuTime& time)
 		// read control byte
 		result = control;
 	} else {
-		result = Rom8kBBlocks::readMem(address, time);
+		result = Rom8kBBlocks::peekMem(address, time);
 	}
 	//PRT_DEBUG("DEBUG read "<<hex<<(int)address<<" "<<
 	//          (int)result<<dec);
 	return result;
 }
 
+byte RomPanasonic::readMem(word address, const EmuTime& time)
+{
+	return peekMem(address, time);
+}
+
 const byte* RomPanasonic::getReadCacheLine(word address) const
 {
-	//return NULL;
 	if ((0x7FF0 & CPU::CACHE_LINE_HIGH) == address) {
 		// TODO check mirrored
 		return NULL;
@@ -111,12 +118,13 @@ void RomPanasonic::writeMem(word address, byte value, const EmuTime& /*time*/)
 		if (sram.get() && (SRAM_BASE <= selectedBank) &&
 			    (selectedBank < maxSRAMBank)) {
 			// SRAM
-			bank[region][address & 0x1FFF] = value;
+			int block = selectedBank - SRAM_BASE;
+			sram->write((block * 0x2000) | (address & 0x1FFF), value);
 		} else if (RAM_BASE <= selectedBank) {
 			// RAM
-			bank[region][address & 0x1FFF] = value;
+			const_cast<byte*>(bank[region])[address & 0x1FFF] = value;
 		}
-	} 
+	}
 }
 
 byte* RomPanasonic::getWriteCacheLine(word address) const
@@ -129,10 +137,13 @@ byte* RomPanasonic::getWriteCacheLine(word address) const
 	} else if ((0x8000 <= address) && (address < 0xC000)) {
 		int region = address >> 13;
 		int selectedBank = bankSelect[region];
-		if ((sram.get() && (SRAM_BASE <= selectedBank) &&
-			     (selectedBank < maxSRAMBank)) ||
-		    (RAM_BASE <= selectedBank)) {
-			return &bank[region][address & 0x1FFF];
+		if (sram.get() && (SRAM_BASE <= selectedBank) &&
+			     (selectedBank < maxSRAMBank)) {
+			// SRAM
+			return NULL;
+		} else if (RAM_BASE <= selectedBank) {
+			// RAM
+			return const_cast<byte*>(&bank[region][address & 0x1FFF]);
 		} else {
 			return unmappedWrite;
 		}
@@ -157,7 +168,7 @@ void RomPanasonic::changeBank(byte region, int bank)
 		setBank(region, &sram->operator[](offset));
 	} else if (RAM_BASE <= bank) {
 		// RAM
-		setBank(region, PanasonicMemory::instance().
+		setBank(region, getMotherBoard().getPanasonicMemory().
 			getRamBlock(bank - RAM_BASE));
 	} else {
 		// ROM
