@@ -1,4 +1,4 @@
-// $Id: RomPlain.cc,v 1.19 2005/02/24 22:44:51 m9710797 Exp $
+// $Id: RomPlain.cc,v 1.24 2006/01/25 19:26:32 m9710797 Exp $
 
 #include "RomPlain.hh"
 #include "Rom.hh"
@@ -17,13 +17,14 @@ static inline bool isInside(unsigned x, unsigned start, unsigned len)
 
 static std::string toString(unsigned start, unsigned len)
 {
-	return "[0x" + StringOp::toHexString(start) + ", " +
-	        "0x" + StringOp::toHexString(start + len) + ']';
+	return "[0x" + StringOp::toHexString(start, 4) + ", " +
+	        "0x" + StringOp::toHexString(start + len, 4) + ']';
 }
 
-RomPlain::RomPlain(const XMLElement& config, const EmuTime& time,
-                   std::auto_ptr<Rom> rom_, MirrorType mirrored, int start)
-	: Rom8kBBlocks(config, time, rom_)
+RomPlain::RomPlain(MSXMotherBoard& motherBoard, const XMLElement& config,
+                   const EmuTime& time, std::auto_ptr<Rom> rom_,
+                   MirrorType mirrored, int start)
+	: Rom8kBBlocks(motherBoard, config, time, rom_)
 {
 	unsigned windowBase =  0x0000;
 	unsigned windowSize = 0x10000;
@@ -34,23 +35,23 @@ RomPlain::RomPlain(const XMLElement& config, const EmuTime& time,
 
 	unsigned romSize = rom->getSize();
 	if ((romSize > 0x10000) || (romSize & 0x1FFF)) {
-		throw FatalError(rom->getName() +
+		throw MSXException(rom->getName() +
 		    ": invalid rom size: must be smaller than or equal to 64kB "
 		    "and must be a multiple of 8kB.");
 	}
-	
+
 	unsigned romBase = (start == -1)
 	                 ? guessLocation(windowBase, windowSize)
 	                 : start;
 	if (!isInside(romBase,               windowBase, windowSize) ||
 	    !isInside(romBase + romSize - 1, windowBase, windowSize)) {
-		throw FatalError(rom->getName() +
+		throw MSXException(rom->getName() +
 		    ": invalid rom position: interval " +
 		    toString(romBase, romSize) + " must fit in " +
 		    toString(windowBase, windowSize) + '.');
 	}
 	if ((romBase & 0x1FFF)) {
-		throw FatalError(rom->getName() +
+		throw MSXException(rom->getName() +
 		    ": invalid rom position: must start at a 8kB boundary.");
 	}
 
@@ -67,12 +68,8 @@ RomPlain::RomPlain(const XMLElement& config, const EmuTime& time,
 				setBank(page, unmappedRead);
 			}
 		}
-		
-	}
-}
 
-RomPlain::~RomPlain()
-{
+	}
 }
 
 void RomPlain::guessHelper(unsigned offset, int* pages)
