@@ -1,4 +1,4 @@
-// $Id: RomRType.cc,v 1.10 2004/12/15 09:09:48 m9710797 Exp $
+// $Id: RomRType.cc,v 1.13 2005/11/12 21:53:21 m9710797 Exp $
 
 // R-Type cartridges
 //
@@ -7,24 +7,20 @@
 //  second 16kb: 0x7000 - 0x7FFF (0x7000 and 0x7800 used)
 //               bit 4 selects ROM chip,
 //                if low  bit 3-0 select page
-//                   high     2-0 
-                   
+//                   high     2-0
+
 #include "RomRType.hh"
 #include "Rom.hh"
 
 namespace openmsx {
 
-RomRType::RomRType(const XMLElement& config, const EmuTime& time,
-                   std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(config, time, rom)
+RomRType::RomRType(MSXMotherBoard& motherBoard, const XMLElement& config,
+                   const EmuTime& time, std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, time, rom)
 {
 	reset(time);
 }
 
-RomRType::~RomRType()
-{
-}
-
 void RomRType::reset(const EmuTime& /*time*/)
 {
 	setBank(0, unmappedRead);
