@@ -1,38 +1,51 @@
-// $Id: SRAM.cc,v 1.15 2005/03/12 20:47:16 m9710797 Exp $
+// $Id: SRAM.cc,v 1.20 2006/02/24 20:14:22 m9710797 Exp $
 
 #include "SRAM.hh"
 #include "XMLElement.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
+#include "MSXMotherBoard.hh"
 #include "CliComm.hh"
 
 using std::string;
 
 namespace openmsx {
 
-SRAM::SRAM(const string& name, int size,
+SRAM::SRAM(MSXMotherBoard& motherBoard, const string& name, int size,
            const XMLElement& config_, const char* header_)
-	: Ram(name, "sram", size), config(config_), header(header_)
+	: ram(motherBoard, name, "sram", size)
+	, config(config_)
+	, header(header_)
+	, cliComm(motherBoard.getCliComm())
 {
-	init();
+	load();
 }
 
-SRAM::SRAM(const string& name, const string& description, int size,
+SRAM::SRAM(MSXMotherBoard& motherBoard, const string& name,
+           const string& description, int size,
 	   const XMLElement& config_, const char* header_)
-	: Ram(name, description, size), config(config_), header(header_)
+	: ram(motherBoard, name, description, size)
+	, config(config_)
+	, header(header_)
+	, cliComm(motherBoard.getCliComm())
 {
-	init();
+	load();
 }
 
-void SRAM::init()
+SRAM::~SRAM()
+{
+	cancel(); // cancel pending alarm
+	save();
+}
+
+void SRAM::load()
 {
 	const string& filename = config.getChildData("sramname");
-	PRT_DEBUG("SRAM: read " << filename);
 	try {
 		bool headerOk = true;
 		File file(config.getFileContext().resolveCreate(filename),
-			  LOAD_PERSISTENT);
+			  File::LOAD_PERSISTENT);
 		if (header) {
 			int length = strlen(header);
 			byte* temp = new byte[length];
@@ -43,35 +56,38 @@ void SRAM::init()
 			delete[] temp;
 		}
 		if (headerOk) {
-			file.read(&(this->operator[](0)), getSize());
+			file.read(&ram[0], getSize());
 		} else {
-			CliComm::instance().printWarning(
+			cliComm.printWarning(
 				"Warning no correct SRAM file: " + filename);
 		}
 	} catch (FileException &e) {
-		CliComm::instance().printWarning(
-			"Couldn't load SRAM " + filename +
-			" (" + e.getMessage() + ").");
+		cliComm.printWarning("Couldn't load SRAM " + filename +
+		                     " (" + e.getMessage() + ").");
 	}
 }
 
-SRAM::~SRAM()
+void SRAM::save()
 {
 	const string& filename = config.getChildData("sramname");
-	PRT_DEBUG("SRAM: save " << filename);
 	try {
 		File file(config.getFileContext().resolveCreate(filename),
-			  SAVE_PERSISTENT);
+			  File::SAVE_PERSISTENT);
 		if (header) {
 			int length = strlen(header);
 			file.write((const byte*)header, length);
 		}
-		file.write(&(this->operator[](0)), getSize());
+		file.write(&ram[0], getSize());
 	} catch (FileException& e) {
-		CliComm::instance().printWarning(
-			"Couldn't save SRAM " + filename +
-			" (" + e.getMessage() + ").");
+		cliComm.printWarning("Couldn't save SRAM " + filename +
+		                     " (" + e.getMessage() + ").");
 	}
 }
 
+bool SRAM::alarm()
+{
+	save();
+	return false; // don't reschedule
+}
+
 } // namespace openmsx
