@@ -1,25 +1,19 @@
-// $Id: ClockPin.cc,v 1.13 2005/05/14 12:49:39 m9710797 Exp $
+// $Id: ClockPin.cc,v 1.17 2006/01/23 19:35:25 m9710797 Exp $
 
 #include <cassert>
 #include "ClockPin.hh"
-#include "Scheduler.hh"
 
 using std::string;
 
 namespace openmsx {
 
-ClockPin::ClockPin(ClockPinListener* listener_)
-	: listener(listener_), periodic(false), status(false),
-	  signalEdge(false), scheduler(Scheduler::instance())
+ClockPin::ClockPin(Scheduler& scheduler, ClockPinListener* listener_)
+	: Schedulable(scheduler), listener(listener_)
+	, referenceTime(EmuTime::zero)
+	, periodic(false) , status(false), signalEdge(false)
 {
 }
 
-ClockPin::~ClockPin()
-{
-	unschedule();
-}
-
-
 void ClockPin::setState(bool newStatus, const EmuTime& time)
 {
 	periodic = false;
@@ -46,7 +40,7 @@ void ClockPin::setPeriodicState(const EmuDuration& total,
 	referenceTime = time;
 	totalDur = total;
 	hiDur = hi;
-	
+
 	if (listener) {
 		if (periodic) {
 			unschedule();
@@ -76,7 +70,7 @@ bool ClockPin::isPeriodic() const
 	return periodic;
 }
 
-const EmuDuration& ClockPin::getTotalDuration() const 
+const EmuDuration& ClockPin::getTotalDuration() const
 {
 	assert(periodic);
 	return totalDur;
@@ -128,13 +122,13 @@ void ClockPin::generateEdgeSignals(bool wanted, const EmuTime& time)
 
 void ClockPin::unschedule()
 {
-	scheduler.removeSyncPoint(*this);
+	removeSyncPoint();
 }
 
 void ClockPin::schedule(const EmuTime& time)
 {
 	assert(signalEdge && periodic && listener);
-	scheduler.setSyncPoint(time, *this);
+	setSyncPoint(time);
 }
 
 void ClockPin::executeUntil(const EmuTime& time, int /*userdata*/)
