@@ -1,9 +1,7 @@
-// $Id: DummyMidiInDevice.cc,v 1.10 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: DummyMidiInDevice.cc,v 1.11 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "DummyMidiInDevice.hh"
 
-using std::string;
-
 namespace openmsx {
 
 void DummyMidiInDevice::signal(const EmuTime& /*time*/)
@@ -11,13 +9,13 @@ void DummyMidiInDevice::signal(const EmuTime& /*time*/)
 	// ignore
 }
 
-const string& DummyMidiInDevice::getDescription() const
+const std::string& DummyMidiInDevice::getDescription() const
 {
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyMidiInDevice::plugHelper(Connector* /*connector*/,
+void DummyMidiInDevice::plugHelper(Connector& /*connector*/,
                                    const EmuTime& /*time*/)
 {
 }
