@@ -1,9 +1,7 @@
-// $Id: DummyMidiOutDevice.cc,v 1.10 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: DummyMidiOutDevice.cc,v 1.11 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "DummyMidiOutDevice.hh"
 
-using std::string;
-
 namespace openmsx {
 
 void DummyMidiOutDevice::recvByte(byte /*value*/, const EmuTime& /*time*/)
@@ -11,13 +9,14 @@ void DummyMidiOutDevice::recvByte(byte /*value*/, const EmuTime& /*time*/)
 	// ignore
 }
 
-const string& DummyMidiOutDevice::getDescription() const
+const std::string& DummyMidiOutDevice::getDescription() const
 {
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyMidiOutDevice::plugHelper(Connector* /*connector*/, const EmuTime& /*time*/)
+void DummyMidiOutDevice::plugHelper(
+		Connector& /*connector*/, const EmuTime& /*time*/)
 {
 }
 
