@@ -1,9 +1,7 @@
-// $Id: DummyRS232Device.cc,v 1.8 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: DummyRS232Device.cc,v 1.9 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "DummyRS232Device.hh"
 
-using std::string;
-
 namespace openmsx {
 
 void DummyRS232Device::signal(const EmuTime& /*time*/)
@@ -11,13 +9,13 @@ void DummyRS232Device::signal(const EmuTime& /*time*/)
 	// ignore
 }
 
-const string& DummyRS232Device::getDescription() const
+const std::string& DummyRS232Device::getDescription() const
 {
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyRS232Device::plugHelper(Connector* /*connector*/,
+void DummyRS232Device::plugHelper(Connector& /*connector*/,
                                   const EmuTime& /*time*/)
 {
 }
