@@ -1,8 +1,8 @@
-// $Id: I8251.cc,v 1.14 2005/05/14 12:49:39 m9710797 Exp $
+// $Id: I8251.cc,v 1.19 2005/12/03 18:58:10 arnoldmnl Exp $
 
-#include <cassert>
 #include "I8251.hh"
 #include "Scheduler.hh"
+#include <cassert>
 
 using std::string;
 
@@ -51,18 +51,12 @@ enum SyncPointType {
 };
 
 
-I8251::I8251(I8251Interface* interf_, const EmuTime& time)
-	: interf(interf_), recvBuf(0)
+I8251::I8251(Scheduler& scheduler, I8251Interface* interf_, const EmuTime& time)
+	: Schedulable(scheduler), interf(interf_), clock(scheduler), recvBuf(0)
 {
 	reset(time);
 }
 
-I8251::~I8251()
-{
-	Scheduler::instance().removeSyncPoint(*this, TRANS);
-	Scheduler::instance().removeSyncPoint(*this, RECV);
-}
-
 void I8251::reset(const EmuTime& time)
 {
 	status = STAT_TXRDY | STAT_TXEMPTY;
@@ -71,7 +65,7 @@ void I8251::reset(const EmuTime& time)
 	cmdFaze = FAZE_MODE;
 }
 
-byte I8251::readIO(byte port, const EmuTime& time)
+byte I8251::readIO(word port, const EmuTime& time)
 {
 	byte result;
 	switch (port & 1) {
@@ -89,7 +83,7 @@ byte I8251::readIO(byte port, const EmuTime& time)
 	return result;
 }
 
-byte I8251::peekIO(byte port, const EmuTime& /*time*/) const
+byte I8251::peekIO(word port, const EmuTime& /*time*/) const
 {
 	byte result;
 	switch (port & 1) {
@@ -107,7 +101,7 @@ byte I8251::peekIO(byte port, const EmuTime& /*time*/) const
 }
 
 
-void I8251::writeIO(byte port, byte value, const EmuTime& time)
+void I8251::writeIO(word port, byte value, const EmuTime& time)
 {
 	//PRT_DEBUG("I8251: write " << (int)port << " " << (int)value);
 	switch (port & 1) {
@@ -228,15 +222,15 @@ void I8251::writeCommand(byte value, const EmuTime& time)
 {
 	byte oldCommand = command;
 	command = value;
-	
+
 	// CMD_RESET, CMD_TXEN, CMD_RXE  handled in other routines
-	
+
 	interf->setRTS(command & CMD_RTS, time);
 	interf->setDTR(command & CMD_DTR, time);
 
 	if (!(command & CMD_TXEN)) {
 		// disable transmitter
-		Scheduler::instance().removeSyncPoint(*this, TRANS);
+		removeSyncPoint(TRANS);
 		status |= STAT_TXRDY | STAT_TXEMPTY;
 	}
 	if (command & CMD_RSTERR) {
@@ -256,7 +250,7 @@ void I8251::writeCommand(byte value, const EmuTime& time)
 			recvReady = true;
 		} else {
 			// disable receiver
-			Scheduler::instance().removeSyncPoint(*this, RECV);
+			removeSyncPoint(RECV);
 			status &= ~(STAT_PE | STAT_OE | STAT_FE); // TODO
 			status &= ~STAT_RXRDY;
 		}
@@ -330,7 +324,7 @@ void I8251::recvByte(byte value, const EmuTime& time)
 	recvReady = false;
 	if (clock.isPeriodic()) {
 		EmuTime next = time + (clock.getTotalDuration() * charLength);
-		Scheduler::instance().setSyncPoint(next, *this, RECV);
+		setSyncPoint(next, RECV);
 	}
 }
 
@@ -349,11 +343,11 @@ void I8251::send(byte value, const EmuTime& time)
 	sendByte = value;
 	if (clock.isPeriodic()) {
 		EmuTime next = time + (clock.getTotalDuration() * charLength);
-		Scheduler::instance().setSyncPoint(next, *this, TRANS);
+		setSyncPoint(next, TRANS);
 	}
 }
 
-void I8251::executeUntil(const EmuTime &time, int userData)
+void I8251::executeUntil(const EmuTime& time, int userData)
 {
 	switch ((SyncPointType)userData) {
 	case RECV:
@@ -361,9 +355,9 @@ void I8251::executeUntil(const EmuTime &time, int userData)
 		recvReady = true;
 		interf->signal(time);
 		break;
-	case TRANS: 
+	case TRANS:
 		assert(!(status & STAT_TXEMPTY) && (command & CMD_TXEN));
-		
+
 		interf->recvByte(sendByte, time);
 		if (status & STAT_TXRDY) {
 			status |= STAT_TXEMPTY;
