@@ -1,4 +1,4 @@
-// $Id: I8254.cc,v 1.9 2004/10/19 19:47:45 m9710797 Exp $
+// $Id: I8254.cc,v 1.13 2006/01/23 19:35:25 m9710797 Exp $
 
 #include <cassert>
 #include "I8254.hh"
@@ -8,11 +8,12 @@ namespace openmsx {
 
 /// class I8254 ///
 
-I8254::I8254(ClockPinListener* output0, ClockPinListener* output1,
-             ClockPinListener* output2, const EmuTime& time)
-	: counter0(output0, time),
-	  counter1(output1, time),
-	  counter2(output2, time)
+I8254::I8254(Scheduler& scheduler, ClockPinListener* output0,
+             ClockPinListener* output1, ClockPinListener* output2,
+             const EmuTime& time)
+	: counter0(scheduler, output0, time)
+	, counter1(scheduler, output1, time)
+	, counter2(scheduler, output2, time)
 {
 }
 
@@ -27,7 +28,7 @@ void I8254::reset(const EmuTime& time)
 	counter2.reset(time);
 }
 
-byte I8254::readIO(byte port, const EmuTime& time)
+byte I8254::readIO(word port, const EmuTime& time)
 {
 	port &= 3;
 	switch (port) {
@@ -45,7 +46,7 @@ byte I8254::readIO(byte port, const EmuTime& time)
 	}
 }
 
-byte I8254::peekIO(byte port, const EmuTime& time) const
+byte I8254::peekIO(word port, const EmuTime& time) const
 {
 	port &= 3;
 	switch (port) {
@@ -63,7 +64,7 @@ byte I8254::peekIO(byte port, const EmuTime& time) const
 	}
 }
 
-void I8254::writeIO(byte port, byte value, const EmuTime& time)
+void I8254::writeIO(word port, byte value, const EmuTime& time)
 {
 	port &= 3;
 	switch (port) {
@@ -136,8 +137,10 @@ I8254::Counter& I8254::getCounter(byte cntr)
 
 /// class Counter ///
 
-I8254::Counter::Counter(ClockPinListener* listener, const EmuTime& time)
-	: output(listener)
+I8254::Counter::Counter(Scheduler& scheduler, ClockPinListener* listener,
+                        const EmuTime& time)
+	: clock(scheduler), output(scheduler, listener)
+	, currentTime(time)
 {
 	gate = true;
 	counter = 0;
@@ -192,7 +195,7 @@ byte I8254::Counter::peekIO(const EmuTime& time) const
 	if (ltchCtrl) {
 		return latchedControl;
 	}
-	
+
 	const_cast<I8254::Counter*>(this)->advance(time);
 
 	word readData = ltchCntr ? latchedCounter : counter;
@@ -215,7 +218,7 @@ byte I8254::Counter::peekIO(const EmuTime& time) const
 	}
 }
 
-void I8254::Counter::writeIO(byte value, const EmuTime& time)
+void I8254::Counter::writeIO(word value, const EmuTime& time)
 {
 	advance(time);
 	switch (control & WRT_FRMT) {
@@ -251,7 +254,7 @@ void I8254::Counter::writeLoad(word value, const EmuTime& time)
 	if ((mode==CNTR_M0) || (mode==CNTR_M4)) {
 		counter = counterLoad;
 	}
-	if (!active && ((mode == CNTR_M2) || (mode == CNTR_M2_) || 
+	if (!active && ((mode == CNTR_M2) || (mode == CNTR_M2_) ||
 	                 (mode == CNTR_M3) || (mode == CNTR_M3_))) {
 		if (clock.isPeriodic()) {
 			counter = counterLoad;
@@ -279,8 +282,8 @@ void I8254::Counter::writeControlWord(byte value, const EmuTime& time)
 		// new control mode
 		control = value;
 		writeOrder = LOW;
-		counting = true; 
-		active = false; 
+		counting = true;
+		active = false;
 		triggered = false;
 		switch (control & CNTR_MODE) {
 		case CNTR_M0:
