@@ -1,30 +1,27 @@
-// $Id: MC6850.cc,v 1.1 2004/10/25 17:08:56 mthuurne Exp $
+// $Id: MC6850.cc,v 1.4 2005/12/03 18:58:10 arnoldmnl Exp $
 
-#include <cassert>
 #include "MC6850.hh"
+#include <cassert>
 
 namespace openmsx {
 
-MC6850::MC6850(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
+MC6850::MC6850(MSXMotherBoard& motherBoard, const XMLElement& config,
+               const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
 {
 	reset(time);
 }
 
-MC6850::~MC6850()
-{
-}
-
 void MC6850::reset(const EmuTime& /*time*/)
 {
 }
 
-byte MC6850::readIO(byte port, const EmuTime& time)
+byte MC6850::readIO(word port, const EmuTime& time)
 {
 	return peekIO(port, time);
 }
 
-byte MC6850::peekIO(byte port, const EmuTime& /*time*/) const
+byte MC6850::peekIO(word port, const EmuTime& /*time*/) const
 {
 	byte result;
 	switch (port & 0x1) {
@@ -42,7 +39,7 @@ byte MC6850::peekIO(byte port, const EmuTime& /*time*/) const
 	return result;
 }
 
-void MC6850::writeIO(byte port, byte /*value*/, const EmuTime& /*time*/)
+void MC6850::writeIO(word port, byte /*value*/, const EmuTime& /*time*/)
 {
 	switch (port & 0x01) {
 	case 0: // control
