@@ -1,9 +1,10 @@
-// $Id: MidiInNative.cc,v 1.17 2005/05/14 14:57:01 h_oudejans Exp $
+// $Id: MidiInNative.cc,v 1.24 2006/05/25 14:16:07 m9710797 Exp $
 
 #if defined(_WIN32)
 #include "MidiInNative.hh"
 #include "MidiInConnector.hh"
 #include "PluggingController.hh"
+#include "EventDistributor.hh"
 #include "Scheduler.hh"
 #include <cstring>
 #include <cerrno>
@@ -21,44 +22,52 @@ using std::string;
 
 namespace openmsx {
 
-void MidiInNative::registerAll(PluggingController* controller)
+void MidiInNative::registerAll(EventDistributor& eventDistributor,
+                               Scheduler& scheduler,
+                               PluggingController& controller)
 {
 	w32_midiInInit();
 	unsigned devnum = w32_midiInGetVFNsNum();
 	for (unsigned i = 0 ; i <devnum; ++i) {
-		controller->registerPluggable(new MidiInNative(i));
+		controller.registerPluggable(
+			new MidiInNative(eventDistributor, scheduler, i));
 	}
 }
 
 
-MidiInNative::MidiInNative(unsigned num)
-	: thread(this), lock(1)
+MidiInNative::MidiInNative(EventDistributor& eventDistributor_,
+                           Scheduler& scheduler_, unsigned num)
+	: eventDistributor(eventDistributor_), scheduler(scheduler_)
+	, thread(this), lock(1)
 {
 	name = w32_midiInGetVFN(num);
 	desc = w32_midiInGetRDN(num);
+	
+	eventDistributor.registerEventListener(OPENMSX_MIDI_IN_NATIVE_EVENT, *this);
 }
 
 MidiInNative::~MidiInNative()
 {
-	Scheduler::instance().removeSyncPoint(*this);
+	eventDistributor.unregisterEventListener(OPENMSX_MIDI_IN_NATIVE_EVENT, *this);
+
 	//w32_midiInClean(); // TODO
 }
 
 // Pluggable
-void MidiInNative::plugHelper(Connector* connector_, const EmuTime& time)
+void MidiInNative::plugHelper(Connector& connector_, const EmuTime& time)
 {
 	devidx = w32_midiInOpen(name.c_str(), thrdid);
 	if (devidx == (unsigned)-1) {
 		throw PlugException("Failed to open " + name);
 	}
 
-	MidiInConnector* midiConnector = static_cast<MidiInConnector*>(connector_);
-	midiConnector->setDataBits(SerialDataInterface::DATA_8);	// 8 data bits
-	midiConnector->setStopBits(SerialDataInterface::STOP_1);	// 1 stop bit
-	midiConnector->setParityBit(false, SerialDataInterface::EVEN); // no parity
+	MidiInConnector& midiConnector = static_cast<MidiInConnector&>(connector_);
+	midiConnector.setDataBits(SerialDataInterface::DATA_8); // 8 data bits
+	midiConnector.setStopBits(SerialDataInterface::STOP_1); // 1 stop bit
+	midiConnector.setParityBit(false, SerialDataInterface::EVEN); // no parity
 
-	connector = connector_; // base class will do this in a moment,
-	                        // but thread already needs it
+	connector = &connector_; // base class will do this in a moment,
+	                         // but thread already needs it
 	thread.start();
 }
 
@@ -86,7 +95,8 @@ void MidiInNative::procLongMsg(LPMIDIHDR p)
 		for (unsigned i = 0; i < p->dwBytesRecorded; ++i) {
 			queue.push_back(p->lpData[i]);
 		}
-		Scheduler::instance().setSyncPoint(Scheduler::ASAP, *this);
+		eventDistributor.distributeEvent(
+			new SimpleEvent<OPENMSX_MIDI_IN_NATIVE_EVENT>());
 	}
 }
 
@@ -106,7 +116,8 @@ void MidiInNative::procShortMsg(DWORD param)
 		queue.push_back(param & 0xFF);
 		param >>= 8;
 	}
-	Scheduler::instance().setSyncPoint(Scheduler::ASAP,*this);
+	eventDistributor.distributeEvent(
+		new SimpleEvent<OPENMSX_MIDI_IN_NATIVE_EVENT>());
 }
 
 // Runnable
@@ -153,7 +164,7 @@ void MidiInNative::signal(const EmuTime& time)
 		return;
 	}
 	if (!connector->ready()) return;
-	
+
 	ScopedLock l(lock);
 	if (queue.empty()) return;
 	byte data = queue.front();
@@ -161,22 +172,17 @@ void MidiInNative::signal(const EmuTime& time)
 	connector->recvByte(data, time);
 }
 
-// Schedulable
-void MidiInNative::executeUntil(const EmuTime& time, int userData)
+// EventListener
+void MidiInNative::signalEvent(const Event& /*event*/)
 {
 	if (getConnector()) {
-		signal(time);
+		signal(scheduler.getCurrentTime());
 	} else {
 		ScopedLock l(lock);
 		queue.empty();
 	}
 }
 
-const string &MidiInNative::schedName() const
-{
-	return getName();
-}
-
 } // namespace openmsx
 
 #endif // defined(_WIN32)
