@@ -1,7 +1,8 @@
-// $Id: MidiInReader.cc,v 1.24 2005/05/14 13:29:07 m9710797 Exp $
+// $Id: MidiInReader.cc,v 1.29 2006/05/25 12:37:12 m9710797 Exp $
 
 #include "MidiInReader.hh"
 #include "MidiInConnector.hh"
+#include "EventDistributor.hh"
 #include "Scheduler.hh"
 #include "FilenameSetting.hh"
 #include <cstring>
@@ -11,21 +12,26 @@ using std::string;
 
 namespace openmsx {
 
-MidiInReader::MidiInReader()
-	: thread(this), lock(1)
-	, readFilenameSetting(new FilenameSetting("midi-in-readfilename",
+MidiInReader::MidiInReader(EventDistributor& eventDistributor_,
+                           Scheduler& scheduler_,
+                           CommandController& commandController)
+	: eventDistributor(eventDistributor_), scheduler(scheduler_)
+	, thread(this), lock(1)
+	, readFilenameSetting(new FilenameSetting(
+		commandController, "midi-in-readfilename",
 		"filename of the file where the MIDI input is read from",
 		"/dev/midi"))
 {
+	eventDistributor.registerEventListener(OPENMSX_MIDI_IN_READER_EVENT, *this);
 }
 
 MidiInReader::~MidiInReader()
 {
-	Scheduler::instance().removeSyncPoint(*this);
+	eventDistributor.unregisterEventListener(OPENMSX_MIDI_IN_READER_EVENT, *this);
 }
 
 // Pluggable
-void MidiInReader::plugHelper(Connector* connector_, const EmuTime& /*time*/)
+void MidiInReader::plugHelper(Connector& connector_, const EmuTime& /*time*/)
 {
 	file = fopen(readFilenameSetting->getValue().c_str(), "rb");
 	if (!file) {
@@ -33,13 +39,13 @@ void MidiInReader::plugHelper(Connector* connector_, const EmuTime& /*time*/)
 			+ string(strerror(errno)));
 	}
 
-	MidiInConnector* midiConnector = static_cast<MidiInConnector*>(connector_);
-	midiConnector->setDataBits(SerialDataInterface::DATA_8);	// 8 data bits
-	midiConnector->setStopBits(SerialDataInterface::STOP_1);	// 1 stop bit
-	midiConnector->setParityBit(false, SerialDataInterface::EVEN); // no parity
+	MidiInConnector& midiConnector = static_cast<MidiInConnector&>(connector_);
+	midiConnector.setDataBits(SerialDataInterface::DATA_8); // 8 data bits
+	midiConnector.setStopBits(SerialDataInterface::STOP_1); // 1 stop bit
+	midiConnector.setParityBit(false, SerialDataInterface::EVEN); // no parity
 
-	connector = connector_; // base class will do this in a moment,
-	                        // but thread already needs it
+	connector = &connector_; // base class will do this in a moment,
+	                         // but thread already needs it
 	thread.start();
 }
 
@@ -76,10 +82,11 @@ void MidiInReader::run()
 			continue;
 		}
 		assert(getConnector());
-		
+
 		ScopedLock l(lock);
 		queue.push_back(buf);
-		Scheduler::instance().setSyncPoint(Scheduler::ASAP, *this);
+		eventDistributor.distributeEvent(
+			new SimpleEvent<OPENMSX_MIDI_IN_READER_EVENT>());
 	}
 }
 
@@ -94,7 +101,7 @@ void MidiInReader::signal(const EmuTime& time)
 	if (!connector->ready()) {
 		return;
 	}
-	
+
 	ScopedLock l(lock);
 	if (queue.empty()) return;
 	byte data = queue.front();
@@ -102,20 +109,15 @@ void MidiInReader::signal(const EmuTime& time)
 	connector->recvByte(data, time);
 }
 
-// Schedulable
-void MidiInReader::executeUntil(const EmuTime& time, int /*userData*/)
+// EventListener
+void MidiInReader::signalEvent(const Event& /*event*/)
 {
 	if (getConnector()) {
-		signal(time);
+		signal(scheduler.getCurrentTime());
 	} else {
 		ScopedLock l(lock);
 		queue.empty();
 	}
 }
 
-const string& MidiInReader::schedName() const
-{
-	return getName();
-}
-
 } // namespace openmsx
