@@ -1,4 +1,4 @@
-// $Id: MidiOutConnector.cc,v 1.10 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: MidiOutConnector.cc,v 1.12 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "MidiOutConnector.hh"
 #include "DummyMidiOutDevice.hh"
@@ -8,15 +8,17 @@ using std::string;
 
 namespace openmsx {
 
-MidiOutConnector::MidiOutConnector(const string &name)
+MidiOutConnector::MidiOutConnector(PluggingController& pluggingController_,
+                                   const string &name)
 	: Connector(name, std::auto_ptr<Pluggable>(new DummyMidiOutDevice()))
+	, pluggingController(pluggingController_)
 {
-	PluggingController::instance().registerConnector(this);
+	pluggingController.registerConnector(*this);
 }
 
 MidiOutConnector::~MidiOutConnector()
 {
-	PluggingController::instance().unregisterConnector(this);
+	pluggingController.unregisterConnector(*this);
 }
 
 const string& MidiOutConnector::getDescription() const
