@@ -1,18 +1,16 @@
-// $Id: MidiOutDevice.cc,v 1.7 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: MidiOutDevice.cc,v 1.8 2005/11/15 20:03:46 m9710797 Exp $
 
 #include "MidiOutDevice.hh"
 
-using std::string;
-
 namespace openmsx {
 
 MidiOutDevice::~MidiOutDevice()
 {
 }
 
-const string& MidiOutDevice::getClass() const
+const std::string& MidiOutDevice::getClass() const
 {
-	static const string className("midi out");
+	static const std::string className("midi out");
 	return className;
 }
 
