@@ -1,24 +1,19 @@
-// $Id: MidiOutLogger.cc,v 1.16 2005/01/13 18:25:41 m9710797 Exp $
+// $Id: MidiOutLogger.cc,v 1.18 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "MidiOutLogger.hh"
 #include "FilenameSetting.hh"
 
-using std::string;
-
 namespace openmsx {
 
-MidiOutLogger::MidiOutLogger()
-	: logFilenameSetting(new FilenameSetting("midi-out-logfilename",
+MidiOutLogger::MidiOutLogger(CommandController& commandController)
+	: logFilenameSetting(new FilenameSetting(
+		commandController, "midi-out-logfilename",
 		"filename of the file where the MIDI output is logged to",
 		"/dev/midi"))
 {
 }
 
-MidiOutLogger::~MidiOutLogger()
-{
-}
-
-void MidiOutLogger::plugHelper(Connector* /*connector*/,
+void MidiOutLogger::plugHelper(Connector& /*connector*/,
                                const EmuTime& /*time*/)
 {
 	file.open(logFilenameSetting->getValue().c_str());
@@ -33,15 +28,15 @@ void MidiOutLogger::unplugHelper(const EmuTime& /*time*/)
 	file.close();
 }
 
-const string& MidiOutLogger::getName() const
+const std::string& MidiOutLogger::getName() const
 {
-	static const string name("midi-out-logger");
+	static const std::string name("midi-out-logger");
 	return name;
 }
 
-const string& MidiOutLogger::getDescription() const
+const std::string& MidiOutLogger::getDescription() const
 {
-	static const string desc(
+	static const std::string desc(
 		"Midi output logger. Log all data that is sent to this "
 		"pluggable to a file. The filename is set with the "
 		"'midi-out-logfilename' setting.");
