@@ -1,4 +1,4 @@
-// $Id: MidiOutNative.cc,v 1.7 2004/12/15 18:01:04 m9710797 Exp $
+// $Id: MidiOutNative.cc,v 1.9 2005/11/11 20:53:06 m9710797 Exp $
 
 #if defined(_WIN32)
 
@@ -10,12 +10,12 @@ using std::string;
 
 namespace openmsx {
 
-void MidiOutNative::registerAll(PluggingController* controller)
+void MidiOutNative::registerAll(PluggingController& controller)
 {
 	w32_midiOutInit();
 	unsigned devnum = w32_midiOutGetVFNsNum();
 	for (unsigned i = 0; i < devnum; ++i) {
-		controller->registerPluggable(new MidiOutNative(i));
+		controller.registerPluggable(new MidiOutNative(i));
 	}
 }
 
@@ -31,7 +31,7 @@ MidiOutNative::~MidiOutNative()
 	//w32_midiOutClean(); // TODO
 }
 
-void MidiOutNative::plugHelper(Connector* connector, const EmuTime& time)
+void MidiOutNative::plugHelper(Connector& connector, const EmuTime& time)
 {
 	devidx = w32_midiOutOpen(name.c_str());
 	if (devidx == (unsigned)-1) {
