@@ -1,14 +1,12 @@
-// $Id: RS232Device.cc,v 1.4 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: RS232Device.cc,v 1.5 2005/11/12 21:53:21 m9710797 Exp $
 
 #include "RS232Device.hh"
 
-using std::string;
-
 namespace openmsx {
 
-const string& RS232Device::getClass() const
+const std::string& RS232Device::getClass() const
 {
-	static const string className("RS232");
+	static const std::string className("RS232");
 	return className;
 }
 
