@@ -1,4 +1,4 @@
-// $Id: BooleanSetting.cc,v 1.10 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: BooleanSetting.cc,v 1.11 2005/09/03 21:28:15 m9710797 Exp $
 
 #include "BooleanSetting.hh"
 
@@ -6,9 +6,11 @@ using std::string;
 
 namespace openmsx {
 
-BooleanSetting::BooleanSetting(const string& name, const string& description,
-                               bool initialValue, SaveSetting save)
-	: EnumSetting<bool>(name, description, initialValue, getMap(), save)
+BooleanSetting::BooleanSetting(
+		CommandController& commandController, const string& name,
+		const string& description, bool initialValue, SaveSetting save)
+	: EnumSetting<bool>(commandController, name, description, initialValue,
+	                    getMap(), save)
 {
 }
 
