@@ -1,4 +1,4 @@
-// $Id: FloatSetting.cc,v 1.9 2004/12/15 09:09:49 m9710797 Exp $
+// $Id: FloatSetting.cc,v 1.11 2005/12/08 21:06:36 m9710797 Exp $
 
 #include "FloatSetting.hh"
 #include "CommandException.hh"
@@ -12,8 +12,9 @@ namespace openmsx {
 
 // class FloatSettingPolicy
 
-FloatSettingPolicy::FloatSettingPolicy(double minValue, double maxValue)
-	: SettingRangePolicy<double>(minValue, maxValue)
+FloatSettingPolicy::FloatSettingPolicy(CommandController& commandController,
+                                       double minValue, double maxValue)
+	: SettingRangePolicy<double>(commandController, minValue, maxValue)
 {
 }
 
@@ -34,13 +35,20 @@ double FloatSettingPolicy::fromString(const string& str) const
 	return result;
 }
 
+string FloatSettingPolicy::getTypeString() const
+{
+	return "float";
+}
+
 
 //class FloatSetting
 
-FloatSetting::FloatSetting(const string& name, const string& description,
+FloatSetting::FloatSetting(CommandController& commandController,
+                           const string& name, const string& description,
                            double initialValue, double minValue, double maxValue)
-	: SettingImpl<FloatSettingPolicy>(name, description, initialValue,
-	                                  Setting::SAVE, minValue, maxValue)
+	: SettingImpl<FloatSettingPolicy>(
+		commandController, name, description, initialValue,
+		Setting::SAVE, minValue, maxValue)
 {
 }
 
