@@ -1,4 +1,4 @@
-// $Id: AY8910.cc,v 1.30 2005/04/02 21:25:37 mthuurne Exp $
+// $Id: AY8910.cc,v 1.39 2006/05/22 21:14:15 m9710797 Exp $
 
 /*
  * Emulation of the AY-3-8910
@@ -11,17 +11,29 @@
  */
 
 #include "AY8910.hh"
-#include "Mixer.hh"
-#include "Debugger.hh"
-#include "Scheduler.hh"
 #include "AY8910Periphery.hh"
+#include "Mixer.hh"
+#include "MSXMotherBoard.hh"
+#include "SimpleDebuggable.hh"
+#include "XMLElement.hh"
+#include "MSXException.hh"
+#include "StringOp.hh"
 #include <cassert>
 
 using std::string;
 
 namespace openmsx {
 
-static const bool FORCE_PORTA_INPUT = true;
+class AY8910Debuggable : public SimpleDebuggable
+{
+public:
+	AY8910Debuggable(MSXMotherBoard& motherBoard, AY8910& ay8910);
+	virtual byte read(unsigned address, const EmuTime& time);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	AY8910& ay8910;
+};
+
 
 // Fixed point representation of 1.
 static const int FP_UNIT = 0x8000;
@@ -191,8 +203,17 @@ inline void AY8910::NoiseGenerator::advance(int duration)
 
 // Amplitude:
 
-AY8910::Amplitude::Amplitude()
+AY8910::Amplitude::Amplitude(const XMLElement& config)
 {
+	string type = StringOp::toLower(config.getChildData("type", "ay8910"));
+	if (type == "ay8910") {
+		ay8910 = true;
+	} else if (type == "ym2149") {
+		ay8910 = false;
+	} else {
+		throw FatalError("Unknown PSG type: " + type);
+	}
+
 	vol[0] = vol[1] = vol[2] = 0;
 	envChan[0] = envChan[1] = envChan[2] = false;
 	envVolume = 0;
@@ -212,7 +233,7 @@ inline void AY8910::Amplitude::setChannelVolume(byte chan, byte value)
 
 inline void AY8910::Amplitude::setEnvelopeVolume(byte volume)
 {
-	envVolume = volTable[volume];
+	envVolume = envVolTable[volume];
 	if (envChan[0]) vol[0] = envVolume;
 	if (envChan[1]) vol[1] = envVolume;
 	if (envChan[2]) vol[2] = envVolume;
@@ -222,12 +243,26 @@ inline void AY8910::Amplitude::setMasterVolume(int volume)
 {
 	// Calculate the volume->voltage conversion table.
 	// The AY-3-8910 has 16 levels, in a logarithmic scale (3dB per step).
+	// YM2149 has 32 levels, the 16 extra levels are only used for envelope
+	// volumes
+
 	double out = volume; // avoid clipping
-	for (int i = 15; i > 0; --i) {
-		volTable[i] = (unsigned)(out + 0.5);	// round to nearest
-		out *= 0.707945784384;			// 1/(10^(3/20)) = 1/(3dB)
+	for (int i = 31; i > 0; --i) {
+		envVolTable[i] = (unsigned)(out + 0.5); // round to nearest;
+		out *= 0.841395141645195;               // 1/(10^(1.5/20)) = 1/(1.5dB)
 	}
+	envVolTable[0] = 0;
 	volTable[0] = 0;
+	for (int i = 1; i < 16; ++i) {
+		volTable[i] = envVolTable[2 * i + 1];
+	}
+	if (ay8910) {
+		// only 16 envelope steps, duplicate every step
+		envVolTable[1] = 0;
+		for (int i = 2; i < 32; i += 2) {
+			envVolTable[i] = envVolTable[i + 1];
+		}
+	}
 }
 
 inline bool AY8910::Amplitude::anyEnvelope()
@@ -238,6 +273,11 @@ inline bool AY8910::Amplitude::anyEnvelope()
 
 // Envelope:
 
+// AY8910 and YM2149 behave different here:
+//  YM2149 envelope goes twice as fast and has twice as many levels. Here
+//  we implement the YM2149 behaviour, but to get the AY8910 behaviour we
+//  repeat every level twice in the envVolTable
+
 inline AY8910::Envelope::Envelope(Amplitude& amplitude)
 	: amplitude(amplitude)
 {
@@ -253,11 +293,12 @@ inline void AY8910::Envelope::reset()
 
 inline void AY8910::Envelope::setPeriod(int value, unsigned int updateStep)
 {
+	// twice as fast as AY8910
 	const int old = period;
 	if (value == 0) {
-		period = updateStep;
+		period = updateStep / 2;
 	} else {
-		period = value * 2 * updateStep;
+		period = value * updateStep;
 	}
 	count += period - old;
 	if (count <= 0) count = 1;
@@ -270,6 +311,7 @@ inline byte AY8910::Envelope::getVolume()
 
 inline void AY8910::Envelope::setShape(byte shape)
 {
+	// do 32 steps for both AY8910 and YM2149
 	/*
 	envelope shapes:
 		C AtAlH
@@ -284,7 +326,7 @@ inline void AY8910::Envelope::setShape(byte shape)
 		1 1 1 0  /\/\
 		1 1 1 1  /___
 	*/
-	attack = (shape & 0x04) ? 0x0F : 0x00;
+	attack = (shape & 0x04) ? 0x1F : 0x00;
 	if ((shape & 0x08) == 0) {
 		// If Continue = 0, map the shape to the equivalent one
 		// which has Continue = 1.
@@ -295,7 +337,7 @@ inline void AY8910::Envelope::setShape(byte shape)
 		alternate = shape & 0x02;
 	}
 	count = period;
-	step = 0x0F;
+	step = 0x1F;
 	holding = false;
 	amplitude.setEnvelopeVolume(getVolume());
 }
@@ -317,16 +359,16 @@ inline void AY8910::Envelope::advance(int duration)
 			// Check current envelope position.
 			if (step < 0) {
 				if (hold) {
-					if (alternate) attack ^= 0x0f;
+					if (alternate) attack ^= 0x1F;
 					holding = true;
 					step = 0;
 				} else {
 					// If step has looped an odd number of times
 					// (usually 1), invert the output.
 					if (alternate && (step & 0x10)) {
-						attack ^= 0x0F;
+						attack ^= 0x1F;
 					}
-					step &= 0x0F;
+					step &= 0x1F;
 				}
 			}
 			amplitude.setEnvelopeVolume(getVolume());
@@ -337,10 +379,13 @@ inline void AY8910::Envelope::advance(int duration)
 
 // AY8910 main class:
 
-AY8910::AY8910(AY8910Periphery& periphery_, const XMLElement& config,
-               const EmuTime& time)
-	: periphery(periphery_)
+AY8910::AY8910(MSXMotherBoard& motherBoard, AY8910Periphery& periphery_,
+               const XMLElement& config, const EmuTime& time)
+	: SoundDevice(motherBoard.getMixer(), "PSG", "PSG")
+	, periphery(periphery_)
+	, amplitude(config)
 	, envelope(amplitude)
+	, debuggable(new AY8910Debuggable(motherBoard, *this))
 {
 	// make valgrind happy
 	memset(regs, 0, sizeof(regs));
@@ -348,27 +393,13 @@ AY8910::AY8910(AY8910Periphery& periphery_, const XMLElement& config,
 
 	reset(time);
 	registerSound(config);
-	Debugger::instance().registerDebuggable(getName() + " regs", *this);
 }
 
 AY8910::~AY8910()
 {
-	Debugger::instance().unregisterDebuggable(getName() + " regs", *this);
 	unregisterSound();
 }
 
-const string& AY8910::getName() const
-{
-	static const string name("PSG");
-	return name;
-}
-
-const string& AY8910::getDescription() const
-{
-	static const string desc("PSG");
-	return desc;
-}
-
 void AY8910::reset(const EmuTime& time)
 {
 	// Reset generators and envelope.
@@ -390,8 +421,7 @@ byte AY8910::readRegister(byte reg, const EmuTime& time)
 	assert(reg <= 15);
 	switch (reg) {
 	case AY_PORTA:
-		if (FORCE_PORTA_INPUT ||
-		    !(regs[AY_ENABLE] & PORT_A_DIRECTION)) { //input
+		if (!(regs[AY_ENABLE] & PORT_A_DIRECTION)) { //input
 			regs[reg] = periphery.readA(time);
 		}
 		break;
@@ -409,8 +439,7 @@ byte AY8910::peekRegister(byte reg, const EmuTime& time) const
 	assert(reg <= 15);
 	switch (reg) {
 	case AY_PORTA:
-		if (FORCE_PORTA_INPUT ||
-		    !(regs[AY_ENABLE] & PORT_A_DIRECTION)) { //input
+		if (!(regs[AY_ENABLE] & PORT_A_DIRECTION)) { //input
 			return periphery.readA(time);
 		}
 		break;
@@ -429,14 +458,21 @@ void AY8910::writeRegister(byte reg, byte value, const EmuTime& time)
 	assert(reg <= 15);
 	if ((reg < AY_PORTA) && (reg == AY_ESHAPE || regs[reg] != value)) {
 		// Update the output buffer before changing the register.
-		Mixer::instance().updateStream(time);
+		getMixer().updateStream(time);
 	}
-	Mixer::instance().lock();
+	getMixer().lock();
 	wrtReg(reg, value, time);
-	Mixer::instance().unlock();
+	getMixer().unlock();
 }
 void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 {
+	// Force port directions
+	if (reg == AY_ENABLE) {
+		// portA -> input
+		// portB -> output
+		value = (value & ~PORT_A_DIRECTION) | PORT_B_DIRECTION;
+	}
+
 	// Note: unused bits are stored as well; they can be read back.
 	regs[reg] = value;
 
@@ -468,8 +504,7 @@ void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 		envelope.setShape(value);
 		break;
 	case AY_ENABLE:
-		if (!FORCE_PORTA_INPUT &&
-		    (value      & PORT_A_DIRECTION) &&
+		if ((value      & PORT_A_DIRECTION) &&
 		    !(oldEnable & PORT_A_DIRECTION)) {
 			// Changed from input to output.
 			periphery.writeA(regs[AY_PORTA], time);
@@ -483,8 +518,7 @@ void AY8910::wrtReg(byte reg, byte value, const EmuTime& time)
 		checkMute();
 		break;
 	case AY_PORTA:
-		if (!FORCE_PORTA_INPUT &&
-		    regs[AY_ENABLE] & PORT_A_DIRECTION) { // output
+		if (regs[AY_ENABLE] & PORT_A_DIRECTION) { // output
 			periphery.writeA(value, time);
 		}
 		break;
@@ -522,7 +556,8 @@ void AY8910::setSampleRate(int sampleRate)
 }
 
 
-void AY8910::updateBuffer(int length, int* buffer)
+void AY8910::updateBuffer(unsigned length, int* buffer,
+     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
 {
 	/*
 	static long long totalSamples = 0, noiseOff = 0, toneOff = 0, bothOff = 0;
@@ -649,22 +684,23 @@ void AY8910::updateBuffer(int length, int* buffer)
 }
 
 
-// Debuggable
+// SimpleDebuggable
 
-unsigned AY8910::getSize() const
+AY8910Debuggable::AY8910Debuggable(MSXMotherBoard& motherBoard, AY8910& ay8910_)
+	: SimpleDebuggable(motherBoard, ay8910_.getName() + " regs",
+	                   "PSG", 0x10)
+	, ay8910(ay8910_)
 {
-	return 0x10;
 }
 
-byte AY8910::read(unsigned address)
+byte AY8910Debuggable::read(unsigned address, const EmuTime& time)
 {
-	return readRegister(address, Scheduler::instance().getCurrentTime());
+	return ay8910.readRegister(address, time);
 }
 
-void AY8910::write(unsigned address, byte value)
+void AY8910Debuggable::write(unsigned address, byte value, const EmuTime& time)
 {
-	return writeRegister(address, value,
-	                     Scheduler::instance().getCurrentTime());
+	return ay8910.writeRegister(address, value, time);
 }
 
 } // namespace openmsx
