@@ -1,4 +1,4 @@
-// $Id: DirectXSoundDriver.cc,v 1.16 2005/05/14 14:57:00 h_oudejans Exp $
+// $Id: DirectXSoundDriver.cc,v 1.30 2006/01/23 20:45:40 m9710797 Exp $
 
 #ifdef _WIN32
 
@@ -6,6 +6,7 @@
 #include "GlobalSettings.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
+#include "ThrottleManager.hh"
 #include "Mixer.hh"
 #include "Scheduler.hh"
 #include "MSXException.hh"
@@ -24,6 +25,11 @@ static const int CHANNELS = 2;
 static HWND getWindowHandle()
 {
 	// This is SDL specific code, refactor when needed
+
+	// !! Initialize video system, DirectX needs a handle to the window
+	// !! and this only works when SDL video part is initialized
+	if (!SDL_WasInit(SDL_INIT_VIDEO)) SDL_InitSubSystem(SDL_INIT_VIDEO);
+
 	SDL_SysWMinfo info;
 	SDL_VERSION(&info.version);
 	if (!SDL_GetWMInfo(&info)) {
@@ -32,11 +38,14 @@ static HWND getWindowHandle()
 	return info.window;
 }
 
-DirectXSoundDriver::DirectXSoundDriver(Mixer& mixer_,
+DirectXSoundDriver::DirectXSoundDriver(Scheduler& scheduler,
+		GlobalSettings& globalSettings, Mixer& mixer_,
 		unsigned sampleRate, unsigned samples)
-	: mixer(mixer_)
-	, speedSetting(GlobalSettings::instance().getSpeedSetting())
-	, throttleSetting(GlobalSettings::instance().getThrottleSetting())
+	: Schedulable(scheduler)
+	, mixer(mixer_)
+	, prevTime(EmuTime::zero)
+	, speedSetting(globalSettings.getSpeedSetting())
+	, throttleManager(globalSettings.getThrottleManager())
 {
 	if (DirectSoundCreate(NULL, &directSound, NULL) != DS_OK) {
 		throw MSXException("Couldn't initialize DirectSound driver");
@@ -128,20 +137,19 @@ DirectXSoundDriver::DirectXSoundDriver(Mixer& mixer_,
 	mixBuffer = new short[bufferSize / BYTES_PER_SAMPLE];
 
 	reInit();
-	prevTime = Scheduler::instance().getCurrentTime();
+	prevTime = scheduler.getCurrentTime();
 	EmuDuration interval2 = interval1 * fragmentSize;
-	Scheduler::instance().setSyncPoint(prevTime + interval2, *this);
+	setSyncPoint(prevTime + interval2);
 
-	speedSetting.addListener(this);
-	throttleSetting.addListener(this);
+	speedSetting.attach(*this);
+	throttleManager.attach(*this);
 }
 
 DirectXSoundDriver::~DirectXSoundDriver()
 {
-	throttleSetting.removeListener(this);
-	speedSetting.removeListener(this);
+	throttleManager.detach(*this);
+	speedSetting.detach(*this);
 
-	Scheduler::instance().removeSyncPoint(*this);
 	delete[] mixBuffer;
 
 	IDirectSoundBuffer_Stop(primaryBuffer);
@@ -284,11 +292,12 @@ void DirectXSoundDriver::updateStream(const EmuTime& time)
 	if (count == 0) {
 		return;
 	}
-	prevTime += interval1 * count;
 
 	count = std::min(count, bufferSize / BYTES_PER_SAMPLE / CHANNELS);
-	mixer.generate(mixBuffer, count);
+	mixer.generate(mixBuffer, count, prevTime, interval1);
 	dxWrite(mixBuffer, count * CHANNELS);
+
+	prevTime += interval1 * count;
 }
 
 void DirectXSoundDriver::reInit()
@@ -308,7 +317,7 @@ void DirectXSoundDriver::executeUntil(const EmuTime& time, int /*userData*/)
 		updateStream(time);
 	}
 	EmuDuration interval2 = interval1 * fragmentSize;
-	Scheduler::instance().setSyncPoint(time + interval2, *this);
+	setSyncPoint(time + interval2);
 }
 
 const string& DirectXSoundDriver::schedName() const
@@ -318,19 +327,18 @@ const string& DirectXSoundDriver::schedName() const
 }
 
 
-// SettingListener
+// Observer<Setting>
 
-void DirectXSoundDriver::update(const Setting* setting)
+void DirectXSoundDriver::update(const Setting& /*setting*/)
 {
-	if (setting == &speedSetting) {
-		reInit();
-	} else if (setting == &throttleSetting) {
-		if (throttleSetting.getValue()) {
-			reInit();
-		}
-	} else {
-		assert(false);
-	}
+	reInit();
+}
+
+// Observer<ThrottleManager>
+
+void DirectXSoundDriver::update(const ThrottleManager& /*throttleManager*/)
+{
+	reInit();
 }
 
 } // namespace openmsx
