@@ -1,22 +1,16 @@
-// $Id: DummyAudioInputDevice.cc,v 1.8 2004/12/15 09:09:50 m9710797 Exp $
+// $Id: DummyAudioInputDevice.cc,v 1.9 2005/11/11 20:53:06 m9710797 Exp $
 
 #include "DummyAudioInputDevice.hh"
 
-using std::string;
-
 namespace openmsx {
 
-DummyAudioInputDevice::DummyAudioInputDevice()
+const std::string& DummyAudioInputDevice::getDescription() const
 {
-}
-
-const string& DummyAudioInputDevice::getDescription() const
-{
-	static const string EMPTY;
+	static const std::string EMPTY;
 	return EMPTY;
 }
 
-void DummyAudioInputDevice::plugHelper(Connector* /*connector*/,
+void DummyAudioInputDevice::plugHelper(Connector& /*connector*/,
                                        const EmuTime& /*time*/)
 {
 }
@@ -27,7 +21,7 @@ void DummyAudioInputDevice::unplugHelper(const EmuTime& /*time*/)
 
 short DummyAudioInputDevice::readSample(const EmuTime& /*time*/)
 {
-	return 0;	// silence
+	return 0; // silence
 }
 
 } // namespace openmsx
