@@ -1,4 +1,4 @@
-// $Id: MSXMusic.cc,v 1.33 2005/04/09 07:12:19 manuelbi Exp $
+// $Id: MSXMusic.cc,v 1.37 2005/12/03 18:58:10 arnoldmnl Exp $
 
 #include "MSXMusic.hh"
 #include "YM2413.hh"
@@ -8,14 +8,17 @@
 
 namespace openmsx {
 
-MSXMusic::MSXMusic(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, rom(new Rom(getName() + " ROM", "rom", config))
+MSXMusic::MSXMusic(MSXMotherBoard& motherBoard, const XMLElement& config,
+                   const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, rom(new Rom(motherBoard, getName() + " ROM", "rom", config))
 {
 	if (config.getChildDataAsBool("alternative", false)) {
-		ym2413.reset(new YM2413_2(getName(), config, time));
+		ym2413.reset(new YM2413_2(motherBoard, getName(),
+		                          config, time));
 	} else {
-		ym2413.reset(new YM2413(getName(), config, time));
+		ym2413.reset(new YM2413(motherBoard, getName(),
+		                        config, time));
 	}
 	reset(time);
 }
@@ -31,7 +34,7 @@ void MSXMusic::reset(const EmuTime& time)
 }
 
 
-void MSXMusic::writeIO(byte port, byte value, const EmuTime& time)
+void MSXMusic::writeIO(word port, byte value, const EmuTime& time)
 {
 	switch (port & 0x01) {
 		case 0:
