@@ -1,9 +1,11 @@
-// $Id: MSXPSG.cc,v 1.41 2005/04/02 21:25:37 mthuurne Exp $
+// $Id: MSXPSG.cc,v 1.50 2005/12/27 11:03:51 m9710797 Exp $
 
 #include "MSXPSG.hh"
+#include "AY8910.hh"
 #include "LedEvent.hh"
 #include "EventDistributor.hh"
 #include "CassettePort.hh"
+#include "MSXMotherBoard.hh"
 #include "JoystickPort.hh"
 #include "XMLElement.hh"
 #include "RenShaTurbo.hh"
@@ -11,18 +13,20 @@
 namespace openmsx {
 
 // MSXDevice
-MSXPSG::MSXPSG(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, cassette(CassettePortFactory::instance())
+MSXPSG::MSXPSG(MSXMotherBoard& motherBoard, const XMLElement& config,
+               const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, cassette(motherBoard.getCassettePort())
+	, renShaTurbo(motherBoard.getRenShaTurbo())
 	, prev(255)
 {
 	keyLayoutBit = deviceConfig.getChildData("keyboardlayout", "") == "JIS";
-	ay8910.reset(new AY8910(*this, config, time));
-
 	selectedPort = 0;
-	ports[0].reset(new JoystickPort("joyporta"));
-	ports[1].reset(new JoystickPort("joyportb"));
+	PluggingController& controller = motherBoard.getPluggingController();
+	ports[0].reset(new JoystickPort(controller, "joyporta"));
+	ports[1].reset(new JoystickPort(controller, "joyportb"));
 
+	ay8910.reset(new AY8910(motherBoard, *this, config, time));
 	reset(time);
 }
 
@@ -38,23 +42,23 @@ void MSXPSG::reset(const EmuTime& time)
 
 void MSXPSG::powerDown(const EmuTime& /*time*/)
 {
-	EventDistributor::instance().distributeEvent(
+	getMotherBoard().getEventDistributor().distributeEvent(
 		new LedEvent(LedEvent::KANA, false));
 }
 
-byte MSXPSG::readIO(byte /*port*/, const EmuTime& time)
+byte MSXPSG::readIO(word /*port*/, const EmuTime& time)
 {
 	byte result = ay8910->readRegister(registerLatch, time);
 	//PRT_DEBUG("PSG read R#"<<(int)registerLatch<<" = "<<(int)result);
 	return result;
 }
 
-byte MSXPSG::peekIO(byte /*port*/, const EmuTime& time) const
+byte MSXPSG::peekIO(word /*port*/, const EmuTime& time) const
 {
 	return ay8910->peekRegister(registerLatch, time);
 }
 
-void MSXPSG::writeIO(byte port, byte value, const EmuTime& time)
+void MSXPSG::writeIO(word port, byte value, const EmuTime& time)
 {
 	switch (port & 0x03) {
 	case 0:
@@ -72,7 +76,7 @@ void MSXPSG::writeIO(byte port, byte value, const EmuTime& time)
 byte MSXPSG::readA(const EmuTime& time)
 {
 	byte joystick = ports[selectedPort]->read(time) |
-	                ((RenShaTurbo::instance().getSignal(time)) << 4);
+	                ((renShaTurbo.getSignal(time)) << 4);
 	byte cassetteInput = cassette.cassetteIn(time) ? 0x80 : 0x00;
 	byte keyLayout = keyLayoutBit ? 0x40 : 0x00;
 	return joystick | keyLayout | cassetteInput;
@@ -87,7 +91,7 @@ void MSXPSG::writeB(byte value, const EmuTime& time)
 	selectedPort = (value & 0x40) >> 6;
 
 	if ((prev ^ value) & 0x80) {
-		EventDistributor::instance().distributeEvent(
+		getMotherBoard().getEventDistributor().distributeEvent(
 			new LedEvent(LedEvent::KANA, !(value & 0x80)));
 	}
 	prev = value;
