@@ -1,4 +1,4 @@
-// $Id: MSXSCCPlusCart.cc,v 1.50 2005/01/15 12:55:45 m9710797 Exp $
+// $Id: MSXSCCPlusCart.cc,v 1.56 2006/01/25 19:26:32 m9710797 Exp $
 
 // Note: this device is actually called SCC-II. But this would take a lot of
 // renaming, which isn't worth it right now. TODO rename this :)
@@ -10,30 +10,30 @@
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "MSXCPU.hh"
+#include "MSXMotherBoard.hh"
 #include "CPU.hh"
 #include "XMLElement.hh"
 
-using std::string;
-
 namespace openmsx {
 
-MSXSCCPlusCart::MSXSCCPlusCart(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, ram(new Ram(getName() + " RAM", "SCC+ RAM", 0x20000))
+MSXSCCPlusCart::MSXSCCPlusCart(MSXMotherBoard& motherBoard,
+                               const XMLElement& config, const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, ram(new Ram(motherBoard, getName() + " RAM", "SCC+ RAM", 0x20000))
 {
 	const XMLElement* fileElem = config.findChild("filename");
 	if (fileElem) {
 		// read the rom file
-		const string& filename = fileElem->getData();
+		const std::string& filename = fileElem->getData();
 		try {
 			File file(config.getFileContext().resolve(filename));
 			int romSize = file.getSize();
 			file.read(&(*ram)[0], romSize);
 		} catch (FileException& e) {
-			throw FatalError("Error reading file: " + filename);
+			throw MSXException("Error reading file: " + filename);
 		}
 	}
-	const string subtype = config.getChildData("subtype", "expanded");
+	const std::string subtype = config.getChildData("subtype", "expanded");
 	if (subtype == "Snatcher") {
 		mapperMask = 0x0F;
 		lowRAM  = true;
@@ -57,7 +57,8 @@ MSXSCCPlusCart::MSXSCCPlusCart(const XMLElement& config, const EmuTime& time)
 		mapper[i] = 0;
 	}
 
-	scc.reset(new SCC(getName(), config, time, SCC::SCC_Compatible));
+	scc.reset(new SCC(motherBoard, getName(), config, time,
+	                  SCC::SCC_Compatible));
 
 	reset(time);
 }
@@ -117,24 +118,24 @@ const byte* MSXSCCPlusCart::getReadCacheLine(word start) const
 void MSXSCCPlusCart::writeMem(word address, byte value, const EmuTime& time)
 {
 	//PRT_DEBUG("SCC+ write "<< hex << address << " " << (int)value << dec);
-	
+
 	if ((address < 0x4000) || (0xC000 <= address)) {
 		// outside memory range
 		return;
 	}
-	
+
 	// Mode register is mapped upon 0xBFFE and 0xBFFF
 	if ((address | 0x0001) == 0xBFFF) {
 		setModeRegister(value);
 		return;
 	}
-	
+
 	// Write to RAM
 	int regio = (address >> 13) - 2;
 	if (isRamSegment[regio]) {
 		//According to Sean Young
-		// when the regio's are in RAM mode you can read from 
-		// the SCC(+) but not write to them 
+		// when the regio's are in RAM mode you can read from
+		// the SCC(+) but not write to them
 		// => we assume a write to the memory but maybe
 		//    they are just discarded
 		// TODO check this out => ask Sean...
@@ -155,7 +156,7 @@ void MSXSCCPlusCart::writeMem(word address, byte value, const EmuTime& time)
 		setMapper(regio, value);
 		return;
 	}
-	
+
 	// call writeMemInterface of SCC if needed
 	switch (enable) {
 	case EN_NONE:
@@ -195,7 +196,7 @@ void MSXSCCPlusCart::setMapper(int regio, byte value)
 {
 	mapper[regio] = value;
 	value &= mapperMask;
-	
+
 	byte* block;
 	if ((!lowRAM  && (value <  8)) ||
 	    (!highRAM && (value >= 8))) {
@@ -205,34 +206,36 @@ void MSXSCCPlusCart::setMapper(int regio, byte value)
 		block = &(*ram)[0x2000 * value];
 		isMapped[regio] = true;
 	}
-	
+
 	checkEnable();
 	internalMemoryBank[regio] = block;
-	MSXCPU::instance().invalidateMemCache(0x4000 + regio * 0x2000, 0x2000);
+	getMotherBoard().getCPU().invalidateMemCache(
+		0x4000 + regio * 0x2000, 0x2000);
 }
 
 void MSXSCCPlusCart::setModeRegister(byte value)
 {
 	modeRegister = value;
 	checkEnable();
-	
+
 	if (modeRegister & 0x20) {
 		scc->setChipMode(SCC::SCC_plusmode);
 	} else {
 		scc->setChipMode(SCC::SCC_Compatible);
 	}
-	
+
 	if (modeRegister & 0x10) {
 		isRamSegment[0] = true;
 		isRamSegment[1] = true;
 		isRamSegment[2] = true;
 		isRamSegment[3] = true;
 	} else {
+		MSXCPU& cpu = getMotherBoard().getCPU();
 		if (modeRegister & 0x01) {
 			isRamSegment[0] = true;
 		} else {
 			if (isRamSegment[0]) {
-				MSXCPU::instance().invalidateMemCache(0x4000, 0x2000);
+				cpu.invalidateMemCache(0x4000, 0x2000);
 				isRamSegment[0] = false;
 			}
 		}
@@ -240,7 +243,7 @@ void MSXSCCPlusCart::setModeRegister(byte value)
 			isRamSegment[1] = true;
 		} else {
 			if (isRamSegment[1]) {
-				MSXCPU::instance().invalidateMemCache(0x6000, 0x2000);
+				cpu.invalidateMemCache(0x6000, 0x2000);
 				isRamSegment[1] = false;
 			}
 		}
@@ -249,12 +252,12 @@ void MSXSCCPlusCart::setModeRegister(byte value)
 			isRamSegment[2] = true;
 		} else {
 			if (isRamSegment[2]) {
-				MSXCPU::instance().invalidateMemCache(0x8000, 0x2000);
+				cpu.invalidateMemCache(0x8000, 0x2000);
 				isRamSegment[2] = false;
 			}
 		}
 		if (isRamSegment[3]) {
-			MSXCPU::instance().invalidateMemCache(0xA000, 0x2000);
+			cpu.invalidateMemCache(0xA000, 0x2000);
 			isRamSegment[3] = false;
 		}
 	}
