@@ -1,27 +1,25 @@
-// $Id: Mixer.cc,v 1.106 2005/05/03 17:32:48 m9710797 Exp $
+// $Id: Mixer.cc,v 1.120 2006/01/21 10:51:59 m9710797 Exp $
 
 #include "Mixer.hh"
 #include "NullSoundDriver.hh"
 #include "SDLSoundDriver.hh"
 #include "DirectXSoundDriver.hh"
 #include "SoundDevice.hh"
+#include "WavWriter.hh"
 #include "CliComm.hh"
+#include "CommandController.hh"
+#include "Command.hh"
+#include "InfoTopic.hh"
 #include "InfoCommand.hh"
 #include "GlobalSettings.hh"
-#include "CommandArgument.hh"
-#include "CommandLineParser.hh"
+#include "TclObject.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "EnumSetting.hh"
+#include "FileOperations.hh"
 #include <algorithm>
 #include <cassert>
 
-#include <dirent.h>
-#include "ReadDir.hh"
-#include "FileOperations.hh"
-#include "FileException.hh"
-
-using std::map;
 using std::remove;
 using std::set;
 using std::string;
@@ -29,13 +27,40 @@ using std::vector;
 
 namespace openmsx {
 
-Mixer::Mixer()
+class SoundlogCommand : public SimpleCommand
+{
+public:
+	SoundlogCommand(CommandController& commandController, Mixer& mixer);
+	virtual std::string execute(const std::vector<std::string>& tokens);
+	virtual std::string help(const std::vector<std::string>& tokens) const;
+	virtual void tabCompletion(std::vector<std::string>& tokens) const;
+private:
+	std::string stopSoundLogging(const std::vector<std::string>& tokens);
+	std::string startSoundLogging(const std::vector<std::string>& tokens);
+	std::string toggleSoundLogging(const std::vector<std::string>& tokens);
+	Mixer& mixer;
+};
+
+class SoundDeviceInfoTopic : public InfoTopic
+{
+public:
+	SoundDeviceInfoTopic(CommandController& commandController, Mixer& mixer);
+	virtual void execute(const vector<TclObject*>& tokens,
+	                     TclObject& result) const;
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
+private:
+	Mixer& mixer;
+};
+
+
+Mixer::Mixer(Scheduler& scheduler_, CommandController& commandController_)
 	: muteCount(0)
-	, output(CliComm::instance())
-	, infoCommand(InfoCommand::instance())
-	, pauseSetting(GlobalSettings::instance().getPauseSetting())
-	, soundlogCommand(*this)
-	, soundDeviceInfo(*this)
+	, scheduler(scheduler_)
+	, commandController(commandController_)
+	, pauseSetting(commandController.getGlobalSettings().getPauseSetting())
+	, soundlogCommand(new SoundlogCommand(commandController, *this))
+	, soundDeviceInfo(new SoundDeviceInfoTopic(commandController, *this))
 {
 	driver.reset(new NullSoundDriver());
 	handlingUpdate = false;
@@ -49,17 +74,15 @@ Mixer::Mixer()
 	const int defaultsamples = 1024;
 #endif
 
-	muteSetting.reset(new BooleanSetting(
+	muteSetting.reset(new BooleanSetting(commandController,
 		"mute", "(un)mute the emulation sound", false,
 		Setting::DONT_SAVE));
-	masterVolume.reset(new IntegerSetting(
+	masterVolume.reset(new IntegerSetting(commandController,
 		"master_volume", "master volume", 75, 0, 100));
-	frequencySetting.reset(new IntegerSetting("frequency",
-		"mixer frequency",
-		44100, 11025, 48000));
-	samplesSetting.reset(new IntegerSetting("samples",
-		"mixer samples",
-		defaultsamples, 64, 8192));
+	frequencySetting.reset(new IntegerSetting(commandController,
+		"frequency", "mixer frequency", 44100, 11025, 48000));
+	samplesSetting.reset(new IntegerSetting(commandController,
+		"samples", "mixer samples", defaultsamples, 64, 8192));
 
 	EnumSetting<SoundDriverType>::Map soundDriverMap;
 	soundDriverMap["null"]    = SND_NULL;
@@ -72,22 +95,16 @@ Mixer::Mixer()
 #endif
 
 	soundDriverSetting.reset(new EnumSetting<SoundDriverType>(
-		"sound_driver", "select the sound output driver",
+		commandController, "sound_driver",
+		"select the sound output driver",
 		defaultSoundDriver, soundDriverMap));
-	if (!CommandLineParser::instance().wantSound()) {
-		CliComm::instance().printWarning(
-			"The option '-nosound' is deprecated. "
-			"Use sound_driver=null (see manual).");
-		soundDriverSetting->setValue(SND_NULL);
-	}
 
-	infoCommand.registerTopic("sounddevice", &soundDeviceInfo);
-	muteSetting->addListener(this);
-	masterVolume->addListener(this);
-	frequencySetting->addListener(this);
-	samplesSetting->addListener(this);
-	soundDriverSetting->addListener(this);
-	pauseSetting.addListener(this);
+	muteSetting->attach(*this);
+	masterVolume->attach(*this);
+	frequencySetting->attach(*this);
+	samplesSetting->attach(*this);
+	soundDriverSetting->attach(*this);
+	pauseSetting.attach(*this);
 
 	// Set correct initial mute state.
 	if (muteSetting->getValue()) ++muteCount;
@@ -95,33 +112,20 @@ Mixer::Mixer()
 
 	openSound();
 	muteHelper();
-
-	wavfp = NULL;
-	CommandController::instance().registerCommand(&soundlogCommand, "soundlog");
 }
 
 Mixer::~Mixer()
 {
 	assert(buffers.empty());
-	
-	CommandController::instance().unregisterCommand(&soundlogCommand, "soundlog");
 
-	endSoundLogging();
 	driver.reset();
-	
-	pauseSetting.removeListener(this);
-	soundDriverSetting->removeListener(this);
-	samplesSetting->removeListener(this);
-	frequencySetting->removeListener(this);
-	masterVolume->removeListener(this);
-	muteSetting->removeListener(this);
-	infoCommand.unregisterTopic("sounddevice", &soundDeviceInfo);
-}
 
-Mixer& Mixer::instance()
-{
-	static Mixer oneInstance;
-	return oneInstance;
+	pauseSetting.detach(*this);
+	soundDriverSetting->detach(*this);
+	samplesSetting->detach(*this);
+	frequencySetting->detach(*this);
+	masterVolume->detach(*this);
+	muteSetting->detach(*this);
 }
 
 
@@ -150,15 +154,17 @@ void Mixer::openSound()
 			driver.reset(new NullSoundDriver());
 			break;
 		case SND_SDL:
-			driver.reset(new SDLSoundDriver(*this,
-			                    frequencySetting->getValue(),
-			                    samplesSetting->getValue()));
+			driver.reset(new SDLSoundDriver(scheduler,
+				commandController.getGlobalSettings(), *this,
+				frequencySetting->getValue(),
+				samplesSetting->getValue()));
 			break;
 #ifdef _WIN32
 		case SND_DIRECTX:
-			driver.reset(new DirectXSoundDriver(*this,
-			                    frequencySetting->getValue(),
-			                    samplesSetting->getValue()));
+			driver.reset(new DirectXSoundDriver(scheduler,
+				commandController.getGlobalSettings(), *this,
+				frequencySetting->getValue(),
+				samplesSetting->getValue()));
 			break;
 #endif
 		default:
@@ -169,7 +175,7 @@ void Mixer::openSound()
 		//samplesSetting->setValue(driver->getSamples());
 		handlingUpdate = false;
 	} catch (MSXException& e) {
-		output.printWarning(e.getMessage());
+		commandController.getCliComm().printWarning(e.getMessage());
 	}
 }
 
@@ -178,7 +184,7 @@ void Mixer::registerSound(SoundDevice& device, short volume, ChannelMode mode)
 {
 	const string& name = device.getName();
 	SoundDeviceInfo info;
-	info.volumeSetting = new IntegerSetting(
+	info.volumeSetting = new IntegerSetting(commandController,
 		name + "_volume", "the volume of this sound chip", 75, 0, 100);
 
 	// once we're stereo, stay stereo. Once mono, stay mono.
@@ -196,15 +202,15 @@ void Mixer::registerSound(SoundDevice& device, short volume, ChannelMode mode)
 		modeMap["right"] = MONO_RIGHT;
 	}
 	modeMap["off"] = OFF;
-	info.modeSetting = new EnumSetting<ChannelMode>(
+	info.modeSetting = new EnumSetting<ChannelMode>(commandController,
 		name + "_mode", "the channel mode of this sound chip",
 		modeMap[defaultMode], modeMap, Setting::DONT_SAVE);
 	info.modeSetting->setValue(mode);
-	
+
 	info.mode = mode;
 	info.normalVolume = (volume * 100 * 100) / (75 * 75);
-	info.modeSetting->addListener(this);
-	info.volumeSetting->addListener(this);
+	info.modeSetting->attach(*this);
+	info.volumeSetting->attach(*this);
 	infos[&device] = info;
 
 	lock();
@@ -219,8 +225,7 @@ void Mixer::registerSound(SoundDevice& device, short volume, ChannelMode mode)
 
 void Mixer::unregisterSound(SoundDevice& device)
 {
-	map<SoundDevice*, SoundDeviceInfo>::iterator it=
-		infos.find(&device);
+	Infos::iterator it = infos.find(&device);
 	if (it == infos.end()) {
 		return;
 	}
@@ -230,9 +235,9 @@ void Mixer::unregisterSound(SoundDevice& device)
 	ChannelMode mode = it->second.mode;
 	vector<SoundDevice*> &dev = devices[mode];
 	dev.erase(remove(dev.begin(), dev.end(), &device), dev.end());
-	it->second.volumeSetting->removeListener(this);
+	it->second.volumeSetting->detach(*this);
 	delete it->second.volumeSetting;
-	it->second.modeSetting->removeListener(this);
+	it->second.modeSetting->detach(*this);
 	delete it->second.modeSetting;
 	infos.erase(it);
 
@@ -246,7 +251,8 @@ void Mixer::updateStream(const EmuTime& time)
 	driver->updateStream(time);
 }
 
-void Mixer::generate(short* buffer, unsigned samples)
+void Mixer::generate(short* buffer, unsigned samples,
+	const EmuTime& start, const EmuDuration& sampDur)
 {
 	int modeOffset[NB_MODES];
 	int unmuted = 0;
@@ -256,7 +262,8 @@ void Mixer::generate(short* buffer, unsigned samples)
 		           devices[mode].begin();
 		     it != devices[mode].end(); ++it) {
 			if (!(*it)->isMuted()) {
-				(*it)->updateBuffer(samples, buffers[unmuted++]);
+				(*it)->updateBuffer(samples, buffers[unmuted++],
+				                    start, sampDur);
 			}
 		}
 	}
@@ -279,9 +286,9 @@ void Mixer::generate(short* buffer, unsigned samples)
 			left  += buffers[buf]  [2*j+0];
 			right += buffers[buf++][2*j+1];
 		}
-		
+
 		// DC removal filter
-		//   y(n) = x(n) - x(n-1) + R * y(n-1) 
+		//   y(n) = x(n) - x(n-1) + R * y(n-1)
 		//   R = 1 - (pi*2 * cut-off-frequency / samplerate)
 		// take R = 1022/1024
 		//   44100Hz --> cutt-off freq = 14Hz
@@ -290,7 +297,7 @@ void Mixer::generate(short* buffer, unsigned samples)
 		prevLeft  =  left;
 		outRight  = right - prevRight + ((1022 * outRight) >> 10);
 		prevRight = right;
-		
+
 		// clip
 		if      (outLeft  > 32767)  outLeft  =  32767;
 		else if (outLeft  < -32768) outLeft  = -32768;
@@ -299,10 +306,8 @@ void Mixer::generate(short* buffer, unsigned samples)
 
 		buffer[2 * j + 0] = static_cast<short>(outLeft);
 		buffer[2 * j + 1] = static_cast<short>(outRight);
-		if (wavfp) {
-			// TODO check write error
-			fwrite(&buffer[2 * j], 4, 1, wavfp);
-			nofWavBytes += 4;
+		if (wavWriter.get()) {
+			wavWriter->write16stereo(outLeft, outRight);
 		}
 	}
 }
@@ -339,55 +344,14 @@ void Mixer::muteHelper()
 
 // stuff for soundlogging
 
-// TODO: generalize this with ScreenShotSaver.cc
-static int getNum(dirent* d)
-{
-	string name(d->d_name);
-	if ((name.length() != 15) ||
-	    (name.substr(0, 7) != "openmsx") ||
-	    (name.substr(11, 4) != ".wav")) {
-		return 0;
-	}
-	string num(name.substr(7, 4));
-	char* endpos;
-	unsigned long n = strtoul(num.c_str(), &endpos, 10);
-	if (*endpos != '\0') {
-		return 0;
-	}
-	return n;
-}
-
-// TODO: generalize this with ScreenShotSaver.cc
-string Mixer::SoundlogCommand::getFileName()
-{
-	int max_num = 0;
-	
-	string dirName = FileOperations::getUserOpenMSXDir() + "/soundlogs";
-	try {
-		FileOperations::mkdirp(dirName);
-	} catch (FileException& e) {
-		// ignore
-	}
-
-	ReadDir dir(dirName.c_str());
-	while (dirent* d = dir.getEntry()) {
-		max_num = std::max(max_num, getNum(d));
-	}
-
-	std::ostringstream os;
-	os << dirName << "/openmsx";
-	os.width(4);
-	os.fill('0');
-	os << (max_num + 1) << ".wav";
-	return os.str();
-}
-
-Mixer::SoundlogCommand::SoundlogCommand(Mixer& outer_)
-	: outer(outer_)
+SoundlogCommand::SoundlogCommand(
+		CommandController& commandController, Mixer& mixer_)
+	: SimpleCommand(commandController, "soundlog")
+	, mixer(mixer_)
 {
 }
 
-string Mixer::SoundlogCommand::execute(const vector<string>& tokens)
+string SoundlogCommand::execute(const vector<string>& tokens)
 {
 	if (tokens.size() < 2) {
 		throw CommandException("Missing argument");
@@ -402,154 +366,122 @@ string Mixer::SoundlogCommand::execute(const vector<string>& tokens)
 	throw SyntaxError();
 }
 
-string Mixer::SoundlogCommand::startSoundLogging(const vector<string>& tokens)
+string SoundlogCommand::startSoundLogging(const vector<string>& tokens)
 {
 	string filename;
 	switch (tokens.size()) {
 	case 2:
-		filename = getFileName();
+		filename = FileOperations::getNextNumberedFileName("soundlogs", "openmsx", ".wav");
 		break;
 	case 3:
-		filename = tokens[2];
+		if (tokens[2] == "-prefix") {
+			throw SyntaxError();
+		} else {
+			filename = tokens[2];
+		}
+		break;
+	case 4:
+		if (tokens[2] == "-prefix") {
+			filename = FileOperations::getNextNumberedFileName("soundlogs", tokens[3], ".wav");
+		} else {
+			throw SyntaxError();
+		}
 		break;
 	default:
 		throw SyntaxError();
 	}
-	
-	if (!outer.wavfp) {
-		outer.startSoundLogging(filename);
-		CliComm::instance().printInfo("Started logging sound to " + filename);
-		return filename;
+
+	if (!mixer.wavWriter.get()) {
+		try {
+			mixer.wavWriter.reset(new WavWriter(filename,
+				2, 16, mixer.frequencySetting->getValue()));
+			mixer.commandController.getCliComm().printInfo(
+				"Started logging sound to " + filename);
+			return filename;
+		} catch (MSXException& e) {
+			throw CommandException(e.getMessage());
+		}
 	} else {
 		return "Already logging!";
 	}
 }
 
-string Mixer::SoundlogCommand::stopSoundLogging(const vector<string>& tokens)
+string SoundlogCommand::stopSoundLogging(const vector<string>& tokens)
 {
 	if (tokens.size() != 2) throw SyntaxError();
-	if (outer.wavfp) {
-		outer.endSoundLogging();
+	if (mixer.wavWriter.get()) {
+		mixer.wavWriter.reset();
 		return "SoundLogging stopped.";
 	} else {
 		return "Sound logging was not enabled, are you trying to fool me?";
 	}
 }
-	
-string Mixer::SoundlogCommand::toggleSoundLogging(const vector<string>& tokens)
+
+string SoundlogCommand::toggleSoundLogging(const vector<string>& tokens)
 {
 	if (tokens.size() != 2) throw SyntaxError();
-	if (!outer.wavfp) {
+	if (!mixer.wavWriter.get()) {
 		return startSoundLogging(tokens);
 	} else {
 		return stopSoundLogging(tokens);
 	}
 }
 
-string Mixer::SoundlogCommand::help(const vector<string>& /*tokens*/) const
+string SoundlogCommand::help(const vector<string>& /*tokens*/) const
 {
 	return "Controls sound logging: writing the openMSX sound to a wav file.\n"
-	       "soundlog start             Log sound to file \"openmsxNNNN.wav\"\n"
-	       "soundlog start <filename>  Log sound to indicated file\n"
-	       "soundlog stop              Stop logging sound\n"
-	       "soundlog toggle            Toggle sound logging state\n";
+	       "soundlog start              Log sound to file \"openmsxNNNN.wav\"\n"
+	       "soundlog start <filename>   Log sound to indicated file\n"
+	       "soundlog start -prefix foo  Log sound to file \"fooNNNN.wav\"\n"
+	       "soundlog stop               Stop logging sound\n"
+	       "soundlog toggle             Toggle sound logging state\n";
 }
 
-void Mixer::SoundlogCommand::tabCompletion(vector<string>& tokens) const
+void SoundlogCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		set<string> cmds;
 		cmds.insert("start");
 		cmds.insert("stop");
 		cmds.insert("toggle");
-		CommandController::completeString(tokens, cmds);
-	}
-}
-
-void Mixer::startSoundLogging(const string& filename)
-{
-	/* TODO on the soundlogger:
-	 * - take care of endianness (this probably won't work on PPC now)
-	 * - properly unite the getFileName stuff with ScreenShotSaver
-	 * - error handling, fwrite or fopen may fail miserably
-	 */
-
-	assert(!wavfp);
-	
-	wavfp = fopen(filename.c_str(), "wb");
-	if (!wavfp) {
-		// TODO
-	}
-	nofWavBytes = 0;
-
-	// write wav header
-	char header[44] = {
-		'R', 'I', 'F', 'F', //
-		0, 0, 0, 0,         // total size (filled in later)
-		'W', 'A', 'V', 'E', //
-		'f', 'm', 't', ' ', //
-		16, 0, 0, 0,        // size of fmt block
-		1, 0,               // format tag = 1
-		2, 0,               // nb of channels = 2
-		0, 0, 0, 0,         // samples per second (filled in)
-		0, 0, 0, 0,         // avg bytes per second (filled in)
-		4, 0,               // block align (2 * bits per samp / 16)
-		16, 0,              // bits per sample
-		'd', 'a', 't', 'a', //
-		0, 0, 0, 0,         // size of data block (filled in later)
-	};
-	const int channels = 2;
-	const int bitsPerSample = 16;
-	uint32 samplesPerSecond = frequencySetting->getValue();
-	*reinterpret_cast<uint32*>(header + 24) = samplesPerSecond;
-	*reinterpret_cast<uint32*>(header + 28) = (channels * samplesPerSecond *
-	                                           bitsPerSample) / 8;
-	fwrite(header, sizeof(header), 1, wavfp);
-}
-
-void Mixer::endSoundLogging()
-{
-	if (wavfp) {
-		// TODO error handling
-		uint32 totalsize = nofWavBytes + 44;
-		fseek(wavfp, 4, SEEK_SET);
-		fwrite(&totalsize, 4, 1, wavfp);
-		fseek(wavfp, 40, SEEK_SET);
-		fwrite(&nofWavBytes, 4, 1, wavfp);
-
-		fclose(wavfp);
-		wavfp = NULL;
+		completeString(tokens, cmds);
+	} else if ((tokens.size() == 3) && (tokens[1] == "start")) {
+		set<string> cmds;
+		cmds.insert("-prefix");
+		completeString(tokens, cmds);
 	}
 }
 
 // end stuff for soundlogging
 
-void Mixer::update(const Setting* setting)
+void Mixer::update(const Setting& setting)
 {
-	if (setting == muteSetting.get()) {
+	if (&setting == muteSetting.get()) {
 		if (muteSetting->getValue()) {
 			mute();
 		} else {
 			unmute();
 		}
-	} else if (setting == &pauseSetting) {
+	} else if (&setting == &pauseSetting) {
 		if (pauseSetting.getValue()) {
 			mute();
 		} else {
 			unmute();
 		}
-	} else if ((setting == samplesSetting.get()) ||
-	           (setting == soundDriverSetting.get())) {
+	} else if ((&setting == samplesSetting.get()) ||
+	           (&setting == soundDriverSetting.get())) {
 		if (handlingUpdate) return;
 		handlingUpdate = true;
 		reopenSound();
 		handlingUpdate = false;
-	} else if (setting == frequencySetting.get()) {
+	} else if (&setting == frequencySetting.get()) {
 		if (handlingUpdate) return;
 		handlingUpdate = true;
-		if (wavfp != 0) {
-			endSoundLogging();
-			CliComm::instance().printWarning("Stopped logging sound, because of change of frequency setting");
+		if (wavWriter.get()) {
+			wavWriter.reset();
+			commandController.getCliComm().printWarning(
+				"Stopped logging sound, because of change of "
+				"frequency setting");
 			// the alternative: ignore the change of setting and keep logging sound
 		}
 		reopenSound();
@@ -561,11 +493,11 @@ void Mixer::update(const Setting* setting)
 			}
 		}
 		handlingUpdate = false;
-	} else if (setting == masterVolume.get()) {
+	} else if (&setting == masterVolume.get()) {
 		updateMasterVolume(masterVolume->getValue());
-	} else if (dynamic_cast<const EnumSetting<ChannelMode>* >(setting)) {
-		map<SoundDevice*, SoundDeviceInfo>::iterator it = infos.begin();
-		while (it != infos.end() && it->second.modeSetting != setting) {
+	} else if (dynamic_cast<const EnumSetting<ChannelMode>* >(&setting)) {
+		Infos::iterator it = infos.begin();
+		while (it != infos.end() && it->second.modeSetting != &setting) {
 			++it;
 		}
 		assert(it != infos.end());
@@ -577,9 +509,9 @@ void Mixer::update(const Setting* setting)
 		dev.erase(remove(dev.begin(), dev.end(), it->first), dev.end());
 		devices[info.mode].push_back(it->first);
 		unlock();
-	} else if (dynamic_cast<const IntegerSetting*>(setting)) {
-		map<SoundDevice*, SoundDeviceInfo>::iterator it = infos.begin();
-		while (it != infos.end() && it->second.volumeSetting != setting) {
+	} else if (dynamic_cast<const IntegerSetting*>(&setting)) {
+		Infos::iterator it = infos.begin();
+		while (it != infos.end() && it->second.volumeSetting != &setting) {
 			++it;
 		}
 		assert(it != infos.end());
@@ -595,7 +527,7 @@ void Mixer::update(const Setting* setting)
 // 0 <= mastervolume <= 100
 void Mixer::updateMasterVolume(int masterVolume)
 {
-	for (map<SoundDevice*, SoundDeviceInfo>::const_iterator it = infos.begin();
+	for (Infos::const_iterator it = infos.begin();
 	     it != infos.end(); ++it) {
 		const SoundDeviceInfo& info = it->second;
 		it->first->setVolume(
@@ -609,8 +541,8 @@ void Mixer::updateMasterVolume(int masterVolume)
 
 SoundDevice* Mixer::getSoundDevice(const string& name)
 {
-	for (map<SoundDevice*, SoundDeviceInfo>::const_iterator it =
-	       infos.begin(); it != infos.end(); ++it) {
+	for (Infos::const_iterator it = infos.begin();
+	     it != infos.end(); ++it) {
 		if (it->first->getName() == name) {
 			return it->first;
 		}
@@ -618,23 +550,25 @@ SoundDevice* Mixer::getSoundDevice(const string& name)
 	return NULL;
 }
 
-Mixer::SoundDeviceInfoTopic::SoundDeviceInfoTopic(Mixer& outer_)
-	: outer(outer_)
+SoundDeviceInfoTopic::SoundDeviceInfoTopic(
+		CommandController& commandController, Mixer& mixer_)
+	: InfoTopic(commandController, "sounddevice")
+	, mixer(mixer_)
 {
 }
 
-void Mixer::SoundDeviceInfoTopic::execute(const vector<CommandArgument>& tokens,
-	CommandArgument& result) const
+void SoundDeviceInfoTopic::execute(const vector<TclObject*>& tokens,
+	TclObject& result) const
 {
 	switch (tokens.size()) {
 	case 2:
-		for (map<SoundDevice*, SoundDeviceInfo>::const_iterator it =
-		       outer.infos.begin(); it != outer.infos.end(); ++it) {
+		for (Mixer::Infos::const_iterator it = mixer.infos.begin();
+		     it != mixer.infos.end(); ++it) {
 			result.addListElement(it->first->getName());
 		}
 		break;
 	case 3: {
-		SoundDevice* device = outer.getSoundDevice(tokens[2].getString());
+		SoundDevice* device = mixer.getSoundDevice(tokens[2]->getString());
 		if (!device) {
 			throw CommandException("Unknown sound device");
 		}
@@ -646,20 +580,20 @@ void Mixer::SoundDeviceInfoTopic::execute(const vector<CommandArgument>& tokens,
 	}
 }
 
-string Mixer::SoundDeviceInfoTopic::help(const vector<string>& /*tokens*/) const
+string SoundDeviceInfoTopic::help(const vector<string>& /*tokens*/) const
 {
 	return "Shows a list of available sound devices.\n";
 }
 
-void Mixer::SoundDeviceInfoTopic::tabCompletion(vector<string>& tokens) const
+void SoundDeviceInfoTopic::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 3) {
 		set<string> devices;
-		for (map<SoundDevice*, SoundDeviceInfo>::const_iterator it =
-		       outer.infos.begin(); it != outer.infos.end(); ++it) {
+		for (Mixer::Infos::const_iterator it =
+		       mixer.infos.begin(); it != mixer.infos.end(); ++it) {
 			devices.insert(it->first->getName());
 		}
-		CommandController::completeString(tokens, devices);
+		completeString(tokens, devices);
 	}
 }
 
