@@ -1,17 +1,9 @@
-// $Id: NullSoundDriver.cc,v 1.1 2005/04/21 19:03:31 m9710797 Exp $
+// $Id: NullSoundDriver.cc,v 1.2 2005/11/12 20:32:42 m9710797 Exp $
 
 #include "NullSoundDriver.hh"
 
 namespace openmsx {
 
-NullSoundDriver::NullSoundDriver()
-{
-}
-
-NullSoundDriver::~NullSoundDriver()
-{
-}
-
 void NullSoundDriver::lock()
 {
 }
