@@ -1,40 +1,40 @@
-// $Id: SCC.cc,v 1.38 2005/04/09 08:03:38 m9710797 Exp $
+// $Id: SCC.cc,v 1.50 2006/01/23 19:35:25 m9710797 Exp $
 
 //-----------------------------------------------------------------------------
-// 
+//
 // On Mon, 24 Feb 2003, Jon De Schrijder wrote:
-// 
+//
 // I've done some measurements with the scope on the output of the SCC.
 // I didn't do timing tests, only amplitude checks:
-// 
+//
 // I know now for sure, the amplitude calculation works as follows:
-// 
+//
 // AmpOut=640+AmpA+AmpB+AmpC+AmpD+AmpE
-// 
+//
 // range AmpOut (11 bits positive number=SCC digital output): [+40...+1235]
-// 
+//
 // AmpA="((SampleValue*VolA) AND #7FF0) div 16"
 // AmpB="((SampleValue*VolB) AND #7FF0) div 16"
 // AmpC="((SampleValue*VolC) AND #7FF0) div 16"
 // AmpD="((SampleValue*VolD) AND #7FF0) div 16"
 // AmpE="((SampleValue*VolE) AND #7FF0) div 16"
-// 
+//
 // Setting the enablebit to zero, corresponds with VolX=0.
-// 
+//
 // SampleValue range [-128...+127]
 // VolX range [0..15]
-// 
-// Notes: 
+//
+// Notes:
 // * SampleValue*VolX is calculated (signed multiplication) and the lower 4
 //   bits are dropped (both in case the value is positive or negative), before
 //   the addition of the 5 values is done. This was tested by setting
 //   SampleValue=+1 and VolX=15 of different channels. The resulting AmpOut=640,
 //   indicating that the 4 lower bits were dropped *before* the addition.
-// 
+//
 //-----------------------------------------------------------------------------
 //
 // On Mon, 14 Apr 2003, Manuel Pazos wrote
-// 
+//
 // I have some info about SCC/SCC+ that I hope you find useful. It is about
 // "Mode Setting Register", also called "Deformation Register" Here it goes:
 //
@@ -59,7 +59,7 @@
 // Reading Mode Setting Register, is equivalent to write #FF to it.
 //
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// 
+//
 // Additions:
 //   - Setting both bit0 and bit1 is equivalent to setting only bit1
 //   - A rotation goes like this:
@@ -67,60 +67,89 @@
 //   - Channel 4-5 rotation speed is set by channel 5 freq (channel 4 freq
 //     is ignored for rotation)
 //
+//-----------------------------------------------------------------------------
+//
+// On Sat, 09 Sep 2005, NYYRIKKI wrote (MRC post)
+//
+// ...
+//
+// One important thing to know is that change of volume is not implemented
+// immediately in SCC. Normally it is changed when next byte from sample memory
+// is played, but writing value to frequency causes current byte to be started
+// again. As in this example we write values very quickly to frequency registers
+// the internal sample counter does not actually move at all.
+//
+// Third method is a variation of first method. As we don't know where SCC is
+// playing, let's update the whole sample memory with one and same new value.
+// To make sample rate not variable in low sample rates we first stop SCC from
+// reading sample memory. This can be done by writing value less than 9 to
+// frequency. Now we can update sample RAM so, that output does not change.
+// After sample RAM has been updated, we start SCC internal counter so that
+// value (where ever the counter was) is sent to output. This routine can be
+// found below as example 3.
+//
+// ...
+//
+//
 //
 // Something completely different: the SCC+ is actually called SCC-I.
 //-----------------------------------------------------------------------------
 
 #include "SCC.hh"
 #include "Mixer.hh"
-#include "Scheduler.hh"
-#include "Debugger.hh"
+#include "SimpleDebuggable.hh"
+#include "MSXMotherBoard.hh"
 
 using std::string;
 
 namespace openmsx {
 
-SCC::SCC(const string& name_, const XMLElement& config, const EmuTime& time,
-	 ChipMode mode)
-	: sccDebuggable(*this), currentChipMode(mode), name(name_)
+class SCCDebuggable : public SimpleDebuggable
+{
+public:
+	SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc);
+	virtual byte read(unsigned address, const EmuTime& time);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	SCC& scc;
+};
+
+
+static string calcDescription(SCC::ChipMode mode)
+{
+	return (mode == SCC::SCC_Real) ? "Konami SCC" : "Konami SCC+";
+}
+
+SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
+         const XMLElement& config, const EmuTime& time, ChipMode mode)
+	: SoundDevice(motherBoard.getMixer(), name, calcDescription(mode))
+	, currentChipMode(mode)
+	, deformTimer(time)
+	, debuggable(new SCCDebuggable(motherBoard, *this))
 {
 	// Clear wave forms.
 	for (unsigned i = 0; i < 5; ++i) {
 		for (unsigned j = 0; j < 32; ++j) {
 			wave[i][j] = 0;
 		}
+		out[i] = 0;
 	}
 
 	nbSamples = 9999;
-	
+
 	in[0] = in[1] = in[2] = in[3] = in[4] = 0;
 	inHp [0] = inHp [1] = inHp [2] = 0;
 	outHp[0] = outHp[1] = outHp[2] = 0;
-	
+
 	reset(time);
 	registerSound(config);
-	Debugger::instance().registerDebuggable(name + " SCC", sccDebuggable);
 }
 
 SCC::~SCC()
 {
-	Debugger::instance().unregisterDebuggable(name + " SCC", sccDebuggable);
 	unregisterSound();
 }
 
-const string& SCC::getName() const
-{
-	return name;
-}
-
-const string& SCC::getDescription() const
-{
-	static const string desc_scc ("Konami SCC");
-	static const string desc_sccp("Konami SCC+");
-	return (currentChipMode == SCC_Real) ? desc_scc
-	                                     : desc_sccp;
-}
-
 void SCC::reset(const EmuTime& /*time*/)
 {
 	if (currentChipMode != SCC_Real) {
@@ -133,11 +162,11 @@ void SCC::reset(const EmuTime& /*time*/)
 			volAdjustedWave[i][j] = 0;
 		}
 		count[i] = 0;
+		pos[i] = (unsigned)-1;
 		freq[i] = 0;
 		volume[i] = 0;
 		rotate[i] = false;
 		readOnly[i] = false;
-		offset[i] = 0;
 	}
 	deformValue = 0;
 	ch_enable = 0xFF;
@@ -264,7 +293,7 @@ byte SCC::getFreqVol(byte address)
 
 void SCC::writeMemInterface(byte address, byte value, const EmuTime& time)
 {
-	Mixer::instance().updateStream(time);
+	getMixer().updateStream(time);
 
 	switch (currentChipMode) {
 	case SCC_Real:
@@ -360,6 +389,9 @@ void SCC::setFreqVol(byte address, byte value)
 			frq >>= 8;
 		}
 		incr[channel] = (frq <= 8) ? 0 : baseStep / (frq + 1);
+		count[channel] &= 0x0F800000; // reset to begin of byte
+		pos[channel] = (unsigned)-1;
+
 	} else if (address < 0x0F) {
 		// change volume
 		byte channel = address - 0x0A;
@@ -383,7 +415,7 @@ void SCC::setDeformReg(byte value, const EmuTime& time)
 	}
 	deformValue = value;
 	deformTimer.advance(time);
-	
+
 	if (currentChipMode != SCC_Real) {
 		value &= ~0x80;
 	}
@@ -392,7 +424,6 @@ void SCC::setDeformReg(byte value, const EmuTime& time)
 			for (unsigned i = 0; i < 5; ++i) {
 				rotate[i] = false;
 				readOnly[i] = false;
-				offset[i] = 0;
 			}
 			break;
 		case 0x40:
@@ -448,7 +479,8 @@ int SCC::filter(int input)
 	return outHp[0];
 }
 
-void SCC::updateBuffer(int length, int* buffer)
+void SCC::updateBuffer(unsigned length, int* buffer,
+     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
 {
 	nbSamples += length;
 	if ((deformValue & 0xC0) == 0x00) {
@@ -458,9 +490,14 @@ void SCC::updateBuffer(int length, int* buffer)
 			int mixed = 0;
 			byte enable = ch_enable;
 			for (int i = 0; i < 5; ++i, enable >>= 1) {
-				count[i] = (count[i] + incr[i]) & 0xFFFFFFF;
+				count[i] = (count[i] + incr[i]) & 0x0FFFFFFF;
+				unsigned newPos = count[i] >> 23;
+				if (newPos != pos[i]) {
+					pos[i] = newPos;
+					out[i] = volAdjustedWave[i][newPos];
+				}
 				if (enable & 1) {
-					mixed += volAdjustedWave[i][(count[i] >> 23)];
+					mixed += out[i];
 				}
 			}
 			*buffer++ = filter((masterVolume * mixed) / 256);
@@ -472,14 +509,18 @@ void SCC::updateBuffer(int length, int* buffer)
 			int mixed = 0;
 			byte enable = ch_enable;
 			for (int i = 0; i < 5; ++i, enable >>= 1) {
-				int cnt = count[i] + incr[i];
+				unsigned cnt = count[i] + incr[i];
 				if (rotate[i]) {
-					offset[i] += cnt >> 28;
+					cnt += (cnt >> 28) << 23;
+				}
+				count[i] = cnt & 0x0FFFFFFF;
+				unsigned newPos = count[i] >> 23;
+				if (newPos != pos[i]) {
+					pos[i] = newPos;
+					out[i] = volAdjustedWave[i][newPos];
 				}
-				count[i] = cnt & 0xFFFFFFF;
 				if (enable & 1) {
-					byte pos = (count[i] >> 23) + offset[i];
-					mixed += volAdjustedWave[i][pos & 0x1F];
+					mixed += out[i];
 				}
 			}
 			*buffer++ = filter((masterVolume * mixed) / 256);
@@ -509,54 +550,42 @@ void SCC::checkMute()
 	}
 }
 
+// SimpleDebuggable
 
-// class SCCDebuggable
-
-SCC::SCCDebuggable::SCCDebuggable(SCC& parent_)
-	: parent(parent_)
+SCCDebuggable::SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc_)
+	: SimpleDebuggable(motherBoard, scc_.getName() + " SCC",
+	                   "SCC registers in SCC+ format", 0x100)
+	, scc(scc_)
 {
 }
 
-unsigned SCC::SCCDebuggable::getSize() const
-{
-	return 0x100;
-}
-
-const string& SCC::SCCDebuggable::getDescription() const
-{
-	static const string desc = "SCC registers in SCC+ format";
-	return desc;
-}
-
-byte SCC::SCCDebuggable::read(unsigned address)
+byte SCCDebuggable::read(unsigned address, const EmuTime& time)
 {
 	if (address < 0xA0) {
 		// read wave form 1..5
-		const EmuTime& time = Scheduler::instance().getCurrentTime();
-		return parent.readWave(address >> 5, address, time);
+		return scc.readWave(address >> 5, address, time);
 	} else if (address < 0xC0) {
 		// freq volume block
-		return parent.getFreqVol(address);
+		return scc.getFreqVol(address);
 	} else if (address < 0xE0) {
 		// peek deformation register
-		return parent.deformValue;
+		return scc.deformValue;
 	} else {
 		return 0xFF;
 	}
 }
 
-void SCC::SCCDebuggable::write(unsigned address, byte value)
+void SCCDebuggable::write(unsigned address, byte value, const EmuTime& time)
 {
 	if (address < 0xA0) {
 		// read wave form 1..5
-		parent.writeWave(address >> 5, address, value);
+		scc.writeWave(address >> 5, address, value);
 	} else if (address < 0xC0) {
 		// freq volume block
-		 parent.setFreqVol(address, value);
+		 scc.setFreqVol(address, value);
 	} else if (address < 0xE0) {
 		// deformation register
-		const EmuTime& time = Scheduler::instance().getCurrentTime();
-		parent.setDeformReg(value, time);
+		scc.setDeformReg(value, time);
 	} else {
 		// ignore
 	}
