@@ -1,29 +1,60 @@
-// $Id: SoundDevice.cc,v 1.12 2005/01/15 12:55:45 m9710797 Exp $
+// $Id: SoundDevice.cc,v 1.15 2006/07/07 21:40:54 m9710797 Exp $
 
 #include "SoundDevice.hh"
 #include "XMLElement.hh"
+#include <cassert>
 
 using std::string;
 
 namespace openmsx {
 
-SoundDevice::SoundDevice()
+SoundDevice::SoundDevice(Mixer& mixer_, const string& name_,
+                         const string& description_)
+	: mixer(mixer_), name(name_), description(description_)
+	, muteCount(1), muted(true)
 {
-	muted = true;
 }
 
 SoundDevice::~SoundDevice()
 {
 }
 
-void SoundDevice::setMute(bool muted)
+const std::string& SoundDevice::getName() const
 {
-	this->muted = muted;
+	return name;
+}
+
+const std::string& SoundDevice::getDescription() const
+{
+	return description;
+}
+
+void SoundDevice::increaseMuteCount()
+{
+	++muteCount;
+}
+
+void SoundDevice::decreaseMuteCount()
+{
+	assert(isMuted());
+	--muteCount;
+}
+
+void SoundDevice::setMute(bool newMuted)
+{
+	if (newMuted != muted) {
+		muted = newMuted;
+		if (muted) {
+			increaseMuteCount();
+		} else {
+			decreaseMuteCount();
+		}
+	}
 }
 
 bool SoundDevice::isMuted() const
 {
-	return muted;
+	return muteCount;
 }
 
 void SoundDevice::registerSound(const XMLElement& config,
@@ -41,12 +72,12 @@ void SoundDevice::registerSound(const XMLElement& config,
 			mode = Mixer::MONO;
 		}
 	}
-	Mixer::instance().registerSound(*this, volume, mode);
+	mixer.registerSound(*this, volume, mode);
 }
 
 void SoundDevice::unregisterSound()
 {
-	Mixer::instance().unregisterSound(*this);
+	mixer.unregisterSound(*this);
 }
 
 } // namespace openmsx
