@@ -1,4 +1,4 @@
-// $Id: WavAudioInput.cc,v 1.22 2005/03/12 20:47:16 m9710797 Exp $
+// $Id: WavAudioInput.cc,v 1.29 2006/01/24 18:03:35 m9710797 Exp $
 
 #include "WavAudioInput.hh"
 #include "MSXException.hh"
@@ -10,20 +10,21 @@ using std::string;
 
 namespace openmsx {
 
-WavAudioInput::WavAudioInput()
+WavAudioInput::WavAudioInput(CommandController& commandController)
 	: length(0), buffer(0), freq(44100)
+	, reference(EmuTime::zero)
 	, plugged(false)
-	, audioInputFilenameSetting(new FilenameSetting("audio-inputfilename",
+	, audioInputFilenameSetting(new FilenameSetting(
+		commandController, "audio-inputfilename",
 		"filename of the file where the sampler reads data from",
 		"audio-input.wav"))
 {
-	audioInputFilenameSetting->addListener(this);
+	audioInputFilenameSetting->attach(*this);
 }
 
-
 WavAudioInput::~WavAudioInput()
 {
-	audioInputFilenameSetting->removeListener(this);
+	audioInputFilenameSetting->detach(*this);
 	freeWave();
 }
 
@@ -36,7 +37,8 @@ void WavAudioInput::loadWave()
 	Uint32 wavLen;
 	if (SDL_LoadWAV(audioInputFilenameSetting->getValueString().c_str(),
 	                &wavSpec, &wavBuf, &wavLen) == NULL) {
-		throw MSXException(string("WavAudioInput error: ") + SDL_GetError());
+		throw MSXException(string("WavAudioInput error: ") +
+		                   SDL_GetError());
 	}
 
 	freq = wavSpec.freq;
@@ -85,13 +87,14 @@ const string& WavAudioInput::getDescription() const
 	return desc;
 }
 
-void WavAudioInput::plugHelper(Connector* /*connector*/, const EmuTime& time)
+void WavAudioInput::plugHelper(Connector& /*connector*/, const EmuTime& time)
 {
 	if (buffer == 0) {
 		try {
 			loadWave();
 		} catch (MSXException &e) {
-			throw PlugException("Load of wave file failed: " + e.getMessage());
+			throw PlugException("Load of wave file failed: " +
+			                    e.getMessage());
 		}
 	}
 	reference = time;
@@ -104,16 +107,16 @@ void WavAudioInput::unplugHelper(const EmuTime& /*time*/)
 	plugged = false;
 }
 
-void WavAudioInput::update(const Setting* setting)
+void WavAudioInput::update(const Setting& setting)
 {
-	if (setting); // avoid warning
-	assert(setting == audioInputFilenameSetting.get());
+	(void)setting;
+	assert(&setting == audioInputFilenameSetting.get());
 	if (plugged) {
 		try {
 			loadWave();
 		} catch (MSXException &e) {
 			// TODO proper error handling, message should go to console
-			CliComm::instance().printWarning(
+			setting.getCommandController().getCliComm().printWarning(
 				"Load of wave file failed: " + e.getMessage());
 		}
 	}
