@@ -1,4 +1,4 @@
-// $Id: YM2413.cc,v 1.52 2005/04/06 20:06:34 m9710797 Exp $
+// $Id: YM2413.cc,v 1.62 2006/01/21 10:51:59 m9710797 Exp $
 
 /*
  * Based on:
@@ -6,21 +6,44 @@
  * heavily rewritten to fit openMSX structure
  */
 
+#include "YM2413.hh"
+#include "Mixer.hh"
+#include "SimpleDebuggable.hh"
+#include "MSXMotherBoard.hh"
 #include <cmath>
 #include <cassert>
 #include <algorithm>
-#include "YM2413.hh"
-#include "Mixer.hh"
-#include "Debugger.hh"
-#include "Scheduler.hh"
-
-using std::string;
 
 namespace openmsx {
 
+class YM2413Debuggable : public SimpleDebuggable
+{
+public:
+	YM2413Debuggable(MSXMotherBoard& motherBoard, YM2413& ym2413);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	YM2413& ym2413;
+};
+
+
 static const int CLOCK_FREQ = 3579545;
 static const double PI = 3.14159265358979323846;
 
+static const double DB_STEP = 48.0 / (1 << 8); //   48 / (1 << DB_BITS)
+static const double EG_STEP = 0.375;
+static const double TL_STEP = 0.75;
+static const double SL_STEP = 3.0;
+
+// PM speed(Hz) and depth(cent)
+static const double PM_SPEED = 6.4;
+static const double PM_DEPTH = 13.75;
+
+// AM speed(Hz) and depth(dB)
+static const double AM_SPEED = 3.6413;
+static const double AM_DEPTH = 4.875;
+
+
 int YM2413::pmtable[PM_PG_WIDTH];
 int YM2413::amtable[AM_PG_WIDTH];
 unsigned int YM2413::tllTable[16][8][1 << TL_BITS][4];
@@ -44,12 +67,12 @@ YM2413::Patch YM2413::nullPatch;
 //                                                   //
 //***************************************************//
 
-int YM2413::Slot::EG2DB(int d) 
+int YM2413::Slot::EG2DB(int d)
 {
 	return d * (int)(EG_STEP / DB_STEP);
 }
 int YM2413::TL2EG(int d)
-{ 
+{
 	return d * (int)(TL_STEP / EG_STEP);
 }
 int YM2413::Slot::SL2EG(int d)
@@ -158,8 +181,8 @@ static inline double saw(double phase)
 void YM2413::makePmTable()
 {
 	for (int i = 0; i < PM_PG_WIDTH; ++i) {
-		 pmtable[i] = (int)((double)PM_AMP * 
-		     pow(2, (double)PM_DEPTH * 
+		 pmtable[i] = (int)((double)PM_AMP *
+		     pow(2, (double)PM_DEPTH *
 		            saw(2.0 * PI * i / PM_PG_WIDTH) / 1200));
 	}
 }
@@ -184,7 +207,7 @@ void YM2413::makeDphaseTable(int sampleRate)
 	for (unsigned fnum = 0; fnum < 512; ++fnum) {
 		for (unsigned block = 0; block < 8; ++block) {
 			for (unsigned ML = 0; ML < 16; ++ML) {
-				dphaseTable[fnum][block][ML] = 
+				dphaseTable[fnum][block][ML] =
 				    rate_adjust(((fnum * mltable[ML]) << block) >>
 				                (20 - DP_BITS),
 				                sampleRate);
@@ -201,7 +224,7 @@ void YM2413::makeTllTable()
 		(18.000 * 2), (18.750 * 2), (19.125 * 2), (19.500 * 2),
 		(19.875 * 2), (20.250 * 2), (20.625 * 2), (21.000 * 2)
 	};
-  
+
 	for (int fnum = 0; fnum < 16; ++fnum) {
 		for (int block = 0; block < 8; ++block) {
 			for (int TL = 0; TL < 64; ++TL) {
@@ -229,7 +252,7 @@ void YM2413::makeDphaseARTable(int sampleRate)
 			int RM = AR + (Rks >> 2);
 			int RL = Rks & 3;
 			if (RM > 15) RM = 15;
-			switch (AR) { 
+			switch (AR) {
 			case 0:
 				dphaseARTable[AR][Rks] = 0;
 				break;
@@ -252,7 +275,7 @@ void YM2413::makeDphaseDRTable(int sampleRate)
 			int RM = DR + (Rks >> 2);
 			int RL = Rks & 3;
 			if (RM > 15) RM = 15;
-			switch(DR) { 
+			switch(DR) {
 			case 0:
 				dphaseDRTable[DR][Rks] = 0;
 				break;
@@ -568,8 +591,10 @@ static byte inst_data[16 + 3][8] = {
 	{ 0x25,0x11,0x00,0x00,0xf8,0xfa,0xf8,0x55 }
 };
 
-YM2413::YM2413(const string& name_, const XMLElement& config, const EmuTime& time)
-	: name(name_)
+YM2413::YM2413(MSXMotherBoard& motherBoard, const std::string& name,
+               const XMLElement& config, const EmuTime& time)
+	: SoundDevice(motherBoard.getMixer(), name, "MSX-MUSIC")
+	, debuggable(new YM2413Debuggable(motherBoard, *this))
 {
 	for (int i = 0; i < 16 + 3; ++i) {
 		patches[2 * i + 0] = Patch(0, inst_data[i]);
@@ -594,26 +619,13 @@ YM2413::YM2413(const string& name_, const XMLElement& config, const EmuTime& tim
 
 	reset(time);
 	registerSound(config);
-	Debugger::instance().registerDebuggable(name + " regs", *this);
 }
 
 YM2413::~YM2413()
 {
-	Debugger::instance().unregisterDebuggable(name + " regs", *this);
 	unregisterSound();
 }
 
-const string& YM2413::getName() const
-{
-	return name;
-}
-
-const string& YM2413::getDescription() const
-{
-	static const string desc("MSX-MUSIC");
-	return desc;
-}
-
 // Reset whole of OPLL except patch datas
 void YM2413::reset(const EmuTime &time)
 {
@@ -724,7 +736,7 @@ void YM2413::update_key_status()
 //******************************************************//
 
 // Convert Amp(0 to EG_HEIGHT) to Phase(0 to 4PI)
-int YM2413::Slot::wave2_4pi(int e) 
+int YM2413::Slot::wave2_4pi(int e)
 {
 	int shift =  SLOT_AMP_BITS - PG_BITS - 1;
 	if (shift > 0) {
@@ -735,7 +747,7 @@ int YM2413::Slot::wave2_4pi(int e)
 }
 
 // Convert Amp(0 to EG_HEIGHT) to Phase(0 to 8PI)
-int YM2413::Slot::wave2_8pi(int e) 
+int YM2413::Slot::wave2_8pi(int e)
 {
 	int shift = SLOT_AMP_BITS - PG_BITS - 2;
 	if (shift > 0) {
@@ -776,7 +788,7 @@ inline void YM2413::update_noise()
 // EG
 void YM2413::Slot::calc_envelope(int lfo_am)
 {
-	#define S2E(x) (SL2EG((int)(x / SL_STEP)) << (EG_DP_BITS - EG_BITS)) 
+	#define S2E(x) (SL2EG((int)(x / SL_STEP)) << (EG_DP_BITS - EG_BITS))
 	static unsigned int SL[16] = {
 		S2E( 0.0), S2E( 3.0), S2E( 6.0), S2E( 9.0),
 		S2E(12.0), S2E(15.0), S2E(18.0), S2E(21.0),
@@ -847,7 +859,7 @@ void YM2413::Slot::calc_envelope(int lfo_am)
 	if (out >= (unsigned)DB_MUTE) {
 		out = DB_MUTE - 1;
 	}
-	
+
 	egout = out | 3;
 }
 
@@ -894,7 +906,7 @@ int YM2413::Slot::calc_slot_snare(bool noise)
 {
 	if (egout >= (DB_MUTE - 1)) {
 		return 0;
-	} 
+	}
 	if (BIT(pgout, 7)) {
 		return dB2LinTab[(noise ? DB_POS(0.0) : DB_POS(15.0)) + egout];
 	} else {
@@ -1012,7 +1024,8 @@ bool YM2413::checkMuteHelper()
 	return true;	// nothing is playing, then mute
 }
 
-void YM2413::updateBuffer(int length, int* buffer)
+void YM2413::updateBuffer(unsigned length, int* buffer,
+     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
 {
 	while (length--) {
 		*(buffer++) = calcSample();
@@ -1037,8 +1050,8 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 	//PRT_DEBUG("YM2413: write reg "<<(int)regis<<" "<<(int)data);
 
 	// update the output buffer before changing the register
-	Mixer::instance().updateStream(time);
-	Mixer::instance().lock();
+	getMixer().updateStream(time);
+	getMixer().lock();
 
 	assert (regis < 0x40);
 	reg[regis] = data;
@@ -1145,7 +1158,7 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 		ch[7].mod.updateAll();
 		ch[7].car.updateAll();
 		ch[8].mod.updateAll();
-		ch[8].car.updateAll();        
+		ch[8].car.updateAll();
 		break;
 
 	case 0x10:  case 0x11:  case 0x12:  case 0x13:
@@ -1180,7 +1193,7 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 		break;
 	}
 	case 0x30: case 0x31: case 0x32: case 0x33: case 0x34:
-	case 0x35: case 0x36: case 0x37: case 0x38: 
+	case 0x35: case 0x36: case 0x37: case 0x38:
 	{
 		int cha = regis & 0x0F;
 		int j = (data >> 4) & 15;
@@ -1194,7 +1207,7 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 				ch[8].mod.setVolume(j << 2);
 				break;
 			}
-		} else { 
+		} else {
 			ch[cha].setPatch(j);
 		}
 		ch[cha].setVol(v << 2);
@@ -1205,26 +1218,28 @@ void YM2413::writeReg(byte regis, byte data, const EmuTime &time)
 	default:
 		break;
 	}
-	Mixer::instance().unlock();
+	getMixer().unlock();
 	checkMute();
 }
 
 
-// Debuggable
+// SimpleDebuggable
 
-unsigned YM2413::getSize() const
+YM2413Debuggable::YM2413Debuggable(MSXMotherBoard& motherBoard, YM2413& ym2413_)
+	: SimpleDebuggable(motherBoard, ym2413_.getName() + " regs",
+	                   "MSX-MUSIC", 0x40)
+	, ym2413(ym2413_)
 {
-	return 0x40;
 }
 
-byte YM2413::read(unsigned address)
+byte YM2413Debuggable::read(unsigned address)
 {
-	return reg[address];
+	return ym2413.reg[address];
 }
 
-void YM2413::write(unsigned address, byte value)
+void YM2413Debuggable::write(unsigned address, byte value, const EmuTime& time)
 {
-	writeReg(address, value, Scheduler::instance().getCurrentTime());
+	ym2413.writeReg(address, value, time);
 }
 
 } // namespace openmsx
