@@ -1,15 +1,34 @@
-// $Id: YMF278.cc,v 1.49 2005/05/30 13:55:17 m9710797 Exp $
+// $Id: YMF278.cc,v 1.60 2006/01/23 19:35:25 m9710797 Exp $
 
 #include "YMF278.hh"
 #include "Rom.hh"
-#include "Debugger.hh"
-#include "Scheduler.hh"
+#include "SimpleDebuggable.hh"
+#include "MSXMotherBoard.hh"
 #include <cmath>
 
-using std::string;
-
 namespace openmsx {
 
+class DebugRegisters : public SimpleDebuggable
+{
+public:
+	DebugRegisters(YMF278& ymf278, MSXMotherBoard& motherBoard);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	YMF278& ymf278;
+};
+
+class DebugMemory : public SimpleDebuggable
+{
+public:
+	DebugMemory(YMF278& ymf278, MSXMotherBoard& motherBoard);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value);
+private:
+	YMF278& ymf278;
+};
+
+
 const EmuDuration YMF278::REG_SELECT_DELAY = MasterClock::duration(88);
 const EmuDuration YMF278::REG_WRITE_DELAY = MasterClock::duration(88);
 const EmuDuration YMF278::MEM_READ_DELAY = MasterClock::duration(38);
@@ -232,10 +251,10 @@ void YMF278::advance()
 	while (eg_timer >= EG_TIMER_OVERFLOW) {
 		eg_timer -= EG_TIMER_OVERFLOW;
 		eg_cnt++;
-		
+
 		for (int i = 0; i < 24; i++) {
 			YMF278Slot &op = slots[i];
-			
+
 			if (op.lfo_active) {
 				op.lfo_cnt++;
 				if (op.lfo_cnt < op.lfo_max) {
@@ -249,7 +268,7 @@ void YMF278::advance()
 					}
 				}
 			}
-			
+
 			// Envelope Generator
 			switch(op.state) {
 			case EG_ATT: {	// attack phase
@@ -272,7 +291,7 @@ void YMF278::advance()
 				}
 				break;
 			}
-			case EG_DEC: {	// decay phase 
+			case EG_DEC: {	// decay phase
 				byte rate = op.compute_rate(op.D1R);
 				if (rate < 4) {
 					break;
@@ -281,7 +300,7 @@ void YMF278::advance()
 				if (!(eg_cnt & ((1 << shift) -1))) {
 					byte select = eg_rate_select[rate];
 					op.env_vol += eg_inc[select + ((eg_cnt >> shift) & 7)];
-					
+
 					if (((unsigned)op.env_vol > dl_tab[6]) && op.PRVB) {
 						op.state = EG_REV;
 					} else {
@@ -292,7 +311,7 @@ void YMF278::advance()
 				}
 				break;
 			}
-			case EG_SUS: {	// sustain phase 
+			case EG_SUS: {	// sustain phase
 				byte rate = op.compute_rate(op.D2R);
 				if (rate < 4) {
 					break;
@@ -301,7 +320,7 @@ void YMF278::advance()
 				if (!(eg_cnt & ((1 << shift) -1))) {
 					byte select = eg_rate_select[rate];
 					op.env_vol += eg_inc[select + ((eg_cnt >> shift) & 7)];
-					
+
 					if (((unsigned)op.env_vol > dl_tab[6]) && op.PRVB) {
 						op.state = EG_REV;
 					} else {
@@ -314,7 +333,7 @@ void YMF278::advance()
 				}
 				break;
 			}
-			case EG_REL: {	// release phase 
+			case EG_REL: {	// release phase
 				byte rate = op.compute_rate(op.RR);
 				if (rate < 4) {
 					break;
@@ -323,7 +342,7 @@ void YMF278::advance()
 				if (!(eg_cnt & ((1 << shift) -1))) {
 					byte select = eg_rate_select[rate];
 					op.env_vol += eg_inc[select + ((eg_cnt >> shift) & 7)];
-					
+
 					if (((unsigned)op.env_vol > dl_tab[6]) && op.PRVB) {
 						op.state = EG_REV;
 					} else {
@@ -346,7 +365,7 @@ void YMF278::advance()
 				if (!(eg_cnt & ((1 << shift) - 1))) {
 					byte select = eg_rate_select[rate];
 					op.env_vol += eg_inc[select + ((eg_cnt >> shift) & 7)];
-					
+
 					if (op.env_vol >= MAX_ATT_INDEX) {
 						op.env_vol = MAX_ATT_INDEX;
 						op.active = false;
@@ -362,7 +381,7 @@ void YMF278::advance()
 				if (!(eg_cnt & ((1 << shift) - 1))) {
 					byte select = eg_rate_select[rate];
 					op.env_vol += eg_inc[select + ((eg_cnt >> shift) & 7)];
-					
+
 					if (op.env_vol >= MAX_ATT_INDEX) {
 						op.env_vol = MAX_ATT_INDEX;
 						op.active = false;
@@ -374,7 +393,7 @@ void YMF278::advance()
 			case EG_OFF:
 				// nothing
 				break;
-			
+
 			default:
 				assert(false);
 				break;
@@ -433,7 +452,8 @@ bool YMF278::anyActive()
 	return false;
 }
 
-void YMF278::updateBuffer(int length, int* buffer)
+void YMF278::updateBuffer(unsigned length, int* buffer,
+     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
 {
 	int vl = mix_level[pcm_l];
 	int vr = mix_level[pcm_r];
@@ -460,10 +480,10 @@ void YMF278::updateBuffer(int length, int* buffer)
 			if (volRight < 0) {
 				volRight = 0;
 			}
-			
+
 			left  += (sample * volume[volLeft] ) >> 16;
 			right += (sample * volume[volRight]) >> 16;
-			
+
 			if (sl.lfo_active && sl.vib) {
 				int oct = sl.OCT;
 				if (oct & 8) {
@@ -499,7 +519,7 @@ void YMF278::keyOnHelper(YMF278Slot& slot)
 {
 	slot.active = true;
 	setMute(false);
-	
+
 	int oct = slot.OCT;
 	if (oct & 8) {
 		oct |= -8;
@@ -532,7 +552,7 @@ void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 		switch ((reg - 8) / 24) {
 		case 0: {
 			loadTime = time + LOAD_DELAY;
-			
+
 			slot.wave = (slot.wave & 0x100) | data;
 			int base = (slot.wave < 384 || !wavetblhdr) ?
 			           (slot.wave * 12) :
@@ -551,7 +571,7 @@ void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 			slot.RC   = buf[10] >> 4;
 			slot.RR   = buf[10] & 0xF;
 			slot.AM   = buf[11] & 7;
-			slot.startaddr = buf[2] | (buf[1] << 8) | 
+			slot.startaddr = buf[2] | (buf[1] << 8) |
 			                 ((buf[0] & 0x3F) << 16);
 			slot.loopaddr = buf[4] + (buf[3] << 8);
 			slot.endaddr  = (((buf[6] + (buf[5] << 8)) ^ 0xFFFF) + 1);
@@ -682,7 +702,7 @@ void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 			writeMem(memadr, data);
 			memadr = (memadr + 1) & 0xFFFFFF;
 			break;
-		
+
 		case 0xF8:
 			// TODO use these
 			fm_l = data & 0x7;
@@ -695,7 +715,7 @@ void YMF278::writeReg(byte reg, byte data, const EmuTime& time)
 			break;
 		}
 	}
-	
+
 	regs[reg] = data;
 }
 
@@ -706,7 +726,7 @@ byte YMF278::readReg(byte reg, const EmuTime& time)
 		case 2: // 3 upper bits are device ID
 			result = (regs[2] & 0x1F) | 0x20;
 			break;
-			
+
 		case 6: // Memory Data Register
 			busyTime = time + MEM_READ_DELAY;
 			result = readMem(memadr);
@@ -727,7 +747,7 @@ byte YMF278::peekReg(byte reg) const
 		case 2: // 3 upper bits are device ID
 			result = (regs[2] & 0x1F) | 0x20;
 			break;
-			
+
 		case 6: // Memory Data Register
 			result = readMem(memadr);
 			break;
@@ -752,11 +772,13 @@ byte YMF278::peekStatus(const EmuTime& time) const
 	return result;
 }
 
-YMF278::YMF278(const string& name_, int ramSize, const XMLElement& config,
-               const EmuTime& time)
-	: debugRegisters(*this), debugMemory(*this)
-	, name(name_)
-	, rom(new Rom(getName() + " ROM", "rom", config))
+YMF278::YMF278(MSXMotherBoard& motherBoard, const std::string& name, int ramSize,
+               const XMLElement& config, const EmuTime& time)
+	: SoundDevice(motherBoard.getMixer(), name, "MoonSound wave-part")
+	, rom(new Rom(motherBoard, name + " ROM", "rom", config))
+	, loadTime(time), busyTime(time)
+	, debugRegisters(new DebugRegisters(*this, motherBoard))
+	, debugMemory   (new DebugMemory   (*this, motherBoard))
 {
 	memadr = 0;	// avoid UMR
 	setSampleRate(44100);	// make valgrind happy
@@ -765,36 +787,17 @@ YMF278::YMF278(const string& name_, int ramSize, const XMLElement& config,
 	ramSize *= 1024;	// in kb
 	ram = new byte[ramSize];
 	endRam = endRom + ramSize;
-	
+
 	reset(time);
 	registerSound(config, Mixer::STEREO);
-	Debugger::instance().registerDebuggable(
-		getName() + " regs", debugRegisters );
-	Debugger::instance().registerDebuggable(
-		getName() + " mem", debugMemory );
 }
 
 YMF278::~YMF278()
 {
-	Debugger::instance().unregisterDebuggable(
-		getName() + " mem", debugMemory);
-	Debugger::instance().unregisterDebuggable(
-		getName() + " regs", debugRegisters);
 	unregisterSound();
 	delete[] ram;
 }
 
-const string& YMF278::getName() const
-{
-	return name;
-}
-
-const string& YMF278::getDescription() const
-{
-	static const string desc("MoonSound wave-part");
-	return desc;
-}
-
 void YMF278::reset(const EmuTime& time)
 {
 	eg_timer = 0;
@@ -856,61 +859,43 @@ void YMF278::writeMem(unsigned address, byte value)
 
 // class DebugRegisters
 
-YMF278::DebugRegisters::DebugRegisters(YMF278& parent_)
-	: parent(parent_)
+DebugRegisters::DebugRegisters(YMF278& ymf278_,
+                                       MSXMotherBoard& motherBoard)
+	: SimpleDebuggable(motherBoard, ymf278_.getName() + " regs",
+	                   "OPL4 registers", 0x100)
+	, ymf278(ymf278_)
 {
 }
 
-unsigned YMF278::DebugRegisters::getSize() const
+byte DebugRegisters::read(unsigned address)
 {
-	return 0x100;
+	return ymf278.peekReg(address);
 }
 
-const string& YMF278::DebugRegisters::getDescription() const
+void DebugRegisters::write(unsigned address, byte value, const EmuTime& time)
 {
-	static const string desc = "OPL4 registers";
-	return desc;
-}
-
-byte YMF278::DebugRegisters::read(unsigned address)
-{
-	return parent.peekReg(address);
-}
-
-void YMF278::DebugRegisters::write(unsigned address, byte value)
-{
-	parent.writeReg(address, value, Scheduler::instance().getCurrentTime());
+	ymf278.writeReg(address, value, time);
 }
 
 
 // class DebugMemory
 
-YMF278::DebugMemory::DebugMemory(YMF278& parent_)
-	: parent(parent_)
+DebugMemory::DebugMemory(YMF278& ymf278_, MSXMotherBoard& motherBoard)
+	: SimpleDebuggable(motherBoard, ymf278_.getName() + " mem",
+	                   "OPL4 memory (includes both ROM and RAM)", 0x400000) // 4MB
+	, ymf278(ymf278_)
 {
 }
 
-unsigned YMF278::DebugMemory::getSize() const
+byte DebugMemory::read(unsigned address)
 {
-	return 0x400000; // 4MB
+	return ymf278.readMem(address);
 }
 
-const string& YMF278::DebugMemory::getDescription() const
+void DebugMemory::write(unsigned address, byte value)
 {
-	static const string desc = "OPL4 memory (includes both ROM and RAM)";
-	return desc;
-}
-
-byte YMF278::DebugMemory::read(unsigned address)
-{
-	return parent.readMem(address);
-}
-
-void YMF278::DebugMemory::write(unsigned address, byte value)
-{
-	parent.writeMem(address, value);
+	ymf278.writeMem(address, value);
 }
 
 
 } // namespace openmsx
-
