@@ -1,4 +1,4 @@
-// $Id: Alarm.cc,v 1.6 2005/05/14 13:29:07 m9710797 Exp $
+// $Id: Alarm.cc,v 1.9 2006/02/24 20:14:22 m9710797 Exp $
 
 #include "Alarm.hh"
 #include <SDL.h>
@@ -35,11 +35,11 @@ Alarm::~Alarm()
 	assert(!pending());
 }
 
-void Alarm::schedule(unsigned interval)
+void Alarm::schedule(unsigned us)
 {
 	ScopedLock lock(sem);
 	do_cancel();
-	id = SDL_AddTimer(interval / 1000, helper, this);
+	id = SDL_AddTimer(us / 1000, helper, this);
 }
 
 void Alarm::cancel()
@@ -62,7 +62,7 @@ bool Alarm::pending() const
 	return id;
 }
 
-unsigned Alarm::helper(unsigned /*interval*/, void* param)
+unsigned Alarm::helper(unsigned interval, void* param)
 {
 	// At this position there is a race condition:
 	//   if the timer thread is suspended before the lock is taken and
@@ -70,14 +70,17 @@ unsigned Alarm::helper(unsigned /*interval*/, void* param)
 	//   free memory read when the timer thread continues.
 	// TODO If the other thread just cancels the alarm there is no problem.
 	//      (Is this correct???) TODO adjust the documentation at the top
-	
+
 	Alarm* alarm = static_cast<Alarm*>(param);
 	ScopedLock lock(alarm->sem);
 	if (alarm->id) {
-		alarm->alarm();
+		if (alarm->alarm()) {
+			// reschedule alarm
+			return interval;
+		}
 		alarm->id = 0;
 	}
-	return 0;
+	return 0; // cancel timer
 }
 
 } // namespace openmsx
