@@ -1,17 +1,28 @@
-// $Id: Thread.cc,v 1.10 2005/03/19 16:43:36 m9710797 Exp $
+// $Id: Thread.cc,v 1.13 2006/07/13 19:49:41 m9710797 Exp $
 
 #include "Thread.hh"
 #include "MSXException.hh"
 #include <iostream>
-#include <cstddef>
 #include <cassert>
 #include <SDL_thread.h>
 
-using std::cerr;
-using std::endl;
-
 namespace openmsx {
 
+static unsigned mainThreadId = (unsigned)-1;
+
+void Thread::setMainThread()
+{
+	assert(mainThreadId == (unsigned)-1);
+	mainThreadId = SDL_ThreadID();
+}
+
+bool Thread::isMainThread()
+{
+	assert(mainThreadId != (unsigned)-1);
+	return mainThreadId == SDL_ThreadID();
+}
+
+
 Thread::Thread(Runnable* runnable_)
 	: runnable(runnable_)
 {
@@ -43,9 +54,10 @@ void Thread::stop()
 //       within the given timeout.
 void Thread::join()
 {
-	assert(thread);
-	SDL_WaitThread(thread, NULL);
-	thread = NULL;
+	if (thread) {
+		SDL_WaitThread(thread, NULL);
+		thread = NULL;
+	}
 }
 
 int Thread::startThread(void* runnable)
@@ -53,12 +65,12 @@ int Thread::startThread(void* runnable)
 	try {
 		static_cast<Runnable*>(runnable)->run();
 	} catch (FatalError& e) {
-		cerr << "Fatal error in subthread: "
-		     << e.getMessage() << endl;
+		std::cerr << "Fatal error in subthread: "
+		     << e.getMessage() << std::endl;
 		assert(false);
 	} catch (MSXException& e) {
-		cerr << "Uncaught exception in subthread: "
-		     << e.getMessage() << endl;
+		std::cerr << "Uncaught exception in subthread: "
+		     << e.getMessage() << std::endl;
 		assert(false);
 	} // don't catch(..), thread cancelation seems to depend on it
 	return 0;
