@@ -1,7 +1,8 @@
-// $Id: GLImage.cc,v 1.7 2005/03/17 23:15:25 m9710797 Exp $
+// $Id: GLImage.cc,v 1.9 2006/03/20 20:26:10 mthuurne Exp $
 
 #include "GLImage.hh"
 #include "MSXException.hh"
+#include "Math.hh"
 #include "SDLImage.hh"
 #include <SDL_image.h>
 #include <SDL.h>
@@ -15,6 +16,14 @@ GLImage::GLImage(SDL_Surface* /*output*/, const string& filename)
 	texture = loadTexture(filename, width, height, texCoord);
 }
 
+GLImage::GLImage(SDL_Surface* /*output*/, const string& filename,
+                 double scalefactor)
+{
+	texture = loadTexture(filename, width, height, texCoord);
+	width  = static_cast<unsigned>(scalefactor * width);
+	height = static_cast<unsigned>(scalefactor * height);
+}
+
 GLImage::~GLImage()
 {
 	glDeleteTextures(1, &texture);
@@ -39,16 +48,6 @@ void GLImage::draw(unsigned x, unsigned y, unsigned char alpha)
 	glPopAttrib();
 }
 
-
-static int powerOfTwo(int a)
-{
-	int res = 1;
-	while (a > res) {
-		res <<= 1;
-	}
-	return res;
-}
-
 GLuint GLImage::loadTexture(const string& filename,
 	unsigned& width, unsigned& height, GLfloat* texCoord)
 {
@@ -59,8 +58,8 @@ GLuint GLImage::loadTexture(const string& filename,
 
 	width  = image1->w;
 	height = image1->h;
-	int w2 = powerOfTwo(width);
-	int h2 = powerOfTwo(height);
+	int w2 = Math::powerOfTwo(width);
+	int h2 = Math::powerOfTwo(height);
 	texCoord[0] = 0.0f;			// Min X
 	texCoord[1] = 0.0f;			// Min Y
 	texCoord[2] = (GLfloat)width  / w2;	// Max X
