@@ -1,19 +1,16 @@
-// $Id: GLSnow.cc,v 1.8 2005/03/17 23:15:26 m9710797 Exp $
+// $Id: GLSnow.cc,v 1.12 2006/06/29 20:21:13 m9710797 Exp $
 
 #include "GLSnow.hh"
-#include "Display.hh"
+#include "openmsx.hh"
 #include <cstdlib>
 
 using std::string;
 
 namespace openmsx {
 
-// Dimensions of screen.
-static const int WIDTH = 640;
-static const int HEIGHT = 480;
-
-GLSnow::GLSnow()
+GLSnow::GLSnow(unsigned width_, unsigned height_)
 	: Layer(COVER_FULL, Z_BACKGROUND)
+	, width(width_), height(height_)
 {
 	// Create noise texture.
 	byte buf[128 * 128];
@@ -26,9 +23,6 @@ GLSnow::GLSnow()
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8, 128, 128, 0,
 	             GL_LUMINANCE, GL_UNSIGNED_BYTE, buf);
-
-	// Register as display layer.
-	Display::instance().addLayer(this);
 }
 
 GLSnow::~GLSnow()
@@ -39,20 +33,39 @@ GLSnow::~GLSnow()
 
 void GLSnow::paint()
 {
+	// Rotate and mirror noise texture in consecutive frames to avoid
+	// seeing 'patterns' in the noise.
+	static const int coord[8][4][2] = {
+		{ { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } },
+		{ { 0, 1 }, { 1, 1 }, { 1, 0 }, { 0, 0 } },
+		{ { 0, 1 }, { 0, 0 }, { 1, 0 }, { 1, 1 } },
+		{ { 1, 1 }, { 1, 0 }, { 0, 0 }, { 0, 1 } },
+		{ { 1, 1 }, { 0, 1 }, { 0, 0 }, { 1, 0 } },
+		{ { 1, 0 }, { 0, 0 }, { 0, 1 }, { 1, 1 } },
+		{ { 1, 0 }, { 1, 1 }, { 0, 1 }, { 0, 0 } },
+		{ { 0, 0 }, { 0, 1 }, { 1, 1 }, { 1, 0 } }
+	};
+	static unsigned cnt = 0;
+
 	// Draw noise texture.
 	double x = (double)rand() / RAND_MAX;
 	double y = (double)rand() / RAND_MAX;
+	cnt = (cnt + 1) % 8;
 	glEnable(GL_TEXTURE_2D);
 	glBindTexture(GL_TEXTURE_2D, noiseTextureId);
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	glBegin(GL_QUADS);
-	glTexCoord2f(0.0f + x, 1.5f + y); glVertex2i(   0, HEIGHT - 512);
-	glTexCoord2f(3.0f + x, 1.5f + y); glVertex2i(1024, HEIGHT - 512);
-	glTexCoord2f(3.0f + x, 0.0f + y); glVertex2i(1024, HEIGHT);
-	glTexCoord2f(0.0f + x, 0.0f + y); glVertex2i(   0, HEIGHT);
+	glTexCoord2f(0.0f + x, 2.0f + y);
+	glVertex2i(coord[cnt][0][0] * width, coord[cnt][0][1] * height);
+	glTexCoord2f(2.0f + x, 2.0f + y);
+	glVertex2i(coord[cnt][1][0] * width, coord[cnt][1][1] * height);
+	glTexCoord2f(2.0f + x, 0.0f + y);
+	glVertex2i(coord[cnt][2][0] * width, coord[cnt][2][1] * height);
+	glTexCoord2f(0.0f + x, 0.0f + y);
+	glVertex2i(coord[cnt][3][0] * width, coord[cnt][3][1] * height);
 	glEnd();
 	glDisable(GL_TEXTURE_2D);
-	
+
 	// TODO: Mark dirty in 100ms.
 }
 
