@@ -1,4 +1,4 @@
-// $Id: Layer.cc,v 1.1 2005/01/04 10:34:08 m9710797 Exp $
+// $Id: Layer.cc,v 1.3 2006/07/23 23:22:24 mthuurne Exp $
 
 #include "Layer.hh"
 
@@ -16,13 +16,12 @@ Layer::~Layer()
 void Layer::setCoverage(Coverage coverage_)
 {
 	coverage = coverage_;
-	if (display) display->updateCoverage(this, coverage);
 }
 
 void Layer::setZ(ZIndex z_)
 {
 	z = z_;
-	if (display) display->updateZ(this, z);
+	if (display) display->updateZ(*this);
 }
 
 Layer::ZIndex Layer::getZ() const
