@@ -1,4 +1,4 @@
-// $Id: PixelRenderer.cc,v 1.71 2005/04/09 15:14:17 m9710797 Exp $
+// $Id: PixelRenderer.cc,v 1.82 2006/02/23 20:57:20 m9710797 Exp $
 
 /*
 TODO:
@@ -18,10 +18,10 @@ TODO:
 #include "EventDistributor.hh"
 #include "FinishFrameEvent.hh"
 #include "RealTime.hh"
+#include "MSXMotherBoard.hh"
 #include "Timer.hh"
 #include <algorithm>
 #include <cassert>
-#include <sstream>
 
 namespace openmsx {
 
@@ -39,11 +39,11 @@ void PixelRenderer::draw(
 		assert(drawType == DRAW_DISPLAY);
 
 		// Calculate display coordinates.
-		int zero = vdp->getLineZero();
-		int displayX = (startX - vdp->getLeftSprites()) / 2;
+		int zero = vdp.getLineZero();
+		int displayX = (startX - vdp.getLeftSprites()) / 2;
 		int displayY = startY - zero;
-		if (!vdp->getDisplayMode().isTextMode()) {
-			displayY += vdp->getVerticalScroll();
+		if (!vdp.getDisplayMode().isTextMode()) {
+			displayY += vdp.getVerticalScroll();
 		} else {
 			// this is not what the real VDP does, but it is good
 			// enough for "Boring scroll" demo part of "Relax"
@@ -64,10 +64,10 @@ void PixelRenderer::draw(
 
 		rasterizer->drawDisplay(
 			startX, startY,
-			displayX - vdp->getHorizontalScrollLow() * 2, displayY,
+			displayX - vdp.getHorizontalScrollLow() * 2, displayY,
 			displayWidth, displayHeight
 			);
-		if (vdp->spritesEnabled()) {
+		if (vdp.spritesEnabled()) {
 			rasterizer->drawSprites(
 				startX, startY,
 				displayX / 2, displayY,
@@ -109,33 +109,35 @@ void PixelRenderer::subdivide(
 	if (drawLast) draw(clipL, endY, endX, endY + 1, drawType, false);
 }
 
-PixelRenderer::PixelRenderer(VDP* vdp)
+PixelRenderer::PixelRenderer(VDP& vdp_, Display& display)
+	: vdp(vdp_), vram(vdp.getVRAM())
+	, eventDistributor(vdp.getMotherBoard().getEventDistributor())
+	, realTime(vdp.getMotherBoard().getRealTime())
+	, renderSettings(display.getRenderSettings())
+	, spriteChecker(vdp.getSpriteChecker())
+	, rasterizer(display.getVideoSystem().createRasterizer(vdp))
 {
-	this->vdp = vdp;
-	vram = vdp->getVRAM();
-	spriteChecker = vdp->getSpriteChecker();
-	rasterizer = Display::instance().getVideoSystem().createRasterizer(vdp);
-
-	frameSkipCounter = 999; // force drawing of frame
 	finishFrameDuration = 0;
-	prevDrawFrame = drawFrame = renderFrame = false; // don't draw before frameStart is called
-	displayEnabled = vdp->isDisplayEnabled();
+	frameSkipCounter = 999; // force drawing of frame
+	renderFrame = false; // don't draw before frameStart is called
+	prevRenderFrame = false;
+	displayEnabled = vdp.isDisplayEnabled();
 	rasterizer->reset();
 
-	settings.getMaxFrameSkip()->addListener(this);
-	settings.getMinFrameSkip()->addListener(this);
+	renderSettings.getMaxFrameSkip().attach(*this);
+	renderSettings.getMinFrameSkip().attach(*this);
 }
 
 PixelRenderer::~PixelRenderer()
 {
-	settings.getMinFrameSkip()->removeListener(this);
-	settings.getMaxFrameSkip()->removeListener(this);
+	renderSettings.getMinFrameSkip().detach(*this);
+	renderSettings.getMaxFrameSkip().detach(*this);
 }
 
 void PixelRenderer::reset(const EmuTime& time)
 {
 	rasterizer->reset();
-	displayEnabled = vdp->isDisplayEnabled();
+	displayEnabled = vdp.isDisplayEnabled();
 	frameStart(time);
 }
 
@@ -147,33 +149,39 @@ void PixelRenderer::updateDisplayEnabled(bool enabled, const EmuTime& time)
 
 void PixelRenderer::frameStart(const EmuTime& time)
 {
-	bool draw = false;
-	if (rasterizer->getZ() == Layer::Z_MSX_PASSIVE) {
-		// V9990 is active
-		frameSkipCounter = 0;
-	} else if (frameSkipCounter < settings.getMinFrameSkip()->getValue()) {
-		++frameSkipCounter;
-	} else if (frameSkipCounter >= settings.getMaxFrameSkip()->getValue()) {
-		frameSkipCounter = 0;
-		draw = true;
+	if (!rasterizer->isActive()) {
+		frameSkipCounter = 999;
+		renderFrame = false;
+		prevRenderFrame = false;
+		return;
+	}
+	prevRenderFrame = renderFrame;
+	if (vdp.isInterlaced() && renderSettings.getDeinterlace().getValue() &&
+	    vdp.getEvenOdd() && vdp.isEvenOddEnabled()) {
+		// deinterlaced odd frame, do same as even frame
 	} else {
-		++frameSkipCounter;
-		draw = RealTime::instance().timeLeft(
-			(unsigned)finishFrameDuration, time);
-		if (draw) {
+		if (frameSkipCounter <
+		              renderSettings.getMinFrameSkip().getValue()) {
+			++frameSkipCounter;
+			renderFrame = false;
+		} else if (frameSkipCounter >=
+		              renderSettings.getMaxFrameSkip().getValue()) {
 			frameSkipCounter = 0;
+			renderFrame = true;
+		} else {
+			++frameSkipCounter;
+			renderFrame = realTime.timeLeft(
+				(unsigned)finishFrameDuration, time);
+			if (renderFrame) {
+				frameSkipCounter = 0;
+			}
 		}
 	}
-	prevDrawFrame = drawFrame;
-	drawFrame = draw;
-	renderFrame = drawFrame ||
-	     (prevDrawFrame && vdp->isInterlaced() &&
-	      RenderSettings::instance().getDeinterlace()->getValue());
 	if (!renderFrame) return;
 
 	rasterizer->frameStart();
 
-	accuracy = settings.getAccuracy()->getValue();
+	accuracy = renderSettings.getAccuracy().getValue();
 
 	nextX = 0;
 	nextY = 0;
@@ -184,6 +192,7 @@ void PixelRenderer::frameStart(const EmuTime& time)
 
 void PixelRenderer::frameEnd(const EmuTime& time)
 {
+	bool skipEvent = !renderFrame;
 	if (renderFrame) {
 		// Render changes from this last frame.
 		sync(time, true);
@@ -197,11 +206,16 @@ void PixelRenderer::frameEnd(const EmuTime& time)
 		finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
 		                      current * ALPHA;
 
-		if (drawFrame) {
-			FinishFrameEvent* f = new FinishFrameEvent(VIDEO_MSX);
-			EventDistributor::instance().distributeEvent(f);
+		if (vdp.isInterlaced() && vdp.isEvenOddEnabled() &&
+		    renderSettings.getDeinterlace().getValue() &&
+		    !prevRenderFrame) {
+			// dont send event in deinterlace mode when
+			// previous frame was not rendered
+			skipEvent = true;
 		}
 	}
+	eventDistributor.distributeEvent(
+		new FinishFrameEvent(VIDEO_MSX, skipEvent));
 }
 
 void PixelRenderer::updateHorizontalScrollLow(
@@ -245,7 +259,7 @@ void PixelRenderer::updateBackgroundColour(
 	int colour, const EmuTime& time)
 {
 	sync(time);
-	if (vdp->getDisplayMode().getByte() != DisplayMode::GRAPHIC7) {
+	if (vdp.getDisplayMode().getByte() != DisplayMode::GRAPHIC7) {
 		rasterizer->setBackgroundColour(colour);
 	}
 }
@@ -279,14 +293,14 @@ void PixelRenderer::updatePalette(
 		sync(time);
 	} else {
 		// Only sync if border colour changed.
-		DisplayMode mode = vdp->getDisplayMode();
+		DisplayMode mode = vdp.getDisplayMode();
 		if (mode.getBase() == DisplayMode::GRAPHIC5) {
-			int bgColour = vdp->getBackgroundColour();
+			int bgColour = vdp.getBackgroundColour();
 			if (index == (bgColour & 3) || (index == (bgColour >> 2))) {
 				sync(time);
 			}
 		} else if (mode.getByte() != DisplayMode::GRAPHIC7) {
-			if (index == vdp->getBackgroundColour()) {
+			if (index == vdp.getBackgroundColour()) {
 				sync(time);
 			}
 		}
@@ -310,7 +324,7 @@ void PixelRenderer::updateDisplayMode(
 	DisplayMode mode, const EmuTime& time)
 {
 	// Sync if in display area or if border drawing process changes.
-	DisplayMode oldMode = vdp->getDisplayMode();
+	DisplayMode oldMode = vdp.getDisplayMode();
 	if (displayEnabled
 	|| oldMode.getByte() == DisplayMode::GRAPHIC5
 	|| oldMode.getByte() == DisplayMode::GRAPHIC7
@@ -378,17 +392,17 @@ inline bool PixelRenderer::checkSync(int offset, const EmuTime& time)
 	// Calculate what display lines are scanned between current
 	// renderer time and update-to time.
 	// Note: displayY1 is inclusive.
-	int deltaY = vdp->getVerticalScroll() - vdp->getLineZero();
-	int limitY = vdp->getTicksThisFrame(time) / VDP::TICKS_PER_LINE;
+	int deltaY = vdp.getVerticalScroll() - vdp.getLineZero();
+	int limitY = vdp.getTicksThisFrame(time) / VDP::TICKS_PER_LINE;
 	int displayY0 = (nextY + deltaY) & 255;
 	int displayY1 = (limitY + deltaY) & 255;
 
-	switch(vdp->getDisplayMode().getBase()) {
+	switch(vdp.getDisplayMode().getBase()) {
 	case DisplayMode::GRAPHIC2:
 	case DisplayMode::GRAPHIC3:
-		if (vram->colourTable.isInside(offset)) {
+		if (vram.colourTable.isInside(offset)) {
 			int vramQuarter = (offset & 0x1800) >> 11;
-			int mask = (vram->colourTable.getMask() & 0x1800) >> 11;
+			int mask = (vram.colourTable.getMask() & 0x1800) >> 11;
 			for (int i = 0; i < 4; i++) {
 				if ( (i & mask) == vramQuarter
 				&& overlap(displayY0, displayY1, i * 64, (i + 1) * 64) ) {
@@ -400,9 +414,9 @@ inline bool PixelRenderer::checkSync(int offset, const EmuTime& time)
 				}
 			}
 		}
-		if (vram->patternTable.isInside(offset)) {
+		if (vram.patternTable.isInside(offset)) {
 			int vramQuarter = (offset & 0x1800) >> 11;
-			int mask = (vram->patternTable.getMask() & 0x1800) >> 11;
+			int mask = (vram.patternTable.getMask() & 0x1800) >> 11;
 			for (int i = 0; i < 4; i++) {
 				if ( (i & mask) == vramQuarter
 				&& overlap(displayY0, displayY1, i * 64, (i + 1) * 64) ) {
@@ -414,7 +428,7 @@ inline bool PixelRenderer::checkSync(int offset, const EmuTime& time)
 				}
 			}
 		}
-		if (vram->nameTable.isInside(offset)) {
+		if (vram.nameTable.isInside(offset)) {
 			int vramLine = ((offset & 0x3FF) / 32) * 8;
 			if (overlap(displayY0, displayY1, vramLine, vramLine + 8)) {
 				/*fprintf(stderr,
@@ -429,9 +443,9 @@ inline bool PixelRenderer::checkSync(int offset, const EmuTime& time)
 	case DisplayMode::GRAPHIC5: {
 		// Is the address inside the visual page(s)?
 		// TODO: Also look at which lines are touched inside pages.
-		int visiblePage = vram->nameTable.getMask()
-			& (0x10000 | (vdp->getEvenOddMask() << 7));
-		if (vdp->isMultiPageScrolling()) {
+		int visiblePage = vram.nameTable.getMask()
+			& (0x10000 | (vdp.getEvenOddMask() << 7));
+		if (vdp.isMultiPageScrolling()) {
 			return (offset & 0x18000) == visiblePage
 				|| (offset & 0x18000) == (visiblePage & 0x10000);
 		} else {
@@ -443,9 +457,9 @@ inline bool PixelRenderer::checkSync(int offset, const EmuTime& time)
 		return true; // TODO: Implement better detection.
 	default:
 		// Range unknown; assume full range.
-		return vram->nameTable.isInside(offset)
-			|| vram->colourTable.isInside(offset)
-			|| vram->patternTable.isInside(offset);
+		return vram.nameTable.isInside(offset)
+			|| vram.colourTable.isInside(offset)
+			|| vram.patternTable.isInside(offset);
 	}
 }
 
@@ -456,7 +470,7 @@ void PixelRenderer::updateVRAM(unsigned offset, const EmuTime& time)
 	if (renderFrame && displayEnabled && checkSync(offset, time)) {
 		/*
 		fprintf(stderr, "vram sync @ line %d\n",
-			vdp->getTicksThisFrame(time) / VDP::TICKS_PER_LINE
+			vdp.getTicksThisFrame(time) / VDP::TICKS_PER_LINE
 			);
 		*/
 		renderUntil(time);
@@ -489,7 +503,7 @@ void PixelRenderer::sync(const EmuTime& time, bool force)
 	//       scheme at a higher level. Probably. But how...
 	//if ((frameSkipCounter == 0) && TODO
 	if (accuracy != RenderSettings::ACC_SCREEN || force) {
-		vram->sync(time);
+		vram.sync(time);
 		renderUntil(time);
 	}
 }
@@ -497,8 +511,8 @@ void PixelRenderer::sync(const EmuTime& time, bool force)
 void PixelRenderer::renderUntil(const EmuTime& time)
 {
 	// Translate from time to pixel position.
-	int limitTicks = vdp->getTicksThisFrame(time);
-	assert(limitTicks <= vdp->getTicksPerFrame());
+	int limitTicks = vdp.getTicksThisFrame(time);
+	assert(limitTicks <= vdp.getTicksPerFrame());
 	int limitX, limitY;
 	switch (accuracy) {
 	case RenderSettings::ACC_PIXEL: {
@@ -530,9 +544,9 @@ void PixelRenderer::renderUntil(const EmuTime& time)
 	if (limitX == nextX && limitY == nextY) return;
 
 	if (displayEnabled) {
-		if (vdp->spritesEnabled()) {
+		if (vdp.spritesEnabled()) {
 			// Update sprite checking, so that rasterizer can call getSprites.
-			spriteChecker->checkUntil(time);
+			spriteChecker.checkUntil(time);
 		}
 
 		// Calculate start and end of borders in ticks since start of line.
@@ -541,10 +555,10 @@ void PixelRenderer::renderUntil(const EmuTime& time)
 		// but sprites above these pixels are clipped at the actual border
 		// rather than the end of the border coloured area.
 		// TODO: Move these calculations and getDisplayLeft() to VDP.
-		int borderL = vdp->getLeftBorder();
+		int borderL = vdp.getLeftBorder();
 		int displayL =
-			vdp->isBorderMasked() ? borderL : vdp->getLeftBackground();
-		int borderR = vdp->getRightBorder();
+			vdp.isBorderMasked() ? borderL : vdp.getLeftBackground();
+		int borderR = vdp.getRightBorder();
 
 		// Left border.
 		subdivide(nextX, nextY, limitX, limitY,
@@ -564,10 +578,10 @@ void PixelRenderer::renderUntil(const EmuTime& time)
 	nextY = limitY;
 }
 
-void PixelRenderer::update(const Setting* setting)
+void PixelRenderer::update(const Setting& setting)
 {
-	if (setting == settings.getMinFrameSkip()
-	|| setting == settings.getMaxFrameSkip() ) {
+	if (&setting == &renderSettings.getMinFrameSkip()
+	|| &setting == &renderSettings.getMaxFrameSkip() ) {
 		// Force drawing of frame.
 		frameSkipCounter = 999;
 	} else {
