@@ -1,11 +1,11 @@
-// $Id: SDLImage.cc,v 1.7 2005/03/12 20:47:17 m9710797 Exp $
+// $Id: SDLImage.cc,v 1.13 2005/11/13 21:05:52 m9710797 Exp $
 
 #include "SDLImage.hh"
 #include "File.hh"
-#include "FileException.hh"
-#include "CliComm.hh"
+#include "MSXException.hh"
 #include <SDL_image.h>
 #include <SDL.h>
+#include <cassert>
 
 using std::string;
 
@@ -18,16 +18,24 @@ SDLImage::SDLImage(SDL_Surface* output, const string& filename)
 	init(filename);
 }
 
-SDLImage::SDLImage(SDL_Surface* output, const string& filename,
-	           unsigned width, unsigned height, byte defaultAlpha)
+SDLImage::SDLImage(SDL_Surface* output, const std::string& filename,
+                   double scaleFactor)
 	: outputScreen(output)
 {
-	image = loadImage(filename, width, height, defaultAlpha);
+	image = loadImage(filename, scaleFactor);
+	init(filename);
+}
+
+SDLImage::SDLImage(SDL_Surface* output, const string& filename,
+	           unsigned width, unsigned height)
+	: outputScreen(output)
+{
+	image = loadImage(filename, width, height);
 	init(filename);
 }
 
 SDLImage::SDLImage(SDL_Surface* output, unsigned width, unsigned height,
-                   byte defaultAlpha)
+                   byte alpha)
 	: outputScreen(output)
 {
 	SDL_PixelFormat* format = outputScreen->format;
@@ -35,16 +43,14 @@ SDLImage::SDLImage(SDL_Surface* output, unsigned width, unsigned height,
 		width, height, format->BitsPerPixel,
 		format->Rmask, format->Gmask, format->Bmask, 0);
 	if (image) {
-		SDL_SetAlpha(image, SDL_SRCALPHA, defaultAlpha);
+		SDL_SetAlpha(image, SDL_SRCALPHA, alpha);
 	}
 	init("");
 }
 
 void SDLImage::init(const string& filename)
 {
-	if (!image) {
-		throw MSXException("Error loading image " + filename);
-	}
+	assert(image);
 	SDL_PixelFormat* format = image->format;
 	workImage = SDL_CreateRGBSurface(SDL_SWSURFACE,
 		image->w, image->h, format->BitsPerPixel,
@@ -84,30 +90,35 @@ void SDLImage::draw(unsigned x, unsigned y, byte alpha)
 }
 
 
-SDL_Surface* SDLImage::loadImage(const string& filename, byte defaultAlpha)
+SDL_Surface* SDLImage::loadImage(const string& filename)
 {
 	SDL_Surface* picture = readImage(filename);
-	if (picture == NULL) {
-		return NULL;
-	}
-
-	SDL_Surface* result = convertToDisplayFormat(picture, defaultAlpha);
+	SDL_Surface* result = convertToDisplayFormat(picture);
 	SDL_FreeSurface(picture);
 	return result;
 }
 
-SDL_Surface* SDLImage::loadImage(const string& filename,
-	unsigned width, unsigned height, byte defaultAlpha)
+SDL_Surface* SDLImage::loadImage(const string& filename, double scaleFactor)
 {
 	SDL_Surface* picture = readImage(filename);
-	if (picture == NULL) {
-		return NULL;
-	}
+	SDL_Surface* scaled = scaleImage32(picture,
+		static_cast<unsigned>(picture->w * scaleFactor),
+		static_cast<unsigned>(picture->h * scaleFactor));
+	SDL_FreeSurface(picture);
 
+	SDL_Surface* result = convertToDisplayFormat(scaled);
+	SDL_FreeSurface(scaled);
+	return result;
+}
+
+SDL_Surface* SDLImage::loadImage(const string& filename,
+                                 unsigned width, unsigned height)
+{
+	SDL_Surface* picture = readImage(filename);
 	SDL_Surface* scaled = scaleImage32(picture, width, height);
 	SDL_FreeSurface(picture);
 
-	SDL_Surface* result = convertToDisplayFormat(scaled, defaultAlpha);
+	SDL_Surface* result = convertToDisplayFormat(scaled);
 	SDL_FreeSurface(scaled);
 	return result;
 }
@@ -127,21 +138,13 @@ SDL_Surface* SDLImage::loadImage(const string& filename,
 
 SDL_Surface* SDLImage::readImage(const string& filename)
 {
-	// If file does exist, but cannot be read as an image,
-	// IMG_Load returns NULL.
-	try {
-		File file(filename);
-		SDL_Surface* result = IMG_Load(file.getLocalName().c_str());
-		if (result == NULL) {
-			CliComm::instance().printWarning("File \"" +
-			        file.getURL() + "\" is not a valid image");
-		}
-		return result;
-	} catch (FileException& e) {
-		CliComm::instance().printWarning("Could not open file \"" +
-		        filename + "\": " + e.getMessage());
-		return NULL;
+	File file(filename);
+	SDL_Surface* result = IMG_Load(file.getLocalName().c_str());
+	if (!result) {
+		throw MSXException("File \"" + file.getURL() +
+		                   "\" is not a valid image");
 	}
+	return result;
 }
 
 SDL_Surface* SDLImage::scaleImage32(
@@ -157,8 +160,7 @@ SDL_Surface* SDLImage::scaleImage32(
 	return result;
 }
 
-SDL_Surface* SDLImage::convertToDisplayFormat(
-	SDL_Surface* input, byte defaultAlpha)
+SDL_Surface* SDLImage::convertToDisplayFormat(SDL_Surface* input)
 {
 	// scan image, are all alpha values the same?
 	const char* pixels = (char*)input->pixels;
@@ -167,7 +169,8 @@ SDL_Surface* SDLImage::convertToDisplayFormat(
 	unsigned pitch = input->pitch;
 	unsigned Amask = input->format->Amask;
 	unsigned bytes = input->format->BytesPerPixel;
-	unsigned alpha = *((Uint32*)pixels) & Amask;
+	unsigned pixel = *((Uint32*)pixels);
+	unsigned alpha = pixel & Amask;
 
 	bool constant = true;
 	for (unsigned y = 0; y < height; ++y) {
@@ -185,7 +188,9 @@ SDL_Surface* SDLImage::convertToDisplayFormat(
 	// convert the background to the right format
 	SDL_Surface* result;
 	if (constant) {
-		SDL_SetAlpha(input, SDL_SRCALPHA, defaultAlpha);
+		Uint8 r, g, b, a;
+		SDL_GetRGBA(pixel, input->format, &r, &g, &b, &a);
+		SDL_SetAlpha(input, SDL_SRCALPHA, a);
 		result = SDL_DisplayFormat(input);
 	} else {
 		result = SDL_DisplayFormatAlpha(input);
