@@ -1,11 +1,9 @@
-// $Id: SDLSnow.cc,v 1.10 2005/05/09 17:38:37 m9710797 Exp $
+// $Id: SDLSnow.cc,v 1.13 2005/11/13 21:05:52 m9710797 Exp $
 
 #include "SDLSnow.hh"
-#include "Display.hh"
+#include "openmsx.hh"
 #include <SDL.h>
 
-using std::string;
-
 namespace openmsx {
 
 template <class Pixel>
@@ -17,14 +15,6 @@ SDLSnow<Pixel>::SDLSnow(SDL_Surface* screen)
 	for (unsigned i = 0; i < 256; i++) {
 		gray[i] = SDL_MapRGB(screen->format, i, i, i);
 	}
-
-	// Register as display layer.
-	Display::instance().addLayer(this);
-}
-
-template <class Pixel>
-SDLSnow<Pixel>::~SDLSnow()
-{
 }
 
 // random routine, less random than libc rand(), but a lot faster
@@ -61,9 +51,9 @@ void SDLSnow<Pixel>::paint()
 }
 
 template <class Pixel>
-const string& SDLSnow<Pixel>::getName()
+const std::string& SDLSnow<Pixel>::getName()
 {
-	static const string NAME = "SDLSnow";
+	static const std::string NAME = "SDLSnow";
 	return NAME;
 }
 
