@@ -1,4 +1,4 @@
-// $Id: SaI2xScaler.cc,v 1.8 2005/05/09 17:38:37 m9710797 Exp $
+// $Id: SaI2xScaler.cc,v 1.23 2006/01/06 14:06:16 m9710797 Exp $
 
 // 2xSaI is Copyright (c) 1999-2001 by Derek Liauw Kie Fa.
 //   http://elektron.its.tudelft.nl/~dalikifa/
@@ -7,40 +7,71 @@
 // Modified for use in openMSX by Maarten ter Huurne.
 
 #include "SaI2xScaler.hh"
+#include "FrameSource.hh"
+#include "OutputSurface.hh"
+#include "MemoryOps.hh"
 #include "openmsx.hh"
 #include <cassert>
 
-using std::min;
-using std::max;
-
 namespace openmsx {
 
 template <class Pixel>
-SaI2xScaler<Pixel>::SaI2xScaler(SDL_PixelFormat* format)
-	: blender(Blender<Pixel>::createFromFormat(format))
+SaI2xScaler<Pixel>::SaI2xScaler(const PixelOperations<Pixel>& pixelOps_)
+	: Scaler2<Pixel>(pixelOps_)
+	, pixelOps(pixelOps_)
 {
 }
 
-const int WIDTH256 = 320; // TODO: Specify this in a clean way.
-const int HEIGHT = 480;
+template <class Pixel>
+void SaI2xScaler<Pixel>::scaleBlank1to2(
+		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
+		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
+{
+	unsigned stopDstY = (dstEndY == dst.getHeight())
+	                  ? dstEndY : dstEndY - 2;
+	unsigned srcY = srcStartY, dstY = dstStartY;
+	for (/* */; dstY < stopDstY; srcY += 1, dstY += 2) {
+		Pixel* dummy = 0;
+		Pixel color = src.getLinePtr(srcY, dummy)[0];
+		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
+			dstLine0, dst.getWidth(), color);
+		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		MemoryOps::memset<Pixel, MemoryOps::STREAMING>(
+			dstLine1, dst.getWidth(), color);
+	}
+	if (dstY != dst.getHeight()) {
+		unsigned nextLineWidth = src.getLineWidth(srcY + 1);
+		assert(src.getLineWidth(srcY) == 1);
+		assert(nextLineWidth != 1);
+		this->scaleImage(src, srcY, srcEndY, nextLineWidth,
+		                 dst, dstY, dstEndY);
+	}
+}
 
 template <class Pixel>
-void SaI2xScaler<Pixel>::scaleLine256(
+inline Pixel SaI2xScaler<Pixel>::blend(Pixel p1, Pixel p2)
+{
+	return pixelOps.template blend<1, 1>(p1, p2);
+}
+
+template <class Pixel>
+void SaI2xScaler<Pixel>::scaleLine1on2(
 	const Pixel* srcLine0, const Pixel* srcLine1,
 	const Pixel* srcLine2, const Pixel* srcLine3,
-	Pixel* dstUpper, Pixel* dstLower )
+	Pixel* dstUpper, Pixel* dstLower, unsigned srcWidth)
 {
 	// TODO: Scale border pixels as well.
-	for (int x = 0; x < WIDTH256; x++) {
+	for (unsigned x = 0; x < srcWidth; x++) {
 		// Map of the pixels:
 		//   I|E F|J
 		//   G|A B|K
 		//   H|C D|L
 		//   M|N O|P
 
-		int xl = max(x - 1, 0);
-		int xr = min(x + 1, WIDTH256 - 1);
-		int xrr = min(x + 2, WIDTH256 - 1);
+		unsigned xl  = x ? x - 1 : 0;
+		unsigned xr  = std::min(x + 1, srcWidth - 1);
+		unsigned xrr = std::min(x + 2, srcWidth - 1);
 
 		// TODO: Possible performance improvements:
 		// - Play with order of fetching (effect on data cache).
@@ -75,7 +106,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				       && colorB != colorE && colorB == colorJ )
 				  )
 				? colorA
-				: blender.blend(colorA, colorB)
+				: blend(colorA, colorB)
 				);
 			product1 =
 				( ( (colorA == colorG && colorC == colorO)
@@ -83,7 +114,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				       && colorG != colorC && colorC == colorM )
 				  )
 				? colorA
-				: blender.blend(colorA, colorC)
+				: blend(colorA, colorC)
 				);
 			product2 = colorA;
 		} else if (colorB == colorC && colorA != colorD) {
@@ -93,7 +124,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				       && colorA != colorF && colorA == colorI )
 				  )
 				? colorB
-				: blender.blend(colorA, colorB)
+				: blend(colorA, colorB)
 				);
 			product1 =
 				( ( (colorC == colorH && colorA == colorF)
@@ -101,7 +132,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				       && colorA != colorH && colorA == colorI )
 				  )
 				? colorC
-				: blender.blend(colorA, colorC)
+				: blend(colorA, colorC)
 				);
 			product2 = colorB;
 		} else if (colorA == colorD && colorB == colorC) {
@@ -121,7 +152,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				if (colorL == colorO) {
 					if (colorA == colorL) r--; else if (colorB == colorL) r++;
 				}
-				product = product1 = blender.blend(colorA, colorB);
+				product = product1 = blend(colorA, colorB);
 				product2 = r > 0 ? colorA : (r < 0 ? colorB : product);
 			}
 		} else {
@@ -132,7 +163,7 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				: ( colorB == colorE && colorB == colorD
 				    && colorA != colorF && colorA == colorI
 				  ? colorB
-				  : blender.blend(colorA, colorB)
+				  : blend(colorA, colorB)
 				  )
 				);
 			product1 =
@@ -142,12 +173,12 @@ void SaI2xScaler<Pixel>::scaleLine256(
 				: ( colorC == colorG && colorC == colorD
 				    && colorA != colorH && colorA == colorI
 				  ? colorC
-				  : blender.blend(colorA, colorC)
+				  : blend(colorA, colorC)
 				  )
 				);
-			product2 = blender.blend( // TODO: Quad-blend may be better?
-				blender.blend(colorA, colorB),
-				blender.blend(colorC, colorD)
+			product2 = blend( // TODO: Quad-blend may be better?
+				blend(colorA, colorB),
+				blend(colorC, colorD)
 				);
 		}
 
@@ -158,20 +189,18 @@ void SaI2xScaler<Pixel>::scaleLine256(
 	}
 }
 
-const int WIDTH512 = 640; // TODO: Specify this in a clean way.
-
 template <class Pixel>
-void SaI2xScaler<Pixel>::scaleLine512(
+void SaI2xScaler<Pixel>::scaleLine1on1(
 	const Pixel* srcLine0, const Pixel* srcLine1,
 	const Pixel* srcLine2, const Pixel* srcLine3,
-	Pixel* dstUpper, Pixel* dstLower )
+	Pixel* dstUpper, Pixel* dstLower, unsigned srcWidth)
 {
 	// Apply 2xSaI and keep the bottom-left pixel.
 	// It's not great, but at least it looks better than doubling the pixel
 	// like SimpleScaler does.
 	dstUpper[0] = srcLine1[0];
-	dstLower[0] = blender.blend(srcLine1[0], srcLine2[0]);
-	for (int x = 1; x < WIDTH512 - 1; x++) {
+	dstLower[0] = blend(srcLine1[0], srcLine2[0]);
+	for (unsigned x = 1; x < srcWidth - 1; x++) {
 		// Map of the pixels:
 		//   I E F
 		//   G A B
@@ -203,7 +232,7 @@ void SaI2xScaler<Pixel>::scaleLine512(
 				       && colorG != colorC && colorC == colorM )
 				  )
 				? colorA
-				: blender.blend(colorA, colorC)
+				: blend(colorA, colorC)
 				);
 		} else if (colorB == colorC && colorA != colorD) {
 			product1 =
@@ -212,13 +241,13 @@ void SaI2xScaler<Pixel>::scaleLine512(
 				       && colorA != colorH && colorA == colorI )
 				  )
 				? colorC
-				: blender.blend(colorA, colorC)
+				: blend(colorA, colorC)
 				);
 		} else if (colorA == colorD && colorB == colorC) {
 			if (colorA == colorC) {
 				product1 = colorA;
 			} else {
-				product1 = blender.blend(colorA, colorC);
+				product1 = blend(colorA, colorC);
 			}
 		} else {
 			product1 =
@@ -228,7 +257,7 @@ void SaI2xScaler<Pixel>::scaleLine512(
 				: ( colorC == colorG && colorC == colorD
 				    && colorA != colorH && colorA == colorI
 				  ? colorC
-				  : blender.blend(colorA, colorC)
+				  : blend(colorA, colorC)
 				  )
 				);
 		}
@@ -236,45 +265,56 @@ void SaI2xScaler<Pixel>::scaleLine512(
 		dstUpper[x] = colorA;
 		dstLower[x] = product1;
 	}
-	dstUpper[WIDTH512 - 1] = srcLine1[WIDTH512 - 1];
-	dstLower[WIDTH512 - 1] =
-		blender.blend(srcLine1[WIDTH512 - 1], srcLine2[WIDTH512 - 1]);
+	dstUpper[srcWidth - 1] = srcLine1[srcWidth - 1];
+	dstLower[srcWidth - 1] =
+		blend(srcLine1[srcWidth - 1], srcLine2[srcWidth - 1]);
 }
 
 template <class Pixel>
-void SaI2xScaler<Pixel>::scale256(
-	SDL_Surface* src, int srcY, int endSrcY, SDL_Surface* dst, int dstY )
+void SaI2xScaler<Pixel>::scale1x1to2x2(FrameSource& src,
+	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	assert(dst->w == WIDTH256 * 2);
-	for (int y = srcY; y < endSrcY; y++) {
-		const Pixel* srcLine0 = Scaler<Pixel>::linePtr(src, max(y - 1, srcY));
-		const Pixel* srcLine1 = Scaler<Pixel>::linePtr(src, y);
-		const Pixel* srcLine2 = Scaler<Pixel>::linePtr(src, min(y + 1, endSrcY - 1));
-		const Pixel* srcLine3 = Scaler<Pixel>::linePtr(src, min(y + 2, endSrcY - 1));
-		Pixel* dstUpper = Scaler<Pixel>::linePtr(dst, dstY++);
-		Pixel* dstLower = Scaler<Pixel>::linePtr(dst, min(dstY++, HEIGHT - 1));
-		
-		scaleLine256(
-			srcLine0, srcLine1, srcLine2, srcLine3, dstUpper, dstLower );
+	assert(dst.getWidth() == srcWidth * 2);
+
+	Pixel* const dummy = 0;
+	int srcY = srcStartY;
+	const Pixel* srcLine0 = src.getLinePtr(srcY - 1, srcWidth, dummy);
+	const Pixel* srcLine1 = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	const Pixel* srcLine2 = src.getLinePtr(srcY + 1, srcWidth, dummy);
+	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
+		const Pixel* srcLine3 = src.getLinePtr(srcY + 2, srcWidth, dummy);
+		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		scaleLine1on2(srcLine0, srcLine1, srcLine2, srcLine3,
+		              dstUpper, dstLower, srcWidth);
+		srcLine0 = srcLine1;
+		srcLine1 = srcLine2;
+		srcLine2 = srcLine3;
 	}
 }
 
 template <class Pixel>
-void SaI2xScaler<Pixel>::scale512(
-	SDL_Surface* src, int srcY, int endSrcY,
-	SDL_Surface* dst, int dstY )
+void SaI2xScaler<Pixel>::scale1x1to1x2(FrameSource& src,
+	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	assert(dst->w == WIDTH512);
-	for (int y = srcY; y < endSrcY; y++) {
-		const Pixel* srcLine0 = Scaler<Pixel>::linePtr(src, max(y - 1, srcY));
-		const Pixel* srcLine1 = Scaler<Pixel>::linePtr(src, y);
-		const Pixel* srcLine2 = Scaler<Pixel>::linePtr(src, min(y + 1, endSrcY - 1));
-		const Pixel* srcLine3 = Scaler<Pixel>::linePtr(src, min(y + 2, endSrcY - 1));
-		Pixel* dstUpper = Scaler<Pixel>::linePtr(dst, dstY++);
-		Pixel* dstLower = Scaler<Pixel>::linePtr(dst, min(dstY++, HEIGHT - 1));
-		
-		scaleLine512(
-			srcLine0, srcLine1, srcLine2, srcLine3, dstUpper, dstLower );
+	assert(dst.getWidth() == srcWidth);
+
+	Pixel* const dummy = 0;
+	int srcY = srcStartY;
+	const Pixel* srcLine0 = src.getLinePtr(srcY - 1, srcWidth, dummy);
+	const Pixel* srcLine1 = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	const Pixel* srcLine2 = src.getLinePtr(srcY + 1, srcWidth, dummy);
+	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
+		const Pixel* srcLine3 = src.getLinePtr(srcY + 2, srcWidth, dummy);
+		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		scaleLine1on1(srcLine0, srcLine1, srcLine2, srcLine3,
+		              dstUpper, dstLower, srcWidth);
+		srcLine0 = srcLine1;
+		srcLine1 = srcLine2;
+		srcLine2 = srcLine3;
 	}
 }
 
