@@ -1,4 +1,4 @@
-// $Id: Scale2xScaler.cc,v 1.12 2005/05/09 17:38:37 m9710797 Exp $
+// $Id: Scale2xScaler.cc,v 1.27 2006/01/06 14:06:16 m9710797 Exp $
 
 /*
 Original code: Copyright (C) 2001-2003 Andrea Mazzoleni
@@ -13,18 +13,23 @@ Visit the Scale2x site for info:
 */
 
 #include "Scale2xScaler.hh"
+#include "FrameSource.hh"
+#include "OutputSurface.hh"
 #include "HostCPU.hh"
 #include "openmsx.hh"
-#include <cassert>
-#include <algorithm>
-
-using std::min;
 
 namespace openmsx {
 
 template <class Pixel>
-void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
-	const Pixel* src0, const Pixel* src1, const Pixel* src2)
+Scale2xScaler<Pixel>::Scale2xScaler(const PixelOperations<Pixel>& pixelOps)
+	: Scaler2<Pixel>(pixelOps)
+{
+}
+
+template <class Pixel>
+void Scale2xScaler<Pixel>::scaleLineHalf_1on2(Pixel* dst,
+	const Pixel* src0, const Pixel* src1, const Pixel* src2,
+	unsigned srcWidth)
 {
 	//   n      m is expaned to a b
 	// w m e                    c d
@@ -37,15 +42,14 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 	const HostCPU& cpu = HostCPU::getInstance();
 	if ((sizeof(Pixel) == 4) && cpu.hasMMXEXT()) {
 		asm (
-			"movq	(%0), %%mm1;"        // m1 | e1  or  w2 | m2
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // m1 | e1  or  w2 | m2
 			"pshufw	$238, %%mm1, %%mm0;" // xx | w1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm2;"  // n1 | n2
+			"movq	(%1,%4), %%mm2;"     // n1 | n2
 			"movq	%%mm2, %%mm3;"       // n1 | n2
 			"pshufw	$68, %%mm2, %%mm5;"  // n1 | n1
-			"pcmpeqd (%2,%%eax), %%mm3;" // n1 = s1 | n2 = s2
+			"pcmpeqd (%2,%4), %%mm3;"    // n1 = s1 | n2 = s2
 			"movq	%%mm0, %%mm7;"
 			"pshufw	$68, %%mm3, %%mm4;"  // n1 = s1 | n1 = s1
 			"punpckhdq %%mm1, %%mm7;"    // w1 | e1
@@ -57,7 +61,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 						     // n1 = e1 & n1 != s1 & n1 != w1
 			"pshufw	$68, %%mm1, %%mm7;"  // m1 | m1
 			"pand	%%mm6, %%mm5;"       // c  & n1
-			"movq	8(%0,%%eax), %%mm0;" // e2 | xx
+			"movq	8(%0,%4), %%mm0;"    // e2 | xx
 			"pandn	%%mm7, %%mm6;"       // !c & m1
 			"por	%%mm6, %%mm5;"       // c ? n1 : m1
 
@@ -77,22 +81,21 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pandn  %%mm3, %%mm6;"       // !c & m2
 			"movq	%%mm1, %%mm0;"
 			"por	%%mm6, %%mm7;"       // c ? n2 : m2
-			"movq	%%mm2, %%mm1;"       // swap mm0,mm1 
-			     
-			"movntq	%%mm5, (%3,%%eax,2);"
-			"movntq	%%mm7, 8(%3,%%eax,2);"
-		
-			"addl	$8, %%eax;"
-			"cmpl   $1272, %%eax;"
-			"jl	0b;"
-			
+			"movq	%%mm2, %%mm1;"       // swap mm0,mm1
+
+			"movntq	%%mm5,  (%3,%4,2);"
+			"movntq	%%mm7, 8(%3,%4,2);"
+
+			"addl	$8, %4;"
+			"jnz	0b;"
+
 			// last pixel
-			"movq	(%1,%%eax), %%mm2;"  // n1 | n2
+			"movq	(%1), %%mm2;"        // n1 | n2
 			"movq	%%mm2, %%mm3;"       // n1 | n2
 			"pshufw	$78, %%mm2, %%mm5;"  // n1 | n1
-			"pcmpeqd (%2,%%eax), %%mm3;" // n1 = s1 | n2 = s2
+			"pcmpeqd (%2), %%mm3;"       // n1 = s1 | n2 = s2
 			"movq	%%mm0, %%mm7;"
-			"pshufw	$78, %%mm3, %%mm4;"   // n1 = s1 | n1 = s1
+			"pshufw	$78, %%mm3, %%mm4;"  // n1 = s1 | n1 = s1
 			"punpckhdq %%mm1, %%mm7;"    // w1 | e1
 			"pcmpeqd %%mm5, %%mm7;"      // n1 = w1 | n1 = e1
 			"pandn	%%mm7, %%mm4;"       // n1 = w1 & n1 != s1 |
@@ -120,35 +123,36 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm6, %%mm7;"       // c  & n2
 			"pandn  %%mm3, %%mm6;"       // !c & m2
 			"por	%%mm6, %%mm7;"       // c ? n2 : m2
-			     
-			"movntq	%%mm5, (%3,%%eax,2);"
-			"movntq	%%mm7, 8(%3,%%eax,2);"
+
+			"movntq	%%mm5,  (%3);"
+			"movntq	%%mm7, 8(%3);"
 
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 2) // 0
+			, "r" (src0 + srcWidth - 2) // 1
+			, "r" (src2 + srcWidth - 2) // 2
+			, "r" (dst + 2 * (srcWidth - 2)) // 3
+			, "r" (-4 * (srcWidth - 2)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
 	if ((sizeof(Pixel) == 4) && cpu.hasMMX()) {
 		asm (
-			"movq	(%0), %%mm1;"        // m1 | e1  or  w2 | m2
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // m1 | e1  or  w2 | m2
 			"movq	%%mm1, %%mm0;"       // w1 | xx
 			"punpckldq %%mm0, %%mm0;"    // xx | w1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm2;"  // n1 | n2
+			"movq	(%1,%4), %%mm2;"     // n1 | n2
 			"movq	%%mm2, %%mm3;"       // n1 | n2
 			"movq	%%mm2, %%mm5;"       // n1 | xx
-			"pcmpeqd (%2,%%eax), %%mm3;" // n1 = s1 | n2 = s2
+			"pcmpeqd (%2,%4), %%mm3;"    // n1 = s1 | n2 = s2
 			"punpckldq %%mm5, %%mm5;"    // n1 | n1
 			"movq	%%mm3, %%mm4;"       // n1 = s1 | n2 = s2
 			"movq	%%mm0, %%mm7;"       // xx | w1
@@ -165,7 +169,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 						     // n1 = e1 & n1 != s1 & n1 != w1
 			"punpckldq %%mm7, %%mm7;"    // m1 | m1
 			"pand	%%mm6, %%mm5;"       // c  & n1
-			"movq	8(%0,%%eax), %%mm0;" // e2 | xx
+			"movq	8(%0,%4), %%mm0;"    // e2 | xx
 			"pandn	%%mm7, %%mm6;"       // !c & m1
 			"movq	%%mm2, %%mm7;"       // xx | n2
 			"por	%%mm6, %%mm5;"       // c ? n1 : m1
@@ -189,22 +193,21 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pandn  %%mm4, %%mm6;"       // !c & m2
 			"movq	%%mm1, %%mm0;"
 			"por	%%mm6, %%mm7;"       // c ? n2 : m2
-			"movq	%%mm2, %%mm1;"       // swap mm0,mm1 
-			     
-			"movq	%%mm5, (%3,%%eax,2);"
-			"movq	%%mm7, 8(%3,%%eax,2);"
-		
-			"addl	$8, %%eax;"
-			"cmpl   $1272, %%eax;"
-			"jl	0b;"
-			
+			"movq	%%mm2, %%mm1;"       // swap mm0,mm1
+
+			"movq	%%mm5,  (%3,%4,2);"
+			"movq	%%mm7, 8(%3,%4,2);"
+
+			"addl	$8, %4;"
+			"jnz	0b;"
+
 			// last pixel
-			"movq	(%1,%%eax), %%mm2;"  // n1 | n2
+			"movq	(%1), %%mm2;"        // n1 | n2
 			"movq	%%mm2, %%mm3;"       // n1 | n2
 			"movq	%%mm2, %%mm5;"       // n1 | xx
-			"pcmpeqd (%2,%%eax), %%mm3;" // n1 = s1 | n2 = s2
+			"pcmpeqd (%2), %%mm3;"       // n1 = s1 | n2 = s2
 			"punpckldq %%mm5, %%mm5;"    // n1 | n1
-			"movq	%%mm3, %%mm4;"	     // n1 = s1 | n2 = s2  
+			"movq	%%mm3, %%mm4;"	     // n1 = s1 | n2 = s2
 			"movq	%%mm0, %%mm7;"       // xx | w1
 			"punpckldq %%mm4, %%mm4;"    // n1 = s1 | n1 = s1
 			"punpckhdq %%mm1, %%mm7;"    // w1 | e1
@@ -242,20 +245,22 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm6, %%mm7;"       // c  & n2
 			"pandn  %%mm4, %%mm6;"       // !c & m2
 			"por	%%mm6, %%mm7;"       // c ? n2 : m2
-			     
-			"movq	%%mm5, (%3,%%eax,2);"
-			"movq	%%mm7, 8(%3,%%eax,2);"
+
+			"movq	%%mm5,  (%3);"
+			"movq	%%mm7, 8(%3);"
 
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 2) // 0
+			, "r" (src0 + srcWidth - 2) // 1
+			, "r" (src2 + srcWidth - 2) // 2
+			, "r" (dst + 2 * (srcWidth - 2)) // 3
+			, "r" (-4 * (srcWidth - 2)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
@@ -264,14 +269,13 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 		//mm0: xxx0  mm1: 1234  mm0: 5xxx
 		//                efgh
 		asm (
-			"movq	(%0), %%mm1;"        // 1 2 3 4
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // 1 2 3 4
 			"pshufw	$0, %%mm1, %%mm0;"   // x x x 1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm2;"  // a b c d
+			"movq	(%1,%4), %%mm2;"     // a b c d
 			"movq	%%mm2, %%mm3;"       // a b c d
-			"pcmpeqw (%2,%%eax), %%mm3;" // a=e b=f c=g d=h
+			"pcmpeqw (%2,%4), %%mm3;"    // a=e b=f c=g d=h
 			"pshufw	$33, %%mm1, %%mm7;"  // 2 1 3 x
 			"movq	%%mm0, %%mm6;"       // x x x 0
 			"pshufw	$80, %%mm3, %%mm4;"  // a=e a=e b=f b=f
@@ -288,7 +292,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pandn	%%mm6, %%mm7;"       // !cond & (1 1 2 2)
 			"por	%%mm5, %%mm7;"       // cond ? (a a b b) : (1 1 2 2)
 
-			"movq	8(%0,%%eax), %%mm0;" // 5 x x x
+			"movq	8(%0,%4), %%mm0;"    // 5 x x x
 			"pshufw	$180, %%mm1, %%mm4;" // x 2 4 3
 			"movq	%%mm0, %%mm6;"       // 5 x x x
 			"pshufw	$250, %%mm3, %%mm3;" // c=g c=g d=h d=h
@@ -304,22 +308,21 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm4, %%mm5;"       //  cond & (c c d d)
 			"pandn  %%mm6, %%mm4;"       // !cond & (3 3 4 4)
 			"por	%%mm5, %%mm4;"       // cond ? (c c d d) : (3 3 4 4)
-			
-			"movntq	%%mm7,  (%3,%%eax,2);"
-			"movntq	%%mm4, 8(%3,%%eax,2);"
+
+			"movntq	%%mm7,  (%3,%4,2);"
+			"movntq	%%mm4, 8(%3,%4,2);"
 
 			"movq	%%mm0, %%mm2;"
 			"movq	%%mm1, %%mm0;"
-			"movq	%%mm2, %%mm1;"       // swap mm0,mm1 
-			     
-			"addl	$8, %%eax;"
-			"cmpl   $632, %%eax;"
-			"jl	0b;"
-			
+			"movq	%%mm2, %%mm1;"       // swap mm0,mm1
+
+			"addl	$8, %4;"
+			"jnz	0b;"
+
 			// last pixel
-			"movq	(%1,%%eax), %%mm2;"  // a b c d
+			"movq	(%1), %%mm2;"        // a b c d
 			"movq	%%mm2, %%mm3;"       // a b c d
-			"pcmpeqw (%2,%%eax), %%mm3;" // a=e b=f c=g d=h
+			"pcmpeqw (%2), %%mm3;"       // a=e b=f c=g d=h
 			"pshufw	$33, %%mm1, %%mm7;"  // 2 1 3 x
 			"pshufw	$80, %%mm3, %%mm4;"  // a=e a=e b=f b=f
 			"movq	%%mm0, %%mm6;"       // x x x 0
@@ -352,20 +355,22 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm4, %%mm5;"       //  cond & (c c d d)
 			"pandn  %%mm6, %%mm4;"       // !cond & (3 3 4 4)
 			"por	%%mm5, %%mm4;"       // cond ? (c c d d) : (3 3 4 4)
-			
-			"movntq	%%mm7, (%3,%%eax,2);"
-			"movntq	%%mm4, 8(%3,%%eax,2);"
+
+			"movntq	%%mm7,  (%3);"
+			"movntq	%%mm4, 8(%3);"
 
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 4) // 0
+			, "r" (src0 + srcWidth - 4) // 1
+			, "r" (src2 + srcWidth - 4) // 2
+			, "r" (dst + 2 * (srcWidth - 4)) // 3
+			, "r" (-2 * (srcWidth - 4)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
@@ -374,15 +379,14 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 		//mm0: xxx0  mm1: 1234  mm0: 5xxx
 		//                efgh
 		asm (
-			"movq	(%0), %%mm1;"        // 1 2 3 4
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // 1 2 3 4
 			"movq	%%mm1, %%mm0;"       // 1 x x x
 			"psllq	$48, %%mm0;"         // x x x 1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm2;"  // a b c d
+			"movq	(%1,%4), %%mm2;"     // a b c d
 			"movq	%%mm2, %%mm3;"       // a b c d
-			"pcmpeqw (%2,%%eax), %%mm3;" // a=e b=f c=g d=h
+			"pcmpeqw (%2,%4), %%mm3;"    // a=e b=f c=g d=h
 
 			"movq	%%mm0, %%mm6;"       // x x x 0
 			"psrlq	$48, %%mm6;"         // 0 x x x
@@ -397,7 +401,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpcklwd %%mm5, %%mm5;"    // a a b b
 			"pcmpeqw %%mm5, %%mm6;"      // 0=a 2=a 1=b 3=b
-			
+
 			"movq	%%mm1, %%mm7;"       // 1 2 3 4
 			"psllq	$16, %%mm7;"         // x 1 2 3
 			"pxor	%%mm5, %%mm5;"       // x x x x
@@ -412,7 +416,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpcklwd %%mm5, %%mm5;"    // a a b b
 			"pcmpeqw %%mm5, %%mm7;"      // 2=a 0=a 3=b 1=b
-			
+
 			"movq	%%mm3, %%mm4;"       // a=e b=f c=g d=h
 			"punpcklwd %%mm4, %%mm4;"    // a=e a=e b=f b=f
 			"pandn	%%mm6, %%mm4;"       // 0=a & a!=e  .. .. ..
@@ -422,9 +426,9 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm7, %%mm5;"       //  cond & (a a b b)
 			"pandn	%%mm6, %%mm7;"       // !cond & (1 1 2 2)
 			"por	%%mm5, %%mm7;"       // cond ? (a a b b) : (1 1 2 2)
-			"movq	%%mm7,  (%3,%%eax,2);"
+			"movq	%%mm7,  (%3,%4,2);"
 
-			"movq	8(%0,%%eax), %%mm0;" // 5 x x x
+			"movq	8(%0,%4), %%mm0;"    // 5 x x x
 			"punpckhwd %%mm3, %%mm3;"    // c=g c=g d=h d=h
 
 			"movq	%%mm0, %%mm6;"       // 5 x x x
@@ -455,7 +459,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpckhwd %%mm5, %%mm5;"    // c c d d
 			"pcmpeqw %%mm5, %%mm4;"      // 4=c 2=c 5=d 3=d
-			
+
 			"pandn	%%mm6, %%mm3;"       // 2=c & c!=g .. .. ..
 			"pandn	%%mm3, %%mm4;"       // 2=c & c!=g & 4!=c
 			"movq	%%mm1, %%mm6;"       // 1 2 3 4
@@ -463,21 +467,20 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm4, %%mm5;"       //  cond & (c c d d)
 			"pandn  %%mm6, %%mm4;"       // !cond & (3 3 4 4)
 			"por	%%mm5, %%mm4;"       // cond ? (c c d d) : (3 3 4 4)
-			
-			"movq	%%mm4, 8(%3,%%eax,2);"
+
+			"movq	%%mm4, 8(%3,%4,2);"
 
 			"movq	%%mm0, %%mm2;"
 			"movq	%%mm1, %%mm0;"
-			"movq	%%mm2, %%mm1;"       // swap mm0,mm1 
-			     
-			"addl	$8, %%eax;"
-			"cmpl   $632, %%eax;"
-			"jl	0b;"
-			
+			"movq	%%mm2, %%mm1;"       // swap mm0,mm1
+
+			"addl	$8, %4;"
+			"jnz	0b;"
+
 			// last pixel
-			"movq	(%1,%%eax), %%mm2;"  // a b c d
+			"movq	(%1), %%mm2;"        // a b c d
 			"movq	%%mm2, %%mm3;"       // a b c d
-			"pcmpeqw (%2,%%eax), %%mm3;" // a=e b=f c=g d=h
+			"pcmpeqw (%2), %%mm3;"       // a=e b=f c=g d=h
 
 			"movq	%%mm0, %%mm6;"       // x x x 0
 			"psrlq	$48, %%mm6;"         // 0 x x x
@@ -492,7 +495,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpcklwd %%mm5, %%mm5;"    // a a b b
 			"pcmpeqw %%mm5, %%mm6;"      // 0=a 2=a 1=b 3=b
-			
+
 			"movq	%%mm1, %%mm7;"       // 1 2 3 4
 			"psllq	$16, %%mm7;"         // x 1 2 3
 			"pxor	%%mm5, %%mm5;"       // x x x x
@@ -507,7 +510,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpcklwd %%mm5, %%mm5;"    // a a b b
 			"pcmpeqw %%mm5, %%mm7;"      // 2=a 0=a 3=b 1=b
-			
+
 			"movq	%%mm3, %%mm4;"       // a=e b=f c=g d=h
 			"punpcklwd %%mm4, %%mm4;"    // a=e a=e b=f b=f
 			"pandn	%%mm6, %%mm4;"       // 0=a & a!=e  .. .. ..
@@ -517,7 +520,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm7, %%mm5;"       //  cond & (a a b b)
 			"pandn	%%mm6, %%mm7;"       // !cond & (1 1 2 2)
 			"por	%%mm5, %%mm7;"       // cond ? (a a b b) : (1 1 2 2)
-			"movq	%%mm7,  (%3,%%eax,2);"
+			"movq	%%mm7,  (%3);"
 
 			"movq	%%mm1, %%mm0;"       // x x x 5
 			"psrlq  $48, %%mm0;"         // 5 x x x
@@ -551,7 +554,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // a b c d
 			"punpckhwd %%mm5, %%mm5;"    // c c d d
 			"pcmpeqw %%mm5, %%mm4;"      // 4=c 2=c 5=d 3=d
-			
+
 			"pandn	%%mm6, %%mm3;"       // 2=c & c!=g .. .. ..
 			"pandn	%%mm3, %%mm4;"       // 2=c & c!=g & 4!=c
 			"movq	%%mm1, %%mm6;"       // 1 2 3 4
@@ -559,19 +562,21 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 			"pand	%%mm4, %%mm5;"       //  cond & (c c d d)
 			"pandn  %%mm6, %%mm4;"       // !cond & (3 3 4 4)
 			"por	%%mm5, %%mm4;"       // cond ? (c c d d) : (3 3 4 4)
-			
-			"movq	%%mm4, 8(%3,%%eax,2);"
+
+			"movq	%%mm4, 8(%3);"
 
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 4) // 0
+			, "r" (src0 + srcWidth - 4) // 1
+			, "r" (src2 + srcWidth - 4) // 2
+			, "r" (dst + 2 * (srcWidth - 4)) // 3
+			, "r" (-2 * (srcWidth - 4)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
@@ -584,7 +589,7 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 	dst[1] = (right == src0[0] && src2[0] != src0[0]) ? src0[0] : mid;
 
 	// Central pixels.
-	for (unsigned x = 1; x < 319; ++x) {
+	for (unsigned x = 1; x < srcWidth - 1; ++x) {
 		Pixel left = mid;
 		mid   = right;
 		right = src1[x + 1];
@@ -595,89 +600,91 @@ void Scale2xScaler<Pixel>::scaleLine256Half(Pixel* dst,
 	}
 
 	// Last pixel.
-	dst[638] = (mid == src0[319] && src2[319] != src0[319]) ? src0[319] : right;
-	dst[639] = src1[319];
+	dst[2 * srcWidth - 2] =
+		(mid == src0[srcWidth - 1] && src2[srcWidth - 1] != src0[srcWidth - 1])
+		? src0[srcWidth - 1] : right;
+	dst[2 * srcWidth - 1] =
+		src1[srcWidth - 1];
 }
 
 template <class Pixel>
-void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
-	const Pixel* src0, const Pixel* src1, const Pixel* src2)
+void Scale2xScaler<Pixel>::scaleLineHalf_1on1(Pixel* dst,
+	const Pixel* src0, const Pixel* src1, const Pixel* src2,
+	unsigned srcWidth)
 {
 	//    ab ef
 	// x0 12 34 5x
 	//    cd gh
-	
+
 	#ifdef ASM_X86
 	const HostCPU& cpu = HostCPU::getInstance();
-	if (false && (sizeof(Pixel) == 4) && cpu.hasMMXEXT()) {
+	if ((sizeof(Pixel) == 4) && cpu.hasMMXEXT()) {
 		asm (
-			"movq	(%0), %%mm0;"        // 1 2
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm0;"     // 1 2
 			"pshufw	$68, %%mm0, %%mm2;"  // 1 1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm4;"  // a b
+			"movq	(%1,%4), %%mm4;"     // a b
 			"pshufw	$238, %%mm0, %%mm3;" // 2 2
 			"pcmpeqd %%mm4, %%mm2;"      // a=0 b=1
-			"movq	(%2,%%eax), %%mm5;"  // c d
+			"movq	(%2,%4), %%mm5;"     // c d
 			"movq	%%mm4, %%mm6;"       // a b
 			"pcmpeqd %%mm4, %%mm5;"      // a=c b=d
-			"movq	8(%0,%%eax), %%mm1;" // 3 4
+			"movq	8(%0,%4), %%mm1;"    // 3 4
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=c
 			                             // b=1 & b!=d
 			"punpckldq %%mm1, %%mm3;"    // 2 3
 			"pcmpeqd %%mm3, %%mm6;"      // a=2 b=3
-			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2 
+			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2
 			                             // b=1 & b!=d & b!=3
 			"pand	%%mm6, %%mm4;"       //  cond & (a b)
 			"pandn	%%mm0, %%mm6;"       // !cond & (1 2)
 			"por	%%mm4, %%mm6;"       // cond ? (a b) : (1 2)
-			
-			"movq	8(%1,%%eax), %%mm4;" // e f
+
+			"movq	8(%1,%4), %%mm4;"    // e f
 			"pshufw	$238, %%mm1, %%mm2;" // 4 4
 			"pcmpeqd %%mm4, %%mm3;"      // e=2 f=3
-			"movq	8(%2,%%eax), %%mm5;" // g h
+			"movq	8(%2,%4), %%mm5;"    // g h
 			"movq	%%mm4, %%mm7;"       // e f
 			"pcmpeqd %%mm4, %%mm5;"      // e=g f=h
-			"movq	16(%0,%%eax), %%mm0;"// 5 6
+			"movq	16(%0,%4), %%mm0;"   // 5 6
 			"pandn	%%mm3, %%mm5;"       // e=2 & e!=g
 			                             // f=3 & f!=h
 			"punpckldq %%mm0, %%mm2;"    // 4 5
 			"pcmpeqd %%mm2, %%mm7;"      // e=4 f=5
-			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4 
+			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4
 			                             // f=3 & f!=h & f!=5
 			"pand	%%mm7, %%mm4;"       //  cond & (e f)
 			"pandn	%%mm1, %%mm7;"       // !cond & (3 4)
 			"por	%%mm4, %%mm7;"       // cond ? (e f) : (3 4)
-			
-			"movntq	%%mm6, (%3,%%eax);"
-			"movntq	%%mm7, 8(%3,%%eax);"
-		
-			"addl	$16, %%eax;"
-			"cmpl   $2544, %%eax;"
-			"jl	0b;"
 
-			"movq	(%1,%%eax), %%mm4;"  // a b
+			"movntq	%%mm6,  (%3,%4);"
+			"movntq	%%mm7, 8(%3,%4);"
+
+			"addl	$16, %4;"
+			"jnz	0b;"
+
+			"movq	(%1), %%mm4;"        // a b
 			"pshufw	$238, %%mm0, %%mm3;" // 2 2
 			"pcmpeqd %%mm4, %%mm2;"      // a=0 b=1
-			"movq	(%2,%%eax), %%mm5;"  // c d
+			"movq	(%2), %%mm5;"        // c d
 			"movq	%%mm4, %%mm6;"       // a b
 			"pcmpeqd %%mm4, %%mm5;"      // a=c b=d
-			"movq	8(%0,%%eax), %%mm1;" // 3 4
+			"movq	8(%0), %%mm1;"       // 3 4
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=c
 			                             // b=1 & b!=d
 			"punpckldq %%mm1, %%mm3;"    // 2 3
 			"pcmpeqd %%mm3, %%mm6;"      // a=2 b=3
-			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2 
+			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2
 			                             // b=1 & b!=d & b!=3
 			"pand	%%mm6, %%mm4;"       //  cond & (a b)
 			"pandn	%%mm0, %%mm6;"       // !cond & (1 2)
 			"por	%%mm4, %%mm6;"       // cond ? (a b) : (1 2)
-			
-			"movq	8(%1,%%eax), %%mm4;" // e f
+
+			"movq	8(%1), %%mm4;"       // e f
 			"pshufw	$238, %%mm1, %%mm2;" // 4 4
 			"pcmpeqd %%mm4, %%mm3;"      // e=2 f=3
-			"movq	8(%2,%%eax), %%mm5;" // g h
+			"movq	8(%2), %%mm5;"       // g h
 			"movq	%%mm4, %%mm7;"       // e f
 			"pcmpeqd %%mm4, %%mm5;"      // e=g f=h
 			"movq	%%mm2, %%mm0;"       // 4 4
@@ -685,102 +692,102 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			                             // f=3 & f!=h
 			"punpckldq %%mm0, %%mm2;"    // 4 5
 			"pcmpeqd %%mm2, %%mm7;"      // e=4 f=5
-			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4 
+			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4
 			                             // f=3 & f!=h & f!=5
 			"pand	%%mm7, %%mm4;"       //  cond & (e f)
 			"pandn	%%mm1, %%mm7;"       // !cond & (3 4)
 			"por	%%mm4, %%mm7;"       // cond ? (e f) : (3 4)
-			
-			"movntq	%%mm6, (%3,%%eax);"
-			"movntq	%%mm7, 8(%3,%%eax);"
-			
+
+			"movntq	%%mm6,  (%3);"
+			"movntq	%%mm7, 8(%3);"
+
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 4) // 0
+			, "r" (src0 + srcWidth - 4) // 1
+			, "r" (src2 + srcWidth - 4) // 2
+			, "r" (dst  + srcWidth - 4) // 3
+			, "r" (-4 * (srcWidth - 4)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
 	if ((sizeof(Pixel) == 4) && cpu.hasMMX()) {
 		asm (
-			"movq	(%0), %%mm0;"        // 1 2
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm0;"     // 1 2
 			"movq	%%mm0, %%mm2;"       // 1 2
 			"punpckldq %%mm2, %%mm2;"    // 1 1
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm4;"  // a b
+			"movq	(%1,%4), %%mm4;"     // a b
 			"movq	%%mm0, %%mm3;"       // 1 2
 			"punpckhdq %%mm3, %%mm3;"    // 2 2
 			"pcmpeqd %%mm4, %%mm2;"      // a=0 b=1
-			"movq	(%2,%%eax), %%mm5;"  // c d
+			"movq	(%2,%4), %%mm5;"     // c d
 			"movq	%%mm4, %%mm6;"       // a b
 			"pcmpeqd %%mm4, %%mm5;"      // a=c b=d
-			"movq	8(%0,%%eax), %%mm1;" // 3 4
+			"movq	8(%0,%4), %%mm1;"    // 3 4
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=c
 			                             // b=1 & b!=d
 			"punpckldq %%mm1, %%mm3;"    // 2 3
 			"pcmpeqd %%mm3, %%mm6;"      // a=2 b=3
-			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2 
+			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2
 			                             // b=1 & b!=d & b!=3
 			"pand	%%mm6, %%mm4;"       //  cond & (a b)
 			"pandn	%%mm0, %%mm6;"       // !cond & (1 2)
 			"por	%%mm4, %%mm6;"       // cond ? (a b) : (1 2)
-			
-			"movq	8(%1,%%eax), %%mm4;" // e f
+
+			"movq	8(%1,%4), %%mm4;"    // e f
 			"movq	%%mm1, %%mm2;"       // 3 4
 			"punpckldq %%mm2, %%mm2;"    // 4 4
 			"pcmpeqd %%mm4, %%mm3;"      // e=2 f=3
-			"movq	8(%2,%%eax), %%mm5;" // g h
+			"movq	8(%2,%4), %%mm5;"    // g h
 			"movq	%%mm4, %%mm7;"       // e f
 			"pcmpeqd %%mm4, %%mm5;"      // e=g f=h
-			"movq	16(%0,%%eax), %%mm0;"// 5 6
+			"movq	16(%0,%4), %%mm0;"   // 5 6
 			"pandn	%%mm3, %%mm5;"       // e=2 & e!=g
 			                             // f=3 & f!=h
 			"punpckldq %%mm0, %%mm2;"    // 4 5
 			"pcmpeqd %%mm2, %%mm7;"      // e=4 f=5
-			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4 
+			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4
 			                             // f=3 & f!=h & f!=5
 			"pand	%%mm7, %%mm4;"       //  cond & (e f)
 			"pandn	%%mm1, %%mm7;"       // !cond & (3 4)
 			"por	%%mm4, %%mm7;"       // cond ? (e f) : (3 4)
-			
-			"movq	%%mm6, (%3,%%eax);"
-			"movq	%%mm7, 8(%3,%%eax);"
-		
-			"addl	$16, %%eax;"
-			"cmpl   $2544, %%eax;"
-			"jl	0b;"
 
-			"movq	(%1,%%eax), %%mm4;"  // a b
+			"movq	%%mm6,  (%3,%4);"
+			"movq	%%mm7, 8(%3,%4);"
+
+			"addl	$16, %4;"
+			"jnz	0b;"
+
+			"movq	(%1), %%mm4;"        // a b
 			"movq	%%mm0, %%mm3;"       // 1 2
 			"punpckhdq %%mm3, %%mm3;"    // 2 2
 			"pcmpeqd %%mm4, %%mm2;"      // a=0 b=1
-			"movq	(%2,%%eax), %%mm5;"  // c d
+			"movq	(%2), %%mm5;"        // c d
 			"movq	%%mm4, %%mm6;"       // a b
 			"pcmpeqd %%mm4, %%mm5;"      // a=c b=d
-			"movq	8(%0,%%eax), %%mm1;" // 3 4
+			"movq	8(%0), %%mm1;"       // 3 4
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=c
 			                             // b=1 & b!=d
 			"punpckldq %%mm1, %%mm3;"    // 2 3
 			"pcmpeqd %%mm3, %%mm6;"      // a=2 b=3
-			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2 
+			"pandn	%%mm5, %%mm6;"       // a=0 & a!=c & a!=2
 			                             // b=1 & b!=d & b!=3
 			"pand	%%mm6, %%mm4;"       //  cond & (a b)
 			"pandn	%%mm0, %%mm6;"       // !cond & (1 2)
 			"por	%%mm4, %%mm6;"       // cond ? (a b) : (1 2)
-			
-			"movq	8(%1,%%eax), %%mm4;" // e f
+
+			"movq	8(%1), %%mm4;"       // e f
 			"movq	%%mm1, %%mm2;"       // 3 4
 			"punpckldq %%mm2, %%mm2;"    // 4 4
 			"pcmpeqd %%mm4, %%mm3;"      // e=2 f=3
-			"movq	8(%2,%%eax), %%mm5;" // g h
+			"movq	8(%2), %%mm5;"       // g h
 			"movq	%%mm4, %%mm7;"       // e f
 			"pcmpeqd %%mm4, %%mm5;"      // e=g f=h
 			"movq	%%mm2, %%mm0;"       // 4 4
@@ -788,25 +795,27 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			                             // f=3 & f!=h
 			"punpckldq %%mm0, %%mm2;"    // 4 5
 			"pcmpeqd %%mm2, %%mm7;"      // e=4 f=5
-			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4 
+			"pandn	%%mm5, %%mm7;"       // e=2 & e!=g & e!=4
 			                             // f=3 & f!=h & f!=5
 			"pand	%%mm7, %%mm4;"       //  cond & (e f)
 			"pandn	%%mm1, %%mm7;"       // !cond & (3 4)
 			"por	%%mm4, %%mm7;"       // cond ? (e f) : (3 4)
-			
-			"movq	%%mm6, (%3,%%eax);"
-			"movq	%%mm7, 8(%3,%%eax);"
-			
+
+			"movq	%%mm6,  (%3);"
+			"movq	%%mm7, 8(%3);"
+
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 4) // 0
+			, "r" (src0 + srcWidth - 4) // 1
+			, "r" (src2 + srcWidth - 4) // 2
+			, "r" (dst  + srcWidth - 4) // 3
+			, "r" (-4 * (srcWidth - 4)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
@@ -815,14 +824,13 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 	//      bdfh jlnp
 	if ((sizeof(Pixel) == 2) && cpu.hasMMXEXT()) {
 		asm (
-			"movq	(%0), %%mm1;"        // 1234
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // 1234
 			"pshufw	$0, %%mm1, %%mm0;"   // ...0
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm4;"  // aceg
+			"movq	(%1,%4), %%mm4;"     // aceg
 			"psrlq	$48, %%mm0;"         // 0...
-			"movq	(%2,%%eax), %%mm5;"  // bdfh
+			"movq	(%2,%4), %%mm5;"     // bdfh
 			"movq	%%mm1, %%mm2;"       // 1234
 			"pcmpeqw %%mm4, %%mm5;"      // a=b c=d e=f g=h
 			"psllq	$16, %%mm2;"         // .123
@@ -830,7 +838,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm0, %%mm2;"       // 0123
 			"psrlq	$16, %%mm6;"         // 234.
 			"pcmpeqw %%mm4, %%mm2;"      // a=0 c=1 e=2 g=3
-			"movq	8(%0,%%eax), %%mm0;" // 5678
+			"movq	8(%0,%4), %%mm0;"    // 5678
 			"movq	%%mm0, %%mm3;"       // 5678
 			"psllq	$48, %%mm3;"         // ...5
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=b .. .. ..
@@ -840,10 +848,10 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm6, %%mm4;"       //  cond & aceg
 			"pandn	%%mm1, %%mm6;"       // !cond & 1234
 			"por	%%mm4, %%mm6;"       // cond ? aceg : 1234
-			
-			"movq	8(%1,%%eax), %%mm4;" // ikmo
+
+			"movq	8(%1,%4), %%mm4;"    // ikmo
 			"psrlq	$48, %%mm1;"         // 4...
-			"movq	8(%2,%%eax), %%mm5;" // jlnp
+			"movq	8(%2,%4), %%mm5;"    // jlnp
 			"movq	%%mm0, %%mm2;"       // 5678
 			"pcmpeqw %%mm4, %%mm5;"      // i=j k=l m=n o=p
 			"psllq	$16, %%mm2;"         // .567
@@ -851,7 +859,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm1, %%mm2;"       // 4567
 			"psrlq	$16, %%mm7;"         // 678.
 			"pcmpeqw %%mm4, %%mm2;"      // i=4 k=5 m=6 o=7
-			"movq	16(%0,%%eax), %%mm1;"// 9 10 11 12
+			"movq	16(%0,%4), %%mm1;"   // 9 10 11 12
 			"movq	%%mm1, %%mm3;"       // 9 10 11 12
 			"psllq	$48, %%mm3;"         // ...9
 			"pandn	%%mm2, %%mm5;"       // i=4 & i!=j .. .. ..
@@ -861,17 +869,16 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm7, %%mm4;"       //  cond & ikmo
 			"pandn	%%mm0, %%mm7;"       // !cond & 5678
 			"por	%%mm4, %%mm7;"       // cond ? ikmo : 5678
-			
-			"movntq	%%mm6, (%3,%%eax);"
-			"movntq	%%mm7, 8(%3,%%eax);"
-		
-			"addl	$16, %%eax;"
-			"cmpl   $2544, %%eax;"
-			"jl	0b;"
 
-			"movq	(%1,%%eax), %%mm4;"  // aceg
+			"movntq	%%mm6,  (%3,%4);"
+			"movntq	%%mm7, 8(%3,%4);"
+
+			"addl	$16, %4;"
+			"jnz	0b;"
+
+			"movq	(%1), %%mm4;"        // aceg
 			"psrlq	$48, %%mm0;"         // 0...
-			"movq	(%2,%%eax), %%mm5;"  // bdfh
+			"movq	(%2), %%mm5;"        // bdfh
 			"movq	%%mm1, %%mm2;"       // 1234
 			"pcmpeqw %%mm4, %%mm5;"      // a=b c=d e=f g=h
 			"psllq	$16, %%mm2;"         // .123
@@ -879,7 +886,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm0, %%mm2;"       // 0123
 			"psrlq	$16, %%mm6;"         // 234.
 			"pcmpeqw %%mm4, %%mm2;"      // a=0 c=1 e=2 g=3
-			"movq	8(%0,%%eax), %%mm0;" // 5678
+			"movq	8(%0), %%mm0;"       // 5678
 			"movq	%%mm0, %%mm3;"       // 5678
 			"psllq	$48, %%mm3;"         // ...5
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=b .. .. ..
@@ -889,10 +896,10 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm6, %%mm4;"       //  cond & aceg
 			"pandn	%%mm1, %%mm6;"       // !cond & 1234
 			"por	%%mm4, %%mm6;"       // cond ? aceg : 1234
-			
-			"movq	8(%1,%%eax), %%mm4;" // ikmo
+
+			"movq	8(%1), %%mm4;"       // ikmo
 			"psrlq	$48, %%mm1;"         // 4...
-			"movq	8(%2,%%eax), %%mm5;" // jlnp
+			"movq	8(%2), %%mm5;"       // jlnp
 			"movq	%%mm0, %%mm2;"       // 5678
 			"pcmpeqw %%mm4, %%mm5;"      // i=j k=l m=n o=p
 			"psllq	$16, %%mm2;"         // .567
@@ -910,34 +917,35 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm7, %%mm4;"       //  cond & ikmo
 			"pandn	%%mm0, %%mm7;"       // !cond & 5678
 			"por	%%mm4, %%mm7;"       // cond ? ikmo : 5678
-			
-			"movntq	%%mm6, (%3,%%eax);"
-			"movntq	%%mm7, 8(%3,%%eax);"
-			
+
+			"movntq	%%mm6,  (%3);"
+			"movntq	%%mm7, 8(%3);"
+
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 8) // 0
+			, "r" (src0 + srcWidth - 8) // 1
+			, "r" (src2 + srcWidth - 8) // 2
+			, "r" (dst  + srcWidth - 8) // 3
+			, "r" (-2 * (srcWidth - 8)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
 	if ((sizeof(Pixel) == 2) && cpu.hasMMX()) {
 		asm (
-			"movq	(%0), %%mm1;"        // 1234
-			"xorl	%%eax, %%eax;"
+			"movq	(%0,%4), %%mm1;"     // 1234
 			"movq	%%mm1, %%mm0;"       // 1234
 			"psllq	$48, %%mm0;"         // ...0
 			".p2align 4,,15;"
 		"0:"
-			"movq	(%1,%%eax), %%mm4;"  // aceg
+			"movq	(%1,%4), %%mm4;"     // aceg
 			"psrlq	$48, %%mm0;"         // 0...
-			"movq	(%2,%%eax), %%mm5;"  // bdfh
+			"movq	(%2,%4), %%mm5;"     // bdfh
 			"movq	%%mm1, %%mm2;"       // 1234
 			"pcmpeqw %%mm4, %%mm5;"      // a=b c=d e=f g=h
 			"psllq	$16, %%mm2;"         // .123
@@ -945,7 +953,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm0, %%mm2;"       // 0123
 			"psrlq	$16, %%mm6;"         // 234.
 			"pcmpeqw %%mm4, %%mm2;"      // a=0 c=1 e=2 g=3
-			"movq	8(%0,%%eax), %%mm0;" // 5678
+			"movq	8(%0,%4), %%mm0;"    // 5678
 			"movq	%%mm0, %%mm3;"       // 5678
 			"psllq	$48, %%mm3;"         // ...5
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=b .. .. ..
@@ -955,10 +963,10 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm6, %%mm4;"       //  cond & aceg
 			"pandn	%%mm1, %%mm6;"       // !cond & 1234
 			"por	%%mm4, %%mm6;"       // cond ? aceg : 1234
-			
-			"movq	8(%1,%%eax), %%mm4;" // ikmo
+
+			"movq	8(%1,%4), %%mm4;"    // ikmo
 			"psrlq	$48, %%mm1;"         // 4...
-			"movq	8(%2,%%eax), %%mm5;" // jlnp
+			"movq	8(%2,%4), %%mm5;"    // jlnp
 			"movq	%%mm0, %%mm2;"       // 5678
 			"pcmpeqw %%mm4, %%mm5;"      // i=j k=l m=n o=p
 			"psllq	$16, %%mm2;"         // .567
@@ -966,7 +974,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm1, %%mm2;"       // 4567
 			"psrlq	$16, %%mm7;"         // 678.
 			"pcmpeqw %%mm4, %%mm2;"      // i=4 k=5 m=6 o=7
-			"movq	16(%0,%%eax), %%mm1;"// 9 10 11 12
+			"movq	16(%0,%4), %%mm1;"   // 9 10 11 12
 			"movq	%%mm1, %%mm3;"       // 9 10 11 12
 			"psllq	$48, %%mm3;"         // ...9
 			"pandn	%%mm2, %%mm5;"       // i=4 & i!=j .. .. ..
@@ -976,17 +984,16 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm7, %%mm4;"       //  cond & ikmo
 			"pandn	%%mm0, %%mm7;"       // !cond & 5678
 			"por	%%mm4, %%mm7;"       // cond ? ikmo : 5678
-			
-			"movq	%%mm6, (%3,%%eax);"
-			"movq	%%mm7, 8(%3,%%eax);"
-		
-			"addl	$16, %%eax;"
-			"cmpl   $2544, %%eax;"
-			"jl	0b;"
 
-			"movq	(%1,%%eax), %%mm4;"  // aceg
+			"movq	%%mm6,  (%3,%4);"
+			"movq	%%mm7, 8(%3,%4);"
+
+			"addl	$16, %4;"
+			"jnz	0b;"
+
+			"movq	(%1), %%mm4;"        // aceg
 			"psrlq	$48, %%mm0;"         // 0...
-			"movq	(%2,%%eax), %%mm5;"  // bdfh
+			"movq	(%2), %%mm5;"        // bdfh
 			"movq	%%mm1, %%mm2;"       // 1234
 			"pcmpeqw %%mm4, %%mm5;"      // a=b c=d e=f g=h
 			"psllq	$16, %%mm2;"         // .123
@@ -994,7 +1001,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"por	%%mm0, %%mm2;"       // 0123
 			"psrlq	$16, %%mm6;"         // 234.
 			"pcmpeqw %%mm4, %%mm2;"      // a=0 c=1 e=2 g=3
-			"movq	8(%0,%%eax), %%mm0;" // 5678
+			"movq	8(%0), %%mm0;"       // 5678
 			"movq	%%mm0, %%mm3;"       // 5678
 			"psllq	$48, %%mm3;"         // ...5
 			"pandn	%%mm2, %%mm5;"       // a=0 & a!=b .. .. ..
@@ -1004,10 +1011,10 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm6, %%mm4;"       //  cond & aceg
 			"pandn	%%mm1, %%mm6;"       // !cond & 1234
 			"por	%%mm4, %%mm6;"       // cond ? aceg : 1234
-			
-			"movq	8(%1,%%eax), %%mm4;" // ikmo
+
+			"movq	8(%1), %%mm4;"       // ikmo
 			"psrlq	$48, %%mm1;"         // 4...
-			"movq	8(%2,%%eax), %%mm5;" // jlnp
+			"movq	8(%2), %%mm5;"       // jlnp
 			"movq	%%mm0, %%mm2;"       // 5678
 			"pcmpeqw %%mm4, %%mm5;"      // i=j k=l m=n o=p
 			"psllq	$16, %%mm2;"         // .567
@@ -1025,20 +1032,22 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 			"pand	%%mm7, %%mm4;"       //  cond & ikmo
 			"pandn	%%mm0, %%mm7;"       // !cond & 5678
 			"por	%%mm4, %%mm7;"       // cond ? ikmo : 5678
-			
-			"movq	%%mm6, (%3,%%eax);"
-			"movq	%%mm7, 8(%3,%%eax);"
-			
+
+			"movq	%%mm6,  (%3);"
+			"movq	%%mm7, 8(%3);"
+
 			"emms;"
 
 			: // no output
-			: "r" (src1) // 0
-			, "r" (src0) // 1
-			, "r" (src2) // 2
-			, "r" (dst)  // 3
-			: "eax"
-			, "mm0", "mm1", "mm2", "mm3"
+			: "r" (src1 + srcWidth - 8) // 0
+			, "r" (src0 + srcWidth - 8) // 1
+			, "r" (src2 + srcWidth - 8) // 2
+			, "r" (dst  + srcWidth - 8) // 3
+			, "r" (-2 * (srcWidth - 8)) // 4
+			#ifdef __MMX__
+			: "mm0", "mm1", "mm2", "mm3"
 			, "mm4", "mm5", "mm6", "mm7"
+			#endif
 		);
 		return;
 	};
@@ -1050,7 +1059,7 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 	dst[0] = mid;
 
 	// Central pixels.
-	for (unsigned x = 1; x < 639; ++x) {
+	for (unsigned x = 1; x < srcWidth - 1; ++x) {
 		Pixel left = mid;
 		mid   = right;
 		right = src1[x + 1];
@@ -1060,46 +1069,48 @@ void Scale2xScaler<Pixel>::scaleLine512Half(Pixel* dst,
 	}
 
 	// Last pixel.
-	dst[639] = (mid == src0[639] && src2[639] != src0[639]) ? src0[639] : right;
+	dst[srcWidth - 1] =
+		(mid == src0[srcWidth - 1] && src2[srcWidth - 1] != src0[srcWidth - 1])
+		? src0[srcWidth - 1] : right;
 }
 
 template <class Pixel>
-void Scale2xScaler<Pixel>::scale256(
-	SDL_Surface* src, int srcY, int endSrcY,
-	SDL_Surface* dst, int dstY )
+void Scale2xScaler<Pixel>::scale1x1to2x2(FrameSource& src,
+	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	int prevY = srcY;
-	while (srcY < endSrcY) {
-		Pixel* srcPrev = Scaler<Pixel>::linePtr(src, prevY);
-		Pixel* srcCurr = Scaler<Pixel>::linePtr(src, srcY);
-		Pixel* srcNext = Scaler<Pixel>::linePtr(src, min(srcY + 1, endSrcY - 1));
-		Pixel* dstUpper = Scaler<Pixel>::linePtr(dst, dstY++);
-		scaleLine256Half(dstUpper, srcPrev, srcCurr, srcNext);
-		if (dstY == dst->h) break;
-		Pixel* dstLower = Scaler<Pixel>::linePtr(dst, dstY++);
-		scaleLine256Half(dstLower, srcNext, srcCurr, srcPrev);
-		prevY = srcY;
-		srcY++;
+	Pixel* const dummy = 0;
+	int srcY = srcStartY;
+	const Pixel* srcPrev = src.getLinePtr(srcY - 1, srcWidth, dummy);
+	const Pixel* srcCurr = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
+		const Pixel* srcNext = src.getLinePtr(srcY + 1, srcWidth, dummy);
+		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		scaleLineHalf_1on2(dstUpper, srcPrev, srcCurr, srcNext, srcWidth);
+		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		scaleLineHalf_1on2(dstLower, srcNext, srcCurr, srcPrev, srcWidth);
+		srcPrev = srcCurr;
+		srcCurr = srcNext;
 	}
 }
 
 template <class Pixel>
-void Scale2xScaler<Pixel>::scale512(
-	SDL_Surface* src, int srcY, int endSrcY,
-	SDL_Surface* dst, int dstY)
+void Scale2xScaler<Pixel>::scale1x1to1x2(FrameSource& src,
+	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
+	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	int prevY = srcY;
-	while (srcY < endSrcY) {
-		Pixel* srcPrev = Scaler<Pixel>::linePtr(src, prevY);
-		Pixel* srcCurr = Scaler<Pixel>::linePtr(src, srcY);
-		Pixel* srcNext = Scaler<Pixel>::linePtr(src, min(srcY + 1, endSrcY - 1));
-		Pixel* dstUpper = Scaler<Pixel>::linePtr(dst, dstY++);
-		scaleLine512Half(dstUpper, srcPrev, srcCurr, srcNext);
-		if (dstY == dst->h) break;
-		Pixel* dstLower = Scaler<Pixel>::linePtr(dst, dstY++);
-		scaleLine512Half(dstLower, srcNext, srcCurr, srcPrev);
-		prevY = srcY;
-		srcY++;
+	Pixel* const dummy = 0;
+	int srcY = srcStartY;
+	const Pixel* srcPrev = src.getLinePtr(srcY - 1, srcWidth, dummy);
+	const Pixel* srcCurr = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
+		const Pixel* srcNext = src.getLinePtr(srcY + 1, srcWidth, dummy);
+		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		scaleLineHalf_1on1(dstUpper, srcPrev, srcCurr, srcNext, srcWidth);
+		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		scaleLineHalf_1on1(dstLower, srcNext, srcCurr, srcPrev, srcWidth);
+		srcPrev = srcCurr;
+		srcCurr = srcNext;
 	}
 }
 
