@@ -1,4 +1,4 @@
-// $Id: VDP.cc,v 1.81 2005/06/02 22:18:10 mthuurne Exp $
+// $Id: VDP.cc,v 1.104 2006/05/20 10:11:57 m9710797 Exp $
 
 /*
 TODO:
@@ -23,58 +23,81 @@ TODO:
 #include "VDPVRAM.hh"
 #include "VDPCmdEngine.hh"
 #include "SpriteChecker.hh"
-#include "EventDistributor.hh"
-#include "CommandController.hh"
-#include "Scheduler.hh"
+#include "Display.hh"
 #include "XMLElement.hh"
-#include "Debugger.hh"
 #include "RendererFactory.hh"
 #include "Renderer.hh"
+#include "SimpleDebuggable.hh"
+#include "MSXMotherBoard.hh"
 #include "MSXException.hh"
 #include <sstream>
-#include <iomanip>
 #include <cassert>
 
-using std::setw;
-using std::setfill;
-using std::string;
-using std::vector;
-
 namespace openmsx {
 
-// Inlined methods first, to make sure they are actually inlined:
-// TODO: None left?
-
-// Init and cleanup:
-
-VDP::VDP(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, vdpRegDebug(*this)
-	, vdpStatusRegDebug(*this)
-	, vdpRegsCmd(*this)
-	, paletteCmd(*this)
+class VDPRegDebug : public SimpleDebuggable
 {
-	PRT_DEBUG("Creating a VDP object");
+public:
+	VDPRegDebug(VDP& vdp);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	VDP& vdp;
+};
 
-	interlaced=false;
+class VDPStatusRegDebug : public SimpleDebuggable
+{
+public:
+	VDPStatusRegDebug(VDP& vdp);
+	virtual byte read(unsigned address, const EmuTime& time);
+private:
+	VDP& vdp;
+};
 
-	string versionString = deviceConfig.getChildData("version");
+class VDPPaletteDebug : public SimpleDebuggable
+{
+public:
+	VDPPaletteDebug(VDP& vdp);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	VDP& vdp;
+};
+
+
+VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config,
+         const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, Schedulable(motherBoard.getScheduler())
+	, frameStartTime(time)
+	, irqVertical(motherBoard.getCPU())
+	, irqHorizontal(motherBoard.getCPU())
+	, displayStartSyncTime(time)
+	, vScanSyncTime(time)
+	, hScanSyncTime(time)
+	, vdpRegDebug      (new VDPRegDebug      (*this))
+	, vdpStatusRegDebug(new VDPStatusRegDebug(*this))
+	, vdpPaletteDebug  (new VDPPaletteDebug  (*this))
+{
+	interlaced = false;
+
+	std::string versionString = deviceConfig.getChildData("version");
 	if (versionString == "TMS99X8A") version = TMS99X8A;
 	else if (versionString == "TMS9929A") version = TMS9929A;
 	else if (versionString == "V9938") version = V9938;
 	else if (versionString == "V9958") version = V9958;
-	else throw FatalError("Unknown VDP version \"" + versionString + "\"");
+	else throw MSXException("Unknown VDP version \"" + versionString + "\"");
 
 	// Set up control register availability.
 	static const byte VALUE_MASKS_MSX1[32] = {
 		0x03, 0xFB, 0x0F, 0xFF, 0x07, 0x7F, 0x07, 0xFF  // 00..07
-		};
+	};
 	static const byte VALUE_MASKS_MSX2[32] = {
 		0x7E, 0x7B, 0x7F, 0xFF, 0x3F, 0xFF, 0x3F, 0xFF, // 00..07
 		0xFB, 0xBF, 0x07, 0x03, 0xFF, 0xFF, 0x07, 0x0F, // 08..15
 		0x0F, 0xBF, 0xFF, 0xFF, 0x3F, 0x3F, 0x3F, 0xFF, // 16..23
 		0,    0,    0,    0,    0,    0,    0,    0,    // 24..31
-		};
+	};
 	controlRegMask = (isMSX1VDP() ? 0x07 : 0x3F);
 	memcpy(controlValueMasks,
 		isMSX1VDP() ? VALUE_MASKS_MSX1 : VALUE_MASKS_MSX2, 32);
@@ -86,84 +109,62 @@ VDP::VDP(const XMLElement& config, const EmuTime& time)
 	}
 
 	// Video RAM.
-	int vramSize =
+	unsigned vramSize =
 		(isMSX1VDP() ? 16 : deviceConfig.getChildDataAsInt("vram"));
-	if (vramSize != 16 && vramSize != 64 && vramSize != 128) {
+	if ((vramSize !=  16) && (vramSize !=  64) &&
+	    (vramSize != 128) && (vramSize != 192)) {
 		std::ostringstream out;
 		out << "VRAM size of " << vramSize << "kB is not supported!";
-		throw FatalError(out.str());
+		throw MSXException(out.str());
 	}
-	vramSize *= 1024;
-	vramMask = vramSize - 1;
-	vram.reset(new VDPVRAM(this, vramSize, time));
+	vram.reset(new VDPVRAM(*this, vramSize * 1024, time));
+
+	Display& display = getMotherBoard().getDisplay();
+	RenderSettings& renderSettings = display.getRenderSettings();
 
 	// Create sprite checker.
-	spriteChecker.reset(new SpriteChecker(this));
+	spriteChecker.reset(new SpriteChecker(*this, renderSettings, time));
 	vram->setSpriteChecker(spriteChecker.get());
 
 	// Create command engine.
-	cmdEngine.reset(new VDPCmdEngine(this));
+	cmdEngine.reset(new VDPCmdEngine(*this, renderSettings,
+		getMotherBoard().getCommandController() ));
 	vram->setCmdEngine(cmdEngine.get());
 
 	resetInit(time); // must be done early to avoid UMRs
 
-	// Initialise time stamps.
-	// This will be done again by frameStart, but these have to be
-	// initialised before createRenderer() and reset() are called.
-	// TODO: Can this be simplified with a different design?
-	frameStartTime.advance(time);
-	displayStartSyncTime = time;
-	vScanSyncTime = time;
-	hScanSyncTime = time;
-
 	// Initialise renderer.
 	createRenderer();
 
 	// Reset state.
 	reset(time);
 
-	// Register console commands.
-	CommandController::instance().registerCommand(&vdpRegsCmd, "vdpregs");
-	CommandController::instance().registerCommand(&paletteCmd, "palette");
-
-	Debugger::instance().registerDebuggable("VDP regs", vdpRegDebug);
-	Debugger::instance().registerDebuggable(
-		"VDP status regs", vdpStatusRegDebug );
-
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_RENDERER_SWITCH2_EVENT, *this, EventDistributor::DETACHED);
+	display.attach(*this);
 }
 
 VDP::~VDP()
 {
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_RENDERER_SWITCH2_EVENT, *this, EventDistributor::DETACHED);
-
-	Debugger::instance().unregisterDebuggable("VDP status regs", vdpStatusRegDebug);
-	Debugger::instance().unregisterDebuggable("VDP regs", vdpRegDebug);
-
-	CommandController::instance().unregisterCommand(&vdpRegsCmd, "vdpregs");
-	CommandController::instance().unregisterCommand(&paletteCmd, "palette");
-
-	delete renderer;
+	getMotherBoard().getDisplay().detach(*this);
 }
 
-bool VDP::signalEvent(const Event& event)
+void VDP::preVideoSystemChange()
+{
+	renderer.reset();
+}
+
+void VDP::postVideoSystemChange()
 {
-	if (&event); // avoid warning
-	assert(event.getType() == OPENMSX_RENDERER_SWITCH2_EVENT);
-	delete renderer;
 	createRenderer();
-	return true;
 }
 
 void VDP::createRenderer()
 {
-	renderer = RendererFactory::createRenderer(this);
+	Display& display = getMotherBoard().getDisplay();
+	renderer.reset(RendererFactory::createRenderer(*this, display));
 	// TODO: Is it safe to use frameStartTime,
 	//       which is most likely in the past?
 	//renderer->reset(frameStartTime.getTime());
-	vram->setRenderer(renderer, frameStartTime.getTime());
+	vram->setRenderer(renderer.get(), frameStartTime.getTime());
 }
 
 void VDP::resetInit(const EmuTime& /*time*/)
@@ -233,13 +234,13 @@ void VDP::resetMasks(const EmuTime& time)
 
 void VDP::reset(const EmuTime& time)
 {
-	Scheduler::instance().removeSyncPoint(*this, VSYNC);
-	Scheduler::instance().removeSyncPoint(*this, DISPLAY_START);
-	Scheduler::instance().removeSyncPoint(*this, VSCAN);
-	Scheduler::instance().removeSyncPoint(*this, HSCAN);
-	Scheduler::instance().removeSyncPoint(*this, HOR_ADJUST);
-	Scheduler::instance().removeSyncPoint(*this, SET_MODE);
-	Scheduler::instance().removeSyncPoint(*this, SET_BLANK);
+	removeSyncPoint(VSYNC);
+	removeSyncPoint(DISPLAY_START);
+	removeSyncPoint(VSCAN);
+	removeSyncPoint(HSCAN);
+	removeSyncPoint(HOR_ADJUST);
+	removeSyncPoint(SET_MODE);
+	removeSyncPoint(SET_BLANK);
 
 	// Reset subsystems.
 	cmdEngine->sync(time);
@@ -341,9 +342,9 @@ void VDP::executeUntil(const EmuTime& time, int userData)
 	}
 }
 
-const string& VDP::schedName() const
+const std::string& VDP::schedName() const
 {
-	static const string name("VDP");
+	static const std::string name("VDP");
 	return name;
 }
 
@@ -354,7 +355,7 @@ void VDP::scheduleDisplayStart(const EmuTime& time)
 {
 	// Remove pending DISPLAY_START sync point, if any.
 	if (displayStartSyncTime > time) {
-		Scheduler::instance().removeSyncPoint(*this, DISPLAY_START);
+		removeSyncPoint(DISPLAY_START);
 		//cerr << "removing predicted DISPLAY_START sync point\n";
 	}
 
@@ -375,8 +376,7 @@ void VDP::scheduleDisplayStart(const EmuTime& time)
 
 	// Register new DISPLAY_START sync point.
 	if (displayStartSyncTime > time) {
-		Scheduler::instance().setSyncPoint(
-			displayStartSyncTime, *this, DISPLAY_START);
+		setSyncPoint(displayStartSyncTime, DISPLAY_START);
 		//cerr << "inserting new DISPLAY_START sync point\n";
 	}
 
@@ -398,7 +398,7 @@ void VDP::scheduleVScan(const EmuTime& time)
 
 	// Remove pending VSCAN sync point, if any.
 	if (vScanSyncTime > time) {
-		Scheduler::instance().removeSyncPoint(*this, VSCAN);
+		removeSyncPoint(VSCAN);
 		//cerr << "removing predicted VSCAN sync point\n";
 	}
 
@@ -409,7 +409,7 @@ void VDP::scheduleVScan(const EmuTime& time)
 
 	// Register new VSCAN sync point.
 	if (vScanSyncTime > time) {
-		Scheduler::instance().setSyncPoint(vScanSyncTime, *this, VSCAN);
+		setSyncPoint(vScanSyncTime, VSCAN);
 		//cerr << "inserting new VSCAN sync point\n";
 	}
 }
@@ -418,7 +418,7 @@ void VDP::scheduleHScan(const EmuTime& time)
 {
 	// Remove pending HSCAN sync point, if any.
 	if (hScanSyncTime > time) {
-		Scheduler::instance().removeSyncPoint(*this, HSCAN);
+		removeSyncPoint(HSCAN);
 		hScanSyncTime = time;
 	}
 
@@ -454,7 +454,7 @@ void VDP::scheduleHScan(const EmuTime& time)
 		*/
 		hScanSyncTime = frameStartTime + horizontalScanOffset;
 		if (hScanSyncTime > time) {
-			Scheduler::instance().setSyncPoint(hScanSyncTime, *this, HSCAN);
+			setSyncPoint(hScanSyncTime, HSCAN);
 		}
 	}
 }
@@ -498,8 +498,7 @@ void VDP::frameStart(const EmuTime& time)
 
 	// Schedule next VSYNC.
 	frameStartTime.advance(time);
-	Scheduler::instance().setSyncPoint(
-		frameStartTime + getTicksPerFrame(), *this, VSYNC);
+	setSyncPoint(frameStartTime + getTicksPerFrame(), VSYNC);
 	// Schedule DISPLAY_START, VSCAN and HSCAN.
 	scheduleDisplayStart(time);
 
@@ -520,20 +519,26 @@ void VDP::frameStart(const EmuTime& time)
 
 // The I/O functions.
 
-void VDP::writeIO(byte port, byte value, const EmuTime& time)
+void VDP::writeIO(word port, byte value, const EmuTime& time)
 {
 	assert(isInsideFrame(time));
 	switch (port & 0x03) {
 	case 0: { // VRAM data write
-		int addr = ((controlRegs[14] << 14) | vramPointer) & vramMask;
+		int addr = (controlRegs[14] << 14) | vramPointer;
 		//fprintf(stderr, "VRAM[%05X]=%02X\n", addr, value);
-		// TODO: Is extended VRAM planar?
-		//       I don't think so, because it is limited to 64K.
 		if (displayMode.isPlanar()) {
+			// note: also extended VRAM is interleaved,
+			//       because there is only 64kb it's interleaved
+			//       with itself (every byte repeated twice)
 			addr = ((addr << 16) | (addr >> 1)) & 0x1FFFF;
 		}
-		// TODO: Implement extended VRAM.
-		if (!cpuExtendedVram) vram->cpuWrite(addr, value, time);
+		if (!cpuExtendedVram) {
+			vram->cpuWrite(addr, value, time);
+		} else if (vram->getSize() == 192 * 1024) {
+			vram->cpuWrite(0x20000 | (addr & 0xFFFF), value, time);
+		} else {
+			// ignore
+		}
 		vramPointer = (vramPointer + 1) & 0x3FFF;
 		if (vramPointer == 0 && displayMode.isV9938Mode()) {
 			// In MSX2 video modes, pointer range is 128K.
@@ -576,10 +581,7 @@ void VDP::writeIO(byte port, byte value, const EmuTime& time)
 		if (paletteDataStored) {
 			int index = controlRegs[16];
 			int grb = ((value << 8) | dataLatch) & 0x777;
-			if (palette[index] != grb) {
-				renderer->updatePalette(index, grb, time);
-				palette[index] = grb;
-			}
+			setPalette(index, grb, time);
 			controlRegs[16] = (index + 1) & 0x0F;
 			paletteDataStored = false;
 		} else {
@@ -602,24 +604,35 @@ void VDP::writeIO(byte port, byte value, const EmuTime& time)
 	}
 }
 
+void VDP::setPalette(int index, word grb, const EmuTime& time)
+{
+	if (palette[index] != grb) {
+		renderer->updatePalette(index, grb, time);
+		palette[index] = grb;
+	}
+}
+
 byte VDP::vramRead(const EmuTime& time)
 {
-	byte ret = readAhead;
+	byte result = readAhead;
 	int addr = (controlRegs[14] << 14) | vramPointer;
-	// TODO: Is extended VRAM planar?
-	//       I don't think so, because it is limited to 64K.
 	if (displayMode.isPlanar()) {
 		addr = ((addr << 16) | (addr >> 1)) & 0x1FFFF;
 	}
-	readAhead = vram->cpuRead(addr, time);
+	if (!cpuExtendedVram) {
+		readAhead = vram->cpuRead(addr, time);
+	} else if (vram->getSize() == 192 * 1024) {
+		readAhead = vram->cpuRead(0x20000 | (addr & 0xFFFF), time);
+	} else {
+		readAhead = 0xFF;
+	}
 	vramPointer = (vramPointer + 1) & 0x3FFF;
 	if (vramPointer == 0 && displayMode.isV9938Mode()) {
 		// In MSX2 video modes , pointer range is 128K.
 		controlRegs[14] = (controlRegs[14] + 1) & 0x07;
 	}
 	registerDataStored = false;
-	// TODO: Implement extended VRAM.
-	return cpuExtendedVram ? 0xFF : ret;
+	return result;
 }
 
 byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
@@ -698,7 +711,7 @@ byte VDP::readStatusReg(byte reg, const EmuTime& time)
 	return ret;
 }
 
-byte VDP::readIO(byte port, const EmuTime& time)
+byte VDP::readIO(word port, const EmuTime& time)
 {
 	assert(isInsideFrame(time));
 	switch (port & 0x03) {
@@ -718,7 +731,7 @@ byte VDP::readIO(byte port, const EmuTime& time)
 	}
 }
 
-byte VDP::peekIO(byte /*port*/, const EmuTime& /*time*/) const
+byte VDP::peekIO(word /*port*/, const EmuTime& /*time*/) const
 {
 	// TODO not implemented
 	return 0xFF;
@@ -787,7 +800,7 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 		}
 		break;
 	case 2: {
-		int base = ((val << 10) | ~(-1 << 10)) & vramMask;
+		int base = (val << 10) | ~(-1 << 10);
 		// TODO:
 		// I reverted this fix.
 		// Although the code is correct, there is also a counterpart in the
@@ -801,11 +814,6 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 		}
 		*/
 		renderer->updateNameBase(base, time);
-		int indexMask =
-			  displayMode.isBitmapMode()
-			? -1 << 17 // TODO: Calculate actual value; how to handle planar?
-			: -1 << (displayMode.isTextMode() ? 12 : 10);
-		vram->nameTable.setMask(base, indexMask, time);
 		break;
 	}
 	case 7:
@@ -893,6 +901,9 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 			if (!(val & 0x20)) irqVertical.reset();
 		}
 		break;
+	case 2:
+		updateNameBase(time);
+		break;
 	case 3:
 	case 10:
 		updateColourBase(time);
@@ -931,6 +942,11 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 	case 23:
 		scheduleHScan(time);
 		break;
+	case 25:
+		if (change & 0x01) {
+			updateNameBase(time);
+		}
+		break;
 	}
 }
 
@@ -939,13 +955,40 @@ void VDP::syncAtNextLine(SyncType type, const EmuTime& time)
 	int line = getTicksThisFrame(time) / TICKS_PER_LINE;
 	int ticks = (line + 1) * TICKS_PER_LINE;
 	EmuTime nextTime = frameStartTime + ticks;
-	Scheduler::instance().setSyncPoint(nextTime, *this, type);
+	setSyncPoint(nextTime, type);
+}
+
+void VDP::updateNameBase(const EmuTime& time)
+{
+	int base = (controlRegs[2] << 10) | ~(-1 << 10);
+	// TODO:
+	// I reverted this fix.
+	// Although the code is correct, there is also a counterpart in the
+	// renderer that must be updated. I'm too tired now to find it.
+	// Since name table checking is currently disabled anyway, keeping the
+	// old code does not hurt.
+	// Eventually this line should be re-enabled.
+	/*
+	if (displayMode.isPlanar()) {
+		base = ((base << 16) | (base >> 1)) & 0x1FFFF;
+	}
+	*/
+	int indexMask =
+		  displayMode.isBitmapMode()
+		? -1 << 17 // TODO: Calculate actual value; how to handle planar?
+		: -1 << (displayMode.isTextMode() ? 12 : 10);
+	if (controlRegs[25] & 0x01) {
+		// Multi page scrolling. The same bit is used in character and
+		// (non)planar-bitmap modes.
+		// TODO test text modes
+		indexMask &= ~0x8000;
+	}
+	vram->nameTable.setMask(base, indexMask, time);
 }
 
 void VDP::updateColourBase(const EmuTime& time)
 {
-	int base = vramMask &
-		((controlRegs[10] << 14) | (controlRegs[3] << 6) | ~(-1 << 6));
+	int base = (controlRegs[10] << 14) | (controlRegs[3] << 6) | ~(-1 << 6);
 	renderer->updateColourBase(base, time);
 	switch (displayMode.getBase()) {
 	case 0x09: // Text 2.
@@ -967,7 +1010,7 @@ void VDP::updateColourBase(const EmuTime& time)
 
 void VDP::updatePatternBase(const EmuTime& time)
 {
-	int base = vramMask & ((controlRegs[4] << 11) | ~(-1 << 11));
+	int base = (controlRegs[4] << 11) | ~(-1 << 11);
 	renderer->updatePatternBase(base, time);
 	switch (displayMode.getBase()) {
 	case 0x01: // Text 1.
@@ -995,8 +1038,7 @@ void VDP::updateSpriteAttributeBase(const EmuTime& time)
 		vram->spriteAttribTable.disable(time);
 		return;
 	}
-	int base = vramMask &
-		((controlRegs[11] << 15) | (controlRegs[5] << 7) | ~(-1 << 7));
+	int base = (controlRegs[11] << 15) | (controlRegs[5] << 7) | ~(-1 << 7);
 	if (mode == 1) {
 		vram->spriteAttribTable.setMask(base, -1 << 7, time);
 	} else { // mode == 2
@@ -1015,7 +1057,7 @@ void VDP::updateSpritePatternBase(const EmuTime& time)
 		vram->spritePatternTable.disable(time);
 		return;
 	}
-	int base = ((controlRegs[6] << 11) | ~(-1 << 11)) & vramMask;
+	int base = (controlRegs[6] << 11) | ~(-1 << 11);
 	if (displayMode.isPlanar()) base = ((base << 16) | (base >> 1)) & 0x1FFFF;
 	vram->spritePatternTable.setMask(base, -1 << 11, time);
 }
@@ -1051,6 +1093,7 @@ void VDP::updateDisplayMode(DisplayMode newMode, const EmuTime& time)
 		updateSpritePatternBase(time);
 		updateSpriteAttributeBase(time);
 	}
+	updateNameBase(time);
 
 	// To be extremely accurate, reschedule hscan when changing
 	// from/to text mode. Text mode has different border width,
@@ -1061,127 +1104,67 @@ void VDP::updateDisplayMode(DisplayMode newMode, const EmuTime& time)
 
 // VDPRegDebug
 
-VDP::VDPRegDebug::VDPRegDebug(VDP& parent_)
-	: parent(parent_)
+VDPRegDebug::VDPRegDebug(VDP& vdp_)
+	: SimpleDebuggable(vdp_.getMotherBoard(),
+	                   "VDP regs", "VDP registers.", 0x40)
+	, vdp(vdp_)
 {
 }
 
-unsigned VDP::VDPRegDebug::getSize() const
-{
-	return 0x40;
-}
-
-const string& VDP::VDPRegDebug::getDescription() const
-{
-	static const string desc = "VDP registers.";
-	return desc;
-}
-
-byte VDP::VDPRegDebug::read(unsigned address)
+byte VDPRegDebug::read(unsigned address)
 {
 	if (address < 0x20) {
-		return parent.controlRegs[address];
+		return vdp.controlRegs[address];
 	} else if (address < 0x2F) {
-		return parent.cmdEngine->peekCmdReg(address - 0x20);
+		return vdp.cmdEngine->peekCmdReg(address - 0x20);
 	} else {
 		return 0xFF;
 	}
 }
 
-void VDP::VDPRegDebug::write(unsigned address, byte value)
+void VDPRegDebug::write(unsigned address, byte value, const EmuTime& time)
 {
-	const EmuTime& time = Scheduler::instance().getCurrentTime();
-	parent.changeRegister(address, value, time);
+	vdp.changeRegister(address, value, time);
 }
 
 
 // VDPStatusRegDebug
 
-VDP::VDPStatusRegDebug::VDPStatusRegDebug(VDP& parent_)
-	: parent(parent_)
+VDPStatusRegDebug::VDPStatusRegDebug(VDP& vdp_)
+	: SimpleDebuggable(vdp_.getMotherBoard(),
+	                   "VDP status regs", "VDP status registers.", 0x10)
+	, vdp(vdp_)
 {
 }
 
-unsigned VDP::VDPStatusRegDebug::getSize() const
+byte VDPStatusRegDebug::read(unsigned address, const EmuTime& time)
 {
-	return 0x10;
+	return vdp.peekStatusReg(address, time);
 }
 
-const string& VDP::VDPStatusRegDebug::getDescription() const
-{
-	static const string desc = "VDP status registers.";
-	return desc;
-}
+// VDPPaletteDebug
 
-byte VDP::VDPStatusRegDebug::read(unsigned address)
-{
-	const EmuTime& time = Scheduler::instance().getCurrentTime();
-	return parent.peekStatusReg(address, time);
-}
-
-void VDP::VDPStatusRegDebug::write(unsigned /*address*/, byte /*value*/)
-{
-	// not possible
-}
-
-
-// VDPRegsCmd inner class:
-
-VDP::VDPRegsCmd::VDPRegsCmd(VDP& vdp_)
-	: vdp(vdp_)
+VDPPaletteDebug::VDPPaletteDebug(VDP& vdp_)
+	: SimpleDebuggable(vdp_.getMotherBoard(),
+	                   "VDP palette", "V99x8 palette (RBG format)", 0x20)
+	, vdp(vdp_)
 {
 }
 
-string VDP::VDPRegsCmd::execute(const vector<string>& /*tokens*/)
+byte VDPPaletteDebug::read(unsigned address)
 {
-	// Print palette in 4x4 table.
-	std::ostringstream out;
-	for (int row = 0; row < 8; row++) {
-		for (int col = 0; col < 4; col++) {
-			int reg = col * 8 + row;
-			int value = vdp.controlRegs[reg];
-			out << std::dec << std::setw(2) << std::setfill(' ')<< reg;
-			out << " : ";
-			out << std::hex << "0x" << std::setw(2) << std::setfill('0') << value;
-			out << "   ";
-		}
-		out << "\n";
-	}
-	return out.str();
+	word grb = vdp.getPalette(address / 2);
+	return (address & 1) ? (grb >> 8) : (grb & 0xff);
 }
 
-string VDP::VDPRegsCmd::help(const vector<string>& /*tokens*/) const
+void VDPPaletteDebug::write(unsigned address, byte value, const EmuTime& time)
 {
-	return "Prints the current state of the VDP registers.\n";
-}
-
-// PaletteCmd inner class:
-
-VDP::PaletteCmd::PaletteCmd(VDP& vdp_)
-	: vdp(vdp_)
-{
-}
-
-string VDP::PaletteCmd::execute(const vector<string>& /*tokens*/)
-{
-	// Print palette in 4x4 table.
-	std::ostringstream out;
-	for (int row = 0; row < 4; row++) {
-		for (int col = 0; col < 4; col++) {
-			int i = col * 4 + row;
-			int grb = vdp.getPalette(i);
-			out << std::hex << i << std::dec << ":"
-				<< ((grb >> 4) & 7) << ((grb >> 8) & 7) << (grb & 7)
-				<< "  ";
-		}
-		out << "\n";
-	}
-	return out.str();
-}
-
-string VDP::PaletteCmd::help(const vector<string>& /*tokens*/) const
-{
-	return "Prints the current VDP palette (i:rgb).\n";
+	int index = address / 2;
+	word grb = vdp.getPalette(index);
+	grb = (address & 1)
+	    ? (grb & 0x0077) | ((value & 0x07) << 8)
+	    : (grb & 0x0700) |  (value & 0x77);
+	vdp.setPalette(index, grb, time);
 }
 
 } // namespace openmsx
