@@ -1,18 +1,16 @@
-// $Id: VideoSystem.cc,v 1.7 2005/02/28 18:57:37 m9710797 Exp $
+// $Id: VideoSystem.cc,v 1.9 2005/11/12 19:29:04 m9710797 Exp $
 
 #include "VideoSystem.hh"
 #include "CommandException.hh"
 #include <cassert>
 
-using std::string;
-
 namespace openmsx {
 
 VideoSystem::~VideoSystem()
 {
 }
 
-Rasterizer* VideoSystem::createRasterizer(VDP* /*vdp*/)
+Rasterizer* VideoSystem::createRasterizer(VDP& /*vdp*/)
 {
 	assert(false);
 	return 0;
@@ -28,11 +26,16 @@ bool VideoSystem::prepare()
 	return true;
 }
 
-void VideoSystem::takeScreenShot(const string& /*filename*/)
+void VideoSystem::takeScreenShot(const std::string& /*filename*/)
 {
 	throw CommandException(
 		"Taking screenshot not possible with current renderer." );
 }
 
+void VideoSystem::setWindowTitle(const std::string& /*title*/)
+{
+	// ignore
+}
+
 } // namespace openmsx
 
