@@ -1,23 +1,39 @@
-// $Id: V9990.cc,v 1.45 2005/05/24 20:39:47 m9710797 Exp $ 
+// $Id: V9990.cc,v 1.71 2006/06/05 09:44:00 m9710797 Exp $
 
 #include "V9990.hh"
 #include "Scheduler.hh"
-#include "EventDistributor.hh"
-#include "Debugger.hh"
-#include "CommandController.hh"
+#include "Display.hh"
 #include "RendererFactory.hh"
 #include "V9990VRAM.hh"
 #include "V9990CmdEngine.hh"
 #include "V9990Renderer.hh"
+#include "SimpleDebuggable.hh"
+#include "MSXMotherBoard.hh"
 #include <cassert>
-#include <iomanip>
-
-using std::setw;
-using std::string;
-using std::vector;
 
 namespace openmsx {
 
+class V9990RegDebug : public SimpleDebuggable
+{
+public:
+	V9990RegDebug(V9990& v9990);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	V9990& v9990;
+};
+
+class V9990PalDebug : public SimpleDebuggable
+{
+public:
+	V9990PalDebug(V9990& v9990);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, const EmuTime& time);
+private:
+	V9990& v9990;
+};
+
+
 enum RegisterAccess { NO_ACCESS, RD_ONLY, WR_ONLY, RD_WR };
 static const RegisterAccess regAccess[64] = {
 	WR_ONLY, WR_ONLY, WR_ONLY,          // VRAM Write Address
@@ -41,7 +57,7 @@ static const RegisterAccess regAccess[64] = {
 	WR_ONLY, WR_ONLY, WR_ONLY, WR_ONLY, // Cmd Parameter Size XY
 	WR_ONLY, WR_ONLY, WR_ONLY, WR_ONLY, // Cmd Parameter Arg, LogOp, WrtMask
 	WR_ONLY, WR_ONLY, WR_ONLY, WR_ONLY, // Cmd Parameter Font Color
-	WR_ONLY, RD_ONLY, RD_ONLY,          // Cmd Parameter OpCode, Border X 
+	WR_ONLY, RD_ONLY, RD_ONLY,          // Cmd Parameter OpCode, Border X
 	NO_ACCESS, NO_ACCESS, NO_ACCESS,    // registers 55-63
 	NO_ACCESS, NO_ACCESS, NO_ACCESS,
 	NO_ACCESS, NO_ACCESS, NO_ACCESS
@@ -51,16 +67,17 @@ static const RegisterAccess regAccess[64] = {
 // Constructor & Destructor
 // -------------------------------------------------------------------------
 
-V9990::V9990(const XMLElement& config, const EmuTime& time)
-	: MSXDevice(config, time)
-	, v9990RegDebug(*this)
-	, v9990PalDebug(*this)
-	, v9990RegsCmd(*this)
+V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config,
+             const EmuTime& time)
+	: MSXDevice(motherBoard, config, time)
+	, Schedulable(motherBoard.getScheduler())
+	, irq(motherBoard.getCPU())
 	, pendingIRQs(0)
+	, frameStartTime(time)
 	, hScanSyncTime(time)
+	, v9990RegDebug(new V9990RegDebug(*this))
+	, v9990PalDebug(new V9990PalDebug(*this))
 {
-	PRT_DEBUG("[" << time << "] V9990::Create");
-
         // clear regs TODO find realistic init values
         memset(regs, 0, sizeof(regs));
 	calcDisplayMode();
@@ -74,41 +91,26 @@ V9990::V9990(const XMLElement& config, const EmuTime& time)
 	}
 
 	// create VRAM
-	vram.reset(new V9990VRAM(this, time));
+	vram.reset(new V9990VRAM(*this, time));
 
 	// create Command Engine
-	cmdEngine.reset(new V9990CmdEngine(this, time));
+	cmdEngine.reset(new V9990CmdEngine(*this, time,
+		getMotherBoard().getDisplay().getRenderSettings()));
 
 	// Start with NTSC timing
 	palTiming = false;
 	setVerticalTiming();
-	
-	// Register debuggable
-	Debugger::instance().registerDebuggable("V9990 regs",    v9990RegDebug);
-	Debugger::instance().registerDebuggable("V9990 palette", v9990PalDebug);
-
-	// Register console commands
-	CommandController::instance().registerCommand(&v9990RegsCmd, "v9990regs");
 
 	// Initialise rendering system
 	createRenderer(time);
 
 	reset(time);
-	EventDistributor::instance().registerEventListener(
-		OPENMSX_RENDERER_SWITCH2_EVENT, *this, EventDistributor::DETACHED);
+	getMotherBoard().getDisplay().attach(*this);
 }
 
-
 V9990::~V9990()
 {
-	PRT_DEBUG("[--now--] V9990::Destroy");
-	
-	// Unregister everything that needs to be unregistered
-	CommandController::instance().unregisterCommand(&v9990RegsCmd, "v9990regs");
-	Debugger::instance().unregisterDebuggable("V9990 palette", v9990PalDebug);
-	Debugger::instance().unregisterDebuggable("V9990 regs", v9990RegDebug);
-	EventDistributor::instance().unregisterEventListener(
-		OPENMSX_RENDERER_SWITCH2_EVENT, *this, EventDistributor::DETACHED);
+	getMotherBoard().getDisplay().detach(*this);
 }
 
 // -------------------------------------------------------------------------
@@ -117,20 +119,22 @@ V9990::~V9990()
 
 void V9990::reset(const EmuTime& time)
 {
-	PRT_DEBUG("[" << time << "] V9990::reset");
-
-	Scheduler::instance().removeSyncPoint(*this, V9990_VSYNC);
-	Scheduler::instance().removeSyncPoint(*this, V9990_DISPLAY_START);
-	Scheduler::instance().removeSyncPoint(*this, V9990_VSCAN);
-	Scheduler::instance().removeSyncPoint(*this, V9990_HSCAN);
-	Scheduler::instance().removeSyncPoint(*this, V9990_SET_MODE);
+	removeSyncPoint(V9990_VSYNC);
+	removeSyncPoint(V9990_DISPLAY_START);
+	removeSyncPoint(V9990_VSCAN);
+	removeSyncPoint(V9990_HSCAN);
+	removeSyncPoint(V9990_SET_MODE);
+	removeSyncPoint(V9990_SET_BLANK);
 
 	// Clear registers / ports
 	memset(regs, 0, sizeof(regs));
 	status = 0;
 	regSelect = 0xFF; // TODO check value for power-on and reset
 	calcDisplayMode();
-	
+
+	isDisplayArea = false;
+	displayEnabled = false;
+
 	// Reset IRQs
 	writeIO(INTERRUPT_FLAG, 0xFF, time);
 
@@ -143,16 +147,16 @@ void V9990::reset(const EmuTime& time)
 	frameStart(time);
 }
 
-byte V9990::readIO(byte port, const EmuTime& time)
+byte V9990::readIO(word port, const EmuTime& time)
 {
 	port &= 0x0F;
-	
+
 	byte result = 0;
 	switch (port) {
 		case VRAM_DATA: {
 			// read from VRAM
 			unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
-			result = vram->readVRAM(addr);
+			result = vram->readVRAMSlow(addr);
 			if (!(regs[VRAM_READ_ADDRESS_2] & 0x80)) {
 				setVRAMAddr(VRAM_READ_ADDRESS_0, addr + 1);
 			}
@@ -189,7 +193,7 @@ byte V9990::readIO(byte port, const EmuTime& time)
 		case INTERRUPT_FLAG:
 			result = pendingIRQs;
 			break;
-		    
+
 		case STATUS: {
 			int ticks = getUCTicksThisFrame(time);
 			int x = UCtoX(ticks, getDisplayMode());
@@ -217,7 +221,7 @@ byte V9990::readIO(byte port, const EmuTime& time)
 			result = 0xFF;
 			break;
 	}
-	
+
 	//PRT_DEBUG("[" << time << "] "
 	//	  "V9990::readIO - port=0x" << std::hex << (int)port <<
 	//	                  " val=0x" << std::hex << (int)result);
@@ -225,25 +229,25 @@ byte V9990::readIO(byte port, const EmuTime& time)
 	return result;
 }
 
-byte V9990::peekIO(byte /*port*/, const EmuTime& /*time*/) const
+byte V9990::peekIO(word /*port*/, const EmuTime& /*time*/) const
 {
 	// TODO not implemented
 	return 0xFF;
 }
 
-void V9990::writeIO(byte port, byte val, const EmuTime& time)
+void V9990::writeIO(word port, byte val, const EmuTime& time)
 {
 	port &= 0x0F;
-	
+
 	//PRT_DEBUG("[" << time << "] "
-	//	  "V9990::writeIO - port=0x" << std::hex << int(port) << 
+	//	  "V9990::writeIO - port=0x" << std::hex << int(port) <<
 	//	                   " val=0x" << std::hex << int(val));
 
 	switch (port) {
 		case VRAM_DATA: {
 			// write VRAM
 			unsigned addr = getVRAMAddr(VRAM_WRITE_ADDRESS_0);
-			vram->writeVRAM(addr, val);
+			vram->writeVRAMSlow(addr, val);
 			if (!(regs[VRAM_WRITE_ADDRESS_2] & 0x80)) {
 				setVRAMAddr(VRAM_WRITE_ADDRESS_0, addr + 1);
 			}
@@ -281,7 +285,7 @@ void V9990::writeIO(byte port, byte val, const EmuTime& time)
 		case STATUS:
 			// read-only, ignore writes
 			break;
-		
+
 		case INTERRUPT_FLAG:
 			pendingIRQs &= ~val;
 			if (!(pendingIRQs & regs[INTERRUPT_0])) {
@@ -289,19 +293,19 @@ void V9990::writeIO(byte port, byte val, const EmuTime& time)
 			}
 			scheduleHscan(time);
 			break;
-		
+
 		case SYSTEM_CONTROL:
-			status = (status & 0xFB) | ((val & 1) << 2); 
+			status = (status & 0xFB) | ((val & 1) << 2);
 			syncAtNextLine(V9990_SET_MODE, time);
 		break;
-		
+
 		case KANJI_ROM_0:
 		case KANJI_ROM_1:
 		case KANJI_ROM_2:
 		case KANJI_ROM_3:
 			// not used in Gfx9000, ignore
 			break;
-			
+
 		default:
 			// ignore
 			break;
@@ -318,161 +322,116 @@ void V9990::writeIO(byte port, byte val, const EmuTime& time)
 
 void V9990::executeUntil(const EmuTime& time, int userData)
 {
-	PRT_DEBUG("[" << time << "] "
-	          "V9990::executeUntil - data=0x" << std::hex << userData);
+	//PRT_DEBUG("[" << time << "] "
+	//          "V9990::executeUntil - data=0x" << std::hex << userData);
 	switch (userData)  {
-		case V9990_VSYNC:
-			// Transition from one frame to the next
-			renderer->frameEnd(time);
-			frameStart(time);
-			break;
+	case V9990_VSYNC:
+		// Transition from one frame to the next
+		renderer->frameEnd(time);
+		frameStart(time);
+		break;
 
-		case V9990_DISPLAY_START:
-			if (isDisplayEnabled()) {
-				renderer->updateDisplayEnabled(true, time);
-			}
-			break;
+	case V9990_DISPLAY_START:
+		if (displayEnabled) {
+			renderer->updateDisplayEnabled(true, time);
+		}
+		isDisplayArea = true;
+		break;
 
-		case V9990_VSCAN:
-			if (isDisplayEnabled()) {
-				renderer->updateDisplayEnabled(false, time);
-			}
-			raiseIRQ(VER_IRQ);
-			break;
+	case V9990_VSCAN:
+		if (isDisplayEnabled()) {
+			renderer->updateDisplayEnabled(false, time);
+		}
+		isDisplayArea = false;
+		raiseIRQ(VER_IRQ);
+		break;
 
-		case V9990_HSCAN:
-			raiseIRQ(HOR_IRQ);
-			break;
+	case V9990_HSCAN:
+		raiseIRQ(HOR_IRQ);
+		break;
 
-		case V9990_SET_MODE:
-			calcDisplayMode();
-			renderer->setDisplayMode(getDisplayMode(), time);
-			renderer->setImageWidth(getImageWidth());
-			renderer->setColorMode(getColorMode(), time);
-			break;
+	case V9990_SET_MODE:
+		calcDisplayMode();
+		renderer->setDisplayMode(getDisplayMode(), time);
+		renderer->setColorMode(getColorMode(), time);
+		break;
 
-		default:
-			assert(false);
+	case V9990_SET_BLANK: {
+		bool newDisplayEnabled = regs[CONTROL] & 0x80;
+		if (isDisplayArea) {
+			renderer->updateDisplayEnabled(newDisplayEnabled, time);
+		}
+		displayEnabled = newDisplayEnabled;
+		break;
+	}
+	default:
+		assert(false);
 	}
 }
 
-const string& V9990::schedName() const
+const std::string& V9990::schedName() const
 {
-	static const string name("V9990");
-	PRT_DEBUG("[--now---] V9990::SchedName - \"" << name << "\"");
+	static const std::string name("V9990");
 	return name;
 }
 
 // -------------------------------------------------------------------------
-// EventListener
+// VideoSystemChangeListener
 // -------------------------------------------------------------------------
 
-bool V9990::signalEvent(const Event& event)
+void V9990::preVideoSystemChange()
 {
-	if (&event); // avoid warning
-	assert(event.getType() == OPENMSX_RENDERER_SWITCH2_EVENT);
-	const EmuTime& time = Scheduler::instance().getCurrentTime();
+	renderer.reset();
+}
+
+void V9990::postVideoSystemChange()
+{
+	const EmuTime& time = getMotherBoard().getScheduler().getCurrentTime();
 	createRenderer(time);
 	renderer->frameStart(time);
-	return true;
 }
 
 // -------------------------------------------------------------------------
 // V9990RegDebug
 // -------------------------------------------------------------------------
 
-V9990::V9990RegDebug::V9990RegDebug(V9990& parent_)
-	: parent(parent_)
+V9990RegDebug::V9990RegDebug(V9990& v9990_)
+	: SimpleDebuggable(v9990_.getMotherBoard(),
+	                   v9990_.getName() + " regs", "V9990 registers", 0x40)
+	, v9990(v9990_)
 {
 }
 
-unsigned V9990::V9990RegDebug::getSize() const
+byte V9990RegDebug::read(unsigned address)
 {
-	return 0x40;
+	return v9990.regs[address];
 }
 
-const string& V9990::V9990RegDebug::getDescription() const
+void V9990RegDebug::write(unsigned address, byte value, const EmuTime& time)
 {
-	static const string desc = "V9990 registers.";
-	return desc;
-}
-
-byte V9990::V9990RegDebug::read(unsigned address)
-{
-	return parent.regs[address];
-}
-
-void V9990::V9990RegDebug::write(unsigned address, byte value)
-{
-	const EmuTime& time = Scheduler::instance().getCurrentTime();
-	parent.writeRegister(address, value, time);
+	v9990.writeRegister(address, value, time);
 }
 
 // -------------------------------------------------------------------------
 // V9990PalDebug
 // -------------------------------------------------------------------------
 
-V9990::V9990PalDebug::V9990PalDebug(V9990& parent_)
-	: parent(parent_)
+V9990PalDebug::V9990PalDebug(V9990& v9990_)
+	: SimpleDebuggable(v9990_.getMotherBoard(),
+	                   v9990_.getName() + " palette",
+	                   "V9990 palette (format is R, G, B, 0).", 0x100)
+	, v9990(v9990_)
 {
 }
 
-unsigned V9990::V9990PalDebug::getSize() const
+byte V9990PalDebug::read(unsigned address)
 {
-	return 0x100;
+	return v9990.palette[address];
 }
 
-const string& V9990::V9990PalDebug::getDescription() const
+void V9990PalDebug::write(unsigned address, byte value, const EmuTime& time)
 {
-	static const string desc = "V9990 palette (format is R, G, B, 0).";
-	return desc;
-}
-
-byte V9990::V9990PalDebug::read(unsigned address)
-{
-	return parent.palette[address];
-}
-
-void V9990::V9990PalDebug::write(unsigned address, byte value)
-{
-	const EmuTime& time = Scheduler::instance().getCurrentTime();
-	parent.writePaletteRegister(address, value, time);
-}
-
-// -------------------------------------------------------------------------
-// V9990RegsCmd
-// -------------------------------------------------------------------------
-
-V9990::V9990RegsCmd::V9990RegsCmd(V9990& v9990_)
-	: v9990(v9990_)
-{
-}
-
-string V9990::V9990RegsCmd::execute(const vector<string>& /*tokens*/)
-{
-	// Print 55 registers in 4 colums
-	
-	static const int NCOLS = 4;
-	static const int NROWS = (55 + (NCOLS-1))/NCOLS;
-	
-	std::ostringstream out;
-	for(int row = 0; row < NROWS; row++) {
-		for(int col = 0; col < NCOLS; col++) {
-			int reg   = col * NROWS + row;
-			if(reg < 55) {
-				int value = v9990.regs[reg];
-				out << std::dec << std::setw(2) << reg << " : "
-			    	<< std::hex << std::setw(2) << value << "   ";
-			}
-		}
-		out << "\n";
-	}
-	return out.str();
-}
-
-string V9990::V9990RegsCmd::help(const vector<string>& /*tokens*/) const
-{
-	return "Prints the current state of the V9990 registers.\n";
+	v9990.writePaletteRegister(address, value, time);
 }
 
 // -------------------------------------------------------------------------
@@ -497,7 +456,7 @@ inline void V9990::setVRAMAddr(RegisterId base, unsigned addr)
 byte V9990::readRegister(byte reg, const EmuTime& time)
 {
 	// TODO sync(time) (if needed at all)
-	
+
 	assert(reg < 64);
 	byte result;
 	if (regAccess[reg] != NO_ACCESS && regAccess[reg] != WR_ONLY) {
@@ -512,9 +471,9 @@ byte V9990::readRegister(byte reg, const EmuTime& time)
 		result = 0xFF;
 	}
 
-	PRT_DEBUG("[" << time << "] "
-		  "V9990::readRegister - reg=0x" << std::hex << (int)reg <<
-		                       " val=0x" << std::hex << (int)result);
+	//PRT_DEBUG("[" << time << "] "
+	//	  "V9990::readRegister - reg=0x" << std::hex << (int)reg <<
+	//	                       " val=0x" << std::hex << (int)result);
 	return result;
 }
 
@@ -523,14 +482,23 @@ void V9990::syncAtNextLine(V9990SyncType type, const EmuTime& time)
 	int line = getUCTicksThisFrame(time) / V9990DisplayTiming::UC_TICKS_PER_LINE;
 	int ticks = (line + 1) * V9990DisplayTiming::UC_TICKS_PER_LINE;
 	EmuTime nextTime = frameStartTime + ticks;
-	Scheduler::instance().setSyncPoint(nextTime, *this, type);
+	setSyncPoint(nextTime, type);
 }
 
 void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 {
-	PRT_DEBUG("[" << time << "] "
-		  "V9990::writeRegister - reg=0x" << std::hex << int(reg) << 
-		                        " val=0x" << std::hex << int(val));
+	// Found this table by writing 0xFF to a register and reading
+	// back the value (only works for read/write registers)
+	static const byte regWriteMask[32] = {
+		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
+		0xFF, 0x87, 0xFF, 0x83, 0x0F, 0xFF, 0xFF, 0xFF,
+		0xFF, 0xFF, 0xDF, 0x07, 0xFF, 0xFF, 0xC1, 0x07,
+		0x3F, 0xCF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
+	};
+
+	//PRT_DEBUG("[" << time << "] "
+	//	  "V9990::writeRegister - reg=0x" << std::hex << int(reg) <<
+	//	                        " val=0x" << std::hex << int(val));
 
 	assert(reg < 64);
 	if ((regAccess[reg] == NO_ACCESS) || (regAccess[reg] == RD_ONLY)) {
@@ -541,9 +509,13 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 		cmdEngine->setCmdReg(reg, val, time);
 		return;
 	}
-	
+
+	val &= regWriteMask[reg];
+
 	byte change = regs[reg] ^ val;
-	if (!change) return;
+	// This optimization is not valid for the vertical scroll registers
+	// TODO is this optimization still useful for other registers?
+	//if (!change) return;
 
 	// Perform additional tasks before new value becomes active
 	switch (reg) {
@@ -552,6 +524,11 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 			// TODO verify this on real V9990
 			syncAtNextLine(V9990_SET_MODE, time);
 			break;
+		case CONTROL:
+			if (change & 0x80) {
+				syncAtNextLine(V9990_SET_BLANK, time);
+			}
+			break;
 		case PALETTE_CONTROL:
 			renderer->setColorMode(getColorMode(val), time);
 			break;
@@ -559,8 +536,10 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 			renderer->updateBackgroundColor(val & 63, time);
 			break;
 		case SCROLL_CONTROL_AY0:
+			renderer->updateScrollAYLow(time);
+			break;
 		case SCROLL_CONTROL_AY1:
-			renderer->updateScrollAY(time);
+			renderer->updateScrollAYHigh(time);
 			break;
 		case SCROLL_CONTROL_AX0:
 		case SCROLL_CONTROL_AX1:
@@ -622,8 +601,9 @@ void V9990::getPalette(int index, byte& r, byte& g, byte& b)
 
 void V9990::createRenderer(const EmuTime& time)
 {
-	renderer.reset(); // delete old renderer before creating new one
-	renderer.reset(RendererFactory::createV9990Renderer(this));
+	assert(!renderer.get());
+	Display& display = getMotherBoard().getDisplay();
+	renderer.reset(RendererFactory::createV9990Renderer(*this, display));
 	renderer->reset(time);
 }
 
@@ -634,26 +614,26 @@ void V9990::frameStart(const EmuTime& time)
 	setVerticalTiming();
 	status ^= 0x02; // flip EO bit
 
-	renderer->frameStart(time);
-	// Schedule next VSYNC
 	frameStartTime.advance(time);
-	
+
 	// schedule next VSYNC
-	Scheduler::instance().setSyncPoint(
+	setSyncPoint(
 		frameStartTime + V9990DisplayTiming::getUCTicksPerFrame(palTiming),
-		*this, V9990_VSYNC);
+		V9990_VSYNC);
 
 	// schedule DISPLAY_START
 	int topBorder = getVerticalTiming().blank + getVerticalTiming().border1;
-	Scheduler::instance().setSyncPoint(
+	setSyncPoint(
 	        frameStartTime + topBorder * V9990DisplayTiming::UC_TICKS_PER_LINE,
-	        *this, V9990_DISPLAY_START);
-	
+	        V9990_DISPLAY_START);
+
 	// schedule VSCAN
 	int bottomBorder = topBorder + getVerticalTiming().display;
-	Scheduler::instance().setSyncPoint(
+	setSyncPoint(
 	        frameStartTime + bottomBorder * V9990DisplayTiming::UC_TICKS_PER_LINE,
-	        *this, V9990_VSCAN);
+	        V9990_VSCAN);
+
+	renderer->frameStart(time);
 }
 
 void V9990::raiseIRQ(IRQType irqType)
@@ -710,7 +690,7 @@ V9990ColorMode V9990::getColorMode(byte pal_ctrl) const
 		switch (regs[SCREEN_MODE_0] & 0x03) {
 			case 0x00: mode = BP2; break;
 			case 0x01: mode = BP4; break;
-			case 0x02: 
+			case 0x02:
 				switch (pal_ctrl & 0xC0) {
 					case 0x00: mode = BP6; break;
 					case 0x40: mode = BD8; break;
@@ -777,10 +757,10 @@ void V9990::scheduleHscan(const EmuTime& time)
 {
 	// remove pending HSCAN, if any
 	if (hScanSyncTime > time) {
-		Scheduler::instance().removeSyncPoint(*this, V9990_HSCAN);
+		removeSyncPoint(V9990_HSCAN);
 		hScanSyncTime = time;
 	}
-	
+
 	if (pendingIRQs & HOR_IRQ) {
 		// flag already set, no need to schedule
 		return;
@@ -803,7 +783,7 @@ void V9990::scheduleHscan(const EmuTime& time)
 	}
 
 	hScanSyncTime = frameStartTime + offset;
-	Scheduler::instance().setSyncPoint(hScanSyncTime, *this, V9990_HSCAN);
+	setSyncPoint(hScanSyncTime, V9990_HSCAN);
 }
 
 } // namespace openmsx
