@@ -1,23 +1,12 @@
-// $Id: V9990DummyRasterizer.cc,v 1.7 2005/04/12 20:11:40 m9710797 Exp $
+// $Id: V9990DummyRasterizer.cc,v 1.10 2005/11/14 21:18:06 m9710797 Exp $
 
 #include "V9990DummyRasterizer.hh"
 
-using std::string;
-
 namespace openmsx {
 
-void V9990DummyRasterizer::paint()
+bool V9990DummyRasterizer::isActive()
 {
-}
-
-V9990DummyRasterizer::~V9990DummyRasterizer()
-{
-}
-
-const string& V9990DummyRasterizer::getName()
-{
-	static const string NAME = "V9990DummyRasterizer";
-	return NAME;
+	return false;
 }
 
 void V9990DummyRasterizer::reset()
@@ -45,10 +34,6 @@ void V9990DummyRasterizer::setPalette(
 {
 }
 
-void V9990DummyRasterizer::setImageWidth(int /*width*/)
-{
-}
-
 void V9990DummyRasterizer::drawBorder(int /*fromX*/, int /*fromY*/,
                                       int /*limitX*/, int /*limitY*/)
 {
