@@ -1,4 +1,4 @@
-// $Id: V9990DummyRenderer.cc,v 1.7 2005/04/13 19:51:55 m9710797 Exp $
+// $Id: V9990DummyRenderer.cc,v 1.9 2006/06/03 10:22:08 m9710797 Exp $
 
 #include "V9990DummyRenderer.hh"
 
@@ -46,14 +46,13 @@ void V9990DummyRenderer::updateBackgroundColor(
 {
 }
 
-void V9990DummyRenderer::setImageWidth(int /*width*/)
-{
-}
-
 void V9990DummyRenderer::updateScrollAX(const EmuTime& /*time*/)
 {
 }
-void V9990DummyRenderer::updateScrollAY(const EmuTime& /*time*/)
+void V9990DummyRenderer::updateScrollAYLow(const EmuTime& /*time*/)
+{
+}
+void V9990DummyRenderer::updateScrollAYHigh(const EmuTime& /*time*/)
 {
 }
 void V9990DummyRenderer::updateScrollBX(const EmuTime& /*time*/)
