@@ -1,9 +1,9 @@
-// $Id: V9990PixelRenderer.cc,v 1.17 2005/04/13 19:51:55 m9710797 Exp $
+// $Id: V9990PixelRenderer.cc,v 1.34 2006/06/03 10:22:08 m9710797 Exp $
 
+#include "V9990PixelRenderer.hh"
 #include "V9990.hh"
 #include "V9990VRAM.hh"
 #include "V9990DisplayTiming.hh"
-#include "V9990PixelRenderer.hh"
 #include "V9990Rasterizer.hh"
 #include "Scheduler.hh"
 #include "Display.hh"
@@ -12,95 +12,93 @@
 #include "RealTime.hh"
 #include "Timer.hh"
 #include "EventDistributor.hh"
+#include "MSXMotherBoard.hh"
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
-#include "openmsx.hh"
-
 
 namespace openmsx {
 
-V9990PixelRenderer::V9990PixelRenderer(V9990* vdp_)
+V9990PixelRenderer::V9990PixelRenderer(V9990& vdp_)
 	: vdp(vdp_)
-	, rasterizer(Display::instance().getVideoSystem()
-	                                .createV9990Rasterizer(vdp_))
+	, eventDistributor(vdp.getMotherBoard().getEventDistributor())
+	, realTime(vdp.getMotherBoard().getRealTime())
+	, renderSettings(vdp.getMotherBoard().getDisplay().getRenderSettings())
+	, rasterizer(vdp.getMotherBoard().getDisplay().getVideoSystem().
+	                createV9990Rasterizer(vdp))
 {
 	frameSkipCounter = 999; // force drawing of frame;
 	finishFrameDuration = 0;
 	drawFrame = false; // don't draw before frameStart is called
-	
-	reset(Scheduler::instance().getCurrentTime());
 
-	settings.getMaxFrameSkip()->addListener(this);
-	settings.getMinFrameSkip()->addListener(this);
+	reset(vdp.getMotherBoard().getScheduler().getCurrentTime());
+
+	renderSettings.getMaxFrameSkip().attach(*this);
+	renderSettings.getMinFrameSkip().attach(*this);
 }
 
 V9990PixelRenderer::~V9990PixelRenderer()
 {
-	settings.getMaxFrameSkip()->removeListener(this);
-	settings.getMinFrameSkip()->removeListener(this);
+	renderSettings.getMaxFrameSkip().detach(*this);
+	renderSettings.getMinFrameSkip().detach(*this);
 }
 
 void V9990PixelRenderer::reset(const EmuTime& time)
 {
-	PRT_DEBUG("V9990PixelRenderer::reset");
-
-	displayEnabled = vdp->isDisplayEnabled();
-	setDisplayMode(vdp->getDisplayMode(), time);
-	setColorMode(vdp->getColorMode(), time);
+	displayEnabled = vdp.isDisplayEnabled();
+	setDisplayMode(vdp.getDisplayMode(), time);
+	setColorMode(vdp.getColorMode(), time);
 
 	rasterizer->reset();
 }
 
 void V9990PixelRenderer::frameStart(const EmuTime& time)
 {
-	if (rasterizer->getZ() == Layer::Z_MSX_PASSIVE) {
-		// V99x8 is active
+	if (!rasterizer->isActive()) {
 		frameSkipCounter = 0;
 		drawFrame = false;
-	} else if (frameSkipCounter < settings.getMinFrameSkip()->getValue()) {
+	} else if (frameSkipCounter < renderSettings.getMinFrameSkip().getValue()) {
 		++frameSkipCounter;
 		drawFrame = false;
-	} else if (frameSkipCounter >= settings.getMaxFrameSkip()->getValue()) {
+	} else if (frameSkipCounter >= renderSettings.getMaxFrameSkip().getValue()) {
 		frameSkipCounter = 0;
 		drawFrame = true;
 	} else {
 		++frameSkipCounter;
-		drawFrame = RealTime::instance().timeLeft(
-			(unsigned)finishFrameDuration, time);
+		drawFrame = realTime.timeLeft((unsigned)finishFrameDuration, time);
 		if (drawFrame) {
 			frameSkipCounter = 0;
 		}
 	}
 	if (!drawFrame) return;
-	
-	accuracy = settings.getAccuracy()->getValue();
+
+	accuracy = renderSettings.getAccuracy().getValue();
 	lastX = 0;
 	lastY = 0;
+	const V9990DisplayPeriod& verTiming = vdp.getVerticalTiming();
+	verticalOffset = verTiming.blank + verTiming.border1;
 
 	// Make sure that the correct timing is used
-	setDisplayMode(vdp->getDisplayMode(), time);
+	setDisplayMode(vdp.getDisplayMode(), time);
 	rasterizer->frameStart();
 }
 
 void V9990PixelRenderer::frameEnd(const EmuTime& time)
 {
-	PRT_DEBUG("V9990PixelRenderer::frameEnd");
-	
-	if (!drawFrame) return;
+	if (drawFrame) {
+		// Render last changes in this frame before starting a new frame
+		sync(time, true);
 
-	// Render last changes in this frame before starting a new frame
-	sync(time, true);
-	
-	unsigned long long time1 = Timer::getTime();
-	rasterizer->frameEnd();
-	unsigned long long time2 = Timer::getTime();
-	unsigned long long current = time2 - time1;
-	const double ALPHA = 0.2;
-	finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
-	                      current * ALPHA;
-	
-	FinishFrameEvent* f = new FinishFrameEvent(VIDEO_GFX9000);
-	EventDistributor::instance().distributeEvent(f);
+		unsigned long long time1 = Timer::getTime();
+		rasterizer->frameEnd();
+		unsigned long long time2 = Timer::getTime();
+		unsigned long long current = time2 - time1;
+		const double ALPHA = 0.2;
+		finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
+		                      current * ALPHA;
+	}
+	eventDistributor.distributeEvent(
+		new FinishFrameEvent(VIDEO_GFX9000, !drawFrame)
+		);
 }
 
 void V9990PixelRenderer::sync(const EmuTime& time, bool force)
@@ -108,35 +106,31 @@ void V9990PixelRenderer::sync(const EmuTime& time, bool force)
 	if (!drawFrame) return;
 
 	if (accuracy != RenderSettings::ACC_SCREEN || force) {
-		vdp->getVRAM()->sync(time);
+		vdp.getVRAM().sync(time);
 		renderUntil(time);
 	}
 }
 
 void V9990PixelRenderer::renderUntil(const EmuTime& time)
 {
-	const V9990DisplayPeriod& horTiming = vdp->getHorizontalTiming();
-	const V9990DisplayPeriod& verTiming = vdp->getVerticalTiming();
+	const V9990DisplayPeriod& horTiming = vdp.getHorizontalTiming();
 
 	// Translate time to pixel position
-	int limitTicks = vdp->getUCTicksThisFrame(time);
+	int limitTicks = vdp.getUCTicksThisFrame(time);
 	assert(limitTicks <=
-	       V9990DisplayTiming::getUCTicksPerFrame(vdp->isPalTiming()));
+	       V9990DisplayTiming::getUCTicksPerFrame(vdp.isPalTiming()));
 	int toX, toY;
 	switch (accuracy) {
 	case RenderSettings::ACC_PIXEL:
-		toX = limitTicks % V9990DisplayTiming::UC_TICKS_PER_LINE -
-		      horTiming.blank;
-		toY = limitTicks / V9990DisplayTiming::UC_TICKS_PER_LINE -
-		      verTiming.blank;
+		toX = limitTicks % V9990DisplayTiming::UC_TICKS_PER_LINE;
+		toY = limitTicks / V9990DisplayTiming::UC_TICKS_PER_LINE;
 		break;
 	case RenderSettings::ACC_LINE:
 	case RenderSettings::ACC_SCREEN:
 		// TODO figure out rounding point
 		toX = 0;
 		toY = (limitTicks + V9990DisplayTiming::UC_TICKS_PER_LINE - 400) /
-		             V9990DisplayTiming::UC_TICKS_PER_LINE -
-		      verTiming.blank;
+		             V9990DisplayTiming::UC_TICKS_PER_LINE;
 		break;
 	default:
 		assert(false);
@@ -144,11 +138,11 @@ void V9990PixelRenderer::renderUntil(const EmuTime& time)
 	}
 
 	if ((toX == lastX) && (toY == lastY)) return;
-	
+
 	// edges of the DISPLAY part of the vdp output
-	int left       = horTiming.border1;
+	int left       = horTiming.blank + horTiming.border1;
 	int right      = left   + horTiming.display;
-	int rightEdge  = right  + horTiming.border2;
+	int rightEdge  = V9990DisplayTiming::UC_TICKS_PER_LINE;
 
 	if (displayEnabled) {
 		// left border
@@ -190,7 +184,7 @@ void V9990PixelRenderer::subdivide(int fromX, int fromY, int toX, int toY,
 	if (fromY < toY) {
 		draw(clipL, fromY, clipR, toY, drawType);
 	}
-	
+
 	// partial last line
 	if (drawLast) draw(clipL, toY, toX, toY + 1, drawType);
 }
@@ -198,21 +192,19 @@ void V9990PixelRenderer::subdivide(int fromX, int fromY, int toX, int toY,
 void V9990PixelRenderer::draw(int fromX, int fromY, int toX, int toY,
                               DrawType type)
 {
-	PRT_DEBUG("V9990PixelRenderer::draw(" << std::dec <<
-	          fromX << "," << fromY << "," << toX << "," << toY << "," <<
-	          ((type == DRAW_BORDER)? "BORDER": "DISPLAY") << ")");
-	
+	//PRT_DEBUG("V9990PixelRenderer::draw(" << std::dec <<
+	//          fromX << "," << fromY << "," << toX << "," << toY << "," <<
+	//          ((type == DRAW_BORDER)? "BORDER": "DISPLAY") << ")");
+
 	if (type == DRAW_BORDER) {
 		rasterizer->drawBorder(fromX, fromY, toX, toY);
 
 	} else {
 		assert(type == DRAW_DISPLAY);
-		
-		const V9990DisplayPeriod& horTiming = vdp->getHorizontalTiming();
-		const V9990DisplayPeriod& verTiming = vdp->getVerticalTiming();
 
-		int displayX = fromX - horTiming.border1;
-		int displayY = fromY - verTiming.border1;
+		const V9990DisplayPeriod& horTiming = vdp.getHorizontalTiming();
+		int displayX = fromX - horTiming.blank - horTiming.border1;
+		int displayY = fromY - verticalOffset;
 		int displayWidth = toX - fromX;
 		int displayHeight = toY - fromY;
 
@@ -255,19 +247,23 @@ void V9990PixelRenderer::updateBackgroundColor(int /*index*/, const EmuTime& tim
 	sync(time);
 }
 
-void V9990PixelRenderer::setImageWidth(int width)
-{
-	if (displayEnabled) rasterizer->setImageWidth(width);
-}
-
 void V9990PixelRenderer::updateScrollAX(const EmuTime& time)
 {
 	if (displayEnabled) sync(time);
 }
-void V9990PixelRenderer::updateScrollAY(const EmuTime& time)
+void V9990PixelRenderer::updateScrollAYHigh(const EmuTime& time)
 {
 	if (displayEnabled) sync(time);
 }
+void V9990PixelRenderer::updateScrollAYLow(const EmuTime& time)
+{
+	if (displayEnabled) {
+		sync(time);
+		if (vdp.getDisplayMode() != P1) {
+			verticalOffset = lastY;
+		}
+	}
+}
 void V9990PixelRenderer::updateScrollBX(const EmuTime& time)
 {
 	// TODO only in P1 mode
@@ -279,10 +275,10 @@ void V9990PixelRenderer::updateScrollBY(const EmuTime& time)
 	if (displayEnabled) sync(time);
 }
 
-void V9990PixelRenderer::update(const Setting* setting)
+void V9990PixelRenderer::update(const Setting& setting)
 {
-	if (setting == settings.getMinFrameSkip() ||
-	    setting == settings.getMaxFrameSkip()) {
+	if (&setting == &renderSettings.getMinFrameSkip() ||
+	    &setting == &renderSettings.getMaxFrameSkip()) {
 		// Force drawing of frame
 		frameSkipCounter = 999;
 	} else {
