@@ -1,7 +1,6 @@
-// $Id: V9990Renderer.cc,v 1.5 2005/02/28 18:57:38 m9710797 Exp $
+// $Id: V9990Renderer.cc,v 1.7 2005/08/27 22:06:01 m9710797 Exp $
 
 #include "V9990Renderer.hh"
-#include "RenderSettings.hh"
 
 namespace openmsx {
 
@@ -9,14 +8,13 @@ namespace openmsx {
   * methods mentioned here...
   */
 V9990Renderer::V9990Renderer()
-	: settings(RenderSettings::instance())
 {
 }
 
 V9990Renderer::~V9990Renderer()
 {
 }
-	
+
 void V9990Renderer::reset(const EmuTime& /*time*/)
 {
 }
